
#------------------------------------------------------------------------------
# STV_BIN_DIR: See if the user gave us the location of Cumulvs bin
# directory
# one must be present and contain the correct files, or else the user
# must give the directory that contains the correct files.
#------------------------------------------------------------------------------
# STV bin directory, must contain stvmk.in
AC_DEFUN(STV_FIND_BIN_DIR,
[

#  CUMULVS bin search paths
stv_bin_dirs="
    $STV_ROOT/bin    
    $prefix/CUMULVS/bin
    /usr/local/CUMULVS/bin
    $HOME/projects/CUMULVS/bin
    $CURRDIR_PARENT/bin
    $CURRDIR_GRANDPARENT/bin
    $CURRDIR_GREATGRANDPARENT/bin
    $CURRDIR_GREATGRANDPARENT/../bin
    $CURRDIR_ANCESTOR/bin
    $CURRDIR/bin    
"

AC_ARG_WITH(stv_bin, 
    [Location of the CUMULVS bin dir:
  --with-stv_bin=STV_BIN_DIR
                          The location of CUMULVS bin directory.
                          Give the full path:
                              --with-stv_bin='/usr/local/CUMULVS/bin'
                          ] ,
    , [with_stv_bin=no])

case "$with_stv_bin" in
    no)
        # lUser input error
        with_stv_bin=_searching
        ;;
    yes)
        #  User didn't give the option or didn't give useful
        #  information, search for it ourselves
        with_stv_bin=_searching
        ;;
    *)
        STV_BIN_DIR="$with_stv_bin"
        ;;
esac

if test "$with_stv_bin" = _searching ; then
    AC_MSG_CHECKING([for CUMULVS bin directory])
    AC_MSG_RESULT([])
    stv_bin_dir_found=0
    for chome in $stv_bin_dirs ; do
        if test -d $chome ; then
	    AC_MSG_CHECKING([for stvmk in $chome])
	    if test ! -f "$chome/stvmk"  ; then
		AC_MSG_RESULT([no.])
	    else
		AC_MSG_RESULT([yes.])
		stv_bin_dir_found=1
		with_stv_bin=$chome
	    fi

	    if test "$stv_bin_dir_found" = 1; then
	        break
	    fi
        fi
    done
fi
if test "$with_stv_bin" = _searching ; then
    AC_MSG_RESULT([Cannot find Cumulvs bin directory.])
    AC_MSG_ERROR([Please reinstall the CUMULVS rpm or source and/or use --with-stv_bin=STV_ROOT/bin.])
fi

if test ! -f "$with_stv_bin/stvmk"  ; then
    AC_MSG_RESULT([No stvmk in $with_stv_bin.])
    AC_MSG_ERROR([Please reinstall the CUMULVS rpm or source and/or use --with-stv_bin=STV_ROOT/bin.])
fi

STV_BIN_DIR="$with_stv_bin"
AC_SUBST(STV_BIN_DIR)
]
)
###########################################################################
# END: STV FIND BIN DIR M4 MACRO					  #
###########################################################################

