#------------------------------------------------------------------------------
# STV_CONFIG_DIR: See if the user gave us the location of Cumulvs config
# directory
# one must be present and contain the correct files, or else the user
# must give the directory that contains the correct files.
#------------------------------------------------------------------------------
# STV configure directory, must contain install-sh
AC_DEFUN(STV_FIND_CONFIG_DIR,
[

#  CUMULVS config search paths
stv_inst_config_dirs="
    $STV_ROOT/config    
    $prefix/CUMULVS/config
    /usr/local/CUMULVS/config
    $HOME/projects/CUMULVS/config
    $CURRDIR_PARENT/config
    $CURRDIR_GRANDPARENT/config
    $CURRDIR_GREATGRANDPARENT/config
    $CURRDIR_GREATGRANDPARENT/../config
    $CURRDIR_ANCESTOR/config
    $CURRDIR/config    
"

AC_ARG_WITH(stv_inst_config, 
    [Location of the CUMULVS config dir:
  --with-stv_inst_config=STV_CONFIG_DIR
                          The location of CUMULVS config directory.
                          Give the full path:
                              --with-cca='/usr/local/CUMULVS/config'
                          ] ,
    , [with_stv_inst_config=no])

case "$with_stv_inst_config" in
    no)
        # lUser input error
        with_stv_inst_config=_searching
        ;;
    yes)
        #  User didn't give the option or didn't give useful
        #  information, search for it ourselves
        with_stv_inst_config=_searching
        ;;
    *)
        STV_INST_CONFIG="$with_stv_inst_config"
        ;;
esac

if test "$with_stv_inst_config" = _searching ; then
    AC_MSG_CHECKING([for CUMULVS config directory])
    AC_MSG_RESULT([])
    stv_inst_config_dir_found=0
    for chome in $stv_inst_config_dirs ; do
        if test -d $chome ; then
	    AC_MSG_CHECKING([for install-sh in $chome])
	    if test ! -f "$chome/install-sh"  ; then
		AC_MSG_RESULT([no.])
	    else
		AC_MSG_RESULT([yes.])
		stv_inst_config_dir_found=1
		with_stv_inst_config=$chome
	    fi

	    if test "$stv_inst_config_dir_found" = 1; then
	        break
	    fi
        fi
    done
fi
if test "$with_stv_inst_config" = _searching ; then
    AC_MSG_RESULT([Cannot find Cumulvs config directory.])
    AC_MSG_ERROR([Please reinstall the CUMULVS rpm or source and/or use --with-stv_inst_config=STV_INST_ROOT/config.])
fi

if test ! -f "$with_stv_inst_config/install-sh"  ; then
    AC_MSG_RESULT([No install-sh in $with_stv_inst_config.])
    AC_MSG_ERROR([Please reinstall the CUMULVS rpm or source and/or use --with-stv_inst_config=STV_INST_ROOT/config.])
fi

STV_INST_CONFIG="$with_stv_inst_config"
AC_SUBST(STV_INST_CONFIG)
]
)
