#------------------------------------------------------------------------------
#  Find CUMULVS library infos
#------------------------------------------------------------------------------
AC_DEFUN(STV_FIND_LIB_CUMULVS,
[

AC_MSG_CHECKING([for CUMULVS rpm or source distribution !])
AC_MSG_RESULT([])
AC_MSG_RESULT([    (override default order using --with-cumulvs-source option)]) 

AC_ARG_WITH(cumulvs-source, 
        [Overwrite default, use source tree only:
      --with-cumulvs-source=yes
                          To use only source tree distribution
                          Select yes: e.g.
                              --with-cumulvs-source=yes ] ,
        , [with_cumulvs_source=not_selected])

    case "$with_cumulvs_source" in
        no)
            ;;
        yes)
            ;;
        *)
            ;;
    esac

    if test "$with_cumulvs_source" = yes; then
	STV_FIND_SOURCE_ROOT_DIR
	if test "$STV_SOURCE_DIST_FOUND" = 0; then
	    AC_MSG_ERROR([NO source distribution found. Make sure you installed the source tree correctly!])
	fi

	STV_FIND_CUMULVS
	if test "$STV_FOUND" = 0; then
	    AC_MSG_ERROR([NO source distribution found. Make sure that you build the source tree first!])
	fi
    else
	STV_FIND_RPM_ROOT_DIR_NB
	if test "$STV_RPM_DIST_FOUND" = 1; then
	    STV_FIND_CUMULVS
	    if test "$STV_FOUND" = 0; then
		if test "$with_cumulvs_source" = no; then
		    AC_MSG_ERROR([NO valid RPM distribution found. Please install a valid distribution first!])
		else
		    STV_FIND_SOURCE_ROOT_DIR
		    if test "$STV_SOURCE_DIST_FOUND" = 0; then
	    	    	AC_MSG_ERROR([NO valid RPM distribution found. Please install a valid distribution first!])
		    else
		    	STV_FIND_CUMULVS
		    	if test "$STV_FOUND" = 0; then
			    AC_MSG_ERROR([NO valid RPM and SOURCE distribution found. Please install a valid distribution first!])
		    	fi
		    fi
	    	fi
	    fi
	else
	    if test "$with_cumulvs_source" = no; then
		AC_MSG_ERROR([NO RPM distribution found. Please install a valid distribution first!])
	    else
		STV_FIND_SOURCE_ROOT_DIR
	    	if test "$STV_SOURCE_DIST_FOUND" = 0; then
		    AC_MSG_ERROR([NO RPM and SOURCE distribution found. Please install a valid distribution first!])
	    	else
		    STV_FIND_CUMULVS
		    if test "$STV_FOUND" = 0; then
		    	AC_MSG_ERROR([NO valid RPM or SOURCE distribution found. Please install a valid distribution first!])
		    fi
	    	fi
	    fi
	fi
    fi

]
)


AC_DEFUN(STV_FIND_CUMULVS,
[
# requires STV_SOURCE_DIST_FOUND from stv_find_rpm_root_dir.m4
# or stv_find_source_root_dir.m4

# requires also STV_FIND_SOURCE_ROOT_DIR or STV_FIND_RPM_ROOT_DIR)
AC_REQUIRE([STV_LIB_PVM])
AC_REQUIRE([STV_LIB_MPI])

stv_found=0;

# if source tree distribution found, set search paths accordingly
if test ! -z "$STV_SOURCE_DIST_FOUND"; then
#echo "STV_SOURCE_DIST_FOUND is not ZERO"
    if test "$STV_SOURCE_DIST_FOUND" = 1; then
    	AC_MSG_RESULT([Found CUMULVS source tree in $STV_ROOT])

	#  CUMULVS static libraries search paths
    	stv_static_lib_dirs="$STV_ROOT/src/${STV_ARCH}"
    	stv_static_cpd_lib_dirs="$STV_ROOT/cpd/${STV_ARCH}"

    	#  CUMULVS shared libraries search paths
    	stv_shared_lib_dirs="$STV_ROOT/src/${STV_ARCH}"
    	stv_shared_cpd_lib_dirs="$STV_ROOT/cpd/${STV_ARCH}"

    	#  CUMULVS include search paths
    	stv_include_dirs="$STV_ROOT/include"

    	#  CUMULVS make include search paths
    	stv_make_incl_dirs="$STV_ROOT/src/${STV_ARCH}"
    else
    	AC_MSG_RESULT([Found CUMULVS RPM installation in $STV_ROOT])
    	#  CUMULVS static libraries search paths
    	stv_static_lib_dirs="
    		/usr/local/lib
		$prefix/lib
    	"
    	#  CUMULVS shared libraries search paths
    	stv_shared_lib_dirs="
		/usr/local/lib
    		$prefix/lib
    	"
    	#  CUMULVS include search paths
    	stv_include_dirs="
    		/usr/local/include
    		$prefix/include
    	"
    	#  CUMULVS make include search paths
    	stv_make_incl_dirs="
    		/usr/local/CUMULVS/makeincl
    		$prefix/CUMULVS/makeincl
    	"
    fi
else
    AC_MSG_RESULT([Assuming CUMULVS RPM installation])
    #  CUMULVS static libraries search paths
    stv_static_lib_dirs="
    	/usr/local/lib
	$prefix/lib
    "
    #  CUMULVS shared libraries search paths
    stv_shared_lib_dirs="
	/usr/local/lib
    	$prefix/lib
    "
    #  CUMULVS include search paths
    stv_include_dirs="
    	/usr/local/include
    	$prefix/include
    "
    #  CUMULVS make include search paths
    stv_make_incl_dirs="
    	/usr/local/CUMULVS/makeincl
    	$prefix/CUMULVS/makeincl
    "
fi

    # 1. check for static libraries
    AC_ARG_WITH(cumulvs-static-libs, 
        [Location of CUMULVS libraries:
      --with-cumulvs-static-libs=CUMULVS_LIBRARIES
                          The location of the CUMULVS static libraries
                          Give the full path: e.g.
                              --with-cumulvs-static-libs=/usr/local/lib ] ,
        , [with_cumulvs_static_libs=yes])

    case "$with_cumulvs_static_libs" in
        no)
            # lUser input error
            with_cumulvs_static_libs=_searching
            ;;
        yes)
            #  User didn't give the option or didn't give useful
            #  information, search for it ourselves
            with_cumulvs_static_libs=_searching
            ;;
        *)
            STV_STATIC_LIB_DIR="$with_cumulvs_static_libs"
            ;;
    esac

    if test "$with_cumulvs_static_libs" = _searching ; then
    	AC_MSG_CHECKING([for CUMULVS static libraries])
    	AC_MSG_RESULT([])
    	stv_static_libs_found=0
    	for chome in $stv_static_lib_dirs ; do
	    if test "$source_dist_found" = 1; then
		dhome=$stv_static_cpd_lib_dirs
	    else
		dhome=$chome
	    fi

	    if test -d $chome ; then
	    	# check for static libraries
            	AC_MSG_CHECKING([for CUMULVS static libraries in $chome ])
	    	if test -f $chome/libstv.a;then
		    if test -f $chome/libfstv.a; then
	    	        if test -f $dhome/libcpd.a;then
	    	    	    if test -f $dhome/libfcpd.a;then
			        AC_MSG_RESULT([yes.])
				stv_static_libs_found=1
			    else
		    	        AC_MSG_RESULT([no. (couldn't find libfcpd.a)])
			    fi
		    	else
			    AC_MSG_RESULT([no. (couldn't find libcpd.a)])
			fi
		    else
		    	AC_MSG_RESULT([no. (couldn't find libfstv.a)])
		    fi
	    	else
		    AC_MSG_RESULT([no. (couldn't find libstv.a)])
	    	fi

            	if test "$stv_static_libs_found" = 1; then
		    with_cumulvs_static_libs=$chome
            	    break
	    	fi
	    fi
        done
    fi

    if test "$with_cumulvs_static_libs" = _searching ; then
    	AC_MSG_RESULT([Cannot find CUMULVS static libraries.])
	AC_MSG_RESULT([if CUMULVS source tree was used, please build it first])
   	AC_MSG_RESULT([or install CUMULVS RPM and/or use:])
    	AC_MSG_RESULT([ --with-cumulvs-static-libs=CUMULVS_STATIC_LIBS_DIR or])
    	AC_MSG_RESULT([ --with-cumulvs=CUMULVS_ROOT_DIR for source installation])
    	with_cumulvs_static_libs="no_static_libs_found"
    else
	stv_static_libs_found=1
    fi

    STV_STATIC_LIB_DIR="$with_cumulvs_static_libs"
    AC_SUBST(STV_STATIC_LIB_DIR)


    # 2. check for shared libraries
    AC_ARG_WITH(cumulvs-shared-libs, 
    	[Location of CUMULVS libraries:
  	--with-cumulvs-shared-libs=CUMULVS_LIBRARIES
                          The location of the CUMULVS shared libraries
                          Give the full path: e.g.
                              --with-cumulvs-shared-libs=/usr/local/lib ] ,
    	, [with_cumulvs_shared_libs=yes])

    case "$with_cumulvs_shared_libs" in
    	no)
            # lUser input error
            with_cumulvs_shared_libs=_searching
            ;;
    	yes)
            #  User didn't give the option or didn't give useful
            #  information, search for it ourselves
	    with_cumulvs_shared_libs=_searching
	    ;;
    	*)
	    STV_SHARED_LIB_DIR="$with_cumulvs_shared_libs"
            ;;
    esac

    if test "$with_cumulvs_shared_libs" = _searching ; then
    	AC_MSG_CHECKING([for CUMULVS shared libraries ])
	AC_MSG_RESULT([])
    	stv_shared_libs_found=0
    	for chome in $stv_shared_lib_dirs ; do
	    if test "$source_dist_found" = 1; then
		dhome=$stv_shared_cpd_lib_dirs
	    else
		dhome=$chome
	    fi
	    if test -d $chome ; then
	    	# check for shared libraries
            	AC_MSG_CHECKING([for CUMULVS shared libraries in $chome ])
	    	if test -f $chome/libstv.so;then
		    if test -f $chome/libfstv.so; then
			if test -f $dhome/libcpd.so;then
			    if test -f $dhome/libcpd.so;then
			    	AC_MSG_RESULT([yes.])
				stv_shared_libs_found=1
			    else
		    	    	AC_MSG_RESULT([no. (couldn't find libfcpd.so)])
			    fi
		    	else
		    	    AC_MSG_RESULT([no. (couldn't find libcpd.so)])
			fi
		    else
		    	AC_MSG_RESULT([no. (couldn't find libfstv.so)])
		    fi
	    	else
		    AC_MSG_RESULT([no. (couldn't find libstv.so)])
	    	fi

	    	if test "$stv_shared_libs_found" = 1; then
		    with_cumulvs_shared_libs=$chome
            	    break
	    	fi
	    fi
    	done
    fi

    if test "$with_cumulvs_shared_libs" = _searching ; then
    	AC_MSG_RESULT([Cannot find CUMULVS shared libraries.])
	AC_MSG_RESULT([if CUMULVS source tree was used, please build it first])
   	AC_MSG_RESULT([or install CUMULVS RPM and/or use:])
    	AC_MSG_RESULT([ --with-cumulvs-shared-libs=CUMULVS_SHARED_LIBS_DIR or])
    	AC_MSG_RESULT([ --with-cumulvs=CUMULVS_ROOT_DIR for source installation])
    	with_cumulvs_shared_libs="no_shared_libs_found"
    else
	stv_shared_libs_found=1
    fi

    STV_SHARED_LIB_DIR="$with_cumulvs_shared_libs"
    AC_SUBST(STV_SHARED_LIB_DIR)

    if test "$stv_shared_libs_found" = 0 -a "$stv_static_libs_found" = 0; then
    	AC_MSG_RESULT([Cannot find CUMULVS static and shared libraries.])
	AC_MSG_RESULT([if CUMULVS source tree was used, please build it first])
   	AC_MSG_RESULT([or install CUMULVS RPM and/or use:])
    	AC_MSG_RESULT([ --with-cumulvs-shared-libs=CUMULVS_SHARED_LIBS_DIR or])
	AC_MSG_RESULT([ --with-cumulvs-static-libs=CUMULVS_SHARED_LIBS_DIR or])
    	AC_MSG_RESULT([ --with-cumulvs=CUMULVS_ROOT_DIR for source installation])
	stv_found=0
	STV_FOUND="$stv_found"
    	AC_SUBST(STV_FOUND)
    fi

    # 3. check for include path
    AC_ARG_WITH(cumulvs-incl, 
    	[Location of CUMULVS include files:
  	--with-cumulvs-incl=CUMULVS_INCLUDE
                          The location of the CUMULVS include files
                          Give the full path: e.g.
                              --with-cumulvs-incl=/usr/local/include ] ,
    	, [with_cumulvs_incl=yes])

    case "$with_cumulvs_incl" in
    	no)
            # lUser input error
            with_cumulvs_incl=_searching
            ;;
    	yes)
            #  User didn't give the option or didn't give useful
            #  information, search for it ourselves
            with_cumulvs_incl=_searching
            ;;
    	*)
	    STV_INCL_DIR="$with_cumulvs_incl"
            ;;
    esac

    if test "$with_cumulvs_incl" = _searching ; then
    	AC_MSG_CHECKING([for CUMULVS include files])
    	AC_MSG_RESULT([])
    	stv_incl_found=0
    	for chome in $stv_include_dirs ; do
	    if test -d $chome ; then
	    	# check for include files
            	AC_MSG_CHECKING([$chome for stv.h])
	    	if test -f $chome/stv.h;then
		    AC_MSG_RESULT([yes.])
            	    AC_MSG_CHECKING([$chome for stv_type.h])
	    	    if test -f $chome/stv_type.h;then
            	        AC_MSG_RESULT([yes.])
			AC_MSG_CHECKING([$chome for stv_proto.h])
	    		if test -f $chome/stv_proto.h;then
            	    	    AC_MSG_RESULT([yes.])
			    AC_MSG_CHECKING([$chome for fstv.h])
	    		    if test -f $chome/fstv.h;then
            	    		AC_MSG_RESULT([yes.])
				stv_incl_found=1
	    		    else
		    		AC_MSG_RESULT([no.])
	    		    fi
	    		else
		    	    AC_MSG_RESULT([no.])
	    		fi
	    	    else
			AC_MSG_RESULT([no.])
	    	    fi
	    	else
		    AC_MSG_RESULT([no.])
	    	fi
	    
	    	if test "$stv_incl_found" = 1; then
		    with_cumulvs_incl=$chome
            	    break
	    	fi
	    fi
    	done
    fi

    if test "$with_cumulvs_incl" = _searching ; then
    	AC_MSG_RESULT([Cannot find CUMULVS include files.])
 	AC_MSG_RESULT([if CUMULVS source tree was used, please build it first])
   	AC_MSG_RESULT([or install CUMULVS RPM and/or use:])
    	AC_MSG_RESULT([ --with-cumulvs-incl=CUMULVS_INCLUDE_DIR or])
    	AC_MSG_RESULT([ --with-cumulvs=CUMULVS_ROOT_DIR for source installation])
	stv_found=0
	STV_FOUND="$stv_found"
    	AC_SUBST(STV_FOUND)	

    	with_cumulvs_incl="no include files found"
    fi

    if test ! "$chome" = /usr/local/include; then
	STV_INCL_DIR="$with_cumulvs_incl"
	STV_INCL="-I$with_cumulvs_incl"
    else
	STV_INCL_DIR="$with_cumulvs_incl"
	STV_INCL=
    fi
    AC_SUBST(STV_INCL_DIR)
    AC_SUBST(STV_INCL)
   

    # 4. check for make incl file path
    AC_ARG_WITH(cumulvs-make-incl, 
    	[Location of CUMULVS make incl files:
  	--with-cumulvs-make-incl=CUMULVS_INCLUDE
                          The location of the CUMULVS makeincl files
                          Give the full path: e.g.
                              --with-cumulvs-make-incl=/usr/local/CUMULVS/makeincl ] ,
    	, [with_cumulvs_make_incl=yes])

    case "$with_cumulvs_make_incl" in
    	no)
            # lUser input error
            with_cumulvs_make_incl=_searching
            ;;
    	yes)
            #  User didn't give the option or didn't give useful
            #  information, search for it ourselves
            with_cumulvs_make_incl=_searching
            ;;
    	*)
	    STV_INCL_MAKE_DIR="$with_cumulvs_make_incl"
            ;;
    esac


    if test "$with_cumulvs_make_incl" = _searching ; then
    	AC_MSG_CHECKING([for CUMULVS make incl files])
    	AC_MSG_RESULT([])
    	stv_make_incl_found=0
    	for chome in $stv_make_incl_dirs ; do
	    if test -d $chome ; then
	    	# check for make incl files
            	AC_MSG_CHECKING([$chome for Makeincl.stv])
	    	if test -f $chome/Makeincl.stv;then
            	    AC_MSG_RESULT([yes.])
		    AC_MSG_CHECKING([$chome for Makeincl.vis.NONE])
	    	    if test -f $chome/Makeincl.vis.NONE;then
            	        AC_MSG_RESULT([yes.])
			AC_MSG_CHECKING([$chome for Makeincl.mp.PVM])
	    		if test -f $chome/Makeincl.mp.PVM;then
            	    	    AC_MSG_RESULT([yes.])
			    AC_MSG_CHECKING([$chome for Makeincl.vis.TCLTK])
	    		    if test -f $chome/Makeincl.vis.TCLTK;then
            	    		AC_MSG_RESULT([yes.])
				stv_make_incl_found=1
	    		    else
		    		AC_MSG_RESULT([no.])
	    		    fi
	    		else
		    	    AC_MSG_RESULT([no.])
	        	fi
	    	    else
		    	AC_MSG_RESULT([no.])
	    	    fi
	    	else
		    AC_MSG_RESULT([no.])
	    	fi

	    	if test "$stv_incl_found" = 1; then
		    with_cumulvs_make_incl=$chome
            	    break
	    	fi
	    fi
    	done
    fi

    if test "$with_cumulvs_make_incl" = _searching ; then
    	AC_MSG_RESULT([Cannot find CUMULVS make incl files.])
	AC_MSG_RESULT([if CUMULVS source tree was used, please build it first])
   	AC_MSG_RESULT([or install CUMULVS RPM and/or use:])
    	AC_MSG_RESULT([ --with-cumulvs-make-incl=CUMULVS_MAKE_INCL_DIR or])
    	AC_MSG_RESULT([ --with-cumulvs=CUMULVS_ROOT_DIR for source installation])
	stv_found=0
	STV_FOUND="$stv_found"
    	AC_SUBST(STV_FOUND)

    	with_cumulvs_make_incl="no make incl files found"
    fi

    STV_MAKE_INCL_DIR="$with_cumulvs_make_incl"
    AC_MSG_RESULT([Makeincl files in: $STV_MAKE_INCL_DIR])
    AC_SUBST(STV_MAKE_INCL_DIR)

]
)

#------------------------------------------------------------------------------
# END: CUMULVS library MACRO M4
#------------------------------------------------------------------------------

