#------------------------------------------------------------------------------
#  Find CUMULVS RPM root directory
#------------------------------------------------------------------------------
AC_DEFUN(STV_FIND_RPM_ROOT_DIR,
[
#  CUMULVS_RPM search paths
stv_rpm_cumulvs_dirs="
    $prefix/CUMULVS
    /usr/local/CUMULVS
    $HOME/projects/CUMULVS
    $CURRDIR_PARENT/CUMULVS
    $CURRDIR_GRANDPARENT/CUMULVS
    $CURRDIR_GREATGRANDPARENT/CUMULVS
    $CURRDIR_GREATGRANDPARENT/../CUMULVS
    $CURRDIR_ANCESTOR/CUMULVS
    $STV_ROOT
"

AC_ARG_WITH(cumulvs_rpm, 
    [Location of CUMULVS_RPM:
  --with-cumulvs_rpm=CUMULVS_TREE
                          The location of CUMULVS RPM root directory
                          Give the full path: e.g.
                              --with-cumulvs_rpm=/usr/local/CUMULVS ] ,
    , [with_cumulvs_rpm=yes])

case "$with_cumulvs_rpm" in
    no)
        # lUser input error
        with_cumulvs_rpm=_searching
        ;;
    yes)
        #  User didn't give the option or didn't give useful
        #  information, search for it ourselves
        with_cumulvs_rpm=_searching
        ;;
    *)
        ;;
esac

if test "$with_cumulvs_rpm" = _searching ; then
    AC_MSG_CHECKING([for CUMULVS rpm install directory])
    AC_MSG_RESULT([])
    stv_rpm_cumulvs_dir_found=0
    for chome in $stv_rpm_cumulvs_dirs ; do
        if test -d $chome ; then
	    AC_MSG_CHECKING([for stvconfig in $chome])
	    if test ! -f "$chome/stvconfig"  ; then
		AC_MSG_RESULT([no])
	    else
		AC_MSG_RESULT([yes.])
		stv_rpm_cumulvs_dir_found=1
		with_cumulvs_rpm=$chome
	    fi

	    if test "$stv_rpm_cumulvs_dir_found" = 1; then
	        break
	    fi
        fi
    done
fi

AC_MSG_CHECKING([for CUMULVS STV_ROOT environment variable])
if test ! "$STV_ROOT" = "$with_cumulvs_rpm"; then
    if test ! "$STV_ROOT" = ""; then	
	AC_MSG_RESULT([found. Is set to $STV_ROOT. It is different from the source directory.])
	AC_MSG_ERROR([Please reset STV_ROOT to $with_cumulvs_rpm.])
    else
	AC_MSG_RESULT([ok])
    fi
else
    AC_MSG_RESULT([ok])
fi

if test "$with_cumulvs_rpm" = _searching ; then
    AC_MSG_RESULT([Cannot find CUMULVS RPM root directory.])
    AC_MSG_ERROR([Please install it and/or use --with-cumulvs_rpm=STV_ROOT.])
fi

if test ! -f "$with_cumulvs_rpm/stvconfig"  ; then
    AC_MSG_RESULT([No stvconfig in $with_cumulvs_rpm.])
    AC_MSG_ERROR([Please install a correct CUMULVS rpm distribution or use --with-cumulvs_rpm=CUMULVS_ROOT.])
else
    with_make_install=install
fi

STV_ROOT="$with_cumulvs_rpm"
STV_SOURCE_DIST_FOUND=0
STV_MAKE_INST="$with_make_install"
AC_SUBST(STV_ROOT)
AC_SUBST(STV_SOURCE_DIST_FOUND)
AC_SUBST(STV_MAKE_INST)
]
)

#---------------------------------------------------------------------------
# END: STV FIND RPM ROOT DIR M4 AUTOCONF MACRO
#---------------------------------------------------------------------------