#------------------------------------------------------------------------------
#  Find CUMULVS source root directory
#------------------------------------------------------------------------------
AC_DEFUN(STV_FIND_SOURCE_ROOT_DIR,
[
#  CUMULVS search paths
stv_source_cumulvs_dirs="
    $CURRDIR
    $HOME/projects/CUMULVS
    $CURRDIR_PARENT/CUMULVS
    $CURRDIR_GRANDPARENT/CUMULVS
    $CURRDIR_GREATGRANDPARENT/CUMULVS
    $CURRDIR_GREATGRANDPARENT/../CUMULVS
    $CURRDIR_ANCESTOR/CUMULVS
    /home/wilde/projects/CUMULVS
    $prefix/CUMULVS
    /usr/local/CUMULVS
    $STV_ROOT
"

AC_ARG_WITH(cumulvs, 
    [Location of CUMULVS:
  --with-cumulvs=CUMULVS_TREE
                          The location of CUMULVS source directory
                          Give the full path: e.g.
                              --with-cumulvs=/usr/local/CUMULVS ] ,
    , [with_cumulvs=yes])

case "$with_cumulvs" in
    no)
        # lUser input error
        with_cumulvs=_searching
        ;;
    yes)
        #  User didn't give the option or didn't give useful
        #  information, search for it ourselves
        with_cumulvs=_searching
        ;;
    *)
	stv_source_cumulvs_dir_found=1;	
        ;;
esac

if test "$with_cumulvs" = _searching ; then
    AC_MSG_CHECKING([for CUMULVS source directory])
    AC_MSG_RESULT([])
    stv_source_cumulvs_dir_found=0
    for chome in $stv_source_cumulvs_dirs ; do
        if test -d $chome ; then
	    AC_MSG_CHECKING([for include/stv.h in $chome])
	    if test ! -f "$chome/include/stv.h"  ; then
		AC_MSG_RESULT([no])
		with_cumulvs=
	    else
		AC_MSG_RESULT([yes.])
		stv_source_cumulvs_dir_found=1
		with_cumulvs=$chome
	    fi

	    if test "$stv_source_cumulvs_dir_found" = 1; then
	        break
	    fi
        fi
    done
fi

AC_MSG_CHECKING([for CUMULVS STV_ROOT environment variable])
if test ! "$STV_ROOT" = "$with_cumulvs"; then
    if test ! "$STV_ROOT" = ""; then	
	AC_MSG_RESULT([found. Is set to $STV_ROOT. It is different from the source directory found by configure.])
	AC_MSG_ERROR([Please unset or reset STV_ROOT to $with_cumulvs.])
    else
	AC_MSG_RESULT([ok])
    fi
else
    AC_MSG_RESULT([ok])
fi

if test "$with_cumulvs" = _searching ; then
    AC_MSG_RESULT([Cannot find CUMULVS root directory.])
    AC_MSG_ERROR([Please install it and/or use --with-cumulvs=STV_ROOT.])
fi

if test ! -f "$with_cumulvs/include/stv.h"  ; then
    if test ! -f "$with_cumulvs/stvconfig" ; then
    	AC_MSG_RESULT([No include/stv.h or stvconfig in $with_cumulvs!])
    	AC_MSG_ERROR([Please install a correct CUMULVS source or rpm distribution or use --with-cumulvs=CUMULVS_ROOT.])
    else
	AC_MSG_RESULT([Using CUMULVS RPM installation in $with_cumulvs.])
	source_dist_found=0
    fi
else
    AC_MSG_RESULT([Using CUMULVS source installation in $with_cumulvs.])
    source_dist_found=1
fi

STV_SOURCE_DIST_FOUND="$source_dist_found"
STV_ROOT="$with_cumulvs"
AC_SUBST(STV_SOURCE_DIST_FOUND)
AC_SUBST(STV_ROOT)
]
)

#---------------------------------------------------------------------------
# END: STV FIND SOURCE ROOT DIR M4 AUTOCONF MACRO
#---------------------------------------------------------------------------

