
#------------------------------------------------------------------------------
# STV_LIB_CAVE: See if the user gave us the location of the CAVE lib
# then search $stv_inst_cave_dirs.
# One of these directories must be
# present and contain the correct files, or else the user
# must give the directory that contains the correct files.
#------------------------------------------------------------------------------
AC_DEFUN(STV_LIB_CAVE,
[

AC_REQUIRE([STV_LIB_VTK])

stv_inst_cave_dirs="
    /usr/local/cave
    /usr/local/CAVE
    /usr/local/lib/cave
    /usr/local/lib/CAVE
    /usr/lib
    /usr/local/lib
    $prefix/cave
    $HOME/cave
    $HOME/CAVE
"

AC_ARG_WITH(cave-libs,
    [Location of the CAVE Library:	
  --with-cave-libs=CAVE_ROOT	
                          The location of the CAVE libraries
                          Give the full path: e.g.
                          --with-cave-libs=/usr/local/cave/lib32 ] ,
    , [with_cave_libs=yes])

case "$with_cave_libs" in
    no)
        # lUser input error
        with_cave_libs=_searching
        ;;
    yes)
        #  User didn't give the option or didn't give useful
        #  information, search for it ourselves
        with_cave_libs=_searching
        ;;
    *)
        CAVE_LIB_ROOT="$with_cave_libs"
        ;;
esac


if test $VTK_INCL_FOUND="1"; then

	if test "$with_cave_libs" = _searching ; then
	    AC_MSG_CHECKING([for OPTIONAL CAVE library])
	    AC_MSG_RESULT([])
	    for chome in $stv_inst_cave_dirs ; do
	        if test -d $chome ; then
		    AC_MSG_CHECKING([in $chome.])

		    if test "$arch" = "IRIX" -o "$arch" = "IRIX64"; then
		    	case "$STV_PVM_LIB_STYLE" in
			    64)
			    	#64 bit libraries required
			    	with_cave_libs=$chome/lib64
			    	;;

			    N32)
			    	#new 32 bit libraries required
			    	with_cave_libs=$chome/lib32	
			    	;;

			    O32)
			    	#old 32 bit libraries required
			    	with_cave_libs=$chome/lib32	
			    	;;
		    	esac

		    	cave_lib_found=1

		    	#check for library
		    	cave_lib_dir=$chome
		    	if test -f "$cave_lib_dir/libpfcave_ogl.a"; then
			    AC_MSG_RESULT([yes])
		    	else
			    AC_MSG_RESULT([no])
			    cave_lib_found=0
			    with_cave_libs=_searching
		    	fi
	    
		    	if test "$cave_lib_found" = "1"; then
			    #checking for CAVE library binary format (only for N32 and O32)
			    if test ! $STV_PVM_LIB_STYLE=64; then
		 	    	AC_MSG_CHECKING(for 32 bit library format)
		    	    	#check for library
			    	tmp=`file $cave_lib_dir/libpfcave_ogl.a`
	
			    	if test `echo $tmp | grep -c '32-bit'` -eq 1; then
				    AC_MSG_RESULT([CAVE Library is 32-Bit])
				    if test ! $STV_PVM_LIB_STYLE=O32; then
				    	AC_MSG_RESULT([PVM Library and CAVE Library are not binary compatible! Will not build CAVE examples])
					cave_lib_found=0
				    fi
				else
				    AC_MSG_RESULT([CAVE Library is N32])
				    if test ! $STV_PVM_LIB_STYLE=N32; then
				    	AC_MSG_RESULT([PVM Library and CAVE Library are not binary compatible! Will not build CAVE examples])
				    	cave_lib_found=0
				    fi
				fi
			    fi
		    	fi
		    else
			#no irix os
			cave_lib_found=1

		    	#check for library
		    	cave_lib_dir=$chome
		    	if test -f "$cave_lib_dir/libpfcave_ogl.a"; then
			    AC_MSG_RESULT([yes])
			else
			    AC_MSG_RESULT([no])
			    cave_lib_found=0
			    with_cave_libs=_searching
		    	fi
		    fi
		
		    if test "$cave_lib_found" = "1"; then
		    	# set library path
			CAVE_VIEWER_MAKE="cum_viewerV1.0"
		    	LIBS_CAVE="-lX11 -lpfcave_ogl -lpfutil -lpf -limage"
			CAVE_LIB_ROOT="$cave_lib_dir"
			CAVE_LIB_FILE="-lpfcave_ogl"
			CAVE_INCL_DIR="$chome/include"
		    	break
		    fi
		fi

		cave_lib_found=0
		with_cave_libs=_searching
	    done
	fi

	if test "$with_cave_libs" = _searching ; then
	    AC_MSG_RESULT([Cannot find CAVE Library files.])
	    AC_MSG_RESULT([Please install it and/or use --with-cave-libs=CAVE_LIB_ROOT.])
	    LIBS_CAVE=
	    CAVE_LIB_ROOT=
	    CAVE_LIB_FILE=
	    VTK_CAVE_VIEWER_MAKE=
	    CAVE_INCL_DIR=
	fi

else
	LIBS_CAVE=
	CAVE_LIB_ROOT=
	CAVE_LIB_FILE=
	VTK_CAVE_VIEWER_MAKE=
	CAVE_INCL_DIR=
fi

AC_SUBST(CAVE_LIB_ROOT)
AC_SUBST(CAVE_LIB_FILE)
AC_SUBST(LIBS_CAVE)
AC_SUBST(VTK_CAVE_VIEWER_MAKE)
AC_SUBST(CAVE_INCL_DIR)
]
)
#---------------------------------------------------------------------------
# END: STV LIB CAVE MACRO M4
#---------------------------------------------------------------------------
