
#------------------------------------------------------------------------------
#
# Some folklore is keyed to hostname, which is probably a very bad
# idea in the long run, but provides examples for frustrated alpha users.
# Most of it is keyed to the answers supplied by AC_CANONICAL_SYSTEM.
#
# We wish MPICH was adapted to use autoconf 2.x so we could steal
# conveniently everything they deduce about fortran, mpi, threads,
# etc in their configure scripts and extend it to our compilers.
#
# Find the mpi dujour if not suppressed.
# Setup pll build targets if available.
# Always:
# MPI, MPI_ARCH, MPICC, MPICXX, MPIRUN
# When mpicc and mpiCC are broken, as is frequently the case
# MPI_LDFLAGS, MPILIB
# Note that we need to finish the failure cases and expand folklore
# to cope with broken mpiCC. Or we need mpich to just work in every
# single release. Or we need MPI forum to standardize on a
# "configuration variables" sh includable file to live in
# $MPI_HOME/lib/config.vars so we don't need mpiCC,
# or ... insert your favorite solution to the vagaries of mpi builds
# here.
#--------------------------------------------------------------------
AC_DEFUN(STV_LIB_MPI,
[

AC_REQUIRE([STV_SELECT_CUMULVS_MP])

# MPICH search paths 
stv_inst_mpich_dirs="
	/usr/local/cca/
	$MPI_HOME
	$STV_INST_HOME/mpich
	$STV_INST_HOME/mpi
	$TOP/mpich
	$TOP/mpi
	$STVFE_PARENT/mpi
	$STVFE_ANCESTOR/mpi
	$STVFE_PARENT/mpich
	$STVFE_ANCESTOR/mpich
	$prefix/mpi
	$prefix/mpich
	$HOME/mpi
	$HOME/mpich
	/usr/local/mpi
	/usr/local/mpich
"

AC_ARG_WITH(mpi,
    [Location of mpi:
  --with-mpi=MPI_HOME
                          The location of mpich used in your framework.
                          Give the full path, e.g.:
                              --with-mpi='/share/bizarre/mpich1.1.2' ],
    , [with_mpi=yes])

case "$with_mpi" in
    no)
        stv_inst_have_mpi=no
        ;;
    yes)
        with_mpi=_searching
        ;;
    *)
        MPI="$with_mpi"
        ;;
esac

if test "$with_mpi" = _searching ; then
    stv_inst_have_mpi=no
    AC_MSG_CHECKING([for OPTIONAL mpi])
    AC_MSG_RESULT([])
    for mhome in $stv_inst_mpich_dirs ; do
	AC_MSG_CHECKING([for mpi in $mhome ...])
        if test -f $mhome/include/mpi.h -a -x $mhome/bin/tarch -a -x $mhome/bin/mpirun; then
		MPI=$mhome
		MPIRUN=$MPI/bin/mpirun
		MPI_ARCH=`$mhome/bin/tarch`
		MPI_INCLUDE="-I$mhome/include"
		MPI_INCL_DIR="$mhome/include"
		AC_MSG_RESULT([$mhome ($MPI_ARCH) found.])
        	stv_inst_have_mpi=yes
        	break
	else
		AC_MSG_RESULT([no])
        fi
    done
    if test -z "$MPI" ; then
        stv_inst_have_mpi=no
        AC_MSG_RESULT([mpi not found. mpi-based components and tests disabled.
           --with-mpi=/path/of/mpi(ch) to enable them.])
    fi
fi

# we think we have something. check that it runs
# and if not, sigh and go to folklore.
stv_inst_try_mpi_folklore=no
if test ! "X$stv_inst_have_mpi" = Xno ; then
    if test -x $MPI/bin/mpirun -a "X$stv_inst_try_mpi_folklore" = Xno ; then
        MPIRUN=$MPI/bin/mpirun
    else
    MPIRUN=broken
        stv_inst_try_mpi_folklore=yes
    fi
    if test -x $MPI/bin/tarch -a "X$stv_inst_try_mpi_folklore" = Xno ; then
        MPI_ARCH=`$MPI/bin/tarch`
    else
        MPI_ARCH="Set_MPI_ARCH_in_Makefile.Vars_manually"
        stv_inst_try_mpi_folklore=yes
    fi
    if test -x $MPI/bin/mpicc -a "X$stv_inst_try_mpi_folklore" = Xno ; then
        AC_MSG_CHECKING([mpicc])
	if test -x $MPI/bin/mpicc -a "X$STV_COMPONENTS_CONFIG" = X ; then
	        $MPI/bin/mpicc -o config.cpi.exe config/cpi.c
	else
		$MPI/bin/mpicc -o config.cpi.exe $STV_COMPONENTS_CONFIG/cpi.c
	fi
        if test -x config.cpi.exe ; then
            AC_MSG_RESULT(yes)
            MPICC=$MPI/bin/mpicc
        else
            MPICC=broken
            BROKEN_MPICC=$MPI/bin/mpicc
            AC_MSG_RESULT(broken.)
	    stv_inst_have_mpi=no
        fi
        rm -f config.cpi.exe cpi.o
    fi
    if test -x $MPI/bin/mpiCC -a "X$stv_inst_try_mpi_folklore" = Xno ; then
        AC_MSG_CHECKING([mpiCC])
	if test -x $MPI/bin/mpiCC -a "X$STV_COMPONENTS_CONFIG" = X ; then
	        $MPI/bin/mpiCC -o config.cppi.exe config/cpi.cpp
	else
		$MPI/bin/mpiCC -o config.cppi.exe $STV_COMPONENTS_CONFIG/cpi.cpp
	fi
        if test -x config.cppi.exe ; then
            AC_MSG_RESULT(yes)
            MPICXX=$MPI/bin/mpiCC
        else
            MPICXX=broken
            BROKEN_MPICXX=$MPI/bin/mpiCC
            AC_MSG_RESULT(broken.)
        fi
        rm -f config.cppi.exe cpi.o
    fi
    if test "X$stv_inst_try_mpi_folklore" = Xno ; then
        MPI_HOME=$MPI
    fi
fi
if test "X$stv_inst_try_mpi_folklore" = Xyes ; then
    echo "SIGH! rummaging about in history for mpi."
    STVFE_MPI_HUNT=yes
    . config/mpi.folklore
fi
if test -d "$MPI/build/$MPI_ARCH/ch_p4/include" ; then
    MPI_INCLUDE="-I$MPI/build/$MPI_ARCH/ch_p4/include $MPI_INCLUDE"
fi
if test -s "$MPI/build/$MPI_ARCH/ch_p4/lib/libmpich.a" ; then
    MPILIB="$MPI/build/$MPI_ARCH/ch_p4/lib/libmpich.a"
    MPI_LIBDIR="$MPI/build/$MPI_ARCH/ch_p4/lib"
fi
if test -d "$MPI/include/mpi2c++" ; then
    MPI_INCLUDE="-I$MPI/include/mpi2c++ $MPI_INCLUDE"
fi
if test -s "$MPI/lib/libmpich.a" ; then
    MPILIB="$MPI/lib/libmpich.a"
    MPI_LIBDIR="$MPI/lib"
fi

if test "$stv_inst_have_mpi" = no ; then
    AC_MSG_RESULT([Building with no mpi.])
    STV_MAKE_MPI_TARGET=
    STV_MAKE_MPI_SHARED_TARGET=
    STV_MPI_CLEAN=
    STV_MPI_INST=
else
    AC_MSG_RESULT([Building with mpi.])
    STV_MAKE_MPI_TARGET="all_mpi"
    STV_MAKE_MPI_SHARED_TARGET="sh_mpi_all"
    STV_MPI_CLEAN="yes"
    STV_MPI_INST="all_mpi_install"
    MPI_MPIF77="$MPI/bin/mpif77"
fi

AC_MSG_CHECKING([for MPI selection as CUMULVS message passing library])
if test "$STV_MP" = MPI; then
    AC_MSG_RESULT([yes. (setting STV_ARCH to $MPI_ARCH)])
    STV_ARCH=$MPI_ARCH
    AC_SUBST(STV_ARCH)
else
    AC_MSG_RESULT([no.])
fi

AC_SUBST(MPI)
AC_SUBST(MPIRUN)
AC_SUBST(MPI_ARCH)
AC_SUBST(MPI_HOME)
AC_SUBST(BROKEN_MPICC)
AC_SUBST(BROKEN_MPICXX)
AC_SUBST(MPICC)
AC_SUBST(MPICXX)
AC_SUBST(MPI_LDFLAGS)
AC_SUBST(MPILIB)
AC_SUBST(MPI_LIBDIR)
AC_SUBST(MPI_INCLUDE)
AC_SUBST(MPI_INCL_DIR)
AC_SUBST(STV_MAKE_MPI_TARGET)
AC_SUBST(STV_MAKE_MPI_SHARED_TARGET)
AC_SUBST(STV_MPI_CLEAN)
AC_SUBST(STV_MPI_INST)
AC_SUBST(MPI_MPIF77)
]
)
#----------------------------------------------------------------------------
# END: STV MPI MACRO M4
#----------------------------------------------------------------------------
