
#------------------------------------------------------------------------------
#  Find PVM libraries
#------------------------------------------------------------------------------
AC_DEFUN(STV_LIB_PVM,
[

AC_REQUIRE([STV_SELECT_CUMULVS_MP])

#  PVM search paths
stv_inst_pvm_dirs="
    $PVM_ROOT
    $STV_INST_HOME/pvm3
    $CURRDIR_PARENT/pvm3
    $CURRDIR_GRANDPARENT/pvm3
    $CURRDIR_GREATGRANDPARENT/pvm3
    $prefix/pvm3
    $HOME/pvm3
    /usr/local/pvm3
"

AC_ARG_WITH(pvm, 
    [Location of PVM:
  --with-pvm=PVM_TREE
                          The location of PVM
                          Give the full path: e.g.
                              --with-pvm=/usr/local/pvm3 ] ,
    , [with_pvm=yes])

case "$with_pvm" in
    no)
        # lUser input error
        with_pvm=_searching
        ;;
    yes)
        #  User didn't give the option or didn't give useful
        #  information, search for it ourselves
        with_pvm=_searching
        ;;
    *)
        PVM_ROOT="$with_pvm"
        ;;
esac

if test "$with_pvm" = _searching ; then
    AC_MSG_CHECKING([for REQUIRED PVM])
    for chome in $stv_inst_pvm_dirs ; do
        if test -d $chome ; then
            with_pvm=$chome
            AC_MSG_RESULT([found $chome.])
            break
        fi
    done
fi
if test "$with_pvm" = _searching ; then
    AC_MSG_RESULT([Cannot find PVM root directory.])
    AC_MSG_ERROR([Please install it and/or use --with-pvm=PVM_ROOT.])
fi

PVM_ROOT="$with_pvm"

# Make sure $PVM_ARCH is set...
if test -z "$PVM_ARCH" ; then
	# Make sure pvmgetarch exists...
	if test -x $PVM_ROOT/lib/pvmgetarch ; then
		PVM_ARCH=`$PVM_ROOT/lib/pvmgetarch`
	else
		AC_MSG_WARN([\$PVM_ROOT/lib/pvmgetarch script not found!])
		AC_MSG_ERROR([Please set PVM_ARCH env var.])
	fi
fi

PVM_XDIR="$PVM_ROOT/bin/$PVM_ARCH"

# Now check for (writable) PVM bin...
AC_MSG_CHECKING([for writable PVM bin])
xdirck="aok"
if test ! -e $PVM_XDIR ; then
	# Hmmm... nothing there - try to create it...
	AC_MSG_WARN([PVM bin directory $PVM_XDIR doesn't exist - trying to create...])
	mkdir -p $PVM_XDIR
	if test $? != 0 ; then
		# Damn, can't create that directory...
		AC_MSG_WARN([Cannot create PVM bin directory $PVM_XDIR !])
		xdirck="notok"
	else
		AC_MSG_RESULT([$PVM_XDIR successfully created.])
	fi
else
	if test ! -d $PVM_XDIR ; then
		# Hmmm... not a directory...!
		# Rather than delete it and make a directory,
		# we'll just defer to alternate $HOME/pvm3 soln...
		AC_MSG_WARN([$PVM_XDIR is *not* a directory!])
		xdirck="notok"
	else
		if test ! -w $PVM_XDIR ; then
			# Hmmm... not writable - system-wide install?
			AC_MSG_WARN([PVM bin directory $PVM_XDIR is not writable!])
			xdirck="notok"
		fi
	fi
fi

# If not O.K., try using $HOME/pvm3/bin/$PVM_ARCH...
if test $xdirck = "notok" ; then
	# First, see if $PVM_ROOT is already $HOME/pvm3...
	homepvm="$HOME/pvm3"
	if test "$PVM_ROOT" = "$homepvm" ; then
		# We're screwwwwed, PVM already *is* in $HOME...
    	AC_MSG_RESULT([Cannot write to PVM bin directory in $PVM_XDIR.])
    	AC_MSG_ERROR([Please make $PVM_XDIR a writable directory.])
	else
		# Try to make bin directory in $HOME/pvm3...
		PVM_XDIR="$homepvm/bin/$PVM_ARCH"
		mkdir -p $PVM_XDIR
		if test $? != 0 ; then
			# Damn, can't create directory...
			AC_MSG_RESULT([Cannot create PVM bin directory $PVM_XDIR.])
    		AC_MSG_ERROR([Please make $PVM_XDIR a writable directory.])
		else
			AC_MSG_RESULT([$PVM_XDIR successfully created.])
		fi
	fi
else
   	AC_MSG_RESULT([$PVM_XDIR is O.K.])
fi

AC_MSG_CHECKING([for PVM selection as CUMULVS message passing library])
if test "$STV_MP" = PVM; then
    AC_MSG_RESULT([yes. (setting STV_ARCH to $PVM_ARCH)])
    STV_ARCH=$PVM_ARCH
    AC_SUBST(STV_ARCH)
else
    AC_MSG_RESULT([no.])
fi

AC_SUBST(PVM_ROOT)
AC_SUBST(PVM_ARCH)
AC_SUBST(PVM_XDIR)
]
)


#------------------------------------------------------------------------------
# getting OS specific PVM info
#------------------------------------------------------------------------------
AC_DEFUN(STV_GET_PVM_FLAGS,
[
AC_MSG_CHECKING(PVM Library Flags for System Architecture)

if test "$PVM_ARCH" = SGI; then
   stv_pvm_glib=
   stv_pvmglib=
   stv_fpvmlib=

   AC_MSG_RESULT([NONE])
   AC_MSG_RESULT([also must: setenv PVM_GROUPS_FULL 1 ; setenv PVM_PE_LIST all])
elif test "$PVM_ARCH" = CRAY; then
   stv_pvm_glib=""
   stv_pvmglib=""
   stv_fpvmlib=""
   AC_MSG_RESULT([NONE])
   AC_MSG_RESULT([also must: setenv PVM_GROUPS_FULL 1 ; setenv PVM_PE_LIST all])
else
   stv_pvm_glib='$(STV_PVMLDIR)/libgpvm3.a'
   stv_pvmglib="-lgpvm3"
   stv_fpvmlib="-lfpvm3"
   AC_MSG_RESULT([Using Standard Flags])
fi

STV_PVM_GLIB=$stv_pvm_glib
STV_PVMGLIB=$stv_pvmglib
STV_FPVMLIB=$stv_fpvmlib

#check for pvm library compile style (32, N32, 64-bit)
if test "$arch" = "IRIX" -o "$arch" = "IRIX64"; then

	AC_MSG_CHECKING(SGI binary format for PVM Library)

	tmp=`file $PVM_ROOT/lib/$PVM_ARCH/libpvm3.a`

	if test `echo $tmp | grep -c '64-bit'` -eq 1; then
		AC_MSG_RESULT([PVM Library is 64-Bit])
		STV_PVM_LIB_STYLE="64"
	elif test `echo $tmp | grep -c '32-bit'` -eq 1; then
		AC_MSG_RESULT([PVM Library is 32-Bit])
		STV_PVM_LIB_STYLE="O32"
	elif test `echo $tmp | grep -c 'N32'` -eq 1; then
		AC_MSG_RESULT([PVM Library is N32])
		STV_PVM_LIB_STYLE="N32"
	else
		AC_MSG_RESULT([PVM Library is of unknown binary format!])
		STV_PVM_LIB_STYLE="NONE"
	fi

fi

AC_SUBST(STV_PVM_GLIB)
AC_SUBST(STV_PVMGLIB)
AC_SUBST(STV_FPVMLIB)
AC_SUBST(STV_PVM_LIB_STYLE)

#
# Determine Architecture Name
#

AC_MSG_CHECKING(System Architecture)

AC_ARG_WITH(arch,
[  --with-arch=<name> logical name of local system architecture],
arch="$withval")

if test -z "$arch"; then
	if test -f /bin/uname; then
		arch=`/bin/uname -s`
	elif test -f /usr/bin/uname; then
		arch=`/usr/bin/uname -s`
	else
		arch="Unknown"
	fi
fi

AC_MSG_RESULT($arch)


#
# Determine Architecture dependent shared library flags
#

AC_MSG_CHECKING(Shared Library CFLAGS for System Architecture)

AC_ARG_WITH(arch_shared_cflags,
[  --with-arch_shared_cflags=<flags> compile flags for shared libraries for system architecture],
arch_shared_cflags="$withval")

if test -n "$arch_shared_cflags"; then
	AC_MSG_RESULT($arch_shared_cflags)
else
	if test "$arch" = Linux; then
	   arch_shared_cflags="-fpic"
	   AC_MSG_RESULT($arch_shared_cflags)
	else
	   arch_shared_cflags=""
	   AC_MSG_RESULT([NONE])
	fi
fi

AC_MSG_CHECKING(Shared Library LDFLAGS for System Architecture)

AC_ARG_WITH(arch_shared_ldflags,
[  --with-arch_shared_ldflags=<flags> linker flags for shared libraries for system architecture],
arch_shared_ldflags="$withval")

if test -n "$arch_shared_ldflags"; then
	AC_MSG_RESULT($arch_shared_ldflags)
else
	if test "$arch" = IRIX -o "$arch" = "IRIX64"; then
	   arch_shared_ldflags="-shared -all"
	   AC_MSG_RESULT([$arch_shared_ldflags])
	else
	   arch_shared_ldflags="-shared"
	   AC_MSG_RESULT($arch_shared_ldflags)
	fi
fi

ARCH="$arch"
ARCH_SHARED_CFLAGS="$arch_shared_cflags"
ARCH_SHARED_LDFLAGS="$arch_shared_ldflags"

AC_SUBST(ARCH)
AC_SUBST(ARCH_SHARED_CFLAGS)
AC_SUBST(ARCH_SHARED_LDFLAGS)

]
)
#---------------------------------------------------------------------------
# END: STV PVM MACRO M4
#---------------------------------------------------------------------------
