
#------------------------------------------------------------------------------
# STV_INST_LIB_VTK: See if the user gave us the location of the VTK libraries
# then search $stv_inst_vtk_dirs.
# One of these directories must be
# present and contain the correct files, or else the user
# must give the directory that contains the correct files.
#------------------------------------------------------------------------------
AC_DEFUN(STV_LIB_VTK,
[

stv_vtk_lib_dirs="
    /usr/local/vtk
    /usr/local/VTK
    /usr/local/lib/vtk
    /usr/local/lib/VTK
    /usr/lib
    /usr/local/lib
    $prefix/vtk
    $HOME/vtk
    $HOME/VTK
"

stv_vtk_libs="
    libvtkImaging
    libvtkGraphics
    libvtkFiltering
    libvtkRendering
    libvtkIO
    libvtkPatented
"


AC_ARG_WITH(vtk-libs,
    [Location of VTK libs:	
  --with-vtk-libs=VTK_ROOT	
                          The location of the VTK libraries
                          Give the full path: e.g.
                          --with-vtk-libs=/usr/local/lib ] ,
    , [with_vtk_libs=yes])

case "$with_vtk_libs" in
    no)
        # lUser input error
        with_vtk_libs=_searching
        ;;
    yes)
        #  User didn't give the option or didn't give useful
        #  information, search for it ourselves
        with_vtk_libs=_searching
        ;;
    *)
        VTK_ROOT="$with_vtk_libs"
        ;;
esac

if test "$with_vtk_libs" = _searching ; then
    AC_MSG_CHECKING([for OPTIONAL VTK distribution])
    AC_MSG_RESULT([])
    vtk_lib_ext_so=".so"
    vtk_lib_ext_a=".a"

    for chome in $stv_vtk_lib_dirs ; do
        if test -d $chome ; then
	    with_vtk_libs=$chome
	    #AC_MSG_RESULT([found $chome.])
	    vtk_lib_found=1

	    #check for library
	    vtk_lib_dir=$chome
	    AC_MSG_CHECKING([for VTK Common Library in $vtk_lib_dir])
	    if test -f "$vtk_lib_dir/libvtkCommon$vtk_lib_ext_so"; then
		AC_MSG_RESULT([found shared library])
		vtk_lib_ext=$vtk_lib_ext_so
	    elif test -f "$vtk_lib_dir/libvtkCommon$vtk_lib_ext_a"; then
		AC_MSG_RESULT([found static library])
		vtk_lib_ext=$vtk_lib_ext_a
	    elif test -f "$vtk_lib_dir/bin/libvtkCommon$vtk_lib_ext_so"; then
		vtk_lib_dir=$vtk_lib_dir/bin
		AC_MSG_RESULT([found shared library in ./bin])
		vtk_lib_ext=$vtk_lib_ext_so
	    elif test -f "$vtk_lib_dir/bin/libvtkCommon$vtk_lib_ext_a"; then
		vtk_lib_dir=$vtk_lib_dir/bin
		AC_MSG_RESULT([found static library in ./bin])
		vtk_lib_ext=$vtk_lib_ext_a
	    else
		AC_MSG_RESULT([couldn't find shared or static libvtkCommon library! Will not buid VTK examples!])
		vtk_lib_found=0
		with_vtk_libs=_searching
	    fi
	    
	    if test "$vtk_lib_found" = "1"; then
		#checking for all required VTK libraries
	    	AC_MSG_CHECKING(for all required VTK Libraries)
	    	for lhome in $stv_vtk_libs ; do
		    if test "$vtk_lib_found" = "1"; then
		    	if test ! -f "$vtk_lib_dir/$lhome$vtk_lib_ext"; then
			    vtk_lib_found=0
			    AC_MSG_RESULT([couldn't find library $vtk_lib_dir/$lhome$vtk_lib_ext. Will not build VTK examples!])
			    STV_BUILD_VTK=""
			fi
		    else
			break
		    fi
	    	done
	    fi

	    #check for IRIX
	    if test "$vtk_lib_found" = "1"; then
		AC_MSG_RESULT([yes.])
		if test "$arch" = "IRIX" -o "$arch" = "IRIX64"; then
		    AC_MSG_CHECKING(SGI binary format for VTK Library)

		    #check for library
		    tmp=`file $vtk_lib_dir/libvtkCommon$vtk_lib_ext`

		    if test `echo $tmp | grep -c '64-bit'` -eq 1; then
			AC_MSG_RESULT([VTK Library is 64-Bit])
			if test ! $STV_PVM_LIB_STYLE = "64"; then
			    AC_MSG_RESULT([PVM Library and VTK Library are not binary compatible!])
			    STV_BUILD_VTK=""
			    vtk_lib_found=0
			else
			    LIBS_VTK="-lXt -lGL -lXi -lX11 -lm -lC"
			    VTK_SYSTEM_FLAGS=-64
			fi
		    elif test `echo $tmp | grep -c '32-bit'` -eq 1; then
			AC_MSG_RESULT([VTK Library is 32-Bit])
			if test ! $STV_PVM_LIB_STYLE = "O32"; then
			    AC_MSG_RESULT([PVM Library and VTK Library are not binary compatible!])
			    STV_BUILD_VTK=""
			    vtk_lib_found=0
			else
			    LIBS_VTK="-lXt -lGL -lXi -lX11 -lm -lC"
			    VTK_SYSTEM_FLAGS=-32
			fi
		    elif test `echo $tmp | grep -c 'N32'` -eq 1; then
			AC_MSG_RESULT([VTK Library is N32])
			if test ! $STV_PVM_LIB_STYLE = "N32"; then
			    AC_MSG_RESULT([PVM Library and VTK Library are not binary compatible!])
			    STV_BUILD_VTK=""
			    vtk_lib_found=0
			else
			    LIBS_VTK="-lXt -lGL -lXi -lX11 -lm -lC"
			    VTK_SYSTEM_FLAGS=-N32
			fi
		    else
			AC_MSG_RESULT([VTK Library is of unknown binary format!])
			STV_BUILD_VTK=""
			vtk_lib_found=0
		    fi
		else
		    #non SGI
		    LIBS_VTK="-lm"
		    VTK_SYSTEM_FLAGS=""
		fi
	    fi
	fi

	if test "$vtk_lib_found" = "1"; then
	    # set library path
	    if test "$vtk_lib_ext" = .a; then
	    	VTK_LIB_FILE="$vtk_lib_dir/libvtkPatented$vtk_lib_ext \
			$vtk_lib_dir/libvtkImaging$vtk_lib_ext \
			$vtk_lib_dir/libvtkGraphics$vtk_lib_ext \
			$vtk_lib_dir/libvtkCommon$vtk_lib_ext \
			$vtk_lib_dir/libvtkFiltering$vtk_lib_ext \
			$vtk_lib_dir/libvtkRendering$vtk_lib_ext \
			$vtk_lib_dir/libvtkIO$vtk_lib_ext"
	    else
		VTK_LIB_FILE="	-L$vtk_lib_dir \
			      	-lvtkPatented \
			      	-lvtkImaging \
				-lvtkGraphics \
				-lvtkCommon \
				-lvtkFiltering \
				-lvtkRendering \
				-lvtkIO"
	    fi
	    break
	fi

	with_vtk_libs=_searching
    done
fi

if test "$with_vtk_libs" = _searching ; then
    AC_MSG_RESULT([Cannot find VTK Library files.])
    AC_MSG_RESULT([Please install it and/or use --with-vtk-libs=VTK_ROOT/bin.])
    vtk_lib_found=0
    STV_BUILD_VTK=""	
    VTK_VIEWER_MAKE=
fi

VTK_LIB_FOUND="$vtk_lib_found"
VTK_LIB_DIR="$vtk_lib_dir"
VTK_ROOT="$with_vtk_libs"
AC_SUBST(VTK_ROOT)
AC_SUBST(VTK_LIB_FILE)
AC_SUBST(VTK_LIB_FOUND)
AC_SUBST(VTK_LIB_DIR)
AC_SUBST(LIBS_VTK)
AC_SUBST(VTK_SYSTEM_FLAGS)



#------------------------------------------------------------------------------
# STV_INST_INCL_VTK: See if the user gave us the location of the VTK includes
# then search $stv_inst_vtk_dirs.
# One of these directories must be
# present and contain the correct files, or else the user
# must give the directory that contains the correct files.
#------------------------------------------------------------------------------
stv_vtk_incl_dirs="
    /usr/local/vtk
    /usr/local/VTK
    /usr/local/include/vtk
    /usr/local/include/VTK
    /usr/include
    /usr/local/include
    $prefix/vtk
    $HOME/vtk
    $HOME/VTK
"

AC_ARG_WITH(vtk-incl,
    [Location of VTK incl files:	
  --with-vtk-incl=VTK_ROOT	
                          The location of the VTK include files
                          Give the full path: e.g.
                          --with-vtk-incl=/usr/local/include/vtk ] ,
    , [with_vtk_incl=yes])

case "$with_vtk_incl" in
    no)
        # lUser input error
        with_vtk_incl=_searching
        ;;
    yes)
        #  User didn't give the option or didn't give useful
        #  information, search for it ourselves
        with_vtk_incl=_searching
        ;;
    *)
        VTK_INCL="$with_vtk_incl"
        ;;
esac

VTK_INCL_FOUND="0"

if test ! $VTK_LIB_FOUND = "1"; then
    STV_BUILD_VTK=""
    STV_VTK_INST=""
    STV_VTK_CLEAN=""
else
    if test "$with_vtk_incl" = _searching ; then
	AC_MSG_CHECKING([for VTK headers])
	AC_MSG_RESULT([])
	for chome in $stv_vtk_incl_dirs ; do
	    if test -d $chome ; then
		with_vtk_incl=$chome
		vtk_incl_found=1

	        #check for headers
	        vtk_incl_dir=$chome
	        AC_MSG_CHECKING([for VTK Common header files in $vtk_incl_dir])
	        if test -f "$vtk_incl_dir/vtkAbstractMapper.h"; then
		    AC_MSG_RESULT([found Common headers])
		    vtk_incl_style=1
		    vtk_incl_found=1
	    	elif test -f "$vtk_incl_dir/Common/vtkAbstractMapper.h"; then
		    AC_MSG_RESULT([found Common headers in ./Common.])
		    vtk_incl_style=2
		    vtk_incl_found=1
	    	else
		    AC_MSG_RESULT([no])
		    vtk_incl_found=0
		    STV_BUILD_VTK=""
		    STV_VTK_CLEAN=""
	    	fi

		if test "$vtk_incl_found" = "1"; then
		    #checking for all required VTK headers
	    	    AC_MSG_CHECKING([for all required VTK header files])
		    AC_MSG_RESULT([])
	    	    if test "$vtk_incl_style" = "1"; then
			AC_MSG_CHECKING(for VTK Filtering header files)
			if test -f "$vtk_incl_dir/vtkCardinalSpline.h"; then
			    AC_MSG_RESULT([yes])
			else
			    AC_MSG_RESULT([no])
			    vtk_incl_found=0
			fi

			AC_MSG_CHECKING(for VTK Graphics header files)
			if test -f "$vtk_incl_dir/vtkMaskPolyData.h"; then
			    AC_MSG_RESULT([yes])
			else
			    AC_MSG_RESULT([no])
			    vtk_incl_found=0
			fi

			AC_MSG_CHECKING(for VTK Hybrid header files)
			if test -f "$vtk_incl_dir/vtkImplicitModeller.h"; then
			    AC_MSG_RESULT([yes])
			else
			    AC_MSG_RESULT([no!])
			    vtk_incl_found=0
			fi

			AC_MSG_CHECKING(for VTK Imaging header files)
			if test -f "$vtk_incl_dir/vtkImageImport.h"; then
			    AC_MSG_RESULT([yes])
			else
			    AC_MSG_RESULT([no])
			    vtk_incl_found=0
			fi

			AC_MSG_CHECKING(for VTK IO header files)
			if test -f "$vtk_incl_dir/vtkVolumeReader.h"; then
			    AC_MSG_RESULT([yes])
			else
			    AC_MSG_RESULT([no])
			    vtk_incl_found=0
			fi

			AC_MSG_CHECKING(for VTK Patented header files)
			if test -f "$vtk_incl_dir/vtkDividingCubes.h"; then
			    AC_MSG_RESULT([yes])
			else
			    AC_MSG_RESULT([no])
			    vtk_incl_found=0
			fi

			AC_MSG_CHECKING(for VTK Rendering header files)
			if test -f "$vtk_incl_dir/vtkXRenderWindowInteractor.h"; then
			    AC_MSG_RESULT([yes])
			else
			    AC_MSG_RESULT([no])
			    vtk_incl_found=0
			fi
			
			if test "$vtk_incl_found" = "1"; then
			    # set library path
			    VTK_INCL="-I$vtk_incl_dir"	
			fi
		    else
			#style 2
			AC_MSG_CHECKING(for VTK Filtering header files in ./Filtering)
			if test -f "$vtk_incl_dir/Filtering/vtkCardinalSpline.h"; then
			    AC_MSG_RESULT([yes])
			else
			    AC_MSG_RESULT([no])
			    vtk_incl_found=0
			fi

			AC_MSG_CHECKING(for VTK Graphics header files in ./Graphics)
			if test -f "$vtk_incl_dir/Graphics/vtkMaskPolyData.h"; then
			    AC_MSG_RESULT([yes])
			else
			    AC_MSG_RESULT([no])
			    vtk_incl_found=0
			fi

			AC_MSG_CHECKING(for VTK Hybrid header files in ./Hybrid)
			if test -f "$vtk_incl_dir/Hybrid/vtkImplicitModeller.h"; then
			    AC_MSG_RESULT([yes])
			else
			    AC_MSG_RESULT([no])
			    vtk_incl_found=0
			fi

			AC_MSG_CHECKING(for VTK Imaging header files in ./Imaging)
			if test -f "$vtk_incl_dir/Imaging/vtkImageImport.h"; then
			    AC_MSG_RESULT([yes])
			else
			    AC_MSG_RESULT([no])
			    vtk_incl_found=0
			fi

			AC_MSG_CHECKING(for VTK IO header files in  ./IO)
			if test -f "$vtk_incl_dir/IO/vtkVolumeReader.h"; then
			    AC_MSG_RESULT([yes])
			else
			    AC_MSG_RESULT([no])
			    vtk_incl_found=0
			fi

			AC_MSG_CHECKING(for VTK Patented header files in ./Patented)
			if test -f "$vtk_incl_dir/Patented/vtkDividingCubes.h"; then
			    AC_MSG_RESULT([yes])
			else
			    AC_MSG_RESULT([no])
			    vtk_incl_found=0
			fi

			AC_MSG_CHECKING(for VTK Rendering header files in ./Rendering)
			if test -f "$vtk_incl_dir/Rendering/vtkXRenderWindow.h"; then
			    AC_MSG_RESULT([yes])
			else
			    AC_MSG_RESULT([no])
			    vtk_incl_found=0
			fi

			if test "$vtk_incl_found" = "1"; then
			    # set library path
			    VTK_INCL="-I$vtk_incl_dir/Common -I$vtk_incl_dir/Filtering\
				  	-I$vtk_incl_dir/Graphics -I$vtk_incl_dir/Hybrid \
					-I$vtk_incl_dir/Imaging -I$vtk_incl_dir/IO \
					-I$vtk_incl_dir/Patented -I$vtk_incl_dir/Rendering"
			fi
		    fi
		fi
	    fi

	    if test "$vtk_incl_found" = "1"; then
		STV_BUILD_VTK="vtk_all"
		STV_VTK_INST="vtk_install"
		VTK_VIEWER_MAKE=cum_viewerV1.0vtk
		VTK_INCL_FOUND="1"
		STV_VTK_CLEAN="yes"
		break
	    fi

	with_vtk_incl=_searching
	done
    fi
fi

if test "$with_vtk_incl" = _searching ; then
    AC_MSG_RESULT([Cannot find VTK header files. Will not build VTK examples.])
    AC_MSG_RESULT([Please install it and/or use --with-vtk-incl=VTK_ROOT.])
    VTK_INCL_FOUND="0"
    VTK_VIEWER_MAKE=
    STV_VTK_INST=
    STV_BUILD_VTK=
    STV_VTK_CLEAN=
fi

AC_SUBST(VTK_INCL)
AC_SUBST(STV_BUILD_VTK)
AC_SUBST(VTK_INCL_FOUND)
AC_SUBST(VTK_VIEWER_MAKE)
AC_SUBST(STV_VTK_INST)
AC_SUBST(STV_VTK_CLEAN)
]
)
#-----------------------------------------------------------------------------
# END: STV VTK LIB MACRO M4
#-----------------------------------------------------------------------------
