##############################################################################
# STV MAIN MACRO M4 defines autoconf macros for building CUMULVS from source #
##############################################################################

# Checks for unix programs
AC_DEFUN(STV_INST_UNIX_PROGRAMS,
[
AC_CANONICAL_SYSTEM
AC_HEADER_STDC
AC_CHECK_HEADERS(fcntl.h strings.h sys/file.h sys/time.h unistd.h)

AC_PROG_LN_S
AC_SUBST(LN_S)

AC_PROG_MAKE_SET
AC_SUBST(SET_MAKE)

AC_PROG_RANLIB
AC_SUBST(RANLIB)

# source tree vars
CURRDIR=`pwd`
# The next two lines entirely quoted to keep autoconf/m4 
# from messing with the [] in the regex.
[CURRDIR_PARENT=`echo $CURRDIR | sed 's%[^/][^/]*$%%' | sed 's%\/$%%'`]
[CURRDIR_GRANDPARENT=`echo $CURRDIR_PARENT | sed 's%[^/][^/]*$%%' | sed 's%\/$%%'`]
[CURRDIR_GREATGRANDPARENT=`echo $CURRDIR_GRANDPARENT | sed 's%[^/][^/]*$%%' | sed 's%\/$%%'`]
[CURRDIR_ANCESTOR=`echo $CURRDIR_GREATGRANDPARENT | sed 's%[^/][^/]*$%%' | sed 's%\/$%%'`]
]
)

## end of paths to search collection

AC_DEFUN(STV_INST_COMPILER,
[
#AC_LANG_CPLUSPLUS

# FIND C compiler
#------------------------------------------------------------------------------
#  see if the user gave us the location of the c compiler
#------------------------------------------------------------------------------
AC_ARG_WITH(cc,
    [Location of alternative c compiler:
  --with-cc=EXECUTABLE
                          The location of C.
                          Give the full path:
                              --with-cc='/usr/local/gcc/bin/gcc' ],
    , [with_cc=yes])

case "$with_cc" in
    no)
        #  User specified --without-cc
        #  print a warning.
        AC_MSG_WARN([Option '--without-cc' Causes demo to be suppressed.])
        AC_MSG_ERROR([A c compiler is required; demo has a c code.])
        ;;
    yes)
        #  User didn't give the option or didn't give useful
        #  information, search for it ourselves
        with_cc=_searching
        ;;
    *)
        CC="$with_cc"
        ;;
esac

if test "$with_cc" = _searching ; then
    AC_PROG_CC(cc gcc)
fi
AC_SUBST(CC)

# FIND c++ compiler
#------------------------------------------------------------------------------
#  see if the user gave us the location of the c++ compiler
#------------------------------------------------------------------------------
AC_ARG_WITH(cPP,
    [Location of alternative c++ compiler:
  --with-c++=EXECUTABLE
                          The location of C++.
                          Give the full path:
                              --with-c++='/usr/local/gcc/bin/g++' ],
    , [with_cPP=yes])

case "$with_cPP" in
    no)
        #  User specified --without-c++
        #  print a warning.
        AC_MSG_WARN([Option '--without-c++' Causes compilation to be suppressed.])
        AC_MSG_ERROR([A c++ compiler is required; current code is a c++ code.])
        ;;
    yes)
        #  User didn't give the option or didn't give useful
        #  information, search for it ourselves
        with_cPP=_searching
        ;;
    *)
        CXX="$with_cPP"
        cca_have_cpp=yes
        ;;
esac

if test "$with_cPP" = _searching ; then
    AC_PROG_CXX(CC gcc)
fi


# set the warning flags depending on whether or not we are using gcc
if test "${GCC}" = "yes" ; then
    CFLAGS_WARNING="-Wall -Wconversion"
else
    CFLAGS_WARNING=""
fi
AC_SUBST(CFLAGS_WARNING)

#------------------------------------------------------------------------------
# If we're using GCC, see if the compiler understands -pipe.  If so, use it.
# It makes compiling go faster.  (This is only a performance feature.)
#------------------------------------------------------------------------------

if test -z "$no_pipe"; then
if test -n "$GCC"; then
  AC_MSG_CHECKING([if c++ compiler understands -pipe])
  OLDCXX="$CXX"
  CXX="$CXX -pipe"
  AC_TRY_COMPILE(,,
    AC_MSG_RESULT(yes),
    CXX="$OLDCXX"
    AC_MSG_RESULT(no))
fi
fi

AC_SUBST(CXX)
AC_SUBST(CXXFLAGS)
AC_SUBST(CXX_TEMPLATES)

# FIND c++ preprocessor
AC_PROG_CXXCPP(CC gcc)
AC_SUBST(CXXCPP)


# FIND Fortran compiler
#------------------------------------------------------------------------------
#  see if the user gave us the location of the c compiler
#------------------------------------------------------------------------------
AC_ARG_WITH(f77,
    [Location of alternative Fotran compiler:
  --with-f77=EXECUTABLE
                          The location of F77.
                          Give the full path:
                              --with-f77='/usr/local/bin/f77' ],
    , [with_f77=yes])

case "$with_f77" in
    no)
        #  User specified --without-f77
        #  print a warning.
        AC_MSG_WARN([Option '--without-f77' Causes some examples to be suppressed.])
        ;;
    yes)
        #  User didn't give the option or didn't give useful
        #  information, search for it ourselves
        with_f77=_searching
        ;;
    *)
        F77="$with_f77"
        ;;
esac

if test "$with_f77" = _searching ; then
    AC_PROG_F77
fi


if test ! $F77 = "no"; then
    AC_SUBST(F77)
    STV_MAKE_FORTRAN_EXAMPLES="fprogs"
    STV_MAKE_FORTRAN_SHARED_EXAMPLES="sh_fprogs"
    STV_MAKE_FORTRAN_TARGET="p_all"
    STV_MAKE_FORTRAN_SHARED_TARGET="sh_p_all"
else
    AC_MSG_RESULT([No Fortran compiler found! Skipping Fortran examples!])
    STV_MAKE_FORTRAN_TARGET=""
    STV_MAKE_FORTRAN_SHARED_TARGET=""
    STV_MAKE_FORTRAN_EXAMPLES=""
    STV_MAKE_FORTRAN_SHARED_EXAMPLES=""
fi


AC_SUBST(STV_MAKE_FORTRAN_EXAMPLES)
AC_SUBST(STV_MAKE_FORTRAN_SHARED_EXAMPLES)
AC_SUBST(STV_MAKE_FORTRAN_TARGET)
AC_SUBST(STV_MAKE_FORTRAN_SHARED_TARGET)
]
)

#
# check for script prefix
#
AC_DEFUN(STV_SCRIPT_DIR,
[
AC_ARG_WITH(stv_script_prefix,
    [scripts prefix:
  --with-stv_script_prefix=script path prefix
                          The prefix for Cumulvs script pathes.
                          Give the full path:
                              --with-stv_script_prefix='/usr/local/' ],
    , [with_stv_script_prefix=yes])

case "$with_stv_script_prefix" in
    no)
        #  User specified --without-script_prefix
	stv_script_prefix="$prefix"
        ;;
    yes)
        #  User didn't give the option or didn't give useful
        #  information, search for it ourselves
        stv_script_prefix="$prefix"
        ;;
    *)
        stv_script_prefix="$with_stv_script_prefix"
        ;;
esac

if test "$with_stv_script_prefix" = _searching ; then
    stv_script_prefix="$prefix"
fi

STV_SCRIPT_PREFIX=$stv_script_prefix
AC_SUBST(STV_SCRIPT_PREFIX)
]
)


#------------------------------------------------------------------------------
#  see if the user gave us the location of clueful make
#------------------------------------------------------------------------------
AC_DEFUN(STV_INST_PROG_GMAKE,
[
AC_ARG_WITH(gmake, 
    [Location of gmake:
  --with-gmake=EXECUTABLE
                          The location of gmake.  Give the full path:
                              --with-gmake='/share/bizarre/bin/gmake' ] ,
    , [with_gmake=yes])

case "$with_gmake" in
    no)
        stv_inst_have_gmake=no
        ;;
    yes)
        #  User didn't give the option or didn't give useful
       #  information, search for it ourselves
        with_gmake=_searching
        ;;
    *)
        #  Only the library was specified
        GMAKE="$with_gmake"
        ;;
esac

if test "$with_gmake" = _searching ; then
    AC_CHECK_PROG(gmake_1, gmake, gmake, NONE)
    if test ! "$gmake_1" = NONE ; then
        GMAKE="$gmake_1"
    fi
fi
AC_SUBST(GMAKE)
]
)
#############################################################################
# END: STV MAIN MACRO M4                                                    #
#############################################################################
