#include <stdio.h>
#include <avs/avs.h>
#include <avs/port.h>
#include <avs/geom.h>
#include <avs/colormap.h>
#include "triangleData.h"

 
void hsv_to_rgb();
 
/* *****************************************/
/*  Module Description                     */
/* *****************************************/
int GenerateTris_desc()
{

	int in_port, param, iresult;
	extern int GenerateTris_compute();

	AVSset_module_name("GenerateTris", MODULE_MAPPER);

        AVScreate_input_port("Input Colormap","colormap",REQUIRED);

        /* Create an input port for the sample user data structure input */
        AVScreate_input_port("input 1", "struct tri_user_data",REQUIRED); 

	/* Output Port Specifications              */
	AVScreate_output_port("Grid", "geom");


	AVSset_compute_proc(GenerateTris_compute);
        AVSload_user_data_types("triangleData.h");
	return(1);
}
 
/* *****************************************/
/* Module Compute Routine                  */
/* *****************************************/
int 
GenerateTris_compute( cmap, input, Grid)
AVScolormap *cmap;
tri_user_data *input;
GEOMedit_list *Grid;
{

/*  THIS IS A 'HINTS' AREA - YOU MAY CUT AND PASTE AT WILL FROM IT       */
    float *rgba;

	GEOMobj *obj0;
        float *verts;
	float *colors;
        float color;
        float minval, maxval;
	int   *plist, dummy, i, nverts, ntri;
        int   index;

    minval = 1.0e+10;
    maxval = -1.0e+10;
    rgba = (float *)malloc(sizeof(float)*cmap->size*4);
    hsv_to_rgb(cmap->hue,cmap->saturation,cmap->value,cmap->alpha,
               rgba,cmap->size);


  nverts = input->nvert;
  ntri   = input->ntri;
  colors = (float *)malloc( nverts*3 * sizeof(float));
  verts = (float *)malloc( nverts*3 * sizeof(float));

  for (i=0;i<nverts;i++){
    verts[3*i+0] = input->vertices[i][0];
    verts[3*i+1] = input->vertices[i][1];
    verts[3*i+2] = input->vertices[i][2];
    if (minval > input->vertData[i]) minval = input->vertData[i];
    if (maxval < input->vertData[i]) maxval = input->vertData[i];
    /* printf("%f \n",input->vertData[i]); */
  }
  printf("%f %f\n",minval,maxval);

  for (i=0;i<nverts;i++){
    color = (input->vertData[i]-minval)/(maxval-minval);
    index = (int)(255.0*color);
    colors[3*i+0] = *(rgba+(index*4+1));
    colors[3*i+1] = *(rgba+(index*4+2));
    colors[3*i+2] = *(rgba+(index*4+3));
  }

  plist = (int *)malloc( 1+ntri*4 * sizeof(int));
  for (i=0;i<ntri;i++){
    plist[4*i+0] = 3;
    plist[4*i+1] = input->triconn[3*i+0]+1;
    plist[4*i+2] = input->triconn[3*i+1]+1;
    plist[4*i+3] = input->triconn[3*i+2]+1;
  }
  plist[4*ntri] = 0;

/*                                                                       */
/* Create the Grid object                                                */
	obj0 = GEOMcreate_obj(GEOM_POLYHEDRON, GEOM_NULL);
      printf("GEOMcreate_obj done\n");
	GEOMadd_polygons(obj0, plist, GEOM_CONCAVE, GEOM_COPY_DATA);
      printf("GEOMadd_polygons done\n");
	GEOMadd_vertices(obj0, verts, nverts, GEOM_COPY_DATA);
      printf("GEOMadd_vertices done\n");
        GEOMadd_float_colors(obj0, colors, nverts, GEOM_COPY_DATA);
      printf("GEOMadd_float_colors done\n");

/*                                                                       */
/* Create the GEOM Edit List for Grid                                    */
	*Grid = GEOMinit_edit_list(*Grid);
	GEOMedit_geometry(*Grid, "Grid", obj0);
	GEOMdestroy_obj(obj0);
/* free any locally allocated data                                       */
   free(colors);
   free(verts);
/* THIS IS THE END OF THE 'HINTS' AREA. ADD YOUR OWN CODE BETWEEN THE    */
/* FOLLOWING COMMENTS. THE CODE YOU SUPPLY WILL NOT BE OVERWRITTEN.      */
 
/* ----> START OF USER-SUPPLIED CODE SECTION #3 (COMPUTE ROUTINE BODY)   */
/* <---- END OF USER-SUPPLIED CODE SECTION #3                            */
	return(1);
}
 
/* ***********************************************************************/
/* Initialization for modules contained in this file.                    */
/* ***********************************************************************/
static int ((*mod_list[])()) = {
	GenerateTris_desc
};
#define NMODS (sizeof(mod_list) / sizeof(char *))

void
AVSinit_modules()
{
	AVSinit_from_module_list(mod_list, NMODS);
}
 
void
hsv_to_rgb(h,s,v,a,rgba,n)
float *h,*s,*v,*a;
float *rgba;
int n;
{
    static void hsv_to_pixel();
    int i;

    for (i=0;i<n;i++)
        hsv_to_pixel(rgba+(i*4),h+i,s+i,v+i,a+i);
}

static void
hsv_to_pixel(rgb,h,s,v,a)
float *rgb;
float *h,*s,*v,*a;
{
    float f, p, q, t;
    float r, g, b;
    float ht;
    int ir,ig,ib,ia,i;

/*  Make sure not to trash the input colormap */
    ht = *h;

    if (*v == 0.)
    {
        r=0.;
        g=0.;
        b=0.;
    }
    else
    {
        if (*s == 0.)
        {
            r = *v;
            g = *v;
            b = *v;
        }
        else
        {
            ht = ht * 6.0;
            if (ht >= 6.0)
                ht = 0.0;

            i = ht;
            f = ht - i;
            p = *v*(1.0-*s);
            q = *v*(1.0-*s*f);
            t = *v*(1.0-*s*(1.0-f));

            if (i == 0)
            {
                r = *v;
                g = t;
                b = p;
            }
            else if (i == 1)
            {
                r = q;
                g = *v;
                b = p;
            }
            else if (i == 2)
            {
                r = p;
                g = *v;
                b = t;
            }
            else if (i == 3)
            {
                r = p;
                g = q;
                b = *v;
            }
            else if (i == 4)
            {
                r = t;
                g = p;
                b = *v;
            }
            else if (i == 5)
            {
                r = *v;
                g = p;
                b = q;
            }
        }
    }
#if 0
    ia = *a*255.0;
    ir = r*255.0;
    ig = g*255.0;
    ib = b*255.0;

    *rgb = (ia<<24) | (ir<<16) | (ig<<8) | (ib);
#endif
    *rgb = *a;
    *(rgb+1) = r;
    *(rgb+2) = g;
    *(rgb+3) = b;
}

/* <---- END OF USER-SUPPLIED CODE SECTION #4                            */
