
/* $Id: stv_viewer_tri.c,v 4.11 2003/09/09 17:56:19 kohl Exp $ */


/*
 *                      CUMULVS version 1.0:
 *      Computational Steering and Interactive Visualization
 *                   in Distributed Applications
 *           Oak Ridge National Laboratory, Oak Ridge TN.
 *      Authors:  James Arthur Kohl and Philip M. Papadopoulos
 *                   (C) 1996 All Rights Reserved
 *
 *                              NOTICE
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted
 * provided that the above copyright notice appear in all copies and
 * that both the copyright notice and this permission notice appear
 * in supporting documentation.
 *
 * Neither the Institution, Oak Ridge National Laboratory, nor the
 * Authors make any representations about the suitability of this
 * software for any purpose.  This software is provided ``as is''
 * without express or implied warranty.
 *
 * CUMULVS was funded by the U.S. Department of Energy.
 */


/* System Header Files */

#include <stdio.h>


/* STV Header Files */

#include "stv.h"
#include "stv_glob.h"
#include "triangleData.h"


/* AVS Header Files */

#include <avs/avs.h>
#include <avs/field.h>


/* Field Definitions */

#define FIELD_DEFN    "struct tri_user_data"

#define CONNECT_FIELD	"_connect"


/* Constants */

#define NUM_PARAMS			16

#define MAXFLOAT			(1.0e+36)

#define DEFAULT_FIELD		"<available fields>"

#define DEFAULT_PARAM		"<available parameters>"

#define UNSELECTED_FIELD	"<not selected>"

#define UNSELECTED_PARAM	"<not selected>"

#define CHOICE_DELIM		"!"

#define PARAM_NAME			"stv param number"


/* Routines */

void	cleanup();
void	free_vis_frame();
void	output_vis_frame();
void	program_init();
void	sleep_interface();
void	update_field_names();
void	update_interface_params();
void	update_interface_region();
void	update_param_names();


/* Globals */

STV_VIEW_FIELD_GROUP CURRENT_VFG;

STV_VIEWER VIEWER;

STV_VIEW_FIELD CURRENT_VF;

STV_VIS_REGION VISREGION;

float FPARAMS[NUM_PARAMS];

float DATAMIN;
float DATAMAX;

int CHANGED_IDS[NUM_PARAMS];
int NCHANGED;

int PARAM_IDS[NUM_PARAMS];

char FIELDNAME[255];
char FIELDLIST[255];
char APPNAME[255];

char SAVE_FIELD[255];
char SAVE_APP[255];

tri_user_data *VISFRAME;

int NUM_ACTIVE_PARAMS;

int FIELD_NAME_HOLD;

int NO_SYNCH;

int RESTART;

int MYTID;

int FREQ;

int INITIALIZED;
int FIELD_HOLD;
int CONNECTED;
int STEERING;
int OUTPUT;
int SLEEP;

int *GLB, *GUB;

int DIM;


/* Main */

main( argc, argv )
int argc;
char *argv[];
{
	int avs_init();

	/* Initialize */

	printf( "Entry stv_viewer_tri\n" );

	program_init();

	mp_init();

	AVScorout_init( argc, argv, avs_init );

	AVScorout_set_sync( 0 );

	/* Main Loop */

	while ( stvTrue )
	{
		if ( !do_stv_viewer() )
			cleanup();

		/* Synchronize */

		AVScorout_exec();
	}
}


do_stv_viewer()
{
	STV_VIEW_FIELD_GROUP VFG;

	STV_VIEW_FIELD VF;

	STV_VIEW_PARAM VP;

	STV_DECOMPPTR D;

	char conn_name[255];

	float *param_vals[NUM_PARAMS];

	int *glb, *gub;

	int ckregion;
	int ckparms;
	int ckfield;
	int ckfreq;
	int ckapp;

	int i, j, k;
	int id;
	int mp;
	int cc;

	if ( OUTPUT )
	{
		printf( "do_stv_viewer entry\n" );
		fflush( stdout );
	}

	/* Check for Interface Pause */

	if ( SLEEP )
	{
		if ( OUTPUT )
		{
			printf( "Sleeping...\n" );
			fflush( stdout );
		}

		AVScorout_wait();
	}

	/* Check User Parameter Inputs */

	if ( OUTPUT )
	{
		printf( "Checking Interface Parms...\n" );
		fflush( stdout );
	}

	check_parms( &ckregion, &ckparms, &ckfield, &ckfreq, &ckapp );

	if ( OUTPUT )
	{
		printf( "\tckregion=%d\n", ckregion );
		printf( "\tckparms=%d\n", ckparms );
		printf( "\tckfield=%d\n", ckfield );
		printf( "\tckfreq=%d\n", ckfreq );
		printf( "\tckapp=%d\n", ckapp );
		fflush( stdout );
	}

	/* Handle New Application */

	if ( ckapp || !INITIALIZED )
	{
		if ( CONNECTED )
		{
			if ( OUTPUT )
			{
				printf( "\nSending FieldHaltAll\n" );
				fflush( stdout );
			}

			stv_viewer_send_FieldHaltAll( CURRENT_VFG, VIEWER );

			/* Release Steering Tokens */

			VP = VIEWER->vparams;

			while ( VP != NULL )
			{
				if ( VP->token != stvSteerNot )
					stv_viewer_steering_release( VP, VIEWER );
				
				VP = VP->next;
			}

			free_vis_frame( &VISFRAME );

			CONNECTED = stvFalse;
		}

		STEERING = stvFalse;

		if ( OUTPUT )
		{
			printf( "Initializing With Application \"%s\".\n",
				APPNAME );
			fflush( stdout );
		}

		/* Reset Saved Field if App Changed */
		if ( strcmp( APPNAME, SAVE_APP ) )
			strcpy( SAVE_FIELD, "" );

		cc = stv_viewer_init( &VIEWER, MYTID, APPNAME, stvFalse );

		update_field_names( VIEWER->vfields, VIEWER->nvfields,
			cc == stvStatusOk );

		update_param_names( VIEWER->vparams, VIEWER->nvparams,
			cc == stvStatusOk );

		if ( cc != stvStatusOk )
			return( stvFalse );

		INITIALIZED = stvTrue;
	}

	/* Handle New Field */

	if ( ckfield || !CONNECTED || FIELD_HOLD )
	{
		if ( CONNECTED )
		{
			if ( OUTPUT )
			{
				printf( "\nSending FieldHaltGroup\n" );
				fflush( stdout );
			}

			stv_viewer_send_FieldHaltGroup( CURRENT_VFG, VIEWER );

			free_vis_frame( &VISFRAME );

			CONNECTED = stvFalse;
		}

		if ( !strcmp( FIELDNAME, "" ) )
			return( stvTrue );

		if ( !FIELD_HOLD )
		{
			/* Get Main Data Field */

			CURRENT_VF = stv_get_view_field_name( FIELDNAME,
				VIEWER->vfields, VIEWER->nvfields );

			if ( CURRENT_VF == NULL )
			{
				printf( "Error: Field \"%s\" Not Found.\n", FIELDNAME );
				fflush( stdout );

				return( stvTrue );
			}

			/* Get Secondary Connectivity Field */

			sprintf( conn_name, "%s%s", FIELDNAME, CONNECT_FIELD );

			VF = stv_get_view_field_name( conn_name,
				VIEWER->vfields, VIEWER->nvfields );

			if ( VF == NULL )
			{
				printf( "Error: Connect Field \"%s\" Not Found.\n",
					conn_name );
				fflush( stdout );

				return( stvTrue );
			}

			/* Get Decomp Info */

			if ( CURRENT_VF->field->decomp != NULL )
			{
				D = CURRENT_VF->field->decomp;

				GLB = D->glb;
				GUB = D->gub;

				DIM = D->dataDim;
			}

			else if ( CURRENT_VF->field->particle != NULL )
			{
				printf( "No Support for Particle Field \"%s\"!!\n",
					FIELDNAME );

				return( stvFalse );
			}

			else
			{
				printf( "Error: Field \"%s\" No Particle/Decomp!!\n",
					FIELDNAME );

				return( stvFalse );
			}

			if ( strcmp( SAVE_APP, APPNAME )
				|| strcmp( SAVE_FIELD, FIELDNAME ) )
			{
				/* Get Overall Vis Region Bounds */
				/* (Have to Get All of Both Data & Connectivity!) */

				if ( VF->field->decomp != NULL )
				{
					D = VF->field->decomp;

					glb = D->glb;
					gub = D->gub;
				}

				else
				{
					printf( "Error: Field \"%s\" No Decomp!!\n",
						conn_name );

					return( stvFalse );
				}

				/* Init Vis Region to Comp Region & Cell Sizes to 1 */

				STV_REGION_MIN( VISREGION->region, 0 ) =
					( GLB[0] < glb[0] ) ? GLB[0] : glb[0];
				STV_REGION_MAX( VISREGION->region, 0 ) =
					( GUB[0] > gub[0] ) ? GUB[0] : gub[0];

				STV_CELL_OF_DIM( VISREGION->cell, 0 ) = 1;

				STV_REGION_MIN( VISREGION->region, 1 ) =
					( GLB[1] < glb[1] ) ? GLB[1] : glb[1];
				STV_REGION_MAX( VISREGION->region, 1 ) =
					( GUB[1] > gub[1] ) ? GUB[1] : gub[1];
				if ( STV_REGION_MAX( VISREGION->region, 1 ) > 2 )
					STV_REGION_MAX( VISREGION->region, 1 ) = 2;

				STV_CELL_OF_DIM( VISREGION->cell, 1 ) = 1;

				for ( i=DIM ; i < STV_MAX_DIM ; i++ )
				{
					STV_REGION_MIN( VISREGION->region, i ) = 0;
					STV_REGION_MAX( VISREGION->region, i ) = 1;

					STV_CELL_OF_DIM( VISREGION->cell, i ) = 1;
				}

				STV_CELL_PARTICLE( VISREGION->cell ) = 1;

				update_interface_region( VISREGION );

				strcpy( SAVE_FIELD, FIELDNAME );
				strcpy( SAVE_APP, APPNAME );

				sleep_interface( stvTrue );
			}
		}

		/* Request Field */

		if ( SLEEP )
			FIELD_HOLD = stvTrue;
		
		else
		{
			FIELD_HOLD = stvFalse;

			if ( OUTPUT )
			{
				printf( "Requesting Field \"%s\".\n", FIELDNAME );
				fflush( stdout );
			}

			stv_clear_view_field_select( VIEWER );

			CURRENT_VF->selected = stvTrue;

			CURRENT_VF->view_type = stvFloat;

			CURRENT_VF->view_storage_order = stvRowMajor;

			/* Sneak In Extra "Connect" Data Field */

			sprintf( conn_name, "%s%s", FIELDNAME, CONNECT_FIELD );

			VF = stv_get_view_field_name( conn_name,
				VIEWER->vfields, VIEWER->nvfields );

			if ( VF == NULL )
			{
				printf( "Error: Connect Field \"%s\" Not Found.\n",
					conn_name );
				fflush( stdout );

				return( stvTrue );
			}

			VF->selected = stvTrue;

			VF->view_type = stvInt;

			VF->view_storage_order = stvRowMajor;

			/* Request Field */

			CURRENT_VFG = stv_viewer_request_field( VIEWER,
				VISREGION->region, VISREGION->cell,
				FREQ, NO_SYNCH, &RESTART );
		
			if ( CURRENT_VFG == NULL )
			{
				GLB = GUB = (int *) NULL;

				DIM = -1;

				return( stvFalse );
			}

			CONNECTED = stvTrue;
		}
	}

	/* Handle New Vis Region or Cell */

	if ( ckregion && !ckfield && !FIELD_HOLD && stvFalse )
	{
		if ( OUTPUT )
		{
			printf( "Sending New Vis Region.\n" );
			fflush( stdout );
		}

		if ( stv_check_region( CURRENT_VF->field, VISREGION ) )
			update_interface_region( VISREGION );

		/* if is_static flag ever set, need to check return code here */
		stv_viewer_set_VisRegion( CURRENT_VFG, VIEWER, VISREGION );

		free_vis_frame( &VISFRAME );
	}

	/* Handle New Frequency */

	if ( ckfreq && !FIELD_HOLD )
	{
		if ( OUTPUT )
		{
			printf( "Sending New Frequency:  FREQ=%d  REGION TIME = ",
				FREQ );
			stv_dump_time_str( stdout,
					CURRENT_VFG->region_time, stvTrue );
			fflush( stdout );
		}

		stv_viewer_set_VisFrequency( CURRENT_VFG, VIEWER, FREQ );
	}

	/* Handle New Parms */

	if ( ckparms )
	{
		if ( OUTPUT )
		{
			printf( "Parameters Changed - Update\n" );
			fflush( stdout );
		}

		/* Send Update Message */

		stv_viewer_send_NewParams( VIEWER );
	}

	/* Get Data from Application for Frame */

	if ( !SLEEP )
	{
		if ( OUTPUT )
		{
			printf( "Collecting Vis Frame.\n" );
			fflush( stdout );
		}

		if ( VISFRAME == NULL )
			allocate_vis_frame( CURRENT_VFG, &VISFRAME );

		do
		{
			cc = stv_viewer_receive_frame( &VFG, VIEWER, &RESTART,
				stvMsgBlock );

			if ( cc != stvStatusOk && cc != stvStatusBadFrame )
			{
				/* Release Steering Tokens */

				VP = VIEWER->vparams;

				while ( VP != NULL )
				{
					if ( VP->token != stvSteerNot )
					{
						if ( OUTPUT )
						{
							printf(
								"\nReleasing Steering Token for %s\n",
								VP->param->name );
							fflush( stdout );
						}

						stv_viewer_steering_release( VP, VIEWER );
					}
			
					VP = VP->next;
				}

				if ( cc == stvStatusReconfigure )
				{
					if ( OUTPUT )
					{
						printf(
							"\nReconfigure - Sending FieldHaltAll\n" );
						fflush( stdout );
					}

					stv_viewer_send_FieldHaltAll( CURRENT_VFG, VIEWER );
				}

				else
				{
					if ( OUTPUT )
					{
						printf(
					"\nApplication Failure - Sending FieldHaltAll\n" );
						fflush( stdout );
					}

					stv_viewer_send_FieldHaltAll( CURRENT_VFG, VIEWER );
				}

				free_vis_frame( &VISFRAME );

				CONNECTED = stvFalse;

				return( stvFalse );
			}

			if ( VFG != CURRENT_VFG )
				stv_viewer_release_field( &VFG, VIEWER );
		}
		while ( VFG != CURRENT_VFG );

		/* Send XON Now */

		if ( OUTPUT )
		{
			printf( "Sending Next XON for Time " );
			stv_dump_time_str( stdout,
					CURRENT_VFG->region_time, stvTrue );
			fflush( stdout );
		}

		stv_viewer_send_XON( CURRENT_VFG, VIEWER );

		/* Pass Frame Data Out to Visualization System */

		if ( cc == stvStatusOk )
		{
			if ( OUTPUT )
			{
				printf( "Sending Output Data Frame.\n" );
				fflush( stdout );
			}

			output_vis_frame( CURRENT_VFG, &VISFRAME );
		}

		else
		{
			if ( OUTPUT )
			{
				printf( "Dropping Bad Data Frame.\n" );
				fflush( stdout );
			}
		}

		/* Check Lock Status (& Reset) */

		if ( CURRENT_VFG->locked == stvAppLocked )
			sleep_interface( stvTrue );
	}

	else
		stvSleep( 1 );

	return( stvTrue );
}


int
allocate_vis_frame( VFG, frame )
STV_VIEW_FIELD_GROUP VFG;
tri_user_data **frame;
{
	int cc;
	int i;

	for ( i=0 ; i < VFG->nvfields ; i++ )
	{
		/* Allocate Array Declaration */

		if ( (cc = stv_allocate_Field_arrayDecl( VFG, i ))
				!= stvStatusOk )
			return( cc );

		/* Allocate Field Storage */

		if ( stv_allocate_Field_Data( VFG, i, VFG->view_types[i] )
				!= stvStatusOk )
			return( cc );
	}

	/* Allocate Field Data */

	*frame = (tri_user_data *) AVSdata_alloc( FIELD_DEFN, 0 );

	return( stvStatusOk );
}


void
free_vis_frame( frame )
tri_user_data **frame;
{
	if ( frame == NULL || *frame == NULL )
		return;

	AVSdata_free( "field", (char *) *frame );

	*frame = (tri_user_data *) NULL;
}


void
output_vis_frame( VFG, frame )
STV_VIEW_FIELD_GROUP VFG;
tri_user_data **frame;
{
	STV_VIEW_FIELD GRID_VF;
	STV_VIEW_FIELD CONN_VF;

	FILE *fin;

	char d1[256], d2[256];

	float *fptr;

	int *iptr;

	int nvert, ntri;
	int i;

	/* Extract Data Here */

	GRID_VF = CURRENT_VFG->view_field_list[0];

	nvert = GRID_VF->field->decomp->gub[0] - GRID_VF->field->decomp->glb[0] + 1;

	printf( "%s:  nvert = %d\n", "output_vis_frame()", nvert );

	if ( nvert > 2500 ) {
		printf( "You need to set 2500 to at least %d in triangleData.h\n",
			nvert );
		exit( -1 );
	}

	(*frame)->nvert = nvert;

	fptr = (float *) CURRENT_VFG->field_values[0][0];

	for ( i=0 ; i < nvert ; i++ )
	{
		(*frame)->vertices[i][0] = *fptr++;
		(*frame)->vertices[i][1] = *fptr++;
		(*frame)->vertices[i][2] = *fptr;

		(*frame)->vertData[i] = *fptr++;
	}

	CONN_VF = CURRENT_VFG->view_field_list[1];

	ntri = CONN_VF->field->decomp->gub[0] - CONN_VF->field->decomp->glb[0] + 1;

	printf( "%s:  ntri = %d\n", "output_vis_frame()", ntri );

	if ( ntri > 5000 ) {
		printf( "You need to set 5000 to at least %d in triangleData.h\n",
			ntri );
		exit( -1 );
	}

	(*frame)->ntri = ntri;

	iptr = (int *) CURRENT_VFG->field_values[1][0];

	for ( i=0 ; i < ntri ; i++ )
	{
		(*frame)->triconn[ ( 3 * i ) + 0 ] = *iptr++;
		(*frame)->triconn[ ( 3 * i ) + 1 ] = *iptr++;
		(*frame)->triconn[ ( 3 * i ) + 2 ] = *iptr++;
	}

	printf( "%s: done with transformation.\n", "output_vis_frame()" );

	/* Set Data Min & Max */

	AVSfield_set_minmax( (AVSfield *) *frame, &DATAMIN, &DATAMAX );

	/* Output Dummy Data Frames */

	AVScorout_output( (AVSfield_float *) *frame );
}


void
program_init()
{
	int i;

	stv_setopt( stvOptNull, stvOptFullUnpack, stvFalse );

	stv_setopt( stvOptNull, stvOptDump, stvFalse );

	stv_setopt( stvOptNull, stvOptVerbose, 0 );

	CURRENT_VFG = (STV_VIEW_FIELD_GROUP) NULL;

	CURRENT_VF = (STV_VIEW_FIELD) NULL;

	VIEWER = (STV_VIEWER) NULL;

	VISREGION = stv_create_vis_region();

	for ( i=0 ; i < NUM_PARAMS ; i++ )
	{
		FPARAMS[i] = 0.0;

		CHANGED_IDS[i] = -1;

		PARAM_IDS[i] = -1;
	}

	DATAMIN = 0.0;
	DATAMAX = 100.0;

	NCHANGED = -1;

	strcpy( FIELDNAME, "" );
	strcpy( FIELDLIST, "" );
	strcpy( APPNAME, "" );

	strcpy( SAVE_FIELD, "" );
	strcpy( SAVE_APP, "" );

	VISFRAME = (tri_user_data *) NULL;

	FIELD_NAME_HOLD = stvTrue;

	NO_SYNCH = 0;

	RESTART = 0;

	FREQ = 1;

	INITIALIZED = stvFalse;
	FIELD_HOLD = stvFalse;
	CONNECTED = stvFalse;
	STEERING = stvFalse;
	OUTPUT = stvFalse;
	SLEEP = stvFalse;

	GLB = GUB = (int *) NULL;

	DIM = -1;
}


void
cleanup()
{
	/* Clear Pointer to AVS Field Data */

	if ( CURRENT_VFG != NULL )
		CURRENT_VFG->field_values[0][0] = (STV_VALUE) NULL;

	/* Reset STV Globals */

	CURRENT_VFG = (STV_VIEW_FIELD_GROUP) NULL;

	CURRENT_VF = (STV_VIEW_FIELD) NULL;

	/* Unset Initialized Flag */

	INITIALIZED = stvFalse;
}


int
mp_init()
{
	/* Get My Task ID */

	MyMP->myid( &MYTID );

	if ( MYTID < 0 )
	{
		MyMP->perr( "Obtaining My Task ID" );

		exit( 0 );
	}

	else
	{
		printf( "Viewer Alive as TID=[ 0x%x / %d ]\n", MYTID, MYTID );
		fflush( stdout );
	}

	/* Connect to Any Message Passing Trace Facility */

	stv_mp_trace;

	return( stvStatusOk );
}


int
avs_init()
{
	char name[255];

	float finit, fmin, fmax;

	int i;

	printf( "Entry avs_init()\n" );
	fflush( stdout );

	fmin = -1.0 * MAXFLOAT;
	fmax = MAXFLOAT;

	/* Set the module name and type */

	AVSset_module_name( "stv_viewer_tri", MODULE_DATA );

	/* There are no input ports for this module */

	/* Create Data Frame Output Ports */

	AVScreate_output_port( "Triangular Mesh Output", FIELD_DEFN );

	AVSload_user_data_types("../avs/triangleData.h");

	/* AVS Parameters */

	/* Application Name */
	AVSadd_parameter( "Application Name", "string", "", "", "" );

	/* Field Name */
	AVSadd_parameter( "Data Field Name", "string", "", "", "" );
	AVSadd_parameter( "Data Field Name List", "choice",
		DEFAULT_FIELD, DEFAULT_FIELD, CHOICE_DELIM );

	/* Frequency */
	AVSadd_parameter( "Frame Frequency", "integer", FREQ, 1, 1000000 );

	/* No Synch */
	AVSadd_parameter( "No Synch", "boolean", NO_SYNCH, 0, 1 );

	/* Enable / Disable Toggle for the Module */
	AVSadd_parameter( "Sleep", "boolean", 0, 0, 1 );

	/* Enable / Disable Toggle for the Module Output */
	AVSadd_parameter( "Verbose", "boolean", 0, 0, 1 );

	/* Cell Size Parameters */
	AVSadd_parameter( "X Cell Size", "integer", 1, 1, 100 );
	AVSadd_parameter( "Y Cell Size", "integer", 1, 1, 100 );
	AVSadd_parameter( "Z Cell Size", "integer", 1, 1, 100 );

	/* Particle Cell Size */
	AVSadd_parameter( "Particle Cell Size", "integer", 1, 1, 100 );

	/* Visualization Region */
	AVSadd_parameter( "X Min", "integer",    0, 0, 1000 );
	AVSadd_parameter( "X Max", "integer", 1000, 0, 1000 );
	AVSadd_parameter( "Y Min", "integer",    0, 0, 1000 );
	AVSadd_parameter( "Y Max", "integer", 1000, 0, 1000 );
	AVSadd_parameter( "Z Min", "integer",    0, 0, 1000 );
	AVSadd_parameter( "Z Max", "integer", 1000, 0, 1000 );

	/* Data Min & Max Values */

	finit = -100.0;
	AVSadd_parameter( "Min Data Value", "real", &finit, &fmin, &fmax );

	finit = 100.0;
	AVSadd_parameter( "Max Data Value", "real", &finit, &fmin, &fmax );

	/* Steering Parameters */

	AVSadd_parameter( "Steering Parameter Request List", "choice",
		DEFAULT_PARAM, DEFAULT_PARAM, CHOICE_DELIM );

	AVSadd_parameter( "Steering Parameter Release List", "choice",
		DEFAULT_PARAM, DEFAULT_PARAM, CHOICE_DELIM );

	finit = 0.0;

	for ( i=0 ; i < NUM_PARAMS ; i++ )
	{
		sprintf( name, "%s %d", PARAM_NAME, i );

		PARAM_IDS[i] = AVSadd_parameter( name, "real",
			&finit, &fmin, &fmax );

		AVSadd_parameter_prop( PARAM_IDS[i], "title", "string", name );
	}

	/* There is no compute function for this module */

	return( stvStatusOk );
}


/* Update Parameters & Check for Changes */

check_parms( ckregion, ckparms, ckfield, ckfreq, ckapp )
int *ckregion, *ckparms, *ckfield, *ckfreq, *ckapp;
{
	STV_VIEW_PARAM VP;

	STV_VIS_REGION vrtmp;

	float *fparamptrs[NUM_PARAMS];
	float *dataminptr;
	float *datamaxptr;

	float fparams[NUM_PARAMS];
	float datamin;
	float datamax;

	char *param_request;
	char *param_release;
	char *fieldname;
	char *fieldlist;
	char *appname;

	int output;
	int freq;
	int flag;
	int cc;
	int i;

	/* Get Interface Parameter Values */

	vrtmp = stv_create_vis_region();

	AVScorout_input( &appname, &fieldname, &fieldlist,
		&freq, &NO_SYNCH, &SLEEP, &output,
		&( STV_CELLX( vrtmp->cell ) ), &( STV_CELLY( vrtmp->cell ) ),
		&( STV_CELLZ( vrtmp->cell ) ),
		&( STV_CELL_PARTICLE( vrtmp->cell ) ),
		&( STV_XMIN( vrtmp->region ) ), &( STV_XMAX( vrtmp->region ) ),
		&( STV_YMIN( vrtmp->region ) ), &( STV_YMAX( vrtmp->region ) ),
		&( STV_ZMIN( vrtmp->region ) ), &( STV_ZMAX( vrtmp->region ) ),
		&dataminptr, &datamaxptr,
		&param_request, &param_release,
		&( fparamptrs[0] ), &( fparamptrs[1] ), &( fparamptrs[2] ),
		&( fparamptrs[3] ), &( fparamptrs[4] ), &( fparamptrs[5] ),
		&( fparamptrs[6] ), &( fparamptrs[7] ), &( fparamptrs[8] ),
		&( fparamptrs[9] ), &( fparamptrs[10] ), &( fparamptrs[11] ),
		&( fparamptrs[12] ), &( fparamptrs[13] ), &( fparamptrs[14] ),
		&( fparamptrs[15] ) );
	/* For NUM_PARAMS = 16 */

	/* Initialize Check Flags */

	*ckregion = *ckparms = *ckfield = *ckfreq = *ckapp = 0;

	/* Check Application Name */

	if ( strcmp( APPNAME, appname ) )
	{
		strcpy( APPNAME, appname );

		(*ckapp)++;
	}

	/* Check Field Name */

	if ( strcmp( FIELDNAME, fieldname ) )
	{
		strcpy( FIELDNAME, fieldname );

		(*ckfield)++;
	}

	/* Check Field List Selection */

	if ( strcmp( FIELDLIST, fieldlist ) )
	{
		strcpy( FIELDLIST, fieldlist );

		if ( !strcmp( FIELDLIST, UNSELECTED_FIELD ) )
		{
			if ( !FIELD_NAME_HOLD )
			{
				strcpy( FIELDNAME, "" );

				AVSmodify_parameter( "Data Field Name",
					AVS_VALUE | AVS_MINVAL | AVS_MAXVAL,
						FIELDNAME, "", "" );

				(*ckfield)++;
			}
		}

		else if ( strcmp( FIELDLIST, DEFAULT_FIELD ) )
		{
			FIELD_NAME_HOLD = stvFalse;

			strcpy( FIELDNAME, fieldlist );

			AVSmodify_parameter( "Data Field Name",
				AVS_VALUE | AVS_MINVAL | AVS_MAXVAL,
				FIELDNAME, "", "" );

			(*ckfield)++;
		}
	}

	/* Check Frequency */

	if ( FREQ != freq )
	{
		FREQ = freq;

		(*ckfreq)++;
	}

	/* STV Verbose Goes With AVS Verbose */

	if ( OUTPUT != output )
	{
		OUTPUT = output;

		stv_setopt( stvOptNull, stvOptVerbose,
			OUTPUT ? STV_DEBUG_ALL : 0 );
	}

	/* Check Vis Region */

	if ( !stv_match_vis_region( VISREGION, vrtmp, DIM ) && stvFalse )
	{
		if ( OUTPUT )
		{
			for ( i=0 ; i < DIM ; i++ )
			{
				printf( "%d min: %d vs. %d\n", i,
					STV_REGION_MIN( VISREGION->region, i ),
					STV_REGION_MIN( vrtmp->region, i ) );

				printf( "%d max: %d vs. %d\n", i,
					STV_REGION_MAX( VISREGION->region, i ),
					STV_REGION_MAX( vrtmp->region, i ) );
			}
		}

		stv_copy_vis_region( VISREGION, vrtmp );

		(*ckregion)++;
	}

	stv_free_vis_region( &vrtmp );

	/* Handle New Data Value Range */

	if ( !STV_EQUIV( (double) DATAMIN, (double) *dataminptr ) )
	{
		if ( OUTPUT )
			printf( "DATAMIN = %f\n", *dataminptr );

		DATAMIN = *dataminptr;
	}

	if ( !STV_EQUIV( (double) DATAMAX, (double) *datamaxptr ) )
	{
		if ( OUTPUT )
			printf( "DATAMAX = %f\n", *datamaxptr );

		DATAMAX = *datamaxptr;
	}

	/* Handle Steering Selections & Releases */

	if ( INITIALIZED )
	{
		if ( OUTPUT )
		{
			printf( "param_request = %s, param_release = %s\n",
				param_request, param_release );
		}

		/* Parameter Requests */

		flag = 0;

		if ( strcmp( param_request, UNSELECTED_PARAM )
			&& strcmp( param_request, DEFAULT_PARAM ) )
		{
			if ( !STEERING )
			{
				printf( "Initializing Steering\n" );

				cc = stv_viewer_steering_init( VIEWER,
					FREQ, NO_SYNCH, &RESTART );

				printf( "cc=%d\n", cc );

				if ( cc != stvStatusOk )
					return( stvFalse );
			
				STEERING = stvTrue;
			}

			if ( STEERING )
			{
				printf( "Requesting Param Steer %s\n", param_request );

				VP = stv_get_view_param_name( param_request,
					VIEWER->vparams, VIEWER->nvparams );

				printf( "VP=%lx\n", (long) VP );

				if ( VP != NULL )
				{
					printf( "\t%s\n", VP->param->name );

					stv_viewer_steering_request( VP, VIEWER,
							stvMsgNoBlock );
				}

				flag++;
			}
		}

		/* Parameter Releases */

		if ( strcmp( param_release, UNSELECTED_PARAM )
			&& strcmp( param_release, DEFAULT_PARAM ) )
		{
			if ( !STEERING )
			{
				printf( "\nWhoa!! Not Steering, Can't Release?!\n\n" );
				fflush( stdout );
			}

			else
			{
				printf( "Releasing Param Steer %s\n", param_release );

				VP = stv_get_view_param_name( param_release,
					VIEWER->vparams, VIEWER->nvparams );

				if ( VP != NULL )
					stv_viewer_steering_release( VP, VIEWER );

				flag++;
			}
		}

		/* Retry Any Pending Requests */

		VP = VIEWER->vparams;

		while ( VP != NULL )
		{
			if ( VP->token == stvSteerRqstd )
			{
				stv_viewer_steering_request( VP, VIEWER,
						stvMsgNoBlock );

				if ( VP->token == stvSteerToken )
					flag++;
			}

			VP = VP->next;
		}

		/* Update Interface */

		if ( flag )
		{
			update_param_names( VIEWER->vparams, VIEWER->nvparams,
				stvTrue );
		}

		/* Get Steering Parameters */

		for ( i=0 ; i < NUM_PARAMS ; i++ )
			fparams[i] = *(fparamptrs[i]);

		/* Check Steering Parameters */

		if ( OUTPUT )
			printf( "checking params_changed()\n" );

		if ( params_changed( FPARAMS, fparams ) )
			(*ckparms)++;
	}

	/* A O K Return */

	return( stvTrue );
}


params_changed( P1, P2 )
float *P1, *P2;
{
	STV_VIEW_PARAM VP;

	STV_PARAMPTR P;

	int changed;
	int i, j;
	int ret;

	changed = stvFalse;

	i = 0;

	VP = VIEWER->vparams;

	while ( VP != NULL )
	{
		VP->changed = stvFalse;

		if ( VP->token == stvSteerToken )
		{
			P = VP->param;

			for ( j=0 ; j < P->num ; j++ )
			{
				if ( !STV_EQUIV( (double) P1[i], (double) P2[i] ) )
				{
					P1[i] = P2[i];

					if ( stv_viewer_steer_vparameter( VP,
						(STV_VALUE) &(P1[i]), stvFloat, j )
						== stvStatusOk )
					{
						changed = stvTrue;
					}
				}

				i++;
			}
		}

		VP = VP->next;
	}

	return( changed );
}


void
sleep_interface( flag )
int flag;
{
	AVSmodify_parameter( "Sleep", AVS_VALUE, flag, -1, -1 );

	SLEEP = flag;
}


void
update_field_names( vfields, nvfields, valid )
STV_VIEW_FIELD vfields;
int nvfields;
int valid;
{
	STV_VIEW_FIELD VF;

	char choice_list[1024];
	char tmp[255];

	char *selected;

	int flags;

	if ( OUTPUT )
	{
		printf( "Updating Field Names List\n" );
		fflush( stdout );
	}

	flags = AVS_VALUE | AVS_MINVAL | AVS_MAXVAL;

	/* Reset Field Name Hold Flag */

	FIELD_NAME_HOLD = stvTrue;

	/* Clear Manual Entry Field Name */

	if ( !strcmp( SAVE_FIELD, "" ) )
	{
		AVSmodify_parameter( "Data Field Name", flags, "", "", "" );

		strcpy( FIELDNAME, "" );
	}

	/* Set Up Field Name Choice List */

	if ( !valid || vfields == NULL )
	{
		if ( OUTPUT )
		{
			printf( "\tResetting List\n" );
			fflush( stdout );
		}

		/* Reset Field Name List */

		AVSmodify_parameter( "Data Field Name List", flags,
			DEFAULT_FIELD, DEFAULT_FIELD, CHOICE_DELIM );

		strcpy( FIELDLIST, DEFAULT_FIELD );
	}

	else
	{
		/* Build Field Name Choice List */

		VF = vfields;

		sprintf( choice_list, "%s", UNSELECTED_FIELD );

		while ( VF != NULL )
		{
			sprintf( tmp, "%s%s", CHOICE_DELIM, VF->field->name );

			strcat( choice_list, tmp );

			VF = VF->next;
		}

		/* Set Field Name List */

		if ( strcmp( SAVE_FIELD, "" ) )
			selected = SAVE_FIELD;
		
		else
			selected = UNSELECTED_FIELD;

		if ( OUTPUT )
		{
			printf( "\tSetting value to \"%s\"\n", selected );
			printf( "\tSetting choices to \"%s\"\n", choice_list );
			fflush( stdout );
		}

		AVSmodify_parameter( "Data Field Name List", flags,
			selected, choice_list, CHOICE_DELIM );

		strcpy( FIELDLIST, selected );
	}
}


void
update_param_names( vparams, nvparams, valid )
STV_VIEW_PARAM vparams;
int nvparams;
int valid;
{
	STV_VIEW_PARAM VP;

	STV_PARAMPTR P;

	char request_choice_list[1024];
	char release_choice_list[1024];

	char pname[255];
	char name[255];
	char tmp[255];

	int flags;
	int num;
	int i;

	flags = AVS_VALUE | AVS_MINVAL | AVS_MAXVAL;

	if ( OUTPUT )
	{
		printf( "Updating Parameter Names\n" );
		fflush( stdout );
	}

	/* Set Up Parameter Names */

	if ( !valid || vparams == NULL )
	{
		if ( OUTPUT )
		{
			printf( "\tResetting Parameters\n" );
			fflush( stdout );
		}

		/* Reset Parameter Lists */

		AVSmodify_parameter( "Steering Parameter Request List", flags,
			DEFAULT_PARAM, DEFAULT_PARAM, CHOICE_DELIM );

		AVSmodify_parameter( "Steering Parameter Release List", flags,
			DEFAULT_PARAM, DEFAULT_PARAM, CHOICE_DELIM );

		/* Reset Parameters */

		for ( i=0 ; i < NUM_PARAMS ; i++ )
		{
			sprintf( name, "%s %d", PARAM_NAME, i );

			AVSmodify_parameter_prop( name, "title", "string", name );

			AVSparameter_visible( name, 0 );

			FPARAMS[i] = 0.0;
		}

		NUM_ACTIVE_PARAMS = 0;
	}

	else
	{
		/* Set Up Parameter Lists & Steered Parameter Names */

		sprintf( request_choice_list, "%s", UNSELECTED_PARAM );
		sprintf( release_choice_list, "%s", UNSELECTED_PARAM );

		VP = vparams;

		num = 0;

		while ( VP != NULL )
		{
			P = VP->param;

			if ( VP->token == stvSteerNot )
			{
				/* Add to Request List */

				sprintf( tmp, "%s%s", CHOICE_DELIM, P->name );

				strcat( request_choice_list, tmp );
			}

			else if ( VP->token == stvSteerRqstd )
			{
				/* Add to Release List */

				sprintf( tmp, "%s%s", CHOICE_DELIM, P->name );

				strcat( release_choice_list, tmp );
			}

			else if ( VP->token == stvSteerToken )
			{
				/* Add to Release List */

				sprintf( tmp, "%s%s", CHOICE_DELIM, P->name );

				strcat( release_choice_list, tmp );

				for ( i=0 ; i < P->num ; i++ )
				{
					if ( P->num == 1 )
						sprintf( pname, "%s", P->name );

					else
					{
						sprintf( pname, "%s:%s",
							P->name, P->pnames[i] );
					}

					/* Set Parameter Name & Visibility */

					sprintf( name, "%s %d", PARAM_NAME, num );

					AVSmodify_parameter_prop( name, "title", "string",
						pname );

					AVSparameter_visible( name, 1 );

					/* Grab Initial Parameter Value */

					if ( stv_copy_converted_data(
						P->vars[i], P->types[i] & stvTypeMask,
						&(FPARAMS[ num ]), stvFloat )
						!= stvStatusOk )
					{
						printf(
							"Error:  Converted Type Copy Failed.\n" );

						FPARAMS[num] = 0.0;
					}

					printf( "init parm %s (%d) = %f\n", name, num,
						FPARAMS[num] );

					/* Next Parameter */

					num++;
				}
			}

			VP = VP->next;
		}

		NUM_ACTIVE_PARAMS = num;

		if ( OUTPUT )
		{
			printf( "\tSetting request value to \"%s\"\n",
				UNSELECTED_PARAM );
			printf( "\tSetting request choices to \"%s\"\n",
				request_choice_list );

			printf( "\tSetting release value to \"%s\"\n",
				UNSELECTED_PARAM );
			printf( "\tSetting release choices to \"%s\"\n",
				release_choice_list );

			fflush( stdout );
		}

		/* Set List Choices */

		AVSmodify_parameter( "Steering Parameter Request List", flags,
			UNSELECTED_PARAM, request_choice_list, CHOICE_DELIM );

		AVSmodify_parameter( "Steering Parameter Release List", flags,
			UNSELECTED_PARAM, release_choice_list, CHOICE_DELIM );

		/* Reset Remaining Parameters */

		for ( i=num ; i < NUM_PARAMS ; i++ )
		{
			sprintf( name, "%s %d", PARAM_NAME, i );

			AVSmodify_parameter_prop( name, "title", "string", name );

			AVSparameter_visible( name, 0 );

			FPARAMS[i] = 0.0;
		}
	}

	update_interface_params();
}


void
update_interface_region( VR )
STV_VIS_REGION VR;
{
	int flags;

	if ( OUTPUT )
	{
		printf( "update_interface_region() - DIM = %d\n", DIM );
		fflush( stdout );
	}

	flags = AVS_VALUE | AVS_MINVAL | AVS_MAXVAL;

	if ( DIM >= 1 )
	{
		if ( OUTPUT )
		{
			printf( "\tX: cell=%d min=%d max=%d\n",
				STV_CELLX( VR->cell ), STV_XMIN( VR->region ),
				STV_XMAX( VR->region ) );
			fflush( stdout );
		}

		AVSmodify_parameter( "X Cell Size", flags,
			STV_CELLX( VR->cell ), 1, GUB[0] - GLB[0] );

		AVSmodify_parameter( "X Min", flags,
			STV_XMIN( VR->region ), GLB[0], GUB[0] );
		AVSmodify_parameter( "X Max", flags,
			STV_XMAX( VR->region ), GLB[0], GUB[0] );
		
		if ( DIM >= 2 )
		{
			if ( OUTPUT )
			{
				printf( "\tY: cell=%d min=%d max=%d\n",
					STV_CELLY( VR->cell ), STV_YMIN( VR->region ),
					STV_YMAX( VR->region ) );
				fflush( stdout );
			}

			AVSmodify_parameter( "Y Cell Size", flags,
				STV_CELLY( VR->cell ), 1, GUB[1] - GLB[1] );

			AVSmodify_parameter( "Y Min", flags,
				STV_YMIN( VR->region ), GLB[1], GUB[1] );
			AVSmodify_parameter( "Y Max", flags,
				STV_YMAX( VR->region ), GLB[1], GUB[1] );

			if ( DIM >= 3 )
			{
				if ( OUTPUT )
				{
					printf( "\tZ: cell=%d min=%d max=%d\n",
						STV_CELLZ( VR->cell ), STV_ZMIN( VR->region ),
						STV_ZMAX( VR->region ) );
					fflush( stdout );
				}

				AVSmodify_parameter( "Z Cell Size", flags,
					STV_CELLZ( VR->cell ), 1, GUB[2] - GLB[2] );

				AVSmodify_parameter( "Z Min", flags,
					STV_ZMIN( VR->region ), GLB[2], GUB[2] );
				AVSmodify_parameter( "Z Max", flags,
					STV_ZMAX( VR->region ), GLB[2], GUB[2] );
			}

			else
			{
				AVSmodify_parameter( "Z Cell Size", flags, 1, 1, 1);

				AVSmodify_parameter( "Z Min", flags, 0, 0, 1 );
				AVSmodify_parameter( "Z Max", flags, 1, 0, 1 );
			}
		}

		else
		{
			/* Y */

			AVSmodify_parameter( "Y Cell Size", flags, 1, 1, 1);

			AVSmodify_parameter( "Y Min", flags, 0, 0, 1 );
			AVSmodify_parameter( "Y Max", flags, 1, 0, 1 );

			/* Z */

			AVSmodify_parameter( "Z Cell Size", flags, 1, 1, 1);

			AVSmodify_parameter( "Z Min", flags, 0, 0, 1 );
			AVSmodify_parameter( "Z Max", flags, 1, 0, 1 );
		}
	}

	else
	{
		/* X */

		AVSmodify_parameter( "X Cell Size", flags, 1, 1, 1);

		AVSmodify_parameter( "X Min", flags, 0, 0, 1 );
		AVSmodify_parameter( "X Max", flags, 1, 0, 1 );

		/* Y */

		AVSmodify_parameter( "Y Cell Size", flags, 1, 1, 1);

		AVSmodify_parameter( "Y Min", flags, 0, 0, 1 );
		AVSmodify_parameter( "Y Max", flags, 1, 0, 1 );

		/* Z */

		AVSmodify_parameter( "Z Cell Size", flags, 1, 1, 1);

		AVSmodify_parameter( "Z Min", flags, 0, 0, 1 );
		AVSmodify_parameter( "Z Max", flags, 1, 0, 1 );
	}

	if ( OUTPUT )
	{
		printf( "\tParticle cell=%d\n", STV_CELL_PARTICLE( VR->cell ) );
		fflush( stdout );
	}

	AVSmodify_parameter( "Particle Cell Size", flags,
		STV_CELL_PARTICLE( VR->cell ), 1, 100 );
}


void
update_interface_params()
{
	char name[255];

	float fmin, fmax;

	int flags;
	int i;

	if ( OUTPUT )
	{
		printf( "update_interface_params() - NUM_ACTIVE_PARAMS = %d\n",
			NUM_ACTIVE_PARAMS );
		fflush( stdout );
	}

	flags = AVS_VALUE | AVS_MINVAL | AVS_MAXVAL;

	fmin = -1.0 * MAXFLOAT;
	fmax = MAXFLOAT;

	for ( i=0 ; i < NUM_ACTIVE_PARAMS ; i++ )
	{
		sprintf( name, "%s %d", PARAM_NAME, i );

		if ( OUTPUT )
		{
			printf( "parm %s (%d) = %f\n", name, i, FPARAMS[i] );
			fflush( stdout );
		}

		AVSmodify_parameter( name, flags, &(FPARAMS[i]), &fmin, &fmax );
	}
}

