
C $Id: flaplace.f,v 4.1 2001/11/11 23:36:58 kohl Exp $


C
C                      CUMULVS version 1.0:
C      Computational Steering and Interactive Visualization
C                   in Distributed Applications
C           Oak Ridge National Laboratory, Oak Ridge TN.
C      Authors:  James Arthur Kohl and Philip M. Papadopoulos
C                   (C) 2001 All Rights Reserved
C
C                              NOTICE
C
C Permission to use, copy, modify, and distribute this software and
C its documentation for any purpose and without fee is hereby granted
C provided that the above copyright notice appear in all copies and
C that both the copyright notice and this permission notice appear
C in supporting documentation.
C
C Neither the Institution, Oak Ridge National Laboratory, nor the
C Authors make any representations about the suitability of this
C software for any purpose.  This software is provided ``as is''
C without express or implied warranty.
C
C CUMULVS was funded by the U.S. Department of Energy.
C


C  flaplace.f - A small 'non-trivial' program for playing with CUMULVS.

      program flaplace

      implicit none
      include 'fpvm3.h'
      include 'fstv.h'
      include 'flaplace.inc'


C     --  Variable declarations  --

      real*4 density(0:IDIM+1, 0:JDIM+1, 0:KDIM+1)
      real*4 bounds(0:3)
      real*4 value
      real*4 dt

      integer axisType(3)
      integer cc
      integer declare(3), decompId(NFIELDS)
      integer fieldId(NFIELDS)
      integer glb(3), gub(3)
      integer i, info
      integer j
      integer k
      integer marker, myinum, mytid
      integer myrow, mycol
      integer n, neigh(0:3)
      integer nspawn
      integer offsets(3)
      integer p, paramIds(NPARAMS), prank, pshape(2), paddr(2)
      integer t, tids(NTASKS)
      integer sequence

      marker = -1

      print *, 'Entry flaplace.'

C     --  PVM Initialization  --

      call pvmfmytid( mytid )
      print *, 'LaPlace Task Joined PVM as tid=', mytid

      call pvmfjoingroup( LAPLACEGRP, myinum )
      print *, 'Joined Group ',LAPLACEGRP,' as Instance ',myinum

C     --- Should I spawn other tasks?

      if ( myinum .eq. 0 ) then 

          if ( NTASKS - 1 .gt. 0 ) then

              call pvmfcatchout( 1, info )

              call pvmfspawn( 'flaplace',
     +                PvmTaskDefault,
C    +                PvmTaskDefault + PvmTaskDebug,
     +                ' ', NTASKS - 1, tids, nspawn )

              if ( nspawn .lt. NTASKS - 1 ) then

                  print *, 'Spawn failed with nspawn = ',nspawn
                  print *, 'Killing orphaned tasks...'

                  do 100 i = 1, nspawn
                      call pvmfkill( tids(i), info )
100               continue

                  print *, 'Failed spawns returned codes:'

                  do 110 i = nspawn + 1, NTASKS - 1
                      print *, 'Info was: ', tids(i)    
110               continue

                  call pvmfexit( info )

                  call exit( -1 )

              endif

          endif

      endif

C     --- Make the group static -- this is also a barrier 

      call pvmffreezegroup( LAPLACEGRP, NTASKS, info )

C     --- Determine neighbors (N,E,S,W) for decomposition

      myrow = myinum / COLUMNS
      mycol = mod( myinum, COLUMNS )

      print *, 'MyRow = ',myrow,', MyCol = ',mycol

C     North
      if ( myrow .gt. 0 ) then
          call pvmfgettid( LAPLACEGRP,
     +            ( ( myrow - 1 ) * COLUMNS ) + mycol, neigh(NORTH) )
      else
          neigh(NORTH) = -1
      endif

C     East
      if ( mycol .lt. COLUMNS - 1 ) then
          call pvmfgettid( LAPLACEGRP,
     +            ( myrow * COLUMNS ) + mycol + 1, neigh(EAST) )
      else
          neigh(EAST) = -1
      endif

C     South
      if ( myrow .lt. ROWS - 1 ) then
          call pvmfgettid( LAPLACEGRP,
     +            ( ( myrow + 1 ) * COLUMNS ) + mycol, neigh(SOUTH) )
      else
          neigh(SOUTH) = -1
      endif

C     West
      if ( mycol .gt. 0 ) then
          call pvmfgettid( LAPLACEGRP,
     +            ( myrow * COLUMNS ) + mycol - 1, neigh(WEST) )
      else
          neigh(WEST) = -1
      endif

      print *, 'North Neighbor = ',neigh(NORTH)
      print *, 'East Neighbor = ',neigh(EAST)
      print *, 'South Neighbor = ',neigh(SOUTH)
      print *, 'West Neighbor = ',neigh(WEST)

C     notify me if one of my neighbors fails
      do 200 n = 0, 3

          if ( neigh(n) .gt. 0 ) then
              call pvmfnotify( PvmTaskExit, NEIGHMSG, 1, neigh(n),
     +            info )
          endif

200   continue

C     --- Initialize the program parameters

      bounds(NORTH) = 1.0
      bounds(EAST) = 2.0
      bounds(SOUTH) = 3.0
      bounds(WEST) = 4.0

      dt = 1.0

      sequence = 0

C     --- Initialize density array

      call initarray( density, myrow, mycol, bounds, stvTrue )

C     --- Initialize CUMULVS steering and visualization

      if ( VERBOSE .ne. 0 ) then
          call stvfsetopt( stvOptNull, stvOptVerbose, STVDEBUGALL,
     +        info )
      else
          call stvfsetopt( stvOptNull, stvOptVerbose, 0, info )
      endif

C     call stvfsetopt( stvOptNull, stvOptAppLock, stvTrue, info )

      call stvfinit( LAPLACEGRP, STVTAG, NTASKS, myinum, info )

C     --- Define the steering parameters

      call stvfparamDefine( dt, 'DeltaT',
     +        stvFloat, stvVisOnly, paramIds(1) )

      call stvfparamDefine( bounds(NORTH), 'Bounds North',
     +        stvFloat, stvVisOnly, paramIds(2) )

      call stvfparamDefine( bounds(EAST), 'Bounds East',
     +        stvFloat, stvVisOnly, paramIds(3) )

      call stvfparamDefine( bounds(SOUTH), 'Bounds South',
     +        stvFloat, stvVisOnly, paramIds(4) )

      call stvfparamDefine( bounds(WEST), 'Bounds West',
     +        stvFloat, stvVisOnly, paramIds(5) )

C     --- Record the size of the computational region

      glb(1) = 1
      glb(2) = 1
      glb(3) = 1

      gub(1) = ROWS * IDIM
      gub(2) = COLUMNS * JDIM
      gub(3) = KDIM

C     --- Declare a decomposition of density
C     -- it is block, block, collapsed

      axisType(1) = stvBlock
      axisType(2) = stvBlock
      axisType(3) = stvCollapse

      prank = 2
      pshape(1) = ROWS
      pshape(2) = COLUMNS

      call stvfdecompDefine( 3, axisType,
     +    stvDefaultInfo, stvDefaultInfo, glb, gub, prank, pshape,
     +    decompId(1) )

C     --- declare the density data field with the above decomp

      paddr(1) = myrow
      paddr(2) = mycol

      offsets(1) = 1
      offsets(2) = 1
      offsets(3) = 1

      declare(1) = IDIM + 2
      declare(2) = JDIM + 2
      declare(3) = KDIM + 2

      call stvffieldDefine( density, 'density', decompId(1),
     +    offsets, declare, stvFloat, paddr, stvVisOnly, fieldId(1) )

C     set simulation time step for density field to 1 msec

      call stvftimeof( 1, stvTimeMilliSec, t )
      call stvfsetopt( fieldId(1), stvOptFieldTimeScale, t, info )

C     set base time to 5 mins + 40 secs + 10 msecs = 340,010 msecs

      call stvftimeof( 5, stvTimeMin, t )
      call stvfsetopt( fieldId(1), stvOptSetFieldTime, t, info )

      call stvftimeof( 40, stvTimeSec, t )
      call stvfsetopt( fieldId(1), stvOptIncrFieldTime, t, info )

      call stvftimeof( 10, stvTimeMilliSec, t )
      call stvfsetopt( fieldId(1), stvOptIncrFieldTime, t, info )

C     --- Verify Local Field Decomposition

      call stvfdumpLocalFieldBounds( fieldId(1), stvVerbose, info )

      call stvfdumpLocalFieldBounds( fieldId(1), stvTerse, info )

C     ---  Top of main computational loop

300   continue

          sequence = sequence + 1

          if ( myinum .eq. 0 ) then
              print *, 'Sequence ',sequence
          endif

C         Calculate Density

          do 330 i = 1, IDIM

              do 320 j = 1, JDIM

                  do 310 k = 1, KDIM

C                     Calculate LaPlace Average
                      value = ( density( i - 1, j, k )
     +                            + density( i, j + 1, k )
     +                            + density( i + 1, j, k )
     +                            + density( i, j - 1, k )
     +                            + density( i, j, k - 1 )
     +                            + density( i, j, k + 1 ) )
     +                        / 6.0

C                     Apply Timestep Attentuation
                      density(i, j, k) = density(i, j, k)
     +                        + ( ( value - density(i, j, k) ) / dt )

C                     if (myinum.eq.0 .and. i.eq.5 .and. j.eq.5) then
C                         print *, i,j,k,density(i,j,k)
C                     endif

310               continue

320           continue

330       continue

C         --- send a message to neighbors, exchange ghost data

          do 450 n = 0, 3

              if ( neigh(n) .gt. 0 ) then
                  call pvmfinitsend( PvmDataDefault, info )

C                 to distinguish notify
                  call pvmfpack( INTEGER4, marker, 1, 1, info )

                  call pvmfpack( INTEGER4, n, 1, 1, info )

                  if ( n .eq. NORTH ) then
                      do 410 j = 1, JDIM
                          call pvmfpack( REAL4, density(1, j, 1),
     +                            KDIM, (IDIM+2) * (JDIM+2), info )
410                   continue

                  else if ( n .eq. EAST ) then
                      do 420 i = 1, IDIM
                          call pvmfpack( REAL4, density(i, JDIM, 1),
     +                            KDIM, (IDIM+2) * (JDIM+2), info )
420                   continue

                  else if ( n .eq. SOUTH ) then
                      do 430 j = 1, JDIM
                          call pvmfpack( REAL4, density(IDIM, j, 1),
     +                            KDIM, (IDIM+2) * (JDIM+2), info )
430                   continue

                  else if ( n .eq. WEST ) then
                      do 440 i = 1, IDIM
                          call pvmfpack( REAL4, density(i, 1, 1),
     +                            KDIM, (IDIM+2) * (JDIM+2), info )
440                   continue

                  endif

                  call pvmfsend( neigh(n), NEIGHMSG, info )

              endif

450       continue

C         --- receive a message from each neighbor

          do 500 k = 0, 3

              if ( neigh(k) .gt. 0 ) then
                  call pvmfrecv( neigh(k), NEIGHMSG, info )

C                 Check Marker (for Notify)
                  call pvmfunpack( INTEGER4, cc, 1, 1, info )

                  if ( cc .gt. 0 ) then
                      print *, 'Notify Message Received for ',cc,'!'
                      print *, 'Bailing Out!'
                      call pvmfexit( info )
                      call exit( -1 )
                  endif

                  call pvmfunpack( INTEGER4, n, 1, 1, info )

                  if ( n .eq. NORTH ) then
                      do 460 j = 1, JDIM
                          call pvmfunpack( REAL4, density(IDIM+1, j, 1),
     +                            KDIM, (IDIM+2) * (JDIM+2), info )
460                   continue

                  else if ( n .eq. EAST ) then
                      do 470 i = 1, IDIM
                          call pvmfunpack( REAL4, density(i, 0, 1),
     +                            KDIM, (IDIM+2) * (JDIM+2), info )
470                   continue

                  else if ( n .eq. SOUTH ) then
                      do 480 j = 1, JDIM
                          call pvmfunpack( REAL4, density(0, j, 1),
     +                            KDIM, (IDIM+2) * (JDIM+2), info )
480                   continue

                  else if ( n .eq. WEST ) then
                      do 490 i = 1, IDIM
                          call pvmfunpack( REAL4, density(i, JDIM+1, 1),
     +                            KDIM, (IDIM+2) * (JDIM+2), info )
490                   continue

                  endif

              endif

500       continue

C         --- send messages to any attached CUMULVS viewers

          call stvfsendReadyData( stvSendDefault, info )

C         --- handle (any) steering parameter updates from viewer

          if ( info .gt. 0 ) then

              call dumpparamschanged( info, paramIds,
     +            density, myrow, mycol, dt, bounds )

          endif

      goto 300

      end


      subroutine dumpparamschanged( nchanged, paramIds,
     +    density, myrow, mycol, dt, bounds )

      include 'fstv.h'
      include 'flaplace.inc'

      integer nchanged
      integer paramIds(NPARAMS)
      real*4 density(0:IDIM+1, 0:JDIM+1, 0:KDIM+1)
      integer myrow, mycol
      real*4 dt
      real*4 bounds(0:3)
	  integer p, info

      print *, 'stvfsendReadyData returned ',nchanged

      do 1000 p = 1, NPARAMS

          call stvfisParamChanged( paramIds(p), info )

          if ( info .ne. 0 ) then

              if ( p .eq. 1 ) then

                  print *, 'Parameter DeltaT was changed to ',dt

              else if ( p .eq. 2 ) then

                  print *, 'Parameter Bounds was changed to:'
                  print *, '\tNorth = ',bounds(NORTH)
                  print *, '\tEast = ',bounds(EAST)
                  print *, '\tSouth = ',bounds(SOUTH)
                  print *, '\tWest = ',bounds(WEST)

                  call initarray( density, myrow, mycol, bounds,
     +                stvFalse )

              endif

          endif

1000  continue

      return

      end


      subroutine initarray( density, myrow, mycol, bounds, clearflag )

      include 'fstv.h'
      include 'flaplace.inc'

      real*4 density(0:IDIM+1, 0:JDIM+1, 0:KDIM+1)
      integer myrow, mycol
      real*4 bounds(0:3)
      integer clearflag

      integer i, j, k

      do 2030 i = 0, IDIM + 1

          do 2020 j = 0, JDIM + 1

              do 2010 k = 0, KDIM + 1

C                 Fixed North Boundary
                  if ( i .eq. 0 .and. myrow .eq. 0 ) then
                      density(i, j, k) = bounds(NORTH)

C                 Fixed East Boundary
                  else if ( j .eq. JDIM + 1
     +                    .and. mycol .eq. COLUMNS - 1 ) then
                      density(i, j, k) = bounds(EAST)

C                 Fixed South Boundary
                  else if ( i .eq. IDIM + 1
     +                    .and. myrow .eq. ROWS - 1 ) then
                      density(i, j, k) = bounds(SOUTH)

C                 Fixed West Boundary
                  else if ( j .eq. 0 .and. mycol .eq. 0 ) then
                      density(i, j, k) = bounds(WEST)

C                 Default (Ghost or Interior)
                  else if ( clearflag .ne. 0 ) then
                      density(i, j, k) = 0.0
                  endif

2010          continue

2020      continue

2030  continue

      return

      end

