
/* $Id: gifvwr.cgi.c,v 4.6 2003/09/05 21:25:07 kohl Exp $ */


/* Headers */

#include <stdio.h>

#ifdef USE_MSG

#include <sys/types.h>
#include <unistd.h>

#include "stv.h"

#endif


/* Constants */

#ifdef USE_MSG

#ifndef GIF_VIEWER_GROUP
#define GIF_VIEWER_GROUP \
	"STV_GIF_VIEWER"
#endif

#endif

#ifndef GIF_VIEWER_ROOT
#define GIF_VIEWER_ROOT \
	"/home/user/.wwwhome"
#endif

#define CGI_TIMEOUT    20


/* Routines */

void regurgitate_file();
void dump_header();
void dump_trailer();
void bail();

void unescape_url();
void plustospace();
void spacetoplus();

char *getenv();

char x2c();

int handle_input();

int getword();


/* Globals */

#ifdef USE_MSG
int MYID;
int VTAG;
int VID;
#endif


/* MAIN */

main()
{
	char fname[255];

	int cc;

#ifdef USE_MSG

	/* Set Proper UID (kohl) */

	if ( setuid( 30729 ) < 0 )
		bail( "setuid failed" );

	/* Attach to Message Passing System */

	if ( !(MyMP->myid( &MYID )) )
		bail( "Error Connecting to Message Passing System" );

	/* Verify and/or Create CUMULVS System Contexts :-) */

	if ( stv_verify_system_contexts( stvTrue, stvTrue ) < 0 )
		bail( "Error Verifying CUMULVS System Contexts" );

	/* Look Up GIF Viewer Group */

	if ( MyMP->get_id_from_inst( GIF_VIEWER_GROUP, 0, &VID ) < 0
			|| VID < 0 )
		bail( "Error Finding Instance 0 of GIF Viewer Group" );
	
	VTAG = 123;

#endif

	/* Handle Form Input */

	if ( handle_input() )
	{
		/* Dump HTML from Viewer */

		sprintf( fname, "%s/stv.html", GIF_VIEWER_ROOT );

		regurgitate_file( fname );
	}

	else
	{
		bail(
		"Error Communicating With Viewer - Check \"gifvwr\" Program." );
	}

	/* Done */

	exit( 0 );
}


/* HTML Dump Functions */

void
dump_header()
{
	printf( "Content-type:  text/html\n\n" );

	printf( "<HTML>\n" );

	printf( "<HEADER>\n" );
	printf( "<TITLE>\n" );
	printf( "CUMULVS Browser (GIF-O-Matic) 1.0\n" );
	printf( "</TITLE>\n" );
	printf( "</HEADER>\n" );

	printf( "<BODY>\n" );
}


void
dump_trailer()
{
	printf( "</BODY>\n" );

	printf( "</HTML>\n" );
}


void
bail( str )
char *str;
{
	dump_header();

	printf( "<CENTER>\n" );
	printf( "<H1>CUMULVS Browser (GIF-O-Matic) 1.0</H1>\n" );
	printf( "</CENTER>\n" );

	printf( "<H3>%s</H3>\n", str );

	dump_trailer();

	exit( 0 );
}


/* CGI Lib Functions */

int
handle_input()
{
	char line[1024];
	char tmp[1024];
	char key[1024];

	char *query;
	char *str;

#ifdef USE_MSG
	int sbuf;
	int rbuf;
#else
	FILE *params;
	FILE *confirm;
	char cfname[1024];
	char fname[1024];
	char cmd[1024];
	int cnt;
#endif

	int more;
	int len;
	int cc;
	int x;

#ifdef USE_MSG

	/* Create Message Buffer */

	MyMP->init_msg_buf( stvDataDefault, &sbuf );

#else

	/* Create Parameters File */

	sprintf( fname, "%s/gifvwr.dat.tmp", GIF_VIEWER_ROOT );

	params = fopen( fname, "w" );

	if ( params == NULL )
		bail( "Failed to Open GIF Viewer Parameters File!" );

#endif

	/* Parse Query String */

	str = (char *) NULL;

	query = getenv( "QUERY_STRING" );

	if ( query == NULL || !strcmp( query, "" ) )
	{
		if ( fgets( line, 1024, stdin ) )
		{
			if ( line[ strlen( line ) - 1 ] == '\n'
					|| line[ strlen( line ) - 1 ] == '^M' )
				line[ strlen( line ) - 1 ] = '\0';

			str = line;

			more = 1;
		}
	}

	else
	{
		str = query;

		more = 0;
	}

	for ( x=0 ; str && str[0] != '\0' ; x++ )
	{
		if ( !getword( tmp, str, '&' ) )
		{
			if ( more )
			{
				strcpy( line, tmp );

				len = strlen( tmp );

				if ( 1024 - len <= 0 )
					bail( "Form Input Overflow!!" );

				if ( fgets( line + len, 1024 - len, stdin ) )
				{
					line[ strlen( line ) - 1 ] = '\0';

					str = line;

					if ( !getword( tmp, str, '&' ) )
						bail( "Form Input Overflow!!" );
				}

				else
					*str = '\0';
			}
		}

		plustospace( tmp );

		unescape_url( tmp );

		getword( key, tmp, '=' );

#ifdef USE_MSG
		MyMP->pack_str( sbuf, key );
		MyMP->pack_str( sbuf, tmp );
#else
		fprintf( params, "%s\n%s\n", key, tmp );
#endif
	}

#ifdef USE_MSG

	/* Pack End of Data String */

	MyMP->pack_str( sbuf, "EOF" );

	/* Send Viewer the Update Message */

	MyMP->send_msg( sbuf, VID, VTAG, STV_BASECTX );

	/* Wait for Acknowledgment Message (HTML Output Ready) */

	MyMP->recv_msg( VID, VTAG, STV_BASECTX, &rbuf );

	/* Verify Valid Result */

	MyMP->unpack_int( rbuf, &cc, 1, 1 );

#else

	/* Pack End of Data String */

	fprintf( params, "EOF\n" );

	fclose( params );

	/* Move Viewer File Into Place */

	sprintf( cmd, "mv %s %s/gifvwr.dat", fname, GIF_VIEWER_ROOT );

	system( cmd );

	/* Wait for New HTML File */

	cnt = 0;

	sprintf( cfname, "%s/gifvwr.confirm", GIF_VIEWER_ROOT );

	while ( (confirm = fopen( cfname, "r" )) == NULL
		&& cnt < CGI_TIMEOUT )
	{
		sleep( 1 );

		cnt++;
	}

	/* Verify Valid Result */

	if ( confirm != NULL )
	{
		fscanf( confirm, "%d", &cc );

		fclose( confirm );

		/* Make Sure Confirm File Gone... */

		cnt = 0;

		while ( unlink( cfname ) < 0 && cnt < CGI_TIMEOUT )
		{
			sleep( 1 );

			cnt++;
		}

		if ( cnt == CGI_TIMEOUT )
			bail( "Error Removing GIFVWR Confirm File!\n" );
	}

	else
		cc = 0;

#endif

	return( cc );
}


void
regurgitate_file( fname )
char *fname;
{
	FILE *html;

	char fname2[1024];
	char line[1024];
	char cmd[1024];

	/* Make a Copy so GifVwr Won't Whack Us! */

	sprintf( fname2, "%s.%d", fname, getpid() );

	sprintf( cmd, "mv %s %s", fname, fname2 );

	system( cmd );

	/* Regurgitate File to Browser */

	html = fopen( fname2, "r" );

	if ( html == NULL )
		bail( "Error Opening Viewer HTML Output" );
	
	printf( "Content-type:  text/html\n\n" );

	while ( fgets( line, 1024, html ) )
		printf( "%s", line );

	fclose( html );

	fflush( stdout );

	unlink( fname2 );
}


int
getword( word, line, stop )
char *word;
char *line;
char stop;
{
    int x = 0, y;
	int ret;

    for( x=0 ; ( (line[x]) && (line[x] != stop) ) ; x++ )
        word[x] = line[x];

	ret = ( line[x] == stop ) ? 1 : 0;

    word[x] = '\0';

    if ( line[x] ) ++x;

    y = 0;

    while( line[ y++ ] = line[ x++ ] );

	return( ret );
}


char
x2c( what )
char *what;
{
    register char digit;

    digit = (what[0] >= 'A' ?
		((what[0] & 0xdf) - 'A')+10 : (what[0] - '0'));
    digit *= 16;
    digit += (what[1] >= 'A' ?
		((what[1] & 0xdf) - 'A')+10 : (what[1] - '0'));

    return( digit );
}


void
unescape_url( url )
char *url;
{
    register int x, y;

    for ( x=0, y=0 ; url[y] ; ++x, ++y ) {
        if ( (url[x] = url[y]) == '%' ) {
            url[x] = x2c( &url[ y + 1 ] );
            y += 2;
        }
    }

    url[x] = '\0';
}


void
plustospace( str )
char *str;
{
    register int x;

    for ( x=0 ; str[x] ; x++ ) if ( str[x] == '+' ) str[x] = ' ';
}


void
spacetoplus( str )
char *str;
{
    register int x;

    for ( x=0 ; str[x] ; x++ ) if ( str[x] == ' ' ) str[x] = '+';
}

