
#include <stdio.h> 

#ifdef USE_GETTIMEOFDAY

#include <time.h> // for gettimeofday 
#include <sys/time.h> 

static struct timeval TimerStartValue = { -1, -1 };

void timer_start()
{
	gettimeofday( &TimerStartValue, NULL ); 
}

long timer_elapsed()
{ 
	struct timeval tv2;

	long elapsed;

	gettimeofday( &tv2, NULL ); 

	elapsed = tv2.tv_usec - TimerStartValue.tv_usec
		+ 1000000 * (tv2.tv_sec - TimerStartValue.tv_sec); 

	return( elapsed ); 
} 

#else

#include <sys/time.h> 
#include <sys/resource.h>
#include <unistd.h>

static struct rusage TimerStartValue;

void timer_start()
{
	getrusage( RUSAGE_SELF, &TimerStartValue ); 

	/*
	printf("timer_start: U=%d/%d S=%d/%d\n",
			TimerStartValue.ru_utime.tv_sec,
			TimerStartValue.ru_utime.tv_usec,
			TimerStartValue.ru_stime.tv_sec,
			TimerStartValue.ru_stime.tv_usec );
	*/
}

long timer_elapsed()
{ 
	struct rusage ru2;

	long elapsed;

	getrusage( RUSAGE_SELF, &ru2 ); 

	/*
	printf("timer_elapsed: U=%d/%d S=%d/%d\n",
			ru2.ru_utime.tv_sec,
			ru2.ru_utime.tv_usec,
			ru2.ru_stime.tv_sec,
			ru2.ru_stime.tv_usec );
	*/

	elapsed = ru2.ru_utime.tv_usec - TimerStartValue.ru_utime.tv_usec
		+ 1000000
			* (ru2.ru_utime.tv_sec - TimerStartValue.ru_utime.tv_sec)
		+ ru2.ru_stime.tv_usec - TimerStartValue.ru_stime.tv_usec
		+ 1000000
			* (ru2.ru_stime.tv_sec - TimerStartValue.ru_stime.tv_sec);

	return( elapsed );
} 

#endif

