
/* $Id: trick.c,v 4.7 2002/02/08 17:08:56 kohl Exp $ */


/*
 *                      CUMULVS version 1.0:
 *      Computational Steering and Interactive Visualization
 *                   in Distributed Applications
 *           Oak Ridge National Laboratory, Oak Ridge TN.
 *      Authors:  James Arthur Kohl and Philip M. Papadopoulos
 *                   (C) 2001 All Rights Reserved
 *
 *                              NOTICE
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted
 * provided that the above copyright notice appear in all copies and
 * that both the copyright notice and this permission notice appear
 * in supporting documentation.
 *
 * Neither the Institution, Oak Ridge National Laboratory, nor the
 * Authors make any representations about the suitability of this
 * software for any purpose.  This software is provided ``as is''
 * without express or implied warranty.
 *
 * CUMULVS was funded by the U.S. Department of Energy.
 */


/*
	trick.c - A small "TRIangle mesh ChecK" program for playing
	with CUMULVS (namely the stv_viewer_tri AVS viewer).
*/


#include <stdio.h>
#include "stv.h"


#define TRICKGRP	"trick"
#define STVTAG		100
#define NFIELDS		2


/* --- Program Options --- */

/* #define VERBOSE    stvTrue */
#define VERBOSE    stvFalse


/* --- Globals --- */

float *grid;

int *triconn;


/* --- Main Program --- */

main()
{
	/* --  Variable declarations  -- */

	int cc;
	int declare[2], decompId[NFIELDS];
	int fieldId[NFIELDS];
	int glb[2], gub[2];
	int i, j, k, info;
	int myinum, mytid;
	int sequence;

	int nvert, ntri, i1, i2, i3;
	float x, y, z;
	char d1[256], d2[256];
	int iresult;
	FILE *fin;

	/* --- Open Data File And Read It In! --- */

	fin = fopen( "tri.dat", "r" );

	if ( fin == NULL )
	{
		printf( "\nError Opening Sample File \"tri.dat\" - Bail!\n" );
		exit( -1 );
	}

	fscanf( fin, "%s", d1 );

	while ( strcmp( d1, "POINTS" ) )
		fscanf( fin, "%s", d1 );

	fscanf( fin, "%d %s\n", &nvert, d1 );

	printf( "Number of Vertices:  %d\n", nvert );

	grid = (float *) malloc( (unsigned) ( 3 * nvert )
			* sizeof( float ) );

	if ( grid == NULL )
	{
		printf( "\nError Allocating Field Array!\n" );
		exit( -1 );
	}

	for ( i=0 ; i < nvert ; i++ )
	{
		fscanf( fin, "%f %f %f", &x, &y, &z );

		*( grid + ( 3 * i ) + 0 ) = x;
		*( grid + ( 3 * i ) + 1 ) = y;
		*( grid + ( 3 * i ) + 2 ) = z;
	}

	fscanf( fin, "%s", d1 );

	while ( strcmp( d1, "CELLS" ) )
		fscanf( fin, "%s", d1 );

	fscanf(	fin, "%d %d", &ntri, &i1 );

	printf( "Number of Triangles:  %d\n", ntri );

	triconn = (int *) malloc( (unsigned) ( 3 * ntri ) * sizeof( int ) );

	if ( triconn == NULL )
	{
		printf( "\nError Allocating Connect Array\n" );
		exit( -1 );
	}

	for ( i=0 ; i < ntri ; i++ )
	{
		fscanf( fin, "%d %d %d", &i1, &i2, &i3 );

		*( triconn + ( 3 * i ) + 0 ) = i1;
		*( triconn + ( 3 * i ) + 1 ) = i2;
		*( triconn + ( 3 * i ) + 2 ) = i3;
	}

	fclose( fin );

	/* --- Initialize CUMULVS steering and visualization */

	stv_setopt( stvOptNull, stvOptVerbose,
		VERBOSE ? STV_DEBUG_ALL : 0 );

	/* stv_setopt( stvOptNull, stvOptAppLock, stvTrue ); */

	stv_init( TRICKGRP, STVTAG, 1, 0 );

	/* --- Record the size of the grid region */

	glb[0] = 1; glb[1] = 1;

	gub[0] = nvert;
	gub[1] = 3;

	/* --- Declare decomposition of grid field */

	decompId[0] = stv_decompDefine( 2, stvLocalArray,
			stvDefaultInfo, stvDefaultInfo, glb, gub,
			1, stvLocalArray );

	/* --- declare the grid data field with the above decomp */

	declare[0] = nvert;
	declare[1] = 3;

	fieldId[0] = stv_fieldDefine( grid, "grid", decompId[0],
			stvNoFieldOffsets, declare, stvFloat, stvLocalArray,
			stvVisOnly );

	/* --- Record the size of the triconn region */

	glb[0] = 1; glb[1] = 1;

	gub[0] = ntri;
	gub[1] = 3;

	/* --- Declare decomposition of triconn field */

	decompId[1] = stv_decompDefine( 2, stvLocalArray,
			stvDefaultInfo, stvDefaultInfo, glb, gub,
			1, stvLocalArray );

	/* --- declare the grid data field with the above decomp */

	declare[0] = ntri;
	declare[1] = 3;

	fieldId[1] = stv_fieldDefine( triconn, "grid_connect", decompId[1],
			stvNoFieldOffsets, declare, stvInt, stvLocalArray,
			stvVisOnly );

	/* --- Verify Local Field Decomposition */

	stv_dumpLocalFieldBounds( fieldId[0], stvVerbose );

	stv_dumpLocalFieldBounds( fieldId[0], stvTerse );

	/* ---  Top of main computational loop */

	sequence = 0;

	for( ; ; )
	{
		sequence = sequence + 1;

		printf( "Sequence %d\n", sequence );

		/*  --- send messages to any attached CUMULVS viewers */

		stv_sendReadyData( stvSendDefault );

		stvSleep( 1 );
	}
}

