
/* $Id: tviewer.c,v 4.5 2003/09/05 21:25:08 kohl Exp $ */


/*
 *                      CUMULVS version 1.3:
 *      Computational Steering and Interactive Visualization
 *                   in Distributed Applications
 *           Oak Ridge National Laboratory, Oak Ridge TN.
 *                    Authors:  James Arthur Kohl
 *                   (C) 2003 All Rights Reserved
 *
 *                              NOTICE
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted
 * provided that the above copyright notice appear in all copies and
 * that both the copyright notice and this permission notice appear
 * in supporting documentation.
 *
 * Neither the Institution, Oak Ridge National Laboratory, nor the
 * Authors make any representations about the suitability of this
 * software for any purpose.  This software is provided ``as is''
 * without express or implied warranty.
 *
 * CUMULVS was funded by the U.S. Department of Energy.
 */


/* tsimple.c - viewer timer test program */


#include <stdio.h>
#include "stv.h"
#include "stv_glob.h"


/* Main */

main()
{
	STV_VIEW_FIELD_GROUP CURRENT_VFG;
	STV_VIEW_FIELD_GROUP VFG;

	STV_VIEW_FIELD VF;

	STV_VIEW_PARTICLE_DATA VPD;
	STV_VIEW_PARTICLE VPT;

	STV_VIEWER VIEWER;

	STV_FIELDPTR F;

	STV_VIS_REGION VR;

	STV_VALUE ptr;

	long elapsed;
	long tick;

	char fieldname[1024];
	char appname[1024];
	char tmp[255];

	float fval;

	int indices[STV_MAX_DIM];

	int *glb, *gub;

	int storage_order;
	int view_type;
	int is_static;
	int no_synch;
	int freq;

	int restart;
	int mytid;
	int vflag;
	int i, j;
	int quit;
	int cnt;
	int dim;
	int cc;

	/* Basic Viewer Parameters & Setup */

	VIEWER = (STV_VIEWER) NULL;

	strcpy( appname, "tsimple" );
	strcpy( fieldname, "density" );

	storage_order = stvRowMajor;
	/* storage_order = -1; use default */

	view_type = stvFloat;
	/* view_type = -1; use default */

	is_static = stvTrue;

	no_synch = stvFalse;

	freq = 1;

	vflag = stvFalse;

	/* Set STV Options */

	stv_setopt( stvOptNull, stvOptVerbose, vflag ? STV_DEBUG_ALL : 0 );
	stv_setopt( stvOptNull, stvOptFullUnpack, stvFalse );
	stv_setopt( stvOptNull, stvOptDump, stvFalse );

	/* Initialize Viewer */

	MyMP->myid( &mytid );

	if ( mytid < 0 )
	{
		MyMP->perr( "Obtaining My Task ID" );

		exit( -1 );
	}

	else
	{
		printf( "Viewer Alive as TID=[ 0x%x / %d ]\n", mytid, mytid );
		fflush( stdout );
	}

	/* Initialize Viewer & Connect to Application */

	cnt = 0;

	do
	{
		cc = stv_viewer_init( &VIEWER, mytid, appname, stvFalse );
	}
	while ( cc != stvStatusOk && ++cnt < 10 );

	if ( cc != stvStatusOk )
	{
		printf( "\nApplication \"%s\" Not Found - Reset.\n", appname );

		exit( -1 );
	}

	/* Field Selection */

	stv_clear_view_field_select( VIEWER );

	VF = stv_get_view_field_name( fieldname,
		VIEWER->vfields, VIEWER->nvfields );
	
	if ( VF == NULL )
	{
		printf( "Error: Field \"%s\" Not Found.\n", fieldname );

		exit( -1 );
	}

	else
	{
		/* Select View Field */

		VF->selected = stvTrue;

		VF->view_type = ( view_type > 0 ) ? view_type : VF->field->type;

		VF->view_storage_order = ( storage_order >= 0 ) ?
				storage_order : VF->field->storage_order;
	}

	/* Get glb, gub */

	F = VF->field;

	if ( F->decomp != NULL )
	{
		glb = F->decomp->glb;
		gub = F->decomp->gub;

		dim = F->decomp->dataDim;
	}

	else if ( F->particle != NULL )
	{
		glb = F->particle->glb;
		gub = F->particle->gub;

		dim = F->particle->dataDim;
	}

	else
	{
		printf( "\nWarning: No Decomp or Particle for Field \"%s\"\n\n",
			F->name );
		fflush( stdout );

		exit( -1 );
	}

	/* Set Vis Region & Cell Size (Sampling Rate) */

	VR = stv_create_vis_region();

	for ( i=0 ; i < dim ; i++ )
	{
		STV_REGION_MIN( VR->region, i ) = glb[i];
		STV_REGION_MAX( VR->region, i ) = gub[i];

		if ( F->decomp != NULL )
			STV_CELL_OF_DIM( VR->cell, i ) = 1;
	}

	if ( F->particle != NULL )
		STV_CELL_PARTICLE( VR->cell ) = 1;

	/* Dump Out Region */

	printf( "\n" );

	for ( i=0 ; i < dim ; i++ )
	{
		if ( F->decomp != NULL )
		{
			printf( "[%d]:  min=%d max=%d cell=%d\n", i,
				STV_REGION_MIN( VR->region, i ),
				STV_REGION_MAX( VR->region, i ),
				STV_CELL_OF_DIM( VR->cell, i ) );
		}

		else if ( F->particle != NULL )
		{
			printf( "[%d]:  min=%d max=%d\n", i,
				STV_REGION_MIN( VR->region, i ),
				STV_REGION_MAX( VR->region, i ) );
		}
	}

	if ( F->particle != NULL )
		printf( "\tparticle cell = %d\n",
			STV_CELL_PARTICLE( VR->cell ) );

	printf( "\n" );

	/* Determine Base Timer Latency */

	printf( "Calibrating timer... " );
	fflush( stdout );

	elapsed = 0;

	for ( i=0 ; i < 1000 ; i++ )
	{
		timer_start();
		elapsed += timer_elapsed();
	}

	printf( "done.  Base timer latency = %ld usecs\n", elapsed / 1000 );

	/* Determine Base Timer Resolution */

	printf( "Measuring timer... " );
	fflush( stdout );

	tick = -1;

	for ( i=0 ; i < 3 ; i++ )
	{
		timer_start();

		while ( (elapsed = timer_elapsed()) == 0 );

		if ( elapsed < tick || tick < 0 )
			tick = elapsed;
	}

	printf( "done.  Base timer resolution = %ld usecs\n", tick );

	/* Request Field from Application */

	CURRENT_VFG = stv_viewer_submit_field_request( VIEWER,
		VR, freq,
		( stvFieldRqstDefault
			| ( ( no_synch ) ? stvFieldRqstNoSynch : 0 )
			| ( ( is_static ) ? stvFieldRqstStatic : 0 ) ),
		&restart );

	if ( CURRENT_VFG == NULL )
		exit( -1 );

	stv_free_vis_region( &VR );

	/* Allocate Initial Field Storage */

	for ( i=0 ; i < CURRENT_VFG->nvfields ; i++ )
	{
		VF = CURRENT_VFG->view_field_list[i];

		if ( stv_allocate_Field_arrayDecl( CURRENT_VFG, i )
				!= stvStatusOk )
			exit( -1 );

		if ( VF->field->decomp != NULL )
		{
			if ( stv_allocate_Field_Data( CURRENT_VFG, i,
					CURRENT_VFG->view_types[i] ) != stvStatusOk )
				exit( -1 );
		}
	}

	/* Ready to Receive Frames */

	printf( "\nReady to Receive Frames\n\n" );
	fflush( stdout );

	quit = 0;

	do
	{
		/* Collect Data Frame from Application */

		do
		{
			timer_start();

			cc = stv_viewer_receive_frame( &VFG, VIEWER, &restart,
				stvMsgBlock );

			elapsed = timer_elapsed();

			printf( "\telapsed = %ld\n", elapsed );

			if ( cc != stvStatusOk && cc != stvStatusBadFrame
					&& cc != stvStatusIncomplete )
			{
				/* Free Data Frame */

				for ( i=0 ; i < CURRENT_VFG->nvfields ; i++ )
					stv_free_field_array( CURRENT_VFG, i );

				stv_reset_Field_Particles( CURRENT_VFG );

				/* Release Application */

				printf( "\nSending FieldHaltAll\n" );
				fflush( stdout );

				stv_viewer_send_FieldHaltAll( CURRENT_VFG, VIEWER );

				exit( -1 );
			}

			if ( VFG != CURRENT_VFG )
				stv_viewer_release_field( &VFG, VIEWER );
		}
		while ( VFG != CURRENT_VFG );

		if ( cc == stvStatusOk && stvFalse )
		{
			/**** Use Data Frame Here ****/

			/* Dump Dense Distributed Data */

			for ( i=0 ; i < VFG->nvfields ; i++ )
			{
				VF = VFG->view_field_list[i];

				if ( VF->field->decomp != NULL )
				{
					/* Initialize Local Index */
					for ( j=0 ; j < VF->field->decomp->dataDim ; j++ )
						indices[j] = 0;
					
					/* Set Data Pointer */
					/* (only one vis region, index == 0) */
					ptr = VFG->field_values[i][0];

					/* Dump Array Values */
					do
					{
						/* Convert to Float (or view_type...) */
						fval = STV_VALUE_OF( ptr, float );

						printf( "%s(%d,%d,%d) = %f\n", VF->field->name,
							indices[0], indices[1], indices[2], fval );

						/* Advance Data Pointer */
						ptr = stv_get_typed_ptr_offset( ptr, 1,
								stvFloat );
					}
					while ( ptr != NULL && stv_incr_field_index(
							VF->field->decomp->dataDim, indices,
							VFG->field_arrayDecls[i][0], VFG->visregion,
							VFG->view_storage_orders[i], stvFalse ) );
				}
			}

			/* Dump (Any) Particle Data */

			VPT = VFG->particle_list;

			while ( VPT != NULL )
			{
				/* Traverse Particle Data Lost For Each Particle */

				VPD = VPT->pdata;

				while ( VPD != NULL )
				{
					for ( i=0 ; i < VPT->nvfields ; i++ )
					{
						/* (only one vis region, index == 0) */
						fval = STV_VALUE_OF( VPD->field_values[i],
								float );

						printf( "%s:%s(%d,%d,%d) = %f\n",
							VPT->particle->name,
							(VPT->view_field_list[i])->field->name,
							VPD->coords[0], VPD->coords[1],
							VPD->coords[2], fval );
					}

					VPD = VPD->next;
				}

				VPT = VPT->next;
			}
		}

		/* Free Particle Field Data */

		stv_reset_Field_Particles( CURRENT_VFG );

		/* Handle Viewer XON */

		stv_viewer_send_XON( CURRENT_VFG, VIEWER );

		/* Prompt to Continue */

		do
		{
			printf( "Continue (y/n)? " );
			fflush( stdout );
		}
		while ( ( scanf( "%s", tmp ) < 1 && printf( "\n" ) )
				|| ( strcmp( tmp, "y" ) && strcmp( tmp, "n" ) ) );
		
		if ( !strcmp( tmp, "n" ) )
			quit++;
	}
	while ( !quit );
}

