
/* $Id: viewer.c,v 4.5 2003/09/05 21:25:08 kohl Exp $ */


/*
 *                      CUMULVS version 1.0:
 *      Computational Steering and Interactive Visualization
 *                   in Distributed Applications
 *           Oak Ridge National Laboratory, Oak Ridge TN.
 *      Authors:  James Arthur Kohl and Philip M. Papadopoulos
 *                   (C) 1996 All Rights Reserved
 *
 *                              NOTICE
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted
 * provided that the above copyright notice appear in all copies and
 * that both the copyright notice and this permission notice appear
 * in supporting documentation.
 *
 * Neither the Institution, Oak Ridge National Laboratory, nor the
 * Authors make any representations about the suitability of this
 * software for any purpose.  This software is provided ``as is''
 * without express or implied warranty.
 *
 * CUMULVS was funded by the U.S. Department of Energy.
 */


/* System Header Files */

#include <stdio.h>
#include <fcntl.h>


/* STV Header Files */

#include "stv.h"
#include "stv_glob.h"


#ifdef GIFVWR

/* GD Library Header Files */

#include "gd.h"
#include <math.h>


/* GIF Viewer CGI URL */

#ifndef GIF_VIEWER_GROUP
#define GIF_VIEWER_GROUP \
	"STV_GIF_VIEWER"
#endif

#ifndef GIF_VIEWER_ROOT
#define GIF_VIEWER_ROOT \
	"/home/user/.wwwhome"
#endif

#ifndef CGI_URL_ROOT
#define CGI_URL_ROOT \
	"http://www/~user/cgi-bin"
#endif

#ifndef GIF_URL_ROOT
#define GIF_URL_ROOT \
	"http://www/~user/images"
#endif

#ifndef GIFOMATIC_CODE
#define GIFOMATIC_CODE \
	"gov.sandia.ccaffeine.dc.user_iface.webViewer.AppletShell.class"
#endif

#ifndef GIFOMATIC_ARCHIVE
#define GIFOMATIC_ARCHIVE \
	"gifomatic.jar"
#endif

#ifndef PLUGINS_PAGE
#define PLUGINS_PAGE \
	"http://java.sun.com/products/plugin/1.2/plugin-install.html"
#endif


/* Floating Point Equivalence Macro */

#define EQUIV_DELTA		0.000001

#define EQUIV( _d1, _d2 ) \
( ( (_d1) != 0.0 ) ? \
	( ( fabs( (_d1) - (_d2) ) / fabs( _d1 ) ) < EQUIV_DELTA ) \
	: ( fabs( (_d1) - (_d2) ) < EQUIV_DELTA ) )


/* Plane Choice Constants */

#define PLANE_XY	1
#define PLANE_YZ	2
#define PLANE_XZ	3

#endif


/* Commands String */

#define CMD_STR    "(y/#/0/v/r/f/U/s/R/V/d/D/C/F/L/l/u/o/O/n/N/q/?)"


/* XON Macro */

#define DO_XON( _vfg, _v ) \
\
	printf( "\nSending XON at " ); \
	stv_dump_time_str( stdout, (_vfg)->region_time, stvTrue ); \
	fflush( stdout ); \
\
	stv_viewer_send_XON( _vfg, _v ); \
\

#define DO_XON_CNT( _vfg, _v, _cnt ) \
\
	printf( "\nSending XON at " ); \
	stv_dump_time_str( stdout, (_vfg)->region_time, stvFalse ); \
	printf( " (%d)\n", xon_cnt ); \
	fflush( stdout ); \
\
	stv_viewer_send_XON( _vfg, _v ); \
\


/* Routines */

#ifdef GIFVWR

void	dump_gif_file();
void	dump_html_file();
void	color_value();
void	handle_cgi_reply();

int		get_plane_indices();
int		handle_cgi_input();

#endif

void	cleanup();
void	dump_region();
void	free_field_names();
void	handle_steering();
void	otf_usage();
void	program_init();
void	prompt_for_vis_region();
void	read_args();
void	read_field_names();
void	read_freq_synch_static();
void	read_name();
void	usage();
void	viewer_init();

void	Delay();
void	NoDelay();


/* Globals */

STV_VIEW_FIELD_GROUP CURRENT_VFG;

STV_VIEW_FIELD VF_PRIMARY;

STV_VIEWER VIEWER;

STV_VIS_REGION VISREGION_DEFAULT;

int VISREGION_DEFAULT_SET;
int VISCELL_DEFAULT_SET;

STV_VIS_REGION VISREGION;

char APPNAME_DEFAULT[1024];
char APPNAME[1024];

char FIELDNAME_DEFAULT[1024];

char **FIELDNAMES;
int NFIELDNAMES;

int STATIC_DEFAULT;
int SYNCH_DEFAULT;
int FREQ_DEFAULT;

#ifdef GIFVWR

char HTML_TMPFILE[1024];
char HTML_FILE[1024];

char GIF_TMPFILE[1024];

char GIF_ROOT[1024];

char *RADIUS_FIELD;
char *COLOR_FIELD;

float COLOR_MIN_VALUE;
float COLOR_MAX_VALUE;

float MIN_VALUE;
float MAX_VALUE;

int AUTO_COLOR_MAP;

int LAZY_COLOR_MAP;

int COLOR_MAP_INITD;

int SHOW_MIN_MAX;

int PLANE_CHOICE;

int GIF_HEIGHT;
int GIF_WIDTH;

int GIF_CNTR;

int USE_JAVA;

int CGI_NEWDATA;

int CGI_RESULT;
int CGI_REPLY;
int CGI_SRC;
int CGI_TAG;

#endif

int STORAGE_ORDER;

int RECONNECT;

int STEERING;

int IS_STATIC;

int NO_SYNCH;

int REALLOC;

int RESTART;

int FLAGS;

int MYTID;

int vflag;
int Dflag;
int dflag;


/* Main */

main( argc, argv )
int argc;
char **argv;
{
	/* Program Initialization */

	program_init();

	read_args( argc, argv );

	viewer_init();

	while ( 1 )
	{
		if ( do_viewer() )
			exit( 0 );

		if ( !RESTART )
			cleanup();
	}
}


do_viewer()
{
	STV_VIEW_FIELD_GROUP VFG;

	STV_VIEW_FIELD VF;

	STV_PARTICLEPTR PT;

	STV_DECOMPPTR D;

	char tmpname[1024];
	char prompt[1024];

	char c;

	int option;
	int value;
	int what;

	int whacked;
	int xon_cnt;
	int newapp;
	int fflag;
	int pflag;
	int i, j;
	int done;
	int freq;
	int quit;
	int cnt;
	int cc;

#ifdef GIFVWR
	int rbuf;
#endif

	/* Set Allocation Flag */

	REALLOC = stvTrue;

	/* Initialize Base Default Frequency */

	freq = 1;

	/* Prompt for Viewer Info */

	fflag = 0;

	if ( !RESTART )
	{
		/* Application Name */

		if ( !strcmp( APPNAME_DEFAULT, "" ) )
		{
			printf( "\nEnter Application Name / %s / %s: ",
				"<return> for unnamed", "\"quit\"" );

			read_name( APPNAME );

			if ( !strcmp( APPNAME, "quit" ) )
				return( stvTrue );
		}

		else
		{
			printf( "\nUsing Default Application Name \"%s\".\n",
				APPNAME_DEFAULT );

			strcpy( APPNAME, APPNAME_DEFAULT );
		}

		/* Data Field */

		if ( !strcmp( FIELDNAME_DEFAULT, "" ) )
			read_field_names( &FIELDNAMES, &NFIELDNAMES, stvFalse );
		
		else
		{
			printf( "\nUsing Default Data Field Name \"%s\".\n",
				FIELDNAME_DEFAULT );

			FIELDNAMES = (char **) malloc( sizeof( char * ) );

			if ( stv_memcheck( FIELDNAMES,
					"Field Name List Ptr" ) != stvStatusOk )
				exit( -1 );

			FIELDNAMES[0] = stv_copy_str( FIELDNAME_DEFAULT );

			NFIELDNAMES = 1;
		}

		if ( NFIELDNAMES )
		{
			read_freq_synch_static( &freq, &NO_SYNCH, &IS_STATIC );

			fflag = 1;
		}
	}

	/* Initialize Viewer & Connect to Application */

	cnt = 0;

	do
	{
		cc = stv_viewer_init( &VIEWER, MYTID, APPNAME, stvFalse );
	}
	while ( cc != stvStatusOk && ++cnt < 10 );

	if ( cc != stvStatusOk )
	{
		printf( "\nApplication \"%s\" Not Found - Reset.\n", APPNAME );

		return( stvFalse );
	}

	/* Check for Field Selection */

	newapp = 0;

	quit = 0;

	do
	{
		if ( !RESTART )
		{
			do
			{
				while ( !NFIELDNAMES )
				{
					printf( "\nAvailable Fields are:\n" );
	
					VF = VIEWER->vfields;
	
					while ( VF != NULL )
					{
						if ( VF->field != NULL
							&& VF->field->name != NULL )
						{
							if ( VF->field->decomp != NULL )
							{
								D = VF->field->decomp;
	
								printf( "\t%s ", VF->field->name );
	
								for ( i=0 ; i < D->dataDim ; i++ )
								{
									printf( "[ %d - %d ]",
										D->glb[i], D->gub[i] );
								}
							}

							else if ( VF->field->particle != NULL )
							{
								PT = VF->field->particle;
	
								printf( "\t%s (%s) ", VF->field->name,
									PT->name );
	
								for ( i=0 ; i < PT->dataDim ; i++ )
								{
									printf( "[ %d - %d ]",
										PT->glb[i], PT->gub[i] );
								}
							}
	
							printf( "\n" );
						}
	
						VF = VF->next;
					}
	
					read_field_names( &FIELDNAMES, &NFIELDNAMES,
						stvTrue );
				}

				whacked = select_view_fields( 
					&FIELDNAMES, &NFIELDNAMES );

				if ( !whacked )
				{
					/* Read in Vis Region Fields */

					prompt_for_vis_region( VF_PRIMARY->field,
						VISREGION, stvTrue );

					dump_region( VF_PRIMARY->field, VISREGION );

					if ( !fflag )
						read_freq_synch_static( &freq, &NO_SYNCH,
							&IS_STATIC );
				}
			}
			while ( whacked );
		}

		else
		{
			RESTART = 0;

			if ( select_view_fields( &FIELDNAMES, &NFIELDNAMES ) )
			{
				return( stvFalse );
			}
		}

		/* Request Field from Application */

		CURRENT_VFG = stv_viewer_submit_field_request( VIEWER,
			VISREGION, freq,
			( stvFieldRqstDefault
				| ( ( NO_SYNCH ) ? stvFieldRqstNoSynch : 0 )
				| ( ( IS_STATIC ) ? stvFieldRqstStatic : 0 ) ),
			&RESTART );

		if ( CURRENT_VFG == NULL )
			return( stvFalse );

		/* Ready to Receive Frames */

		printf( "\nReady to Receive Frames\n\n" );
		fflush( stdout );

#ifdef GIFVWR
		strcpy( prompt, "0" );
		xon_cnt = -1;
#else
		xon_cnt = 0;
#endif

		done = 0;

		do
		{
			/* Allocate Initial Field Storage */

			if ( REALLOC )
			{
				for ( i=0 ; i < CURRENT_VFG->nvfields ; i++ )
				{
					VF = CURRENT_VFG->view_field_list[i];

					if ( stv_allocate_Field_arrayDecl( CURRENT_VFG, i )
						!= stvStatusOk )
					{
						return( stvFalse );
					}

					if ( VF->field->decomp != NULL )
					{
						if ( stv_allocate_Field_Data( CURRENT_VFG, i,
								CURRENT_VFG->view_types[i] )
							!= stvStatusOk )
						{
							return( stvFalse );
						}

						if ( Dflag )
						{
							printf(
						"\nInitial Global \"%s\" Field Frame Data:\n",
								VF->field->name );
							fflush( stdout );

							stv_dump_vfg_field_array( CURRENT_VFG, i );
						}
					}
				}

				REALLOC = stvFalse;
			}

			/* Collect Data Frame from Application */

			do
			{
				cc = stv_viewer_receive_frame( &VFG, VIEWER, &RESTART,
					stvMsgBlock );

				if ( cc != stvStatusOk && cc != stvStatusBadFrame )
				{
					/* Free Data Frame */

					for ( i=0 ; i < CURRENT_VFG->nvfields ; i++ )
						stv_free_field_array( CURRENT_VFG, i );

					stv_reset_Field_Particles( CURRENT_VFG );

					/* Release Application */

					printf( "\nSending FieldHaltAll\n" );
					fflush( stdout );

					stv_viewer_send_FieldHaltAll( CURRENT_VFG, VIEWER );

					/* Handle Re-Connect */

					if ( RECONNECT )
						RESTART++;

					return( stvFalse );
				}

				if ( VFG != CURRENT_VFG )
					stv_viewer_release_field( &VFG, VIEWER );
			}
			while ( VFG != CURRENT_VFG );

			/**** Use Data Frame Here ****/

#ifdef GIFVWR
			dump_gif_file( CURRENT_VFG );

			dump_html_file( CURRENT_VFG, VIEWER );

			if ( CGI_REPLY )
			{
				handle_cgi_reply();

				CGI_REPLY = stvFalse;
			}
#endif

			/* Free Particle Field Data */

			stv_reset_Field_Particles( CURRENT_VFG );

			/* Check App Lock (& Reset) */

			if ( CURRENT_VFG->locked == stvAppLocked && xon_cnt )
			{
				printf(
					"Application Locked!  Resetting XON Count...\n" );
				xon_cnt = 0;
			}

#ifdef GIFVWR
			/* Check for CGI Input */

			if ( MyMP->recv_msg_noblock( -1, -1,
					STV_BASECTX, &rbuf ) )
				cc = handle_cgi_input( rbuf, CURRENT_VFG, VIEWER );
			else
				cc = handle_cgi_input( -1, CURRENT_VFG, VIEWER );
			
			if ( !cc )
				return( stvFalse );

			if ( CGI_NEWDATA )
			{
				printf( "Skipping XON for New CGI Data Request\n" );

				CGI_NEWDATA = stvFalse;

				continue;
			}
#endif

			/* Handle Viewer XON Iterations */

			if ( xon_cnt > 0 )
			{
				DO_XON_CNT( CURRENT_VFG, VIEWER, xon_cnt );

				xon_cnt--;
			}

			/* Prompt Viewer for Action */

			else
			{
				pflag = 1;

				if ( xon_cnt < 0 )
				{
					NoDelay();

					if ( (c = getchar()) != (char) EOF )
					{
						xon_cnt = 0;

						if ( c != '\n' )
						{
							prompt[0] = c;
							i = 1;
							while ( (c=getchar()) != '\n' )
								prompt[i++] = c;
							prompt[i] = '\0';
							pflag = 0;
						}
					}

					Delay();
				}

				if ( xon_cnt >= 0 )
				{
					do
					{
						if ( pflag )
						{
							printf( "Send XON Message %s: ", CMD_STR );

							scanf( "%s", prompt );

							/* Grab '\n' */
							while ( getchar() != '\n' );
						}

						pflag = 1;

						if ( prompt[0] == '?' )
						{
							otf_usage();

							*prompt = '\0';
						}
					}
					while ( !strcmp( prompt, "" ) );
				}

				/* Send Multiple XON Messages */

				if ( sscanf( prompt, "%d", &xon_cnt ) == 1 )
				{
					if ( xon_cnt < 0 )
					{
						DO_XON( CURRENT_VFG, VIEWER );

						xon_cnt = 0;
					}
					
					else if ( !xon_cnt )
					{
						DO_XON( CURRENT_VFG, VIEWER );

						xon_cnt = -1;
					}
					
					else
					{
						DO_XON_CNT( CURRENT_VFG, VIEWER, xon_cnt );

						xon_cnt--;
					}
				}

				/* Send XON Message */

				else if ( prompt[0] == 'y' )
				{
					DO_XON( CURRENT_VFG, VIEWER );
				}

				/* Read in New Vis Region */

				else if ( prompt[0] == 'v'
					|| prompt[0] == 'r' || prompt[0] == 'f' )
				{
					/* Read in New Vis Region Fields */

					if ( prompt[0] == 'v' || prompt[0] == 'r' )
					{
						/* Skip if View Field Group is Static */
						/* (return code handling included below */
						/* anyway as an example...) */

						if ( IS_STATIC )
						{
							printf( "Static View Field Group - " );
							printf( "Can't Change View Region!\n" );
						}

						else
						{
							prompt_for_vis_region( VF_PRIMARY->field,
								VISREGION, stvFalse );

							dump_region( VF_PRIMARY->field, VISREGION );

							printf( "\nSending Vis Region\n" );
							fflush( stdout );

							if ( stv_viewer_set_VisRegion( CURRENT_VFG,
									VIEWER, VISREGION )
								!= stvStatusOk )
							{
								printf(
						"Couldn't Change View Region - Reverting.\n" );

								stv_copy_vis_region( VISREGION,
									CURRENT_VFG->visregion );
							}

							else
								REALLOC = stvTrue;
						}
					}

					if ( prompt[0] == 'v' || prompt[0] == 'f' )
					{
						read_freq_synch_static( &freq,
							(int *) NULL, (int *) NULL );

						printf( "\nSending Vis Frequency\n" );
						fflush( stdout );

						stv_viewer_set_VisFrequency( CURRENT_VFG,
							VIEWER, freq );
					}

					/* Send XON */

					DO_XON( CURRENT_VFG, VIEWER );
				}

				/* Toggle Full Unpack Option */

				else if ( prompt[0] == 'U' )
				{
					cc = stv_setopt( stvOptNull, stvOptFullUnpack,
						!stv_getopt( stvOptNull, stvOptFullUnpack ) );

					printf( "\nTurned Full Unpack %s\n\n",
						cc ? "OFF" : "ON" );
					fflush( stdout );

					DO_XON( CURRENT_VFG, VIEWER );

					REALLOC = stvTrue;
				}

				/* Request Steering */

				else if ( prompt[0] == 's' )
				{
					handle_steering( freq, NO_SYNCH );

					DO_XON( CURRENT_VFG, VIEWER );
				}

				/* Toggle Verbosity Option */

				else if ( prompt[0] == 'V' )
				{
					printf( "\nEnter Verbosity Level %s:",
							"(or <return> for STV_DEBUG_ALL)" );
					fflush( stdout );

					if ( scanf( "%d", &value ) != 1 )
						value = STV_DEBUG_ALL;

					stv_setopt( stvOptNull, stvOptVerbose, value );

					vflag = value ? stvTrue : stvFalse;

					printf( "\nVerbosity Set to 0x%x.\n\n", value );
					fflush( stdout );

					DO_XON( CURRENT_VFG, VIEWER );
				}

				/* Toggle Dump Verbosity Option */

				else if ( prompt[0] == 'D' )
				{
					Dflag = Dflag ? stvFalse : stvTrue;

					stv_setopt( stvOptNull, stvOptDumpAll, Dflag );

					printf( "\nTurned Updated Field Dumping %s\n\n",
						Dflag ? "ON" : "OFF" );
					fflush( stdout );

					DO_XON( CURRENT_VFG, VIEWER );
				}

				/* Toggle Final Dump Option */

				else if ( prompt[0] == 'd' )
				{
					dflag = dflag ? stvFalse : stvTrue;

					stv_setopt( stvOptNull, stvOptDump, dflag );

					printf( "\nTurned Final Field Dumping %s\n\n",
						dflag ? "ON" : "OFF" );
					fflush( stdout );

					DO_XON( CURRENT_VFG, VIEWER );
				}

				/* Lock Application */

				else if ( prompt[0] == 'L' )
				{
					printf( "\nLocking Whole Application...\n" );
					fflush( stdout );

					stv_viewer_send_AppLock(
							(STV_VIEW_FIELD_GROUP) NULL, VIEWER, 0 );

					DO_XON( CURRENT_VFG, VIEWER );
				}

				/* Lock Application */

				else if ( prompt[0] == 'l' )
				{
					printf( "\nLocking Application Group...\n" );
					fflush( stdout );

					stv_viewer_send_AppLock( CURRENT_VFG, VIEWER, 0 );

					DO_XON( CURRENT_VFG, VIEWER );
				}

				/* Unlock Application */

				else if ( prompt[0] == 'u' )
				{
					printf( "\nUnlocking Application...\n" );
					fflush( stdout );

					stv_viewer_send_AppUnlock( CURRENT_VFG, VIEWER );

					DO_XON( CURRENT_VFG, VIEWER );
				}

				/* Set Application's CUMULVS Options On-The-Fly */
				/* (That can't be a problem, can it? :-) */

				else if ( prompt[0] == 'o' || prompt[0] == 'O' )
				{
					printf( "\nSend CUMULVS Options to %s.\n\n",
						( prompt[0] == 'o' )
							? "Current Application Group"
							: "Entire Application" );

					printf( "'What' Constants:\n" );
					printf( "\t%d = %s\n",
						stvOptAll, stv_option_str( stvOptAll ) );
					printf( "\t%d = %s\n",
						stvOptNull, stv_option_str( stvOptNull ) );

					printf( "\nEnter 'what' for option: " );
					fflush( stdout );

					if ( scanf( "%d", &what ) != 1 )
						what = stvOptNull;

					printf( "Options:\n" );

					for ( i=0 ; i < stvOptMax ; i++ )
						printf( "\t%d = %s\n", i, stv_option_str( i ) );
					
					printf( "\nEnter option: " );
					fflush( stdout );

					if ( scanf( "%d", &option ) != 1 )
						option = stvOptNull;

					printf( "\nEnter option value: " );
					fflush( stdout );

					if ( scanf( "%d", &value ) != 1 )
						value = stvOptNull;

					if ( prompt[0] == 'o' )
					{
						stv_viewer_send_Setopt( CURRENT_VFG, VIEWER,
								what, option, value );
					}
					
					else
					{
						stv_viewer_send_Setopt(
								(STV_VIEW_FIELD_GROUP) NULL, VIEWER,
								what, option, value );
					}

					DO_XON( CURRENT_VFG, VIEWER );
				}

				/* Set Storage Order to C, Row Major */

				else if ( prompt[0] == 'C' )
				{
					STORAGE_ORDER = stvRowMajor;

					printf( "\nData Storage Set to Row Major\n\n" );

					printf( "\nSending FieldHaltGroup\n" );
					fflush( stdout );

					stv_viewer_send_FieldHaltGroup( CURRENT_VFG,
						VIEWER );

					REALLOC = stvTrue;

					RESTART++;

					done++;
				}

				/* Set Storage Order to Fortran, Column Major */

				else if ( prompt[0] == 'F' )
				{
					STORAGE_ORDER = stvColumnMajor;

					printf( "\nData Storage Set to Column Major\n\n" );

					printf( "\nSending FieldHaltGroup\n" );
					fflush( stdout );

					stv_viewer_send_FieldHaltGroup( CURRENT_VFG,
						VIEWER );

					REALLOC = stvTrue;

					RESTART++;

					done++;
				}

				/* Send FieldHalt Messages */

				else if ( prompt[0] == 'R' || prompt[0] == 'n'
					|| prompt[0] == 'N' || prompt[0] == 'q' )
				{
					printf( "\nSending FieldHaltAll\n" );
					fflush( stdout );

					stv_viewer_send_FieldHaltAll( CURRENT_VFG, VIEWER );

					REALLOC = stvTrue;

					done++;

					if ( prompt[0] == 'R' )
						RESTART++;

					if ( prompt[0] == 'N' )
						{ newapp++; quit++; }

					if ( prompt[0] == 'q' )
						quit++;
				}

				/* Command Help */

				else
				{
					printf( "\nUnknown Command '%c'.\n\n", prompt[0] );

					otf_usage();

					printf( "Hit Return to Continue: " );
					fflush( stdout );

					/* Grab '\n' */

					while ( getchar() != '\n' );

					/* Send XON & Continue */

					DO_XON( CURRENT_VFG, VIEWER );
				}
			}
		}
		while ( !done );

		if ( !RESTART )
			free_field_names( &FIELDNAMES, &NFIELDNAMES );
	}
	while ( !quit );

	if ( newapp )
		return( stvFalse );

	else
		return( stvTrue );
}


select_view_fields( fieldnames, nfields )
char ***fieldnames;
int *nfields;
{
	STV_VIEW_FIELD VF;

	int whacked;
	int i;

	stv_clear_view_field_select( VIEWER );

	whacked = stvFalse;

	for ( i=0 ; i < *nfields && !whacked ; i++ )
	{
		VF = stv_get_view_field_name( (*fieldnames)[i],
			VIEWER->vfields, VIEWER->nvfields );
	
		if ( VF == NULL )
		{
			printf( "Error: Field \"%s\" Not Found.\n",
				(*fieldnames)[i] );
	
			free_field_names( fieldnames, nfields );

			whacked++;
		}
	
		else
		{
			/* Snag Primary View Field */

			if ( !i )
				VF_PRIMARY = VF;

			/* Select View Field */

			VF->selected = stvTrue;

#ifdef GIFVWR
			VF->view_type = stvFloat;
#else
			VF->view_type = VF->field->type;
#endif

			VF->view_storage_order =
				( ( STORAGE_ORDER >= 0 ) ?
					STORAGE_ORDER
					: VF->field->storage_order );
		}
	}

	return( whacked );
}


void
otf_usage()
{
	printf( "\nViewer Commands:\n\n" );

	printf( "y  -  " );
	printf( "Send XON to Application.\n" );

	printf( "#  -  " );
	printf( "Number of XONs to Send.\n" );

	printf( "0  -  " );
	printf( "Send XONs Until <Return> Is Pressed.\n" );

	printf( "v  -  " );
	printf( "Enter New Vis Region & Frame Freq.\n" );

	printf( "r  -  " );
	printf( "Enter New Visualization Region.\n" );

	printf( "f  -  " );
	printf( "Enter New Vis Frame Frequency.\n" );

	printf( "U  -  " );
	printf( "Toggle \"Full Unpack\" Option.\n" );

	printf( "s  -  " );
	printf( "Request Steering of Parameters.\n" );

	printf( "R  -  " );
	printf( "Initiate Restart of Application Link.\n" );

	printf( "V  -  " );
	printf( "Toggle Verbosity Option.\n" );

	printf( "d  -  " );
	printf( "Toggle Final Data Frame Dump Option.\n" );

	printf( "D  -  " );
	printf( "Toggle Mega Data Dumping Option.\n" );

	printf( "C  -  " );
	printf( "Force Data Storage Order to " );
	printf( "Row Major, a la C.\n" );

	printf( "F  -  " );
	printf( "Force Data Storage Order to " );
	printf( "Column Major, a la Fortran.\n" );

	printf( "L  -  " );
	printf( "Lock Whole Application at Next SendToFE Call.\n" );

	printf( "l  -  " );
	printf( "Lock Application Group at Next SendToFE Call.\n" );

	printf( "u  -  " );
	printf( "Unlock Application, Release to Continue " );
	printf( "Computation.\n" );

	printf( "o  -  " );
	printf( "Send CUMULVS Options to Current Application Group.\n" );

	printf( "O  -  " );
	printf( "Send CUMULVS Options to Entire Application.\n" );

	printf( "n  -  " );
	printf( "Do Not Send XON, Release Data Field.\n" );

	printf( "N  -  " );
	printf( "Do Not Send XON, Release Data Field, " );
	printf( "Select New Application.\n" );

	printf( "q  -  " );
	printf( "Quit Viewer, Disconnect Application.\n" );

	printf( "?  -  " );
	printf( "Generate This Command Help.\n" );

	printf( "\nNote:  All Commands Send XON Unless " );
	printf( "Specified Otherwise.\n\n" );
}


void
handle_steering( freq, no_synch )
int freq;
int no_synch;
{
	STV_VIEW_PARAM VP;

	STV_PARAMPTR P;

	double dval;

	char tmp[1024];

	char *ptr;

	int newsteer;
	int changed;
	int nsteer;
	int rcount;
	int count;
	int cc;
	int i;

	/* Initialize Steered Param Count */

	newsteer = 0;

	nsteer = 0;

	/* Prompt for Requests */

	printf( "\nSteering Parameter Request List:\n" );

	VP = VIEWER->vparams;

	rcount = 0;

	count = 0;

	while ( VP != NULL )
	{
		if ( VP->token == stvSteerNot )
		{
			printf( "\t%s\n", VP->param->name );

			count++;
		}

		else
			rcount++;

		VP = VP->next;
	}

	if ( count )
	{
		printf( "\nRequest New Parameter (or <return>)? " );

		read_name( tmp );

		while ( count && strcmp( tmp, "" ) )
		{
			VP = stv_get_view_param_name( tmp,
				VIEWER->vparams, VIEWER->nvparams );

			if ( VP == NULL )
			{
				printf( "\nError: Parameter \"%s\" Not Found.\n\n",
					tmp );
			}
		
			else
			{
				if ( !STEERING )
				{
					printf( "Initializing Steering\n" );

					cc = stv_viewer_steering_init( VIEWER,
						freq, no_synch, &RESTART );

					printf( "\tcc=%d\n", cc );

					if ( cc != stvStatusOk )
						return;

					STEERING = stvTrue;
				}

				stv_viewer_steering_request( VP, VIEWER,
						stvMsgNoBlock );

				if ( VP->token == stvSteerToken )
					newsteer++;

				else if ( VP->token == stvSteerRqstd )
				{
					printf(
						"* Steering Token for %s Not Yet Available *\n",
						VP->param->name );
				}

				count--;
			}

			if ( count )
			{
				printf(
					"\nRequest Another New Parameter (or <return>)? " );

				read_name( tmp );
			}
		}
	}

	else
		printf( "\t<empty list>\n" );

	/* Prompt for Releases */

	if ( rcount )
	{
		printf( "\nSteering Parameter Release List:\n" );

		VP = VIEWER->vparams;

		count = 0;

		while ( VP != NULL )
		{
			if ( VP->token != stvSteerNot )
			{
				printf( "\t%s\n", VP->param->name );

				if ( VP->token == stvSteerToken )
					nsteer++;

				else
				{
					stv_viewer_steering_request( VP, VIEWER,
							stvMsgNoBlock );

					if ( VP->token == stvSteerToken )
						nsteer++;

					else if ( VP->token == stvSteerRqstd )
					{
						printf(
						"* Steering Token for %s Not Yet Available *\n",
							VP->param->name );
					}
				}

				count++;
			}

			VP = VP->next;
		}

		if ( count )
		{
			printf( "\nRelease a Steering Parameter (or <return>)? " );

			read_name( tmp );

			while ( count && strcmp( tmp, "" ) )
			{
				VP = stv_get_view_param_name( tmp,
					VIEWER->vparams, VIEWER->nvparams );

				if ( VP == NULL )
				{
					printf( "\nError: Parameter \"%s\" Not Found.\n\n",
						tmp );
				}
		
				else
				{
					if ( !STEERING )
						printf( "\nWhoa! Not Steering, Release?!\n\n" );

					if ( VP->token == stvSteerToken )
						nsteer--;

					stv_viewer_steering_release( VP, VIEWER );

					count--;
				}

				if ( count )
				{
					printf(
			"\nRelease Another Steering Parameter (or <return>)? " );

					read_name( tmp );
				}
			}
		}

		else
			printf( "\t<empty list>\n" );
	}

	else
		nsteer += newsteer;

	/* Prompt for Actual Steering */

	if ( nsteer )
	{
		printf(
"\nAdjust the Values of the Steering Parameters (or <return>):\n\n" );

		VP = VIEWER->vparams;

		changed = 0;

		while ( VP != NULL )
		{
			VP->changed = stvFalse;

			if ( VP->token == stvSteerToken )
			{
				P = VP->param;

				for ( i=0 ; i < P->num ; i++ )
				{
					if ( P->num == 1 )
						printf( "%s ", P->name );

					else
						printf( "%s:%s ", P->name, P->pnames[i] );

					ptr = stv_str_of_typed_data( P->vars[i],
						P->types[i] & stvTypeMask );

					printf( "[%s]: ", ptr );

					free( ptr );

					read_name( tmp );

					if ( strcmp( tmp, "" ) )
					{
						if ( sscanf( tmp, "%lf", &dval ) == 1 )
						{
							if ( stv_viewer_steer_vparameter( VP,
								(STV_VALUE) &dval, stvDouble, i )
								!= stvStatusOk )
							{
								printf( "Error steering parameter!\n" );
							}

							else
								changed++;
						}

						else
						{
							printf( "\nError: Value Missing \"%s\"\n\n",
								tmp );
						}
					}

					else
						VP->changed = stvTrue;
				}
			}

			VP = VP->next;
		}

		if ( changed )
			stv_viewer_send_NewParams( VIEWER );
	}
}


void
read_field_names( fieldnames, nfields, known )
char ***fieldnames;
int *nfields;
int known;
{
	char tmp[1024];

	int found = 0;
	int i, j;
	int flag;
	int num;

	/* Only One Field to Choose From?  Now Let's See... */

	if ( VIEWER != NULL
			&& VIEWER->nvfields == 1
			&& VIEWER->vfields->field != NULL
			&& VIEWER->vfields->field->name != NULL )
	{
		*fieldnames = (char **) malloc( sizeof( char * ) );

		if ( stv_memcheck( *fieldnames, "Field Name List Ptr" )
			!= stvStatusOk )
		{
			exit( -1 );
		}

		printf( "\nOnly One Field to Choose From - " );
		printf( "Auto Select \"%s\".\n",
			VIEWER->vfields->field->name );

		(*fieldnames)[0] = stv_copy_str( VIEWER->vfields->field->name );

		*nfields = 1;

		return;
	}

	printf( "\nEnter Number of Fields / Single Field Name%s: ",
		( known ) ? "" : " / (<return> If Not Known)" );

	read_name( tmp );

	flag = sscanf( tmp, "%d", &num );

	if ( flag == 1 )
	{
		*fieldnames = (char **) malloc( (STV_SIZE_T) num
			* sizeof( char * ) );

		if ( stv_memcheck( *fieldnames, "Field Name List Ptr" )
			!= stvStatusOk )
		{
			exit( -1 );
		}

		for ( i=0 ; i < num ; i++ )
		{
			do
			{
				printf(
					"Enter Field Name #%d / (or <return> to reset): ",
					i + 1 );

				read_name( tmp );

				if ( !strcmp( tmp, "" ) )
				{
					*nfields = i;

					free_field_names( fieldnames, nfields );

					return;
				}

				else
					(*fieldnames)[i] = stv_copy_str( tmp );

				if ( known )
				{
					found = 0;

					if ( stv_get_view_field_name( (*fieldnames)[i],
						VIEWER->vfields, VIEWER->nvfields ) != NULL )
					{
						found++;
					}

					else
					{
						printf(
							"\nError: Field \"%s\" Not Found - %s.\n\n",
							(*fieldnames)[i], "Please Re-Enter" );
					}
				}
			}
			while ( known && !found );
		}

		*nfields = num;
	}

	else
	{
		/* Handle Single Field Case */

		if ( strlen( tmp ) > 0 )
		{
			*fieldnames = (char **) malloc( sizeof( char * ) );

			if ( stv_memcheck( *fieldnames, "Field Name List Ptr" )
				!= stvStatusOk )
			{
				exit( -1 );
			}

			(*fieldnames)[0] = stv_copy_str( tmp );

			*nfields = 1;

			return;
		}

		*fieldnames = (char **) NULL;

		*nfields = 0;
	}
}


void
free_field_names( fieldnames, nfields )
char ***fieldnames;
int *nfields;
{
	int i;

	for ( i=0 ; i < *nfields ; i++ )
		free( (*fieldnames)[i] );

	if ( *fieldnames != NULL )
	{
		free( *fieldnames );

		*fieldnames = (char **) NULL;
	}

	*nfields = 0;
}


void
read_freq_synch_static( freq, no_synch, is_static )
int *freq;
int *no_synch;
int *is_static;
{
	STV_VIEW_FIELD VF;

	char unitstr[255];
	char tmp[1024];

	double dnum;

	int units;
	int flag;
	int any;
	int num;
	int i;

	if ( freq != NULL )
	{
		if ( FREQ_DEFAULT > 0 )
		{
			printf( "\nUsing Default Frequency:  %d %s\n",
				stv_value_of_time( FREQ_DEFAULT ),
				stv_time_unit_str( stv_unit_of_time( FREQ_DEFAULT ) ) );

			*freq = FREQ_DEFAULT;
		}
		
		else
		{
			any = 0;

			for ( i=0 ; i < NFIELDNAMES && VIEWER != NULL ; i++ )
			{
				VF = stv_get_view_field_name( FIELDNAMES[i],
					VIEWER->vfields, VIEWER->nvfields );
	
				if ( VF != NULL
						&& stv_is_time_value( VF->field->time_scale ) )
				{
					if ( !(any++) )
						printf( "\nSelected Field Time Scales Are:\n" );
				
					printf( "\t%s:  %d %s\n", VF->field->name,
						stv_value_of_time( VF->field->time_scale ),
						stv_time_unit_str(
							stv_unit_of_time(
								VF->field->time_scale ) ) );
				}
			}

			printf( "\nEnter Visualization Frame Frequency (%d %s): ",
				stv_value_of_time( *freq ),
				stv_time_unit_str( stv_unit_of_time( *freq ) ) );
			read_name( tmp );

			if ( !stv_parse_freq_str( tmp, freq, stvTrue ) )
				*freq = 1;
		}
	}

	if ( no_synch != NULL )
	{
		if ( SYNCH_DEFAULT >= 0 )
		{
			printf( "\nUsing Default No_Synch Value = %d\n",
				SYNCH_DEFAULT );

			*no_synch = SYNCH_DEFAULT;
		}
		
		else
		{
			printf( "\nEnter No_Synch Value [0/1] (%d): ", *no_synch );
			read_name( tmp );

			flag = sscanf( tmp, "%d", &num );

			if ( flag != 1 )
				num = 0;

			*no_synch = num;
		}
	}

	if ( is_static != NULL )
	{
		if ( STATIC_DEFAULT >= 0 )
		{
			printf( "\nUsing Default Is_Static Value = %d\n",
				STATIC_DEFAULT );

			*is_static = STATIC_DEFAULT;
		}
		
		else
		{
			printf( "\nEnter Is_Static Value [0/1] (%d): ",
				*is_static );
			read_name( tmp );

			flag = sscanf( tmp, "%d", &num );

			if ( flag != 1 )
				num = 0;

			*is_static = num;
		}
	}
}


void
prompt_for_vis_region( F, VR, initflag )
STV_FIELDPTR F;
STV_VIS_REGION VR;
int initflag;
{
	char tmp[1024];

	int *glb, *gub;

	int flag;
	int dim;
	int num;
	int i;

	/* Get glb, gub */

	if ( F->decomp != NULL )
	{
		glb = F->decomp->glb;
		gub = F->decomp->gub;

		dim = F->decomp->dataDim;
	}

	else if ( F->particle != NULL )
	{
		glb = F->particle->glb;
		gub = F->particle->gub;

		dim = F->particle->dataDim;
	}

	else
	{
		printf( "\nWarning: No Decomp or Particle for Field \"%s\"\n\n",
			F->name );
		fflush( stdout );

		return;
	}

	/* Read in Vis Region Fields */

	if ( VISREGION_DEFAULT_SET )
	{
		printf( "\nUsing Default Vis Region.\n" );

		stv_copy_region( VISREGION->region, VISREGION_DEFAULT->region );
	}

	if ( VISCELL_DEFAULT_SET )
	{
		printf( "\nUsing Default Cell Sizes.\n" );

		stv_copy_cell( VISREGION->cell, VISREGION_DEFAULT->cell );
	}

	for ( i=0 ; i < dim ; i++ )
	{
		if ( !VISREGION_DEFAULT_SET )
		{
			/* Min */

			if ( initflag )
				STV_REGION_MIN( VR->region, i ) = glb[i];

			printf( "\nEnter Dimension #%d Min [ %d - %d ] (%d): ",
				i, glb[i], gub[i], STV_REGION_MIN( VR->region, i ) );
			read_name( tmp );

			flag = sscanf( tmp, "%d", &num );

			if ( flag == 1 )
			{
				STV_REGION_MIN( VR->region, i ) =
					( num < glb[i] ) ? glb[i] : num;

				if ( STV_REGION_MIN( VR->region, i ) > gub[i] )
					STV_REGION_MIN( VR->region, i ) = gub[i];
			}

			/* Max */

			if ( initflag )
				STV_REGION_MAX( VR->region, i ) = gub[i];

			printf( "Enter Dimension #%d Max [ %d - %d ] (%d): ",
				i, glb[i], gub[i], STV_REGION_MAX( VR->region, i ) );
			read_name( tmp );

			flag = sscanf( tmp, "%d", &num );

			if ( flag == 1 )
			{
				if ( num < STV_REGION_MIN( VR->region, i ) )
					num = STV_REGION_MIN( VR->region, i );

				STV_REGION_MAX( VR->region, i ) =
					( num > gub[i] ) ? gub[i] : num;
			}
		}

		if ( F->decomp != NULL && !VISCELL_DEFAULT_SET )
		{
			if ( initflag )
				STV_CELL_OF_DIM( VR->cell, i ) = 1;

			printf( "Enter Dimension #%d Cell Size (%d): ",
				i, STV_CELL_OF_DIM( VR->cell, i ) );
			read_name( tmp );

			flag = sscanf( tmp, "%d", &num );

			if ( flag == 1 )
				STV_CELL_OF_DIM( VR->cell, i ) = ( num < 1 ) ? 1 : num;
		}
	}

	if ( F->particle != NULL && !VISCELL_DEFAULT_SET )
	{
		if ( initflag )
			STV_CELL_PARTICLE( VR->cell ) = 1;

		printf( "\nEnter Particle Cell Size (%d): ",
			STV_CELL_PARTICLE( VR->cell ) );
		read_name( tmp );

		flag = sscanf( tmp, "%d", &num );

		if ( flag == 1 )
			STV_CELL_PARTICLE( VR->cell ) = ( num < 1 ) ? 1 : num;
	}

	/* Dump Out Region */

	printf( "\n" );

	for ( i=0 ; i < dim ; i++ )
	{
		if ( F->decomp != NULL )
		{
			printf( "[%d]:  min=%d max=%d cell=%d\n", i,
				STV_REGION_MIN( VR->region, i ),
				STV_REGION_MAX( VR->region, i ),
				STV_CELL_OF_DIM( VR->cell, i ) );
		}

		else if ( F->particle != NULL )
		{
			printf( "[%d]:  min=%d max=%d\n", i,
				STV_REGION_MIN( VR->region, i ),
				STV_REGION_MAX( VR->region, i ) );
		}
	}

	if ( F->particle != NULL )
		printf( "\tparticle cell = %d\n",
			STV_CELL_PARTICLE( VR->cell ) );

	printf( "\n" );
}


void
dump_region( F, VR )
STV_FIELDPTR F;
STV_VIS_REGION VR;
{
	int dim;
	int i;

	/* Get Data Dim */

	if ( F->decomp != NULL )
		dim = F->decomp->dataDim;

	else if ( F->particle != NULL )
		dim = F->particle->dataDim;

	else
	{
		printf( "\nWarning: No Decomp or Particle for Field \"%s\"\n\n",
			F->name );
		fflush( stdout );

		return;
	}

	printf( "Vis Region:\n" );

	if ( F->decomp != NULL )
	{
		for ( i=0 ; i < dim ; i++ )
		{
			printf( "\t[%d to %d] by %d\n",
				STV_REGION_MIN( VR->region, i ),
				STV_REGION_MAX( VR->region, i ),
				STV_CELL_OF_DIM( VR->cell, i ) );
		}
	}

	else if ( F->particle != NULL )
	{
		for ( i=0 ; i < dim ; i++ )
		{
			printf( "\t[%d to %d]\n",
				STV_REGION_MIN( VR->region, i ),
				STV_REGION_MAX( VR->region, i ) );
		}

		printf( "\t\tby %d\n", STV_CELL_PARTICLE( VR->cell ) );
	}
}


#ifdef GIFVWR

void
dump_gif_file( VFG )
STV_VIEW_FIELD_GROUP VFG;
{
	STV_VIEW_PARTICLE_DATA VPD;

	STV_VIEW_PARTICLE VPT;

	STV_VIEW_FIELD VF;

	STV_VIS_REGION VR;

	STV_VALUE ptr;

	FILE *gif;

	gdImagePtr img = (gdImagePtr) NULL;

	char cmd[1024];

	char **done_parts;
	int done_already;
	int ndone;

	float cdiff, ddiff, thresh;
	float min, max;
	float pxscale;
	float pyscale;
	float pscale;
	float fval;

	int indices[STV_MAX_DIM];

	int radius_index;
	int color_index;
	int i, j, k;
	int xi, yi, zi;
	int newsize;
	int nrects;
	int xscale;
	int yscale;
	int wt, ht;
	int nx, ny;
	int x1, y1;
	int x2, y2;
	int color;
	int scale;
	int x, y;
	int r;

	/* Set Proper Plane Indices */

	get_plane_indices( &xi, &yi, &zi );

	/* Allocate Done Particle Array */

	done_parts = (char **) malloc( (STV_SIZE_T) VFG->nvfields
			 		* sizeof(char *) );
	if ( stv_memcheck( done_parts, "Done Particles List" )
		!= stvStatusOk )
	{
		exit( -1 );
	}

	for ( i=0 ; i < VFG->nvfields ; i++ )
		done_parts[i] = (char *) NULL;

	ndone = 0;

	/* Go Through All View Fields */

	VR = VFG->visregion;

	for ( i=0 ; i < VFG->nvfields ; i++ )
	{
		VF = VFG->view_field_list[i];

		if ( vflag )
			printf( "\tprocessing field %s:\n", VF->field->name );

		/* Contiguous Data */

		if ( VF->field->decomp != NULL )
		{
			if ( vflag )
				printf( "\t\tdecomposed data field.\n" );

			/* Calc Total Num of Rects Needed */

			nx = ( ( STV_REGION_MAX( VR->region, xi )
						- STV_REGION_MIN( VR->region, xi ) + 1 )
					+ ( STV_CELL_OF_DIM( VR->cell, xi ) - 1 ) )
				/ STV_CELL_OF_DIM( VR->cell, xi );

			ny = ( ( STV_REGION_MAX( VR->region, yi )
						- STV_REGION_MIN( VR->region, yi ) + 1 )
					+ ( STV_CELL_OF_DIM( VR->cell, yi ) - 1 ) )
				/ STV_CELL_OF_DIM( VR->cell, yi );

			nrects = nx * ny;

			xscale = GIF_WIDTH / nx;
			yscale = GIF_HEIGHT / ny;

			scale = ( xscale < yscale ) ? xscale : yscale;
			scale = scale ? scale : 1;

			/* Create GIF Image */

			if ( img == NULL )
			{
				/* Determine Actual Width & Height */

				wt = scale * nx;
				ht = scale * ny;

				/* Create GIF Image */

				img = gdImageCreate( wt, ht );

				/* Set Image to Interlaced */

				gdImageInterlace( img, 1 );
			}

			/* Automatically Determine Color Map Range */

			if ( AUTO_COLOR_MAP || LAZY_COLOR_MAP || SHOW_MIN_MAX )
			{
				if ( vflag )
					printf( "\t\tdetermining color map range.\n" );

				/* Initialize Local Index */

				for ( j=0 ; j < VF->field->decomp->dataDim ; j++ )
					indices[j] = 0;

				/* Set Data Pointers */
				/* (only one vis region, index == 0) */

				ptr = VFG->field_values[i][0];

				/* Determine Min & Max Values */

				j = 0;

				do
				{
					fval = STV_VALUE_OF( ptr, float );

					if ( !j )
						min = max = fval;

					else
					{
						min = ( fval < min ) ? fval : min;
						max = ( fval > max ) ? fval : max;
					}

					ptr = stv_get_typed_ptr_offset( ptr, 1, stvFloat );

					j++;
				}
				while ( ptr != NULL && j < nrects
					&& stv_incr_field_index( VF->field->decomp->dataDim,
						indices, VFG->field_arrayDecls[i][0],
						VFG->visregion, VFG->view_storage_orders[i],
						stvFalse ) );

				if ( AUTO_COLOR_MAP )
				{
					COLOR_MIN_VALUE = min;
					COLOR_MAX_VALUE = max;
				}

				else if ( LAZY_COLOR_MAP )
				{
					if ( !COLOR_MAP_INITD )
					{
						COLOR_MIN_VALUE = min;
						COLOR_MAX_VALUE = max;

						COLOR_MAP_INITD = stvTrue;
					}

					else
					{
						/* Color Range */
						cdiff = (float) fabs( (double)
							( COLOR_MAX_VALUE - COLOR_MIN_VALUE ) );

						/* Data Range */
						ddiff = (float) fabs( (double) ( max - min ) );

						/* Cut-Off is 50% of Max Threshold */
						thresh = 0.50
								* ( ( ddiff > cdiff ) ? ddiff : cdiff );

						if ( max - COLOR_MIN_VALUE <= thresh
							|| COLOR_MAX_VALUE - min <= thresh )
						{
							COLOR_MIN_VALUE = min;
							COLOR_MAX_VALUE = max;
						}
					}
				}

				if ( SHOW_MIN_MAX )
				{
					MIN_VALUE = min;
					MAX_VALUE = max;
				}
			}

			/* Fill Color Map */

			for ( j=0 ; j < 255 ; j++ )
			{
				color_value( img, &color,
					( (float) ( j
							* (COLOR_MAX_VALUE - COLOR_MIN_VALUE + 1) )
						/ (float) 255 )
					+ (float) COLOR_MIN_VALUE,
					stvTrue );
			}

			/* Draw Rects */

			if ( vflag )
				printf( "\t\tdrawing rectangles.\n" );

			/* Initialize Local Index */

			for ( j=0 ; j < VF->field->decomp->dataDim ; j++ )
				indices[j] = 0;

			/* Set Data Pointers */
			/* (only one vis region, index == 0) */

			ptr = VFG->field_values[i][0];

			/* Draw 'Em */

			j = 0;

			do
			{
				x = ( scale * indices[xi] );
				y = ( scale * indices[yi] );

				x1 = x;
				y1 = y;
				x2 = x + scale - 1;
				y2 = y + scale - 1;

				fval = STV_VALUE_OF( ptr, float );

				color_value( img, &color, fval, stvFalse );

				gdImageFilledRectangle( img, x1, y1, x2, y2, color );

				ptr = stv_get_typed_ptr_offset( ptr, 1, stvFloat );

				j++;
			}
			while ( ptr != NULL && j < nrects
				&& stv_incr_field_index( VF->field->decomp->dataDim,
					indices, VFG->field_arrayDecls[i][0],
					VFG->visregion, VFG->view_storage_orders[i],
					stvFalse ) );
		}

		/* Particle Data */

		else if ( VF->field->particle != NULL )
		{
			if ( vflag )
				printf( "\t\tparticle data field.\n" );

			done_already = 0;

			for ( j=0 ; j < ndone && !done_already ; j++ )
			{
				if ( !strcmp( done_parts[j],
					VF->field->particle->name ) )
				{
					done_already++;
				}
			}

			if ( !done_already )
			{
				if ( vflag )
					printf( "\t\tnot already processed.\n" );

				done_parts[ ndone++ ] = VF->field->particle->name;

				/* Calc Drawing Scale */

				nx = STV_REGION_MAX( VR->region, xi )
					- STV_REGION_MIN( VR->region, xi ) + 1;

				ny = STV_REGION_MAX( VR->region, yi )
					- STV_REGION_MIN( VR->region, yi ) + 1;

				pxscale = (float) GIF_WIDTH / (float) nx;
				pyscale = (float) GIF_HEIGHT / (float) ny;

				pscale = ( pxscale < pyscale ) ? pxscale : pyscale;

				/* Create GIF Image */

				if ( img == NULL )
				{
					/* Determine Actual Width & Height */

					wt = scale * nx;
					ht = scale * ny;

					/* Create GIF Image */

					img = gdImageCreate( wt, ht );

					/* Set Image to Interlaced */

					gdImageInterlace( img, 1 );
				}

				/* Automatically Determine Color Map Range */

				if ( AUTO_COLOR_MAP || LAZY_COLOR_MAP || SHOW_MIN_MAX )
				{
					if ( vflag )
						printf( "\t\tdetermining color map range.\n" );

					VPT = VFG->particle_list;
	
					k = 0;
	
					while ( VPT != NULL )
					{
						/* Get Index of Color Value... */
	
						color_index = -1;
	
						for ( j=0 ; j < VPT->nvfields ; j++ )
						{
							if ( COLOR_FIELD != NULL
								&& !strcmp(
								(VPT->view_field_list[j])->field->name,
									COLOR_FIELD ) )
							{
								color_index = j;
							}
						}
	
						if ( color_index >= 0 )
						{
							VPD = VPT->pdata;
	
							while ( VPD != NULL )
							{
								fval = STV_VALUE_OF(
									VPD->field_values[ color_index ],
									float );

								if ( !k )
									min = max = fval;

								else
								{
									min = ( fval < min ) ? fval : min;
									max = ( fval > max ) ? fval : max;
								}

								VPD = VPD->next;
	
								k++;
							}
						}
	
						VPT = VPT->next;
					}

					if ( AUTO_COLOR_MAP )
					{
						COLOR_MIN_VALUE = min;
						COLOR_MAX_VALUE = max;
					}

					else if ( LAZY_COLOR_MAP )
					{
						if ( !COLOR_MAP_INITD )
						{
							COLOR_MIN_VALUE = min;
							COLOR_MAX_VALUE = max;

							COLOR_MAP_INITD = stvTrue;
						}

						else
						{
							/* Color Range */
							cdiff = (float) fabs( (double)
								( COLOR_MAX_VALUE - COLOR_MIN_VALUE ) );

							/* Data Range */
							ddiff = (float) fabs( (double)
									( max - min ) );

							/* Cut-Off is 50% of Max Threshold */
							thresh = 0.50
									* ( ( ddiff > cdiff )
											? ddiff : cdiff );

							if ( max - COLOR_MIN_VALUE <= thresh
								|| COLOR_MAX_VALUE - min <= thresh )
							{
								COLOR_MIN_VALUE = min;
								COLOR_MAX_VALUE = max;
							}
						}
					}

					if ( SHOW_MIN_MAX )
					{
						MIN_VALUE = min;
						MAX_VALUE = max;
					}
				}

				/* Fill Color Map */

				for ( j=0 ; j < 255 ; j++ )
				{
					color_value( img, &color,
						( (float) ( j
								* (COLOR_MAX_VALUE
									- COLOR_MIN_VALUE + 1) )
							/ (float) 255 )
						+ (float) COLOR_MIN_VALUE,
						stvTrue );
				}

				/* Process Particles */

				VPT = VFG->particle_list;

				k = 0;

				while ( VPT != NULL )
				{
					if ( vflag )
						printf(
							"\t\tdrawing circles for particle %s.\n",
							VPT->particle->name );

					/* Get Indicies of Radius and/or Color Values */

					radius_index = -1;
					color_index = -1;

					for ( j=0 ; j < VPT->nvfields ; j++ )
					{
						if ( RADIUS_FIELD != NULL
							&& !strcmp(
								(VPT->view_field_list[j])->field->name,
								RADIUS_FIELD ) )
						{
							radius_index = j;
						}

						else if ( COLOR_FIELD != NULL
							&& !strcmp(
								(VPT->view_field_list[j])->field->name,
								COLOR_FIELD ) )
						{
							color_index = j;
						}
					}

					VPD = VPT->pdata;

					while ( VPD != NULL )
					{
						fval = ( color_index >= 0 )
							? STV_VALUE_OF(
								VPD->field_values[ color_index ],
								float )
							: 0.0;

						x = (int) ( pscale
							* (float) ( VPD->coords[xi]
							- STV_REGION_MIN( VR->region, xi ) ) );

						y = (int) ( pscale
							* (float) ( VPD->coords[yi]
							- STV_REGION_MIN( VR->region, yi ) ) );

						r = (int) ( pscale * 
							( ( radius_index >= 0 )
								? STV_VALUE_OF(
									VPD->field_values[ radius_index ],
									float )
								: 1.0 ) );

						/* Get Color Value */

						color_value( img, &color, fval, stvFalse );

						/* Draw It */

						gdImageArc( img, x, y, 2 * r, 2 * r,
							0, 360, color );
							
						gdImageFill( img, x, y, color );

						VPD = VPD->next;

						k++;
					}

					VPT = VPT->next;
				}
			}
		}
	}

	/* Free Up Done Particle List */

	for ( i=0 ; i < VFG->nvfields ; i++ )
		done_parts[i] = (char *) NULL;

	free( done_parts );

	/* Write Temporary File */

	gif = fopen( GIF_TMPFILE, "w" );

	if ( gif != NULL )
	{
		gdImageGif( img, gif );
	
		fclose( gif );
	}

	/* Free GIF Image */

	gdImageDestroy( img );

	/* Move to Final File - Better Atomic Operation */

	if ( USE_JAVA )
		sprintf( cmd, "mv %s %s/stv.gif", GIF_TMPFILE, GIF_ROOT );
	else
	{
		sprintf( cmd, "mv %s %s/stv.%d.gif",
			GIF_TMPFILE, GIF_ROOT, GIF_CNTR );
	}

	system( cmd );
}


void
dump_html_file( VFG, V )
STV_VIEW_FIELD_GROUP VFG;
STV_VIEWER V;
{
	STV_VIEW_FIELD VF;
	STV_VIEW_FIELD VF2;

	STV_VIS_REGION VR;

	STV_FIELDPTR F;

	FILE *html;

	char fname[1024];
	char cmd[1024];

	int xi, yi, zi;
	int selected;
	int dim;
	int i;

	/* Set Proper Plane Indices */

	get_plane_indices( &xi, &yi, &zi );

	/* Get Data Dim */

	F = VF_PRIMARY->field;

	if ( F->decomp != NULL )
		dim = F->decomp->dataDim;

	else if ( F->particle != NULL )
		dim = F->particle->dataDim;

	else
	{
		printf( "\nWarning: No Decomp or Particle for Field \"%s\"\n\n",
			F->name );
		printf( "Assuming 3-Dimensional Data in %s...\n",
			"dump_html_file()" );
		fflush( stdout );

		dim = 3;
	}

	/* Open Temporary HTML File */

	html = fopen( HTML_TMPFILE, "w" );

	if ( html == NULL )
	{
		printf( "\nError Opening Temporary HTML File \"%s\".\n\n",
			HTML_TMPFILE );
		return;
	}

	/* Dump HTML Header */

	fprintf( html, "<HTML>\n" );

	fprintf( html, "<HEADER>\n" );
	fprintf( html, "<TITLE>\n" );
	fprintf( html, "CUMULVS Browser (GIF-O-Matic) 1.0\n" );
	fprintf( html, "</TITLE>\n" );
	fprintf( html, "</HEADER>\n" );

	fprintf( html, "<BODY BGCOLOR=\"#FFFFFF\" TEXT=\"navy\">\n" );

	fprintf( html, "<CENTER>\n" );
	fprintf( html, "<H1>\n" );
	fprintf( html, "CUMULVS Browser (GIF-O-Matic) 1.0\n" );
	fprintf( html, "</H1>\n" );
	fprintf( html, "</CENTER>\n" );

	/* Dump GIF-O-Matic Applet */

	fprintf( html, "<CENTER>\n" );

	if ( USE_JAVA )
	{

#ifdef NEW_JAVA

		fprintf( html, "<EMBED " );
		fprintf( html,
			"TYPE=\"application/x-java-applet;version=1.2\" " );
		fprintf( html, "CODE=\"%s\" ", GIFOMATIC_CODE );
		fprintf( html, "ARCHIVE=\"%s\" ", GIFOMATIC_ARCHIVE );
		fprintf( html, "PLUGINSPAGE=\"%s\" ", PLUGINS_PAGE );
		fprintf( html, "WIDTH=%d HEIGHT=%d>\n", GIF_WIDTH, GIF_HEIGHT );

		fprintf( html, "<PARAM NAME=\"ARCHIVE\" VALUE=\"%s\">\n",
			GIFOMATIC_ARCHIVE );
		fprintf( html,
			"<PARAM NAME=\"ImageURL\" VALUE=\"%s/stv.gif\">\n",
			GIF_URL_ROOT );
	
		fprintf( html, "</EMBED>\n" );

#else

		fprintf( html, "<APPLET\n" );
		fprintf( html, "\tCODE=\"%s\"\n", GIFOMATIC_CODE );
		fprintf( html, "\tARCHIVE=\"%s\"\n", GIFOMATIC_ARCHIVE );
		fprintf( html, "\tWIDTH=%d\n", GIF_WIDTH );
		fprintf( html, "\tHEIGHT=%d>\n", GIF_HEIGHT );

		fprintf( html,
			"<PARAM NAME=\"ImageURL\" VALUE=\"%s/stv.gif\">\n",
			GIF_URL_ROOT );
		fprintf( html, "<PARAM NAME=\"ImgInterval\" VALUE=\"1\">\n" );
		fprintf( html, "<PARAM NAME=\"Trace\" VALUE=\"false\">\n" );
	
		fprintf( html, "</APPLET>\n" );
#endif

	}

	else
	{
		fprintf( html, "<IMG SRC=\"%s/stv.%d.gif\">\n",
			GIF_URL_ROOT, GIF_CNTR );
	}

	fprintf( html, "</CENTER>\n" );

	/* Show Data Field Time Value */

	fprintf( html, "<CENTER>\n" );
	fprintf( html, "<H4>\n" );

	fprintf( html, "Data Frame for Field \"%s\" at ", F->name );

	stv_dump_base_time_str( html, F->time_base, F->time_units,
		stvTrue );

	fprintf( html, "</H4>\n" );
	fprintf( html, "</CENTER>\n" );

	/* Show Min & Max Data Values */

	if ( SHOW_MIN_MAX )
	{
		fprintf( html, "<CENTER>\n" );
		fprintf( html, "<H4>\n" );
		fprintf( html, "Minimum Data Value = %g, &nbsp;&nbsp; ",
			MIN_VALUE );
		fprintf( html, "Maximum Data Value = %g\n", MAX_VALUE );
		fprintf( html, "</H4>\n" );
		fprintf( html, "</CENTER>\n" );
	}

	/* Refresh Button */

	if ( !USE_JAVA )
	{
		fprintf( html, "<CENTER>\n" );

		fprintf( html,
			"<FORM METHOD=\"POST\" ACTION=\"%s/gifvwr.cgi\">\n",
			CGI_URL_ROOT );

		fprintf( html, "<H4>\n" );
		fprintf( html,
			"<INPUT TYPE=\"SUBMIT\" VALUE=\"Refresh View\">\n" );
		fprintf( html, "</H4>\n" );

		fprintf( html, "</FORM>\n" );

		fprintf( html, "</CENTER>\n" );
	}

	fprintf( html, "<HR>\n" );

	/* Dump View Form */

	fprintf( html, "<FORM METHOD=\"POST\" ACTION=\"%s/gifvwr.cgi\">\n",
		CGI_URL_ROOT );

	/* Application Name */

	fprintf( html, "<H4>\n" );

	fprintf( html, "Application Name:\n" );
	fprintf( html, "<INPUT TYPE=\"TEXT\" NAME=\"appname\" " );
	fprintf( html, "SIZE=\"16\" VALUE=\"%s\">\n", APPNAME );

	/* Data Field Selection */

	fprintf( html, "&nbsp;&nbsp;" );
	fprintf( html, "Data Field: " );
	fprintf( html, "<SELECT NAME=\"data_field\">\n" );

	VF = V->vfields;

	selected = 0;

	while ( VF != NULL )
	{
		fprintf( html, "<OPTION" );

		for ( i=0 ; i < VFG->nvfields && !selected ; i++ )
		{
			VF2 = VFG->view_field_list[i];

			if ( !strcmp( VF->field->name, VF2->field->name ) )
			{
				fprintf( html, " SELECTED" );

				selected++;
			}
		}

		fprintf( html, "> %s\n", VF->field->name );

		VF = VF->next;
	}

	fprintf( html, "</SELECT>\n" );

	/* Plane Selection */

	if ( dim > 2 )
	{
		fprintf( html, "&nbsp;&nbsp;" );
		fprintf( html, "Plane: " );
		fprintf( html, "<SELECT NAME=\"plane_choice\">\n" );

		fprintf( html, "<OPTION VALUE=\"xy\"%s> X-Y Plane\n",
			( PLANE_CHOICE == PLANE_XY ) ? " SELECTED" : "" );

		fprintf( html, "<OPTION VALUE=\"yz\"%s> Y-Z Plane\n",
			( PLANE_CHOICE == PLANE_YZ ) ? " SELECTED" : "" );

		fprintf( html, "<OPTION VALUE=\"xz\"%s> X-Z Plane\n",
			( PLANE_CHOICE == PLANE_XZ ) ? " SELECTED" : "" );

		fprintf( html, "</SELECT>\n" );
	}

	/* Viz Frequency */

	fprintf( html, "&nbsp;&nbsp;" );
	fprintf( html, "Frame Freq:\n" );
	fprintf( html, "<INPUT TYPE=\"TEXT\" NAME=\"freq\" SIZE=\"4\" " );

	if ( stv_is_time_value( VFG->freq ) )
		fprintf( html, "VALUE=\"%d %s\">\n",
			stv_value_of_time( VFG->freq ),
			stv_time_unit_str( stv_unit_of_time( VFG->freq ) ) );
	else
		fprintf( html, "VALUE=\"%d\">\n", VFG->freq );

	fprintf( html, "</H4>\n" );

	/* Viz Region */

	VR = VFG->visregion;

	fprintf( html, "<H4>\n" );

	fprintf( html, "%s Axis: ",
		( PLANE_CHOICE == PLANE_YZ ) ? "Y" : "X" );

	fprintf( html, "Minimum=" );
	fprintf( html, "<INPUT TYPE=\"TEXT\" NAME=\"x_min\" " );
	fprintf( html, "SIZE=\"8\" VALUE=\"%d\">\n",
		STV_REGION_MIN( VR->region, xi ) );

	fprintf( html, "Maximum=" );
	fprintf( html, "<INPUT TYPE=\"TEXT\" NAME=\"x_max\" " );
	fprintf( html, "SIZE=\"8\" VALUE=\"%d\">\n",
		STV_REGION_MAX( VR->region, xi ) );

	fprintf( html, "Cell Size=" );
	fprintf( html, "<INPUT TYPE=\"TEXT\" NAME=\"x_cell\" " );
	fprintf( html, "SIZE=\"8\" VALUE=\"%d\">\n",
		STV_CELL_OF_DIM( VR->cell, xi ) );

	fprintf( html, "<BR>\n" );

	fprintf( html, "%s Axis: ",
		( PLANE_CHOICE == PLANE_XY ) ? "Y" : "Z" );

	fprintf( html, "Minimum=" );
	fprintf( html, "<INPUT TYPE=\"TEXT\" NAME=\"y_min\" " );
	fprintf( html, "SIZE=\"8\" VALUE=\"%d\">\n",
		STV_REGION_MIN( VR->region, yi ) );

	fprintf( html, "Maximum=" );
	fprintf( html, "<INPUT TYPE=\"TEXT\" NAME=\"y_max\" " );
	fprintf( html, "SIZE=\"8\" VALUE=\"%d\">\n",
		STV_REGION_MAX( VR->region, yi ) );

	fprintf( html, "Cell Size=" );
	fprintf( html, "<INPUT TYPE=\"TEXT\" NAME=\"y_cell\" " );
	fprintf( html, "SIZE=\"8\" VALUE=\"%d\">\n",
		STV_CELL_OF_DIM( VR->cell, yi ) );

	fprintf( html, "<BR>\n" );

	if ( dim > 2 )
	{
		fprintf( html, "%s Axis: ",
			( PLANE_CHOICE == PLANE_XY ) ? "Z" :
				( ( PLANE_CHOICE == PLANE_YZ ) ? "X" : "Y" ) );

		fprintf( html, "Value=" );
		fprintf( html, "<INPUT TYPE=\"TEXT\" NAME=\"z_val\" " );
		fprintf( html, "SIZE=\"8\" VALUE=\"%d\">\n",
			STV_REGION_MIN( VR->region, zi ) );

		fprintf( html, "<BR>\n" );
	}

	fprintf( html, "</H4>\n" );

	/* Color Map Range / Mode Selection */

	fprintf( html, "<H4>\n" );

	fprintf( html, "Color Map Range: " );
	fprintf( html, "&nbsp;&nbsp;" );

	fprintf( html, "Minimum=" );
	fprintf( html, "<INPUT TYPE=\"TEXT\" NAME=\"color_min\" " );
	fprintf( html, "SIZE=\"8\" VALUE=\"%g\">\n", COLOR_MIN_VALUE );

	fprintf( html, "Maximum=" );
	fprintf( html, "<INPUT TYPE=\"TEXT\" NAME=\"color_max\" " );
	fprintf( html, "SIZE=\"8\" VALUE=\"%g\">\n", COLOR_MAX_VALUE );

	fprintf( html, "&nbsp;&nbsp;" );
	fprintf( html, "Mode: " );
	fprintf( html, "<SELECT NAME=\"color_mode\">\n" );

	fprintf( html, "<OPTION%s VALUE=\"auto\"> Auto Color Map\n",
			AUTO_COLOR_MAP ? " SELECTED" : "" );
	fprintf( html, "<OPTION%s VALUE=\"lazy\"> Lazy Color Map\n",
			LAZY_COLOR_MAP ? " SELECTED" : "" );
	fprintf( html, "<OPTION%s VALUE=\"manual\"> Manual Color Map\n",
			( AUTO_COLOR_MAP || LAZY_COLOR_MAP ) ? "" : " SELECTED" );

	fprintf( html, "</SELECT>\n" );

	fprintf( html, "</H4>\n" );

	/* Dump End of Form */

	fprintf( html, "<H4>\n" );
	fprintf( html, "<INPUT TYPE=\"SUBMIT\" VALUE=\"Change View\">\n" );
	fprintf( html, "</H4>\n" );

	fprintf( html, "</FORM>\n" );

	/* Dump HTML Trailer */

	fprintf( html, "<HR>\n" );

	fprintf( html, "<H4>\n" );
	fprintf( html,
		"<A HREF=\"http://www.csm.ornl.gov/cs/cumulvs.html\">\n" );
	fprintf( html, "CUMULVS Project</A> - " );
	fprintf( html, "<A HREF=\"http://www.csm.ornl.gov/\">\n" );
	fprintf( html, "ORNL CSMD</A>\n" );
	fprintf( html, "</H4>\n" );

	fprintf( html, "<H4>\n" );
	fprintf( html, "GIF-O-Matic and HTML-O-Matic Courtesy of " );
	fprintf( html, "<A HREF=\"http://z.ca.sandia.gov/\">" );
	fprintf( html, "SNL Distributed Computing Team.</A>\n" );
	fprintf( html, "</H4>\n" );

	fprintf( html, "<H5>\n" );
	fprintf( html, "Research Sponsored by the " );
	fprintf( html, "<A HREF=\"http://www.er.doe.gov/production" );
	fprintf( html, "/octr/mics/index.html\">\n" );
	fprintf( html, "Mathematics, Information and Computational " );
	fprintf( html, "Sciences Office</A>,\n" );
	fprintf( html,
		"<A HREF=\"http://www.er.doe.gov/production/octr/\">\n" );
	fprintf( html, "Office of Advanced Scientific Computing " );
	fprintf( html, "Research</A>,\n" );
	fprintf( html, "<A HREF=\"http://www.er.doe.gov\">" );
	fprintf( html, "Office of Science</A>,\n" );
	fprintf( html, "<A HREF=\"http://www.doe.gov\">" );
	fprintf( html, "U. S. Department of Energy</A>,\n" );
	fprintf( html, "under contract No. DE-AC05-00OR22725 with\n" );
	fprintf( html, "<A HREF=\"http://www.ut-battelle.com/\">" );
	fprintf( html, "UT-Battelle, LLC</A>.\n" );
	fprintf( html, "</H5>\n" );

	fprintf( html, "</BODY>\n" );

	fprintf( html, "</HTML>\n" );

	/* Done */

	fclose( html );

	/* Move to Final File - Better Atomic Operation */

	sprintf( cmd, "mv %s %s", HTML_TMPFILE, HTML_FILE );

	system( cmd );

	/* Clean Up Last GIF Image */

	if ( !USE_JAVA )
	{
		/* Always Delete GIF_CNTR - 2, to avoid race with stv.html... */
		sprintf( fname, "%s/stv.%d.gif", GIF_ROOT, GIF_CNTR - 2 );

		unlink( fname );

		GIF_CNTR++;
	}
}


void
color_value( img, color, value, allocate )
gdImagePtr img;
int *color;
float value;
int allocate;
{
	float ratio;

	float R;
	float G;
	float B;

	if ( EQUIV( COLOR_MIN_VALUE, COLOR_MAX_VALUE ) )
	{
		if ( value > COLOR_MIN_VALUE )
		{
			R = 255.0;
			G = 0.0;
			B = 0.0;
		}

		else if ( value < COLOR_MIN_VALUE )
		{
			R = 0.0;
			G = 0.0;
			B = 255.0;
		}

		else
		{
			R = 0.0;
			G = 255.0;
			B = 0.0;
		}
	}

	else
	{
		ratio = ( value - COLOR_MIN_VALUE )
			 / ( COLOR_MAX_VALUE - COLOR_MIN_VALUE );

		B = 255.0 - ( 255.0 * ratio );

		B = ( B < 0 ) ? 0.0 : B;
		B = ( B > 255.0 ) ? 255.0 : B;

		if ( ratio > 0.5 )
			G = 255.0 - ( 255.0 * ( ratio - 0.5 ) * 2.0 );

		else
			G = 255.0 * ratio * 2.0;

		G = ( G < 0 ) ? 0.0 : G;
		G = ( G > 255.0 ) ? 255.0 : G;

		R = 255.0 * ratio;

		R = ( R < 0 ) ? 0.0 : R;
		R = ( R > 255.0 ) ? 255.0 : R;
	}

	if ( allocate )
	{
		*color = gdImageColorAllocate( img, (int) R, (int) G, (int) B );

		if ( *color >= 0 ) return;
	}

	*color = gdImageColorExact( img, (int) R, (int) G, (int) B );

	if ( *color < 0 )
	{
		*color = gdImageColorClosest( img, (int) R, (int) G, (int) B );

		if ( *color < 0 )
		{
			printf( "*" );  fflush( stdout );
		}
	}
}


int
get_plane_indices( xi, yi, zi )
int *xi;
int *yi;
int *zi;
{
	switch ( PLANE_CHOICE )
	{
		case PLANE_XY:
			*xi = 0;
			*yi = 1;
			*zi = 2;
			break;

		case PLANE_YZ:
			*xi = 1;
			*yi = 2;
			*zi = 0;
			break;

		case PLANE_XZ:
			*xi = 0;
			*yi = 2;
			*zi = 1;
			break;

		default:
		{
			printf( "\nError in %s:  Invalid Plane Selection (%d)\n\n",
				"get_plane_indices()", PLANE_CHOICE );

			return( stvFalse );
		}
	}

	return( stvTrue );
}


int
handle_cgi_input( rbuf, VFG, V )
int rbuf;
STV_VIEW_FIELD_GROUP VFG;
STV_VIEWER V;
{
	STV_VIEW_FIELD VF;

	STV_FIELDPTR F;

	FILE *params;

	char unitstr[255];
	char value[1024];
	char fname[1024];
	char key[1024];

	char **fieldnames;
	int nfields;

	double dnum;

	int appl_changed = 0;
	int data_changed = 0;
	int view_changed = 0;
	int freq_changed = 0;

	int *glb, *gub;

	int region_check;
	int xi, yi, zi;
	int newfreq;
	int units;
	int done;
	int flag;
	int freq;
	int dim;
	int i;

	/* Check for CGI Parameters File */

	if ( rbuf < 0 )
	{
		sprintf( fname, "%s/gifvwr.dat", GIF_VIEWER_ROOT );

		params = fopen( fname, "r" );

		if ( params == NULL )
			return( stvTrue );
	}

	/* Get Initial Plane Indices */

	get_plane_indices( &xi, &yi, &zi );

	/* Get GLB & GUB, dim */

	F = VF_PRIMARY->field;

	region_check = 1;

	if ( F->decomp != NULL )
	{
		glb = F->decomp->glb;
		gub = F->decomp->gub;

		dim = F->decomp->dataDim;
	}

	else if ( F->particle != NULL )
	{
		glb = F->particle->glb;
		gub = F->particle->gub;

		dim = F->particle->dataDim;
	}

	else
	{
		printf(
			"\nWarning: No Decomp or Particle for Field \"%s\"\n\n",
			F->name );
		fflush( stdout );

		region_check = 0;
	}

	/* Parse CGI Message */

	done = 0;

	do
	{
		/* Get Next Key & Value */

		if ( rbuf >= 0 )
		{
			if ( !(MyMP->unpack_str( rbuf, key )) )
				done++;
			
			else if ( !strcmp( key, "EOF" )
					|| !(MyMP->unpack_str( rbuf, value )) )
				done++;
		}

		else
		{
			if ( fgets( key, 1024, params ) )
			{
				key[ strlen( key ) - 1 ] = '\0';

				if ( !strcmp( key, "EOF" )
						|| !fgets( value, 1024, params ) )
					done++;
				else
					value[ strlen( value ) - 1 ] = '\0';
			}

			else
				done++;
		}
		
		/* Handle Key-Value Pair */

		if ( !done )
		{
			if ( !strcmp( key, "appname" ) )
			{
				printf( "appname -> %s\n", value );

				appl_changed++;
			}

			else if ( !strcmp( key, "data_field" ) )
			{
				printf( "data_field -> %s\n", value );

				if ( strcmp( value, F->name ) )
				{
					fieldnames = (char **) malloc( sizeof( char * ) );

					if ( stv_memcheck( *fieldnames,
							"Field Name List Ptr" ) != stvStatusOk )
						exit( -1 );

					fieldnames[0] = stv_copy_str( value );

					nfields = 1;

					data_changed++;
				}
			}

			else if ( !strcmp( key, "plane_choice" ) )
			{
				printf( "plane_choice -> %s\n", value );

				if ( dim > 2 )
				{
					if ( !strcmp( value, "xy" ) )
						PLANE_CHOICE = PLANE_XY;
					else if ( !strcmp( value, "yz" ) )
						PLANE_CHOICE = PLANE_YZ;
					else if ( !strcmp( value, "xz" ) )
						PLANE_CHOICE = PLANE_XZ;
				}

				else
				{
					printf( "No Plane Choice - %d-Dimensional Data!\n",
						dim );

					PLANE_CHOICE = PLANE_XY;
				}

				get_plane_indices( &xi, &yi, &zi );
			}

			else if ( !strcmp( key, "freq" ) )
			{
				printf( "freq -> %s\n", value );

				if ( !stv_parse_freq_str( value, &newfreq, stvTrue ) )
					newfreq = 1;

				if ( VFG->freq != newfreq )
					freq_changed++;
			}

			else if ( !strcmp( key, "x_min" ) )
			{
				printf( "x_min -> %s\n", value );

				STV_REGION_MIN( VISREGION->region, xi ) = atoi( value );

				view_changed++;
			}

			else if ( !strcmp( key, "x_max" ) )
			{
				printf( "x_max -> %s\n", value );

				STV_REGION_MAX( VISREGION->region, xi ) = atoi( value );

				view_changed++;
			}

			else if ( !strcmp( key, "x_cell" ) )
			{
				printf( "x_cell -> %s\n", value );

				STV_CELL_OF_DIM( VISREGION->cell, xi ) = atoi( value );

				view_changed++;
			}

			else if ( !strcmp( key, "y_min" ) )
			{
				printf( "y_min -> %s\n", value );

				STV_REGION_MIN( VISREGION->region, yi ) = atoi( value );

				view_changed++;
			}

			else if ( !strcmp( key, "y_max" ) )
			{
				printf( "y_max -> %s\n", value );

				STV_REGION_MAX( VISREGION->region, yi ) = atoi( value );

				view_changed++;
			}

			else if ( !strcmp( key, "y_cell" ) )
			{
				printf( "y_cell -> %s\n", value );

				STV_CELL_OF_DIM( VISREGION->cell, yi ) = atoi( value );

				view_changed++;
			}

			else if ( !strcmp( key, "z_val" ) )
			{
				printf( "z_val -> %s\n", value );

				STV_REGION_MIN( VISREGION->region, zi ) =
					STV_REGION_MAX( VISREGION->region, zi ) =
						atoi( value );

				view_changed++;
			}

			else if ( !strcmp( key, "color_min" ) )
			{
				printf( "color_min -> %s\n", value );

				COLOR_MIN_VALUE = (float) atof( value );

				COLOR_MAP_INITD = stvTrue;
			}

			else if ( !strcmp( key, "color_max" ) )
			{
				printf( "color_max -> %s\n", value );

				COLOR_MAX_VALUE = (float) atof( value );

				COLOR_MAP_INITD = stvTrue;
			}

			else if ( !strcmp( key, "color_mode" ) )
			{
				printf( "color_mode -> %s\n", value );

				if ( !strcmp( value, "auto" ) )
				{
					AUTO_COLOR_MAP = stvTrue;

					LAZY_COLOR_MAP = stvFalse;
				}

				else if ( !strcmp( value, "lazy" ) )
				{
					LAZY_COLOR_MAP = stvTrue;

					AUTO_COLOR_MAP = stvFalse;
				}

				else if ( !strcmp( value, "manual" ) )
				{
					AUTO_COLOR_MAP = stvFalse;

					LAZY_COLOR_MAP = stvFalse;
				}

				else
				{
					printf(
				"Error in %s:  Unknown Color Mode \"%s\" - Ignored.\n",
						"handle_cgi_input()", value );
				}
			}

			else
			{
				printf( "Error in %s:  Unknown Key \"%s\" - Ignored.\n",
					"handle_cgi_input()", key );
			}
		}
	}
	while ( !done );

	/* Check View Region */

	if ( view_changed && region_check )
	{
		for ( i=0 ; i < dim ; i++ )
		{
			/* Min */

			if ( STV_REGION_MIN( VISREGION->region, i ) < glb[i] )
				STV_REGION_MIN( VISREGION->region, i ) = glb[i];
			else if ( STV_REGION_MIN( VISREGION->region, i ) > gub[i] )
				STV_REGION_MIN( VISREGION->region, i ) = gub[i];

			/* Max */

			if ( STV_REGION_MAX( VISREGION->region, i )
					< STV_REGION_MIN( VISREGION->region, i ) )
				STV_REGION_MAX( VISREGION->region, i ) =
					STV_REGION_MIN( VISREGION->region, i );
			else if ( STV_REGION_MAX( VISREGION->region, i ) > gub[i] )
				STV_REGION_MAX( VISREGION->region, i ) = gub[i];

			if ( F->decomp != NULL )
			{
				if ( STV_CELL_OF_DIM( VISREGION->cell, i ) < 1 )
					STV_CELL_OF_DIM( VISREGION->cell, i ) = 1;
			}
		}

		if ( F->particle != NULL )
		{
			if ( STV_CELL_PARTICLE( VISREGION->cell ) < 1 )
				STV_CELL_PARTICLE( VISREGION->cell ) = 1;
		}
	}

	/* Change Data Field */

	if ( data_changed )
	{
		/* Make Sure New Data Field is Valid */
		/* (Shit, Better Be!  We Made the CGI Select List!) */

		if ( !select_view_fields( &fieldnames, &nfields ) )
		{
			/* Save Revised Field Names */

			free_field_names( &FIELDNAMES, &NFIELDNAMES );

			FIELDNAMES = fieldnames;
			NFIELDNAMES = nfields;

			/* Collect Original View Field Group Parameters */

			freq = ( freq_changed ) ? newfreq : CURRENT_VFG->freq;

			/* Check for Lower Data Dim - Fix Visregion... */

			VF = stv_get_view_field_name( fieldnames[0],
				VIEWER->vfields, VIEWER->nvfields );

			if ( VF->field->decomp != NULL )
				dim = VF->field->decomp->dataDim;

			else if ( VF->field->particle != NULL )
				dim = VF->field->particle->dataDim;

			else
			{
				printf(
				"\nWarning: No Decomp or Particle for Field \"%s\"\n\n",
					VF->field->name );
				fflush( stdout );

				dim = -1;
			}

			if ( dim > 0 )
			{
				if ( dim < 3 && PLANE_CHOICE != PLANE_XY )
				{
					STV_REGION_MIN( VISREGION->region, 0 ) =
						STV_REGION_MIN( VISREGION->region, xi );
					STV_REGION_MAX( VISREGION->region, 0 ) =
						STV_REGION_MAX( VISREGION->region, xi );

					STV_REGION_MIN( VISREGION->region, 1 ) =
						STV_REGION_MIN( VISREGION->region, yi );
					STV_REGION_MAX( VISREGION->region, 1 ) =
						STV_REGION_MAX( VISREGION->region, yi );

					PLANE_CHOICE = PLANE_XY;
				}
			}

			/* Disconnect Old View Field */

			printf( "\nSending FieldHaltGroup\n" );
			fflush( stdout );

			stv_viewer_send_FieldHaltGroup( CURRENT_VFG, VIEWER );

			REALLOC = stvTrue;

			/* Reconnect New View Field */

			CURRENT_VFG = stv_viewer_submit_field_request( VIEWER,
				VISREGION, freq,
				( stvFieldRqstDefault
					| ( ( NO_SYNCH ) ? stvFieldRqstNoSynch : 0 )
					| ( ( IS_STATIC ) ? stvFieldRqstStatic : 0 ) ),
				&RESTART );
			
			if ( CURRENT_VFG == NULL )
			{
				printf( "\nError Re-Attaching to Application!\n\n" );

				return( stvFalse );
			}

			CGI_NEWDATA = stvTrue;
		}
	}

	/* Change View */

	if ( view_changed && !data_changed )
	{
		/* Update View Region */

		printf( "\nSending Vis Region\n" );
		fflush( stdout );

		if ( stv_viewer_set_VisRegion( VFG, V, VISREGION )
			!= stvStatusOk )
		{
			printf( "Couldn't Change View Region - Reverting.\n" );

			stv_copy_vis_region( VISREGION, VFG->visregion );
		}

		else
			REALLOC = stvTrue;
	}

	/* Change Frame Frequency */

	if ( freq_changed && !data_changed )
	{
		printf( "\nSending Vis Frequency\n" );
		fflush( stdout );

		stv_viewer_set_VisFrequency( VFG, V, newfreq );
	}

	/* Trigger CGI Script Reply */

	CGI_REPLY = stvTrue;

	CGI_RESULT = 1;

	if ( rbuf < 0 )
	{
		fclose( params );

		unlink( fname );

		CGI_SRC = CGI_TAG = -1;
	}

	else
	{
		/* Get Source & Tag Info */

		stv_msg_buf_info( rbuf, (int *) NULL, &CGI_TAG, &CGI_SRC );
	}

	/* Things Basically Worked...  :-) */

	return( stvTrue );
}


void
handle_cgi_reply()
{
	FILE *confirm;

	char fname[1024];
	char cmd[1024];

	int sbuf;

	/* Respond to CGI Script */

	if ( CGI_SRC != -1 && CGI_TAG != -1 )
	{
		/* Send Confirm Message Back to CGI */

		MyMP->init_msg_buf( stvDataDefault, &sbuf );

		MyMP->pack_int( sbuf, &CGI_RESULT, 1, 1 );

		MyMP->send_msg( sbuf, CGI_SRC, CGI_TAG, STV_BASECTX );
	}

	else
	{
		/* Confirm Parameter Updates to CGI */

		sprintf( fname, "%s/gifvwr.confirm.tmp", GIF_VIEWER_ROOT );

		confirm = fopen( fname, "w" );

		if ( confirm != NULL )
		{
			fprintf( confirm, "%d", CGI_RESULT );

			fclose( confirm );

			/* Copy to Final Destination */

			sprintf( cmd,
			"mv %s %s/gifvwr.confirm ; chmod og+wx %s/gifvwr.confirm",
				fname, GIF_VIEWER_ROOT, GIF_VIEWER_ROOT );

			system( cmd );
		}
	}
}

#endif


void
program_init()
{
	VIEWER = (STV_VIEWER) NULL;

	VISREGION = stv_create_vis_region();

	STEERING = stvFalse;

	NFIELDNAMES = 0;

	RECONNECT = stvFalse;

	RESTART = 0;

	NO_SYNCH = 0;

	IS_STATIC = 0;

#ifdef GIFVWR

	RADIUS_FIELD = (char *) NULL;
	COLOR_FIELD = (char *) NULL;

	COLOR_MIN_VALUE = -100.0;
	COLOR_MAX_VALUE = 100.0;

	MIN_VALUE = 0.0;
	MAX_VALUE = 0.0;

	AUTO_COLOR_MAP = stvFalse;

	LAZY_COLOR_MAP = stvTrue;

	COLOR_MAP_INITD = stvFalse;

	SHOW_MIN_MAX = stvFalse;

	PLANE_CHOICE = PLANE_XY;

	GIF_CNTR = getpid();

	USE_JAVA = stvFalse;

	CGI_NEWDATA = stvFalse;

	CGI_REPLY = stvFalse;

	CGI_SRC = CGI_TAG = -1;
	CGI_RESULT = -1;

#endif

	/* Set STV Options */

	stv_setopt( stvOptNull, stvOptFullUnpack, stvFalse );
}


void
viewer_init()
{

#ifdef GIFVWR
	int inst;
#endif

	MyMP->myid( &MYTID );

	if ( MYTID < 0 )
	{
		MyMP->perr( "Obtaining My Task ID" );

		exit( 0 );
	}

	else
	{
		printf( "Viewer Alive as TID=[ 0x%x / %d ]\n", MYTID, MYTID );
		fflush( stdout );
	}

#ifdef GIFVWR

	MyMP->join_group( GIF_VIEWER_GROUP, &inst );

	if ( inst != 0 )
		printf( "Warning: Instance %d != 0, Group \"%s\"!\n",
			inst, GIF_VIEWER_GROUP );

#endif

}


void
read_args( argc, argv )
int argc;
char **argv;
{
	char freq_str[255];

	int i, j, k;

	strcpy( APPNAME_DEFAULT, "" );
	strcpy( APPNAME, "" );

	strcpy( FIELDNAME_DEFAULT, "" );

	STATIC_DEFAULT = -1;
	SYNCH_DEFAULT = -1;
	FREQ_DEFAULT = -1;

	VISREGION_DEFAULT = stv_create_vis_region();

	VISREGION_DEFAULT_SET = 0;
	VISCELL_DEFAULT_SET = 0;

#ifdef GIFVWR

	strcpy( GIF_ROOT, GIF_VIEWER_ROOT );

	sprintf( GIF_TMPFILE, "%s/stv.gif.tmp", GIF_ROOT );

	sprintf( HTML_FILE, "%s/stv.html", GIF_ROOT );
	sprintf( HTML_TMPFILE, "%s/stv.html.tmp", GIF_ROOT );

	GIF_HEIGHT = 600;
	GIF_WIDTH = 800;

#endif

	STORAGE_ORDER = -1;

	vflag = 0;

#ifdef GIFVWR
	Dflag = 0;
	dflag = 0;
#else
	Dflag = 0;
	dflag = 1;
#endif

	stv_setopt( stvOptNull, stvOptVerbose, vflag ? STV_DEBUG_ALL : 0 );
	stv_setopt( stvOptNull, stvOptDumpAll, Dflag );
	stv_setopt( stvOptNull, stvOptDump, dflag );

	for ( i=1 ; i < argc ; i++ )
	{
		if ( argv[i][0] == '-' )
		{
			k = i + 1;

			for ( j=1 ; j < strlen( argv[i] ) ; j++ )
			{
				switch ( argv[i][j] )
				{
					case 'A':
					{
						strcpy( APPNAME_DEFAULT, argv[ k++ ] );

						break;
					}

					case 'S':
					{
						strcpy( FIELDNAME_DEFAULT, argv[ k++ ] );

						break;
					}

					case 'f':
					{
						if ( strlen( argv[k] ) < 255 )
						{
							strcpy( freq_str, argv[ k++ ] );

							if ( k < argc && argv[k][0] != '-'
									&& strlen( freq_str )
										+ strlen( argv[k] ) < 255 )
								strcat( freq_str, argv[ k++ ] );

							if ( !stv_parse_freq_str( freq_str,
									&FREQ_DEFAULT, stvTrue ) )
								printf(
							"\nError Parsing Frequency String!\n\n" );
						}

						else
							printf(
							"\nError: Frequency Arg Overflow!\n\n" );

						break;
					}

					case 's':
					{
						SYNCH_DEFAULT = atoi( argv[ k++ ] );

						break;
					}

					case 't':
					{
						STATIC_DEFAULT = atoi( argv[ k++ ] );

						break;
					}

					case 'V':
					{
						STV_REGION_MIN( VISREGION_DEFAULT->region, 0 ) =
							atoi( argv[ k++ ] );

						STV_REGION_MAX( VISREGION_DEFAULT->region, 0 ) =
							atoi( argv[ k++ ] );

						STV_REGION_MIN( VISREGION_DEFAULT->region, 1 ) =
							atoi( argv[ k++ ] );

						STV_REGION_MAX( VISREGION_DEFAULT->region, 1 ) =
							atoi( argv[ k++ ] );

						STV_REGION_MIN( VISREGION_DEFAULT->region, 2 ) =
							atoi( argv[ k++ ] );

						STV_REGION_MAX( VISREGION_DEFAULT->region, 2 ) =
							atoi( argv[ k++ ] );

						VISREGION_DEFAULT_SET = 1;

						break;
					}

					case 'c':
					{
						STV_CELL_OF_DIM( VISREGION_DEFAULT->cell, 0 ) =
							atoi( argv[ k++ ] );

						STV_CELL_OF_DIM( VISREGION_DEFAULT->cell, 1 ) =
							atoi( argv[ k++ ] );

						STV_CELL_OF_DIM( VISREGION_DEFAULT->cell, 2 ) =
							atoi( argv[ k++ ] );

						STV_CELL_PARTICLE( VISREGION_DEFAULT->cell ) =
							atoi( argv[ k++ ] );

						VISCELL_DEFAULT_SET = 1;

						break;
					}

#ifdef GIFVWR
					case 'G':
					{
						strcpy( GIF_ROOT, argv[ k++ ] );

						sprintf( GIF_TMPFILE, "%s/stv.gif.tmp",
							GIF_ROOT );

						sprintf( HTML_FILE, "%s/stv.html", GIF_ROOT );
						sprintf( HTML_TMPFILE, "%s/stv.html.tmp",
							GIF_ROOT );

						break;
					}

					case 'M':
					{
						if ( !strcmp( argv[k], "auto" ) )
						{
							AUTO_COLOR_MAP = stvTrue;

							LAZY_COLOR_MAP = stvFalse;

							k++;
						}

						else if ( !strcmp( argv[k], "lazy" ) )
						{
							LAZY_COLOR_MAP = stvTrue;

							AUTO_COLOR_MAP = stvFalse;

							k++;
						}

						else
						{
							COLOR_MIN_VALUE = atof( argv[ k++ ] );
							COLOR_MAX_VALUE = atof( argv[ k++ ] );

							AUTO_COLOR_MAP = stvFalse;

							LAZY_COLOR_MAP = stvFalse;

							COLOR_MAP_INITD = stvTrue;
						}

						break;
					}

					case 'X':
					{
						SHOW_MIN_MAX = stvTrue;

						break;
					}

					case 'H':
					{
						GIF_HEIGHT = atoi( argv[ k++ ] );

						break;
					}

					case 'W':
					{
						GIF_WIDTH = atoi( argv[ k++ ] );

						break;
					}

					case 'P':
					{
						if ( !strcmp( argv[k], "xy" ) )
							PLANE_CHOICE = PLANE_XY;
						else if ( !strcmp( argv[k], "yz" ) )
							PLANE_CHOICE = PLANE_YZ;
						else if ( !strcmp( argv[k], "xz" ) )
							PLANE_CHOICE = PLANE_XZ;
						else
							usage();

						k++;

						break;
					}

					case 'J':
					{
						USE_JAVA = stvTrue;

						break;
					}
#endif

					case 'C':
					{
						STORAGE_ORDER = stvRowMajor;

						break;
					}

					case 'F':
					{
						STORAGE_ORDER = stvColumnMajor;

						break;
					}

					case 'R':
					{
						RECONNECT = stvTrue;

						break;
					}

					case 'D':
					{
						Dflag = 1 - Dflag;
						
						stv_setopt( stvOptNull, stvOptDumpAll, Dflag );

						break;
					}

					case 'd':
					{
						dflag = 1 - dflag;
						
						stv_setopt( stvOptNull, stvOptDump, dflag );

						break;
					}

					case 'v':
					{
						vflag++;
						
						stv_setopt( stvOptNull, stvOptVerbose,
							vflag ? STV_DEBUG_ALL : 0 );

						break;
					}

					case 'h':
					{
						usage();

						break;
					}

					default:
					{
						usage();

						break;
					}
				}
			}

			i = k - 1;
		}
	}
}


void
usage()
{
	printf( "\nusage:  viewer [-A appname] [-S field]\n" );
	printf( "\t[-f freq [units]] [-s no_synch] [-t is_static]\n" );
	printf( "\t[-V xmin xmax ymin ymax zmin zmax]\n" );
	printf( "\t[-c xcell ycell zcell pcell]\n" );

#ifdef GIFVWR
	printf( "\t[-G gif_file] [-M auto|lazy|min max] [-S]\n" );
	printf( "\t[-H height] [-W width] [-P xy|yz|xz] [-J]\n" );
#else
	printf( "\t[-CFRDdv]\n" );
#endif

	printf( "\nwhere:\n" );

	printf( "\t-A appname = connect to application \"appname\".\n" );
	printf( "\t-S field = select data array \"field\".\n" );

	printf( "\t-f freq [units] = set default frame frequency\n" );
	printf( "\t\tto \"freq\" [with optional time \"units\"].\n" );
	printf( "\t-s no_synch = set default value for no_synch flag.\n" );
	printf(
		"\t-t is_static = set default value for is_static flag.\n" );

	printf(
		"\t-V xmin xmax ... = set default vis region for x, y, z.\n" );
	printf(
		"\t-c xcell ycell ... = set default cell size for x, y, z\n" );
	printf( "\t\t and particles.\n" );

#ifdef GIFVWR
	printf( "\t-G gif_root = file root to dump GIF image frames.\n" );
	printf( "\t-M auto = set auto color range for rendering.\n" );
	printf( "\t-M lazy = set lazy color range for rendering.\n" );
	printf( "\t-M min max = set manual color range to [min,max].\n" );
	printf( "\t-S = show data min and max values.\n" );
	printf( "\t-H height = height of GIF image to dump.\n" );
	printf( "\t-W width = width of GIF image to dump.\n" );
	printf( "\t-P xy|yz|xz = select slice plane for GIF image.\n" );
	printf( "\t-J = don't use Java for viewing, just the GIF.\n" );
#endif

	printf( "\t-C = use C data storage order, Row Major.\n" );
	printf( "\t-F = use Fortran data storage order, Column Major.\n" );
	printf( "\t-R = Reconnect on Application Failure / Recovery.\n" );
	printf( "\t-D = toggle major update dumping (default is %s).\n",
			Dflag ? "on" : "off" );
	printf( "\t-d = toggle final data dumping (default is %s).\n",
			dflag ? "on" : "off" );
	printf( "\t-v = run in verbose mode.\n" );

	printf( "\n" );

	exit( 0 );
}


void
cleanup()
{
	/* Reset STV Globals */

	stv_viewer_reset( VIEWER );

	stv_init_vis_region( VISREGION );
}


void
read_name( str )
char *str;
{
	while ( (*str = getchar()) != '\n' )
		str++;

	*str = '\0';
}


void
NoDelay()
{
	FLAGS = fcntl( 0, F_GETFL, 0 );

	fcntl( 0, F_SETFL, FLAGS | O_NDELAY );
}


void
Delay()
{
	fcntl( 0, F_SETFL, FLAGS );
}

