      Program FExpt
C
C $Id: fexpt.F,v 1.4 2000/10/16 20:41:39 bernhold Exp $
C
C                         CUMULVS version 1.1:
C          Computational Steering and Interactive Visualization
C                       in Distributed Applications
C               Oak Ridge National Laboratory, Oak Ridge TN.
C           Authors:  James Arthur Kohl, Philip M. Papadopoulos
C                          and David E. Bernholdt
C                       (C) 2000 All Rights Reserved
C
C                                  NOTICE
C
C     Permission to use, copy, modify, and distribute this software and
C     its documentation for any purpose and without fee is hereby granted
C     provided that the above copyright notice appear in all copies and
C     that both the copyright notice and this permission notice appear
C     in supporting documentation.
C     
C     Neither the Institution, Oak Ridge National Laboratory, nor the
C     Authors make any representations about the suitability of this
C     software for any purpose.  This software is provided ``as is''
C     without express or implied warranty.
C
C     CUMULVS was funded by the U.S. Department of Energy.
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C     A simple program to demonstrate interfacing an application
C     using the Global Array Toolkit with the CUMULVS visualization
C     package.
C
C     Application name (for CUMULVS) is "stv-ga-expt"
C
C     Creates a small 2-d GA "A" and a 3-d GA "B", fills them with
C     "interesting" values, and does sendToFE calls ad nauseum.
C
C     "Interesting" is defined as follows:
C     1) The integral part is the number of the processor which owns 
C        that element (GA_NodeID)
C     2) The fractional part gives the row/column coordinates of the
C        element, two digits per dimension.
C     So elements look like: p.aabbcc where
C     p is the processor
C     aa is the row (fastest index) dimension
C     bb is the next dimension
C     cc is the last (slowest) dimension
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
      Implicit None
C
#include "global.fh"
#include "mafdecls.fh"
#include "fpvm3.h"
#include "fstv.h"
C
C
      Integer Me, NProc, TID, Info, I, J, K
      Integer Type, NDim, dims(GA_MAX_DIM), chunk(GA_MAX_DIM)
      Integer lo(GA_MAX_DIM), hi(GA_MAX_DIM), elem(GA_MAX_DIM)
      Integer ld(GA_MAX_DIM)
      Integer ga_a, ga_b
      Integer FieldIDa, FieldIDb
      Double precision Val
C
C     Initialize memory allocator used by GA
C
      If ( .NOT. MA_Init( MT_Dbl, 10*10*20*30, 10*10*20*30) ) then
         Write (6, *) 'ma_init failed'
         Call Exit
      EndIf
C
C     Initialize TCGMSG message passing (for GA) and GA itself
C
      Call PBeginF()
      Call GA_Initialize()
C
      Me = GA_NodeID()
      NProc = GA_NNodes()
C
C     Initialize PVM (for CUMULVS) and CUMULVS itself
C
      Call PVMfMyTID( TID )
      Call STVfInit('stv-ga-expt', 42, NProc, Me, Info)
      Write (6, *) "CUMULVS initialized on node ", me,
     $     " with return code", Info
C
C     Create two and three dimensional arrays to give people something
C     to play with
C
      Do I = 1, GA_MAX_DIM
         Dims(i) = 10*i
         Chunk(i) = 0
      EndDo
C
      If ( .NOT. NGA_Create(MT_DBL, 2, Dims, 'A', Chunk, ga_a) )
     $     Call GA_Error('unable to create 2d array A')
      If ( .NOT. NGA_Create(MT_DBL, 3, Dims, 'B', Chunk, ga_b) )
     $     Call GA_Error('unable to create 3d array B')
C
C     Admire our handiwork
C
      Call GA_Summarize(.TRUE.)
C
C     Fill them with interesting values
C
C     1) Integral part should be owning node
C
      Call NGA_Inquire(ga_a, Type, NDim, Dims)
      Do I = 1, NProc
         Call NGA_Distribution(ga_a, i, lo, hi)
         Val = dble(i)
         Call NGA_Fill_Patch(ga_a, lo, hi, Val )
      EndDo
C
      Call NGA_Inquire(ga_b, Type, NDim, Dims)
      Do I = 1, NProc
         Call NGA_Distribution(ga_b, i, lo, hi)
         Val = dble(i)
         Call NGA_Fill_Patch(ga_b, lo, hi, Val )
      EndDo
C
C     2) Fractional part identifies coordinates of element
C
      Do i = 1, GA_MAX_DIM
         ld(i) = 1
      EndDo
C      
      Call NGA_Distribution(ga_a, me, lo, hi)
      Do j = lo(2), hi(2)
         Do i = lo(1), hi(1)
            Elem(1) = i
            Elem(2) = j
            Val = 0.01*i + 0.0001*j
            Call NGA_Acc(ga_a, Elem, Elem, Val, ld, 1.0d0)
         EndDo
      EndDo
C
      Call NGA_Distribution(ga_b, me, lo, hi)
      Do k = lo(3), hi(3)
         Do j = lo(2), hi(2)
            Do i = lo(1), hi(1)
               Elem(1) = i
               Elem(2) = j
               Elem(3) = k
               Val = 0.01*i + 0.0001*j + 0.000001*k
               Call NGA_Acc(ga_b, Elem, Elem, Val, ld, 1.0d0)
            EndDO
         EndDo
      EndDo
C
C     Admire our handiwork
C
      Call GA_Print(ga_a)
      Call GA_Print(ga_b)
C
C     Register GAs with CUMULVS
C
      Call STVfGADefine(ga_a, FieldIDa)
      Write (6, *) Me, ' defined array A, got fieldID ', FieldIDa
      Call STVfGADefine(ga_b, FieldIDb)
      Write (6, *) Me, ' defined array B, got fieldID ', FieldIDb
C
C     Though these arrays are static, we'll just loop forever as if
C     we were sending new data to CUMULVS periodically.
C
 1000 Continue
      Call STVfSendToFE( Info )
      Write (6, *) Me, ' sendToFE returned ', Info
C
      Call Sleep(10)
C
      Goto 1000
C
C     We never actually reach this point, but in principle this is 
C     how it ends.  No formal termination for CUMULVS.
C
      Call GA_Terminate()
      Call PEnd()
      Call PVMfExit(Info)
      Stop
      End
