
/* $Id: stv_type.h,v 4.19 2003/09/05 21:28:32 kohl Exp $ */


/*
 *                      CUMULVS version 1.0:
 *      Computational Steering and Interactive Visualization
 *                   in Distributed Applications
 *           Oak Ridge National Laboratory, Oak Ridge TN.
 *      Authors:  James Arthur Kohl and Philip M. Papadopoulos
 *                   (C) 1996 All Rights Reserved
 *
 *                              NOTICE
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted
 * provided that the above copyright notice appear in all copies and
 * that both the copyright notice and this permission notice appear
 * in supporting documentation.
 *
 * Neither the Institution, Oak Ridge National Laboratory, nor the
 * Authors make any representations about the suitability of this
 * software for any purpose.  This software is provided ``as is''
 * without express or implied warranty.
 *
 * CUMULVS was funded by the U.S. Department of Energy.
 */


#ifndef _STV_TYPE_H_

#define _STV_TYPE_H_


#ifdef __cplusplus
extern "C" {
#endif


/* CUMULVS System Data Types */


/* Encapsulated Message-Passing Interface Structure */
/* (Don't ask...  this is a living nightmare...) */

struct stv_mp_api_struct
{
	char *name;

	int (*myid) __STV_Prototype__(( int * ));
	int (*exit) __STV_Prototype__(( void ));

	int (*create_context) __STV_Prototype__(( int * ));
	int (*free_context) __STV_Prototype__(( int * ));
	int (*get_context) __STV_Prototype__(( int * ));
	int (*set_context) __STV_Prototype__(( int, int * ));

	int (*init_msg_buf) __STV_Prototype__(( int, int * ));
	int (*msg_buf_info) __STV_Prototype__(( int, int *, int *, int * ));
	int (*get_sbuf) __STV_Prototype__(( int * ));
	int (*set_sbuf) __STV_Prototype__(( int ));
	int (*get_rbuf) __STV_Prototype__(( int * ));
	int (*set_rbuf) __STV_Prototype__(( int ));
	int (*free_msg_buf) __STV_Prototype__(( int ));

	int (*pack_byte) __STV_Prototype__(( int, void *, int, int ));
	int (*pack_int) __STV_Prototype__(( int, void *, int, int ));
	int (*pack_float) __STV_Prototype__(( int, void *, int, int ));
	int (*pack_double) __STV_Prototype__(( int, void *, int, int ));
	int (*pack_complex) __STV_Prototype__(( int, void *, int, int ));
	int (*pack_dcomplex) __STV_Prototype__(( int, void *, int, int ));
	int (*pack_short) __STV_Prototype__(( int, void *, int, int ));
	int (*pack_ushort) __STV_Prototype__(( int, void *, int, int ));
	int (*pack_uint) __STV_Prototype__(( int, void *, int, int ));
	int (*pack_long) __STV_Prototype__(( int, void *, int, int ));
	int (*pack_ulong) __STV_Prototype__(( int, void *, int, int ));
	int (*pack_str) __STV_Prototype__(( int, char * ));
	int (*pack_context) __STV_Prototype__(( int, int * ));

	int (*unpack_byte) __STV_Prototype__(( int, void *, int, int ));
	int (*unpack_int) __STV_Prototype__(( int, void *, int, int ));
	int (*unpack_float) __STV_Prototype__(( int, void *, int, int ));
	int (*unpack_double) __STV_Prototype__(( int, void *, int, int ));
	int (*unpack_complex) __STV_Prototype__(( int, void *, int, int ));
	int (*unpack_dcomplex) __STV_Prototype__(( int, void *, int, int ));
	int (*unpack_short) __STV_Prototype__(( int, void *, int, int ));
	int (*unpack_ushort) __STV_Prototype__(( int, void *, int, int ));
	int (*unpack_uint) __STV_Prototype__(( int, void *, int, int ));
	int (*unpack_long) __STV_Prototype__(( int, void *, int, int ));
	int (*unpack_ulong) __STV_Prototype__(( int, void *, int, int ));
	int (*unpack_str) __STV_Prototype__(( int, char * ));
	int (*unpack_context) __STV_Prototype__(( int, int * ));

	int (*join_group) __STV_Prototype__ (( char *, int * ));
	int (*leave_group) __STV_Prototype__ (( char * ));
	int (*group_size) __STV_Prototype__ (( char *, int * ));
	int (*get_id_from_inst) __STV_Prototype__ (( char *, int, int * ));
	int (*get_inst_from_id) __STV_Prototype__ (( char *, int, int * ));
	int (*get_group_tids) __STV_Prototype__ (( char *, int **, int * ));

	int (*send_msg) __STV_Prototype__(( int, int, int, int ));
	int (*mcast_msg) __STV_Prototype__(( int, int *, int, int, int ));
	int (*bcast_msg) __STV_Prototype__(( int, char *, int, int ));

	int (*recv_msg) __STV_Prototype__(( int, int, int, int * ));
	int (*recv_msg_noblock) __STV_Prototype__(( int, int, int, int * ));
	int (*recv_msg_timeout) __STV_Prototype__(( int, int, int,
			struct timeval *, int * ));

	int (*perr) __STV_Prototype__(( char * ));
};

typedef struct stv_mp_api_struct STV_MP_API;


/* (Integrated) Visualization Region Struct Declaration */

typedef int STV_REGION[2 * STV_MAX_DIM];

typedef int STV_CELL[ STV_MAX_DIM + 1 ];

struct stv_vis_region_struct
{
	STV_REGION region;
	STV_CELL cell;
	/* STV_VIS_REGION */ struct stv_vis_region_struct *next;
};

typedef struct stv_vis_region_struct *STV_VIS_REGION;


/* Region Macros */

#define STV_REGION_MIN( _region, _dim ) ((_region)[ 2 * (_dim) ])
#define STV_REGION_MAX( _region, _dim ) ((_region)[ (2 * (_dim)) + 1 ])

/* Number of (Integer) Elements to Pack for Given Dim */
#define STV_REGION_SIZE( _dim )    ( 2 * ( _dim ) )

#define STV_XMIN( _region )    ( (_region)[0] )
#define STV_XMAX( _region )    ( (_region)[1] )
#define STV_YMIN( _region )    ( (_region)[2] )
#define STV_YMAX( _region )    ( (_region)[3] )
#define STV_ZMIN( _region )    ( (_region)[4] )
#define STV_ZMAX( _region )    ( (_region)[5] )


/* Cell Macros */

#define STV_CELL_OF_DIM( _cell, _dim ) ( (_cell)[_dim] )

/* Number of (Integer) Elements to Pack for Given Dim */
/* (better always be called with STV_MAX_DIM, */
/*   else particle cell is gone!) */
#define STV_CELL_SIZE( _dim )   ( _dim + 1 )

#define STV_CELLX( _cell )    ( (_cell)[0] )
#define STV_CELLY( _cell )    ( (_cell)[1] )
#define STV_CELLZ( _cell )    ( (_cell)[2] )

#define STV_CELL_PARTICLE( _cell )    ( (_cell)[ STV_MAX_DIM ] )


/* User Provided Particle Access Routines Types */

typedef void *STV_PARTICLE_ID;

typedef void *STV_VALUE;

typedef void (*STV_GET_PARTICLE) __STV_Prototype__(( int,
		STV_REGION, STV_VALUE, STV_PARTICLE_ID *, int * ));

typedef void (*STV_MAKE_PARTICLE) __STV_Prototype__(( int *,
		STV_VALUE, STV_PARTICLE_ID * ));

typedef void (*STV_GET_PFIELD) __STV_Prototype__((
		STV_PARTICLE_ID, STV_VALUE, STV_VALUE *, int * ));

typedef void (*STV_SET_PFIELD) __STV_Prototype__((
		STV_PARTICLE_ID, STV_VALUE, STV_VALUE, int ));


/* User Provided Data Callback Routine Type */

typedef void (*STV_DATA_CALLBACK) __STV_Prototype__(( int, int *,
		STV_VALUE ));


/* Decomp Data Field Packing Structures / Types */


/* SubRegion Packing Index Structure */

struct stv_pack_index_struct
{
	int gstart;
	int gstride;
	int lstart;
	int lstride;
	int max;
	/* STV_PACK_INDEX */ struct stv_pack_index_struct *next;
};

typedef struct stv_pack_index_struct *STV_PACK_INDEX;


/* SubRegion Packing Dimension Structure */

struct stv_pack_dim_struct
{
	STV_PACK_INDEX pindices;
	STV_PACK_INDEX current;
	STV_VALUE dataPtr;
	int gindex;
	int lindex;
	int type;
	int dim;
	/* STV_PACK_DIM */ struct stv_pack_dim_struct *next_dim;
	/* STV_PACK_DIM */ struct stv_pack_dim_struct *next;
};

typedef struct stv_pack_dim_struct *STV_PACK_DIM;


/* Main Decomp & Data Field Structures / Types */


/* Data Decompositions */

struct stv_decomp_struct
{
	int decompID;		/* ID of this field decomposition            */
	int dataDim;		/* dimension of the data to be distributed   */
	int *axisType;		/* How each axis is distributed:             */
						/*     ( stvBlock, stvCyclic, stvExplicit,   */
						/*          or stvCollapse )                 */
	int *axisInfo;		/* detail information for each dist type     */
	int **axisLBounds;	/* separate index bounds storage for         */
	int **axisUBounds;	/*     multi-explicit axes                   */
	int *saveLBounds[STV_MAX_DIM];	/* for AddExplicitBounds...      */
	int *saveUBounds[STV_MAX_DIM];	/* for AddExplicitBounds...      */
	int axisCounts[STV_MAX_DIM];	/* count # of bounds/axis...     */
	int nbounds;		/* # of sets of bounds (multi-explicit axes) */
	int *glb;			/* global lower bound of comp-region         */
	int *gub;			/* global upper bound of comp-region         */
	int prank;			/* number of processors used in this decomp  */
	int *pshape;		/* how is the grid defined                   */
	int ntasks;			/* total # of tasks for this distribution    */
	int refcount;		/* how many are reffing this field */
	/* STV_DECOMPPTR */ struct stv_decomp_struct *prev;
	/* STV_DECOMPPTR */ struct stv_decomp_struct *next;
};

typedef struct stv_decomp_struct STV_DECOMP;
typedef struct stv_decomp_struct *STV_DECOMPPTR;


/* Particle Definition */

struct stv_particle_struct
{
	int particleID;				/* ID of this particle                */
	char *name;					/* name of particle                   */
	STV_GET_PARTICLE get_particle;
								/* function to obtain particle by num */
	STV_VALUE get_particle_arg;	/* user arg to get_particle() */
	STV_MAKE_PARTICLE make_particle;
								/* function to make new particle      */
	STV_VALUE make_particle_arg;
								/* user arg to make_particle() */
	int dataDim;				/* dimension of the particle's space  */
	int *glb;					/* global lower bound of comp-region  */
	int *gub;					/* global upper bound of comp-region  */
	int *lbounds;				/* lower bound this task's subregion  */
	int *ubounds;				/* upper bound this task's subregion  */
	int ntasks;					/* total # of tasks for this particle */
	unsigned offset;			/* checkpointing reconfig file offset */
	int coupler_cnt;			/* counter - unique coupler IDs per
									particle */
	int refcount;   			/* how many are reffing this field    */
	/* STV_PARTICLEPTR */ struct stv_particle_struct *prev;
	/* STV_PARTICLEPTR */ struct stv_particle_struct *next;
};

typedef struct stv_particle_struct STV_PARTICLE;
typedef struct stv_particle_struct *STV_PARTICLEPTR;


/* Named Fields, Space for Pack Buffer */

struct stv_field_struct
{
	int fieldID;			/* id of the field */
	char *name;				/* name of field */

	int decompID;			/* ID for how this field decomposed */
	STV_DECOMPPTR decomp;	/* decomposition (for contiguous fields) */

	int particleID;			/* ID for this field's particle */
	STV_PARTICLEPTR particle;	/* particle (for particle fields) */
	STV_GET_PFIELD get_pfield;	/* funct to get particle field value */
	STV_VALUE get_pfield_arg;	/* user arg to get_pfield() */
	STV_SET_PFIELD set_pfield;	/* funct to set particle field value */
	STV_VALUE set_pfield_arg;	/* user arg to set_pfield() */
	int nelems;					/* # of elems in particle field array */

	int type;				/* what is the actual type of the data */ 

	STV_DATA_CALLBACK get_data;	/* callback routine to service data
									requests for viewers... */

	STV_VALUE *dataPtrs;	/* pointer(s) to the actual variable(s) */
	int **arrayOffset;		/* offset(s) to first valid data value */
	int **arrayDecl;		/* declared storage sizes (per dataPtr) */
	int numData;			/* number of data pointers */

	int storage_order;		/* data storage order: Row/Column Major */
	int time_scale;			/* time scale value, iteration period */
	int time_base;			/* base time value for field frames */
	int time_units;			/* time units for base time value */

	int data_ready;			/* flag to indicate whether data is ready
								for sending / viewing at front end */

	int *paddr;				/* local address in processor topology */

	int aflag;				/* stvCpOnly -- Checkpoint only,
								stvVisOnly - Vis only,
								stvVisCp -  do both */

	unsigned offset;		/* checkpointing reconfig file offset */

	int coupler_cnt;		/* counter - unique coupler IDs per field */

	int refcount;           /* how many are reffing this field */

	struct stv_viewer_list_struct *vlist; /* list of viewers attached */
										  /* to this field */

	/* STV_FIELDPTR */ struct stv_field_struct *prev;
	/* STV_FIELDPTR */ struct stv_field_struct *next;
};

typedef struct stv_field_struct STV_FIELD;
typedef struct stv_field_struct *STV_FIELDPTR;


/* Field Group to Allow Multiple Fields Per Request / Viewer */

struct stv_field_group_struct
{
	int fgid;					/* field group ID provided by viewer */
	STV_FIELDPTR *field_list;	/* fields that we send to the viewer */
	STV_PACK_DIM **field_pdims;	/* latest pack dimensions for field */
	int *field_types;			/* requested data types for fields */
	int *field_storage_orders;	/* requested data storage orders */
	int *field_ids;				/* copy of viewer field IDs for group */
	int nfields;
	int vRegionId;				/* vis Region Id supplied by FE */
	int vState;			 		/* state of viewer for this group */
	STV_VIS_REGION visRegion;	/* requested vis regions / cell sizes */
	int need_descriptor;		/* flag to trigger descriptor info */
	int OKtoSend;				/* Flow control flag */
	int NeedSend;				/* Flow control assist, for app lock */
	int toffset;				/* offset from field timestamp */
	int eff_freq;				/* simple iteration-based frame freq */
	int freq;					/* (time) frequency to view the field */
	int newEffFreq;				/* New iteration-based frame freq */
	int newFreq;				/* New (time) frequency */
	int nfTimeStamp;			/* timestamp when new freq effective */
	int no_synch;				/* no_synch flag, not used */
	int is_static;				/* static flag, for static vis region */
	/* STV_FIELD_GROUP_PTR */ struct stv_field_group_struct *prev;
	/* STV_FIELD_GROUP_PTR */ struct stv_field_group_struct *next;
};

typedef struct stv_field_group_struct STV_FIELD_GROUP;
typedef struct stv_field_group_struct *STV_FIELD_GROUP_PTR;


/* Field Map from local fields to front end fields */

struct  stv_field_map_struct
{
        int remoteID;      /* id as on the viewer */
        STV_FIELDPTR local;  /* ptr to local information */
        /* STV_FIELD_MAP_PTR */ struct stv_field_map_struct *prev; 
        /* STV_FIELD_MAP_PTR */ struct stv_field_map_struct *next; 
};        

typedef struct stv_field_map_struct STV_FIELD_MAP;
typedef struct stv_field_map_struct *STV_FIELD_MAP_PTR;


/* Named Parameters */

struct stv_param_struct
{
	int paramID;			/* id of this param */
	char *name;
	char **pnames;
	STV_VALUE *vars;
	int *types;
	int num;
	int aflag;				/* visOnly, cpOnly, both marker */
	int refcount;                   /* how many are reffing this */
	int updated;			/* is this parameter updated */
	/* STV_PARAMPTR */ struct stv_param_struct *prev;
	/* STV_PARAMPTR */ struct stv_param_struct *next; 
};

typedef struct stv_param_struct STV_PARAM;
typedef struct stv_param_struct *STV_PARAMPTR;


/* Value Lists */

struct stv_value_list_struct
{
	STV_VALUE *values;
	int num;
	int tvalid;        /* at what timestep parameter is valid */
	int update_fgid;	/* fgid for new params update */
	/* STV_VALUE_LIST */ struct stv_value_list_struct *next;
};

typedef struct stv_value_list_struct *STV_VALUE_LIST;


/* Parameter Map from local parameters to front end parameters */

struct stv_param_map_struct
{
	int remoteID;      /* id as on the viewer */
	STV_PARAMPTR local;  /* ptr to local information */
	STV_VALUE_LIST newVals;  /* temp storage for new data */   
	/* STV_PARAM_MAP_PTR */ struct stv_param_map_struct *prev; 
	/* STV_PARAM_MAP_PTR */ struct stv_param_map_struct *next; 
};        

typedef struct stv_param_map_struct STV_PARAM_MAP;
typedef struct stv_param_map_struct *STV_PARAM_MAP_PTR;


/* Linked-List of Viewers */

struct stv_fe_viewer_struct
{
	int vtid;							/* task id of viewer */
	STV_MP_API *mp;						/* messaging system to viewer */
	int vid;							/* viewer id */
	int nfgs;  							/* number of field groups */
	STV_FIELD_GROUP_PTR fg_list;		/* field group list */
	STV_FIELD_MAP_PTR field_map;
	STV_PARAM_MAP_PTR param_map;
	int nNewParams;
	int refcount;						/* can free this viewer? */
	/* STV_FE_VIEWERPTR */ struct stv_fe_viewer_struct *prev;
	/* STV_FE_VIEWERPTR */ struct stv_fe_viewer_struct *next;
};

typedef struct stv_fe_viewer_struct STV_FE_VIEWER;
typedef struct stv_fe_viewer_struct *STV_FE_VIEWERPTR;


/* A list of viewers that are connected to a single field */

struct stv_viewer_list_struct
{
	STV_FE_VIEWERPTR viewer;
	/* STV_VLISTPTR */ struct stv_viewer_list_struct *prev;
	/* STV_VLISTPTR */ struct stv_viewer_list_struct *next;
};

typedef struct stv_viewer_list_struct STV_VLIST;
typedef struct stv_viewer_list_struct *STV_VLISTPTR;


/* Structure to Keep Track of Received */
/* But Partially Unpacked Messages     */

struct stv_mbuffer_struct
{
     int bufid;
     int fgid;
	 int op;
     int src;
     int tag;
	 STV_MP_API *mp;
	 STV_CONTEXT ctxt;
     /* STV_BUFPTR */ struct stv_mbuffer_struct *prev;
     /* STV_BUFPTR */ struct stv_mbuffer_struct *next;
};

typedef struct stv_mbuffer_struct STV_BUFFER;
typedef struct stv_mbuffer_struct *STV_BUFPTR;


/* Structure to Store Various Message Handlers / Sub-Handlers */

typedef int (*STV_HANDLER_PROC) __STV_Prototype__(( STV_BUFPTR ));

struct stv_handler_struct
{
	STV_HANDLER_PROC handler;
	int op;
	int src;
	int tag;
	STV_CONTEXT ctxt;
	/* STV_HANDLER */ struct stv_handler_struct *next;
};

typedef struct stv_handler_struct *STV_HANDLER;


/* Structure to Register High-Priority Messages */

struct stv_msg_pass_thru_struct
{
	int op;
	/* STV_MSG_PASS_THRU */ struct stv_msg_pass_thru_struct *next;
};

typedef struct stv_msg_pass_thru_struct *STV_MSG_PASS_THRU;


/* Host Structure */

struct stv_host_struct
{
	char name[128];
	char arch[20];
	int tid;
};

typedef struct stv_host_struct STV_HOST;
typedef struct stv_host_struct *STV_HOSTPTR;


/* Viewer Structures */


/* View Frame Descriptor Structures */

struct stv_view_desc_index_struct
{
	int indices[STV_MAX_DIM];
	int stride;
	int first;
	int num;
	/* STV_VIEW_DESC_INDEX */ struct stv_view_desc_index_struct *next;
};

typedef struct stv_view_desc_index_struct *STV_VIEW_DESC_INDEX;

struct stv_view_descriptor_struct
{
	int need_descriptor;
	int type;
	int field_storage_order;
	int dim;
	STV_VIS_REGION VRck;
	STV_VIEW_DESC_INDEX dindices;
	/* STV_VIEW_DESCRIPTOR */ struct stv_view_descriptor_struct *next;
};

typedef struct stv_view_descriptor_struct *STV_VIEW_DESCRIPTOR;


/* Viewer Field Bookkeeping Structure */

struct stv_view_field_struct
{
	STV_FIELDPTR field;
	int view_type;
	int view_storage_order;
	int selected;
	STV_VIEW_DESCRIPTOR *descriptors;
	int *tids;
	int ntids;
	/* STV_VIEW_FIELD */ struct stv_view_field_struct *next;
};

typedef struct stv_view_field_struct *STV_VIEW_FIELD;


/* Viewer Particle Field Data Structure */

struct stv_view_particle_data_struct
{
	int *coords;
	STV_VALUE *field_values;
	int *field_nelems;
	/* STV_VIEW_PARTICLE_DATA */
		struct stv_view_particle_data_struct *next;
};

typedef struct stv_view_particle_data_struct *STV_VIEW_PARTICLE_DATA;


/* Viewer Particle Structure */

struct stv_view_particle_struct
{
	STV_PARTICLEPTR particle;
	STV_VIEW_FIELD *view_field_list;
	int *view_types;
	int nvfields;
	STV_VIEW_PARTICLE_DATA pdata;
	int npdata;
	/* STV_VIEW_PARTICLE */ struct stv_view_particle_struct *next;
};

typedef struct stv_view_particle_struct *STV_VIEW_PARTICLE;


/* Viewer Field Group Bookkeeping Structure */

struct stv_view_field_group_struct
{
	int fgid;
	STV_VIEW_FIELD *view_field_list;
	int *view_types;
	int *view_storage_orders;
	STV_VALUE **field_values;
	int ***field_arrayDecls;
	int nvfields;
	STV_VIEW_PARTICLE particle_list;
	int *tid_list;
	int ntids;
	STV_VIS_REGION visregion;
	int nvisregion;
	int region_time;
	int time_offset;
	int region_cnt;
	int region_id;
	int reply_cnt;
	int frame_ok;
	int ack_cnt;
	int timemax;
	int locked;
	int eff_freq;
	int freq;
	int bail;
	int flag;
	/* STV_VIEW_FIELD_GROUP */ struct stv_view_field_group_struct *next;
};

typedef struct stv_view_field_group_struct *STV_VIEW_FIELD_GROUP;


/* Viewer Parameter Bookkeeping Structure */

struct stv_view_param_struct
{
	STV_PARAMPTR param;
	int token;
	int changed;
	int *tids;
	int ntids;
	/* STV_VIEW_PARAM */ struct stv_view_param_struct *next;
};

typedef struct stv_view_param_struct *STV_VIEW_PARAM;


/* Viewer Task Bookkeeping Structure */

struct stv_view_task_struct
{
	int tid;
	STV_FIELDPTR *fields;
	int nfields;
	STV_PARAMPTR *params;
	int nparams;
	int fieldtime;
	int ack;
	/* STV_VIEW_TASK */ struct stv_view_task_struct *next;
};

typedef struct stv_view_task_struct *STV_VIEW_TASK;


/* Primary Viewer Bookkeeping Structure */

struct stv_viewer_struct
{
	char *appname;
	STV_VIEW_FIELD_GROUP vfglist;
	int nvfgs;
	STV_VIEW_FIELD vfields;
	int nvfields;
	STV_VIEW_PARAM vparams;
	int nvparams;
	STV_VIEW_TASK vtasks;
	int nvtasks;
	STV_PARTICLEPTR particles;
	int nparticles;
	STV_DECOMPPTR decomps;
	int ndecomps;
	STV_FIELDPTR fields;
	int nfields;
	STV_PARAMPTR params;
	int nparams;
	int init_seq;
	int init_gen;
	STV_MP_API *mp;
	int msgtag;
	int ntasks;
	int mytid;
	int id;
	/* STV_VIEWER */ struct stv_viewer_struct *next;
};

typedef struct stv_viewer_struct *STV_VIEWER;


/* Data Coupler Handle Structure */

struct stv_data_coupler_handle_struct
{
	int couplerID;
	int couplerFID;
	STV_VIEWER viewer;
	STV_VIEW_FIELD_GROUP vfg;
	STV_VIEW_FIELD view_field;
	STV_PACK_DIM pack_dims;
	STV_PARTICLEPTR particle;
	STV_FIELDPTR field;
	char *appname;
	char *fieldname;
	int dst_freq;
	int src_freq;
	int dst_no_synch;
	int src_no_synch;
	int timemax;
	int offset;
	int need_transfer;
	int auto_transfer;
	int allow_transfer;
	int cnt;
	int ntasks;
	/* STV_COUPLER */ struct stv_data_coupler_handle_struct *next;
};

typedef struct stv_data_coupler_handle_struct *STV_COUPLER;


/* Time Conversion Structure */

struct stv_time_conversion_struct
{
	int units;
	int value;
	int conv_units;
};

typedef struct stv_time_conversion_struct TIME_CONVERSION;


/* ------------- CPD  data struct declarations  --------------- */


/* Checkpointed Application Information */

struct cpd_application
{
	char *name;		/* name of the application that is checkpointing */
	char *aout;		/* executable name for respawning */
	char **argv;	/* arg list for respawning */
	int scratch;	/* scratch file to write in-progress chkpoint dat */
	int sfname;		/* which buffer holds name of the scratch file */
	int cmtfile;	/* commited checkpoint file */
	int cfname;		/* which buffer holds name of the commit file */
	int ntasks;
	int recovery;	/* this app's recovery choice: restart / rollback */
	int globlcount;	/* # of global tasks for which we have this ckpt */
	int nlocl;		/* # of local tasks for this app */
	int ndesired;	/* # of local tasks that we are supposed to have */
	int loclcount;	/* number of local tasks that have checkpointed */
	int *locllist;	/* list of local tids */
	int *finlist;	/* marker for finished tids */
	int locllistsz;	/* malloc'ed size of the local list */
	int nlocltmp;	/* size of temporary local tasks list */
	int *locltmp;	/* temp local tasks, spawned but not attached yet */
	int *globlist;	/* list of all tids   */
	int globlistsz;
	int needgloblist;	/* flag if we need the global list */
	int lc_from_glob;	/* nlocl based on globlist */
	int *loclmap;
	int szloclmap;
	int timestamp;
	int epoch;		/* epoch for the checkpoint */
	int state;		/* state of this application */
	int msgtag;		/* tag to use in communicating with the app */
	int nrestart;
	int nrerestart;	/* # of failed attempts to restart a task... */
	int nfinished;	/* # of local tasks which have finished */
	struct cpd_task *dloadlist;
				 /* list of local tids that need ckpt data downloaded */
	int dloadcnt;	/* size of the dloadlist */
	struct cpd_peer *peerlist;
	int peercount;	/* # of peers to reply to stabilize msg */
	struct cpd_peer *stablist;
	int stabcount;	/* # of peers to tell us they are stable */
	int verifycount;	/* # of peers to verify checkpoint release */
	int finishcount;	/* # of peers to verify app finished */
	int stabepoch;	/* max epoch of stabilized peers */
	int stabtid;	/* peer tid that has the most recent checkpoint */
	char fnames[2][2 * stvMaxNameLen];
	int utag;		/* tag that user wants for his own notify */
	int uvalue;		/* value to put in the user's notify message */
	/* CPD_APPPTR */ struct cpd_application *prev;
	/* CPD_APPPTR */ struct cpd_application *next;
};

typedef struct cpd_application CPD_APP; 
typedef struct cpd_application *CPD_APPPTR; 


/* Checkpointing Daemon Peer Structure */

struct cpd_peer
{
	int tid;
	int state;
	int value;
	STV_HOST host;
	/* CPD_PEERPTR */ struct cpd_peer *prev;
	/* CPD_PEERPTR */ struct cpd_peer *next;
};

typedef struct cpd_peer CPD_PEER;
typedef struct cpd_peer *CPD_PEERPTR;


/* Reconfigure Checkpointing Region Structure */

struct cpd_reconfig_region_struct
{
	char *name;
	int type;
	STV_VIS_REGION regions;
	int nregions;
	/* CPD_RECONFIG_REGION */ struct cpd_reconfig_region_struct *next;
};

typedef struct cpd_reconfig_region_struct *CPD_RECONFIG_REGION;


/* Task list used for tasks that have asked for a checkpoint */

struct cpd_task
{
	int nodetid;
	int nodeid;
	int reconfig;			/* file to write reconfig chkpoint data */
	char reconfigfname[2 * stvMaxNameLen];
	STV_FIELDPTR *fields;
	int nfields;
	STV_PARAMPTR *params;
	int nparams;
	int *setparams;
	int nsetparams;
	CPD_RECONFIG_REGION rregion;
	int nrregion;
	CPD_PEERPTR reconfiglist;
	int reconfigcount;		/* # of peers to reply to reconfig msg */
	int done;
	/* CPD_TASKPTR */ struct cpd_task *prev;
	/* CPD_TASKPTR */ struct cpd_task *next;
};

typedef struct cpd_task CPD_TASK;
typedef struct cpd_task *CPD_TASKPTR;


/* Checkpointing Header Structure */

struct cpd_header
{
	char name[stvMaxNameLen];
	char arch[20];					/* arch code */
	char aout[stvMaxNameLen];		/* name of executable */
	int ntasks;						/* # of tasks */
	int epoch; 						/* checkpoint number */
	int complete;
	int timestamp;
	int restart;
	int msgtag;						/* tag to talk to application on */
};	

typedef struct cpd_header CPD_HEADER;
typedef struct cpd_header *CPD_HEADERPTR;


/* Checkpointing Particle Coords Structure */

struct cpd_particle_coords
{
	int coords[STV_MAX_DIM];
	STV_PARTICLE_ID ptid;
	/* CPD_PARTICLE_COORDS */ struct cpd_particle_coords *next;
};

typedef struct cpd_particle_coords *CPD_PARTICLE_COORDS;


/* Checkpointing Particle Data Structure */

struct cpd_particle_data
{
	STV_VALUE value;
	int nelems;
	/* CPD_PARTICLE_DATA */ struct cpd_particle_data *next;
};

typedef struct cpd_particle_data *CPD_PARTICLE_DATA;


/* Checkpointing In / Out Type Structure */

struct cpd_hole_struct
{
	STV_MP_API *mp;
	int type;
	int parm;
};

typedef struct cpd_hole_struct CPD_HOLE;


/* Checkpointing Data Structure */

struct cpd_data
{
	STV_PARAMPTR param;
	STV_PARTICLEPTR particle;
	CPD_PARTICLE_COORDS pcoords;
	STV_FIELDPTR field;
	CPD_PARTICLE_DATA pdata;
	/* CPD_DATAPTR */ struct cpd_data *next;
};

typedef struct cpd_data CPD_DATA;
typedef struct cpd_data *CPD_DATAPTR;


/* Reconfigure Checkpointing Data Structure */

struct cpd_reconfig_data_struct
{
	char *name;
	int type;
	int offset[STV_MAX_DIM];
	int dim[STV_MAX_DIM];
	int storage_order;
	int gstart;
	int gstride;
	int num;
	STV_VALUE data;
	/* CPD_RECONFIG_DATA */ struct cpd_reconfig_data_struct *next;
};

typedef struct cpd_reconfig_data_struct *CPD_RECONFIG_DATA;


#ifdef __cplusplus
}
#endif


/* _STV_TYPE_H_ */
#endif

