
/* $Id: stv_mp.c,v 4.3 2003/09/09 17:41:38 kohl Exp $ */


/*
 *                      CUMULVS version 1.0:
 *      Computational Steering and Interactive Visualization
 *                   in Distributed Applications
 *           Oak Ridge National Laboratory, Oak Ridge TN.
 *      Authors:  James Arthur Kohl and Philip M. Papadopoulos
 *                   (C) 1996 All Rights Reserved
 *
 *                              NOTICE
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted
 * provided that the above copyright notice appear in all copies and
 * that both the copyright notice and this permission notice appear
 * in supporting documentation.
 *
 * Neither the Institution, Oak Ridge National Laboratory, nor the
 * Authors make any representations about the suitability of this
 * software for any purpose.  This software is provided ``as is''
 * without express or implied warranty.
 *
 * CUMULVS was funded by the U.S. Department of Energy.
 */


/* System Header Files */

#include <stdio.h>


/* STV Header Files */

#include "stv.h"
#include "stvmac.h"


/* Various Message-Passing API Routines */


#ifdef STV_MP_PVM

/* PVM Message-Passing API Routines */

/* Init & Exit */

int
stv_mp_pvm_myid( int *idp )
{
	int cc;

	if ( idp == NULL )
		return( stvFalse );

	cc = pvm_setopt( PvmAutoErr, 0 );

	*idp = pvm_mytid();

	pvm_setopt( PvmAutoErr, cc );

	return( ( *idp >= 0 ) ? stvTrue : stvFalse );
}


int
stv_mp_pvm_exit()
{
	return( pvm_exit() );
}


/* Message Context */

int
stv_mp_pvm_create_context( int *ctxp )
{
	if ( ctxp == NULL )
		return( stvFalse );

#if ( PVM_MAJOR_VERSION > 3 ) || ( PVM_MINOR_VERSION >= 4 )

	*ctxp = pvm_newcontext();

#else

	*ctxp = stvTrue;

#endif

	return( ( (*ctxp) >= 0 ) ? stvTrue : stvFalse );
}


int
stv_mp_pvm_free_context( int *ctxp )
{
	int cc;

	if ( ctxp == NULL )
		return( stvFalse );

#if ( PVM_MAJOR_VERSION > 3 ) || ( PVM_MINOR_VERSION >= 4 )

	cc = pvm_freecontext( *ctxp );

#endif

	*ctxp = (STV_CONTEXT) -1;

	return( ( cc >= 0 ) ? stvTrue : stvFalse );
}


int
stv_mp_pvm_get_context( int *ctxp )
{
	if ( ctxp == NULL )
		return( stvFalse );

#if ( PVM_MAJOR_VERSION > 3 ) || ( PVM_MINOR_VERSION >= 4 )

	*ctxp = pvm_getcontext();

#else

	*ctxp = stvTrue;

#endif

	return( ( (*ctxp) >= 0 ) ? stvTrue : stvFalse );
}


int
stv_mp_pvm_set_context( int newctx, int *oldctxp )
{
	int cc;

#if ( PVM_MAJOR_VERSION > 3 ) || ( PVM_MINOR_VERSION >= 4 )

	cc = pvm_setcontext( newctx );

#else

	cc = stvTrue;

#endif

	if ( oldctxp )
		*oldctxp = cc;

	return( ( cc >= 0 ) ? stvTrue : stvFalse );
}


/* Message Buffer */

int
stv_mp_pvm_init_msg_buf( int enc, int *bufp )
{
	int cc;

	if ( bufp == NULL )
		return( stvFalse );

	cc = pvm_setsbuf(0);

	*bufp = pvm_initsend( enc );

	pvm_setsbuf( cc );

	return( ( *bufp > 0 ) ? stvTrue : stvFalse );
}


int
stv_mp_pvm_msg_buf_info( int buf, int *nbytesp, int *tagp, int *srcp )
{
	int cc;

	cc = pvm_bufinfo( buf, nbytesp, tagp, srcp );

	return( ( cc >= 0 ) ? stvTrue : stvFalse );
}


int
stv_mp_pvm_get_sbuf( int *bufp )
{
	int cc;

	if ( bufp == NULL )
		return( stvFalse );

	cc = pvm_getsbuf();

	if ( bufp )
		*bufp = cc;

	return( ( cc >= 0 ) ? stvTrue : stvFalse );
}


int
stv_mp_pvm_set_sbuf( int buf )
{
	int cc;

	cc = pvm_setsbuf( buf );

	return( ( cc >= 0 ) ? stvTrue : stvFalse );
}


int
stv_mp_pvm_get_rbuf( int *bufp )
{
	int cc;

	if ( bufp == NULL )
		return( stvFalse );

	cc = pvm_getrbuf();

	if ( bufp )
		*bufp = cc;

	return( ( cc >= 0 ) ? stvTrue : stvFalse );
}


int
stv_mp_pvm_set_rbuf( int buf )
{
	int cc;

	cc = pvm_setrbuf( buf );

	return( ( cc >= 0 ) ? stvTrue : stvFalse );
}


int
stv_mp_pvm_free_msg_buf( int buf )
{
	int cc;
	
	cc = pvm_freebuf( buf );

	return( ( cc >= 0 ) ? stvTrue : stvFalse );
}


/* Message Packing */

int
stv_mp_pvm_pack_byte( int buf, void *datap, int count, int stride )
{
	int oldbuf;
	int cc;

	if ( datap == NULL )
		return( stvFalse );

	oldbuf = pvm_setsbuf( buf );

	cc = pvm_pkbyte( datap, count, stride );

	pvm_setsbuf( oldbuf );

	return( ( cc >= 0 ) ? stvTrue : stvFalse );
}


int
stv_mp_pvm_pack_int( int buf, void *datap, int count, int stride )
{
	int oldbuf;
	int cc;

	if ( datap == NULL )
		return( stvFalse );

	oldbuf = pvm_setsbuf( buf );

	cc = pvm_pkint( datap, count, stride );

	pvm_setsbuf( oldbuf );

	return( ( cc >= 0 ) ? stvTrue : stvFalse );
}


int
stv_mp_pvm_pack_float( int buf, void *datap, int count, int stride )
{
	int oldbuf;
	int cc;

	if ( datap == NULL )
		return( stvFalse );

	oldbuf = pvm_setsbuf( buf );

	cc = pvm_pkfloat( datap, count, stride );

	pvm_setsbuf( oldbuf );

	return( ( cc >= 0 ) ? stvTrue : stvFalse );
}


int
stv_mp_pvm_pack_double( int buf, void *datap, int count, int stride )
{
	int oldbuf;
	int cc;

	if ( datap == NULL )
		return( stvFalse );

	oldbuf = pvm_setsbuf( buf );

	cc = pvm_pkdouble( datap, count, stride );

	pvm_setsbuf( oldbuf );

	return( ( cc >= 0 ) ? stvTrue : stvFalse );
}


int
stv_mp_pvm_pack_complex( int buf, void *datap, int count, int stride )
{
	int oldbuf;
	int cc;

	if ( datap == NULL )
		return( stvFalse );

	oldbuf = pvm_setsbuf( buf );

	cc = pvm_pkcplx( datap, count, stride );

	pvm_setsbuf( oldbuf );

	return( ( cc >= 0 ) ? stvTrue : stvFalse );
}


int
stv_mp_pvm_pack_dcomplex( int buf, void *datap, int count, int stride )
{
	int oldbuf;
	int cc;

	if ( datap == NULL )
		return( stvFalse );

	oldbuf = pvm_setsbuf( buf );

	cc = pvm_pkdcplx( datap, count, stride );

	pvm_setsbuf( oldbuf );

	return( ( cc >= 0 ) ? stvTrue : stvFalse );
}


int
stv_mp_pvm_pack_short( int buf, void *datap, int count, int stride )
{
	int oldbuf;
	int cc;

	if ( datap == NULL )
		return( stvFalse );

	oldbuf = pvm_setsbuf( buf );

	cc = pvm_pkshort( datap, count, stride );

	pvm_setsbuf( oldbuf );

	return( ( cc >= 0 ) ? stvTrue : stvFalse );
}


int
stv_mp_pvm_pack_ushort( int buf, void *datap, int count, int stride )
{
	int oldbuf;
	int cc;

	if ( datap == NULL )
		return( stvFalse );

	oldbuf = pvm_setsbuf( buf );

	cc = pvm_pkushort( datap, count, stride );

	pvm_setsbuf( oldbuf );

	return( ( cc >= 0 ) ? stvTrue : stvFalse );
}


int
stv_mp_pvm_pack_uint( int buf, void *datap, int count, int stride )
{
	int oldbuf;
	int cc;

	if ( datap == NULL )
		return( stvFalse );

	oldbuf = pvm_setsbuf( buf );

	cc = pvm_pkuint( datap, count, stride );

	pvm_setsbuf( oldbuf );

	return( ( cc >= 0 ) ? stvTrue : stvFalse );
}


int
stv_mp_pvm_pack_long( int buf, void *datap, int count, int stride )
{
	int oldbuf;
	int cc;

	if ( datap == NULL )
		return( stvFalse );

	oldbuf = pvm_setsbuf( buf );

	cc = pvm_pklong( datap, count, stride );

	pvm_setsbuf( oldbuf );

	return( ( cc >= 0 ) ? stvTrue : stvFalse );
}


int
stv_mp_pvm_pack_ulong( int buf, void *datap, int count, int stride )
{
	int oldbuf;
	int cc;

	if ( datap == NULL )
		return( stvFalse );

	oldbuf = pvm_setsbuf( buf );

	cc = pvm_pkulong( datap, count, stride );

	pvm_setsbuf( oldbuf );

	return( ( cc >= 0 ) ? stvTrue : stvFalse );
}


int
stv_mp_pvm_pack_str( int buf, char *str )
{
	int oldbuf;
	int cc;

	if ( str == NULL )
		return( stvFalse );

	oldbuf = pvm_setsbuf( buf );

	cc = pvm_pkstr( str );

	pvm_setsbuf( oldbuf );

	return( ( cc >= 0 ) ? stvTrue : stvFalse );
}


int
stv_mp_pvm_pack_context( int buf, int *ctxp )
{
	int oldbuf;
	int cc;

	if ( ctxp == NULL )
		return( stvFalse );

	oldbuf = pvm_setsbuf( buf );

	cc = pvm_pkint( ctxp, 1, 1 );

	pvm_setsbuf( oldbuf );

	return( ( cc >= 0 ) ? stvTrue : stvFalse );
}


/* Message Unpacking */

int
stv_mp_pvm_unpack_byte( int buf, void *datap, int count, int stride )
{
	int oldbuf;
	int cc;

	if ( datap == NULL )
		return( stvFalse );

	oldbuf = pvm_setrbuf( buf );

	cc = pvm_upkbyte( datap, count, stride );

	pvm_setrbuf( oldbuf );

	return( ( cc >= 0 ) ? stvTrue : stvFalse );
}


int
stv_mp_pvm_unpack_int( int buf, void *datap, int count, int stride )
{
	int oldbuf;
	int cc;

	if ( datap == NULL )
		return( stvFalse );

	oldbuf = pvm_setrbuf( buf );

	cc = pvm_upkint( datap, count, stride );

	pvm_setrbuf( oldbuf );

	return( ( cc >= 0 ) ? stvTrue : stvFalse );
}


int
stv_mp_pvm_unpack_float( int buf, void *datap, int count, int stride )
{
	int oldbuf;
	int cc;

	if ( datap == NULL )
		return( stvFalse );

	oldbuf = pvm_setrbuf( buf );

	cc = pvm_upkfloat( datap, count, stride );

	pvm_setrbuf( oldbuf );

	return( ( cc >= 0 ) ? stvTrue : stvFalse );
}


int
stv_mp_pvm_unpack_double( int buf, void *datap, int count, int stride )
{
	int oldbuf;
	int cc;

	if ( datap == NULL )
		return( stvFalse );

	oldbuf = pvm_setrbuf( buf );

	cc = pvm_upkdouble( datap, count, stride );

	pvm_setrbuf( oldbuf );

	return( ( cc >= 0 ) ? stvTrue : stvFalse );
}


int
stv_mp_pvm_unpack_complex( int buf, void *datap, int count, int stride )
{
	int oldbuf;
	int cc;

	if ( datap == NULL )
		return( stvFalse );

	oldbuf = pvm_setrbuf( buf );

	cc = pvm_upkcplx( datap, count, stride );

	pvm_setrbuf( oldbuf );

	return( ( cc >= 0 ) ? stvTrue : stvFalse );
}


int
stv_mp_pvm_unpack_dcomplex( int buf,
		void *datap, int count, int stride )
{
	int oldbuf;
	int cc;

	if ( datap == NULL )
		return( stvFalse );

	oldbuf = pvm_setrbuf( buf );

	cc = pvm_upkdcplx( datap, count, stride );

	pvm_setrbuf( oldbuf );

	return( ( cc >= 0 ) ? stvTrue : stvFalse );
}


int
stv_mp_pvm_unpack_short( int buf, void *datap, int count, int stride )
{
	int oldbuf;
	int cc;

	if ( datap == NULL )
		return( stvFalse );

	oldbuf = pvm_setrbuf( buf );

	cc = pvm_upkshort( datap, count, stride );

	pvm_setrbuf( oldbuf );

	return( ( cc >= 0 ) ? stvTrue : stvFalse );
}


int
stv_mp_pvm_unpack_ushort( int buf, void *datap, int count, int stride )
{
	int oldbuf;
	int cc;

	if ( datap == NULL )
		return( stvFalse );

	oldbuf = pvm_setrbuf( buf );

	cc = pvm_upkushort( datap, count, stride );

	pvm_setrbuf( oldbuf );

	return( ( cc >= 0 ) ? stvTrue : stvFalse );
}


int
stv_mp_pvm_unpack_uint( int buf, void *datap, int count, int stride )
{
	int oldbuf;
	int cc;

	if ( datap == NULL )
		return( stvFalse );

	oldbuf = pvm_setrbuf( buf );

	cc = pvm_upkuint( datap, count, stride );

	pvm_setrbuf( oldbuf );

	return( ( cc >= 0 ) ? stvTrue : stvFalse );
}


int
stv_mp_pvm_unpack_long( int buf, void *datap, int count, int stride )
{
	int oldbuf;
	int cc;

	if ( datap == NULL )
		return( stvFalse );

	oldbuf = pvm_setrbuf( buf );

	cc = pvm_upklong( datap, count, stride );

	pvm_setrbuf( oldbuf );

	return( ( cc >= 0 ) ? stvTrue : stvFalse );
}


int
stv_mp_pvm_unpack_ulong( int buf, void *datap, int count, int stride )
{
	int oldbuf;
	int cc;

	if ( datap == NULL )
		return( stvFalse );

	oldbuf = pvm_setrbuf( buf );

	cc = pvm_upkulong( datap, count, stride );

	pvm_setrbuf( oldbuf );

	return( ( cc >= 0 ) ? stvTrue : stvFalse );
}


int
stv_mp_pvm_unpack_str( int buf, char *str )
{
	int oldbuf;
	int cc;

	if ( str == NULL )
		return( stvFalse );

	oldbuf = pvm_setrbuf( buf );

	cc = pvm_upkstr( str );

	pvm_setrbuf( oldbuf );

	return( ( cc >= 0 ) ? stvTrue : stvFalse );
}


int
stv_mp_pvm_unpack_context( int buf, int *ctxp )
{
	int oldbuf;
	int cc;

	if ( ctxp == NULL )
		return( stvFalse );

	oldbuf = pvm_setrbuf( buf );

	cc = pvm_upkint( ctxp, 1, 1 );

	pvm_setrbuf( oldbuf );

	return( ( cc >= 0 ) ? stvTrue : stvFalse );
}


/* Groups */

int
stv_mp_pvm_join_group( char *group, int *instp )
{
	int cc;

	if ( group == NULL )
		return( stvFalse );

	cc = pvm_joingroup( group );

	if ( instp != NULL )
		*instp = cc;

	return( ( cc >= 0 ) ? stvTrue : stvFalse );
}


int
stv_mp_pvm_leave_group( char *group )
{
	int cc;

	if ( group == NULL )
		return( stvFalse );

	cc = pvm_lvgroup( group );

	return( ( cc >= 0 ) ? stvTrue : stvFalse );
}


int
stv_mp_pvm_group_size( char *group, int *sizep )
{
	int cc;

	if ( group == NULL )
		return( stvFalse );

	cc = pvm_gsize( group );

	if ( sizep != NULL )
		*sizep = cc;

	return( ( cc >= 0 ) ? stvTrue : stvFalse );
}


int
stv_mp_pvm_get_id_from_inst( char *group, int inst, int *idp )
{
	int cc;

	if ( group == NULL )
		return( stvFalse );

	cc = pvm_gettid( group, inst );

	if ( idp != NULL )
		*idp = cc;

	return( ( cc >= 0 ) ? stvTrue : stvFalse );
}


int
stv_mp_pvm_get_inst_from_id( char *group, int id, int *instp )
{
	int cc;

	if ( group == NULL )
		return( stvFalse );

	cc = pvm_getinst( group, id );

	if ( instp != NULL )
		*instp = cc;

	return( ( cc >= 0 ) ? stvTrue : stvFalse );
}


int
stv_mp_pvm_get_group_tids( char *group, int **tidlistp, int *ntidsp )
{
	int cc;

	if ( group == NULL || tidlistp == NULL || ntidsp == NULL )
		return( stvFalse );

	cc = gs_get_tidlist( group, 1, ntidsp, tidlistp, stvFalse );

	return( ( cc >= 0 ) ? stvTrue : stvFalse );
}


/* Send & Receive */

int
stv_mp_pvm_send_msg( int buf, int dst, int tag, int ctx )
{
	STV_CONTEXT tmpctx;

	int oldbuf;
	int cc;

	tmpctx = pvm_setcontext( ctx );

	oldbuf = pvm_setsbuf( buf );

	cc = pvm_send( dst, tag );

	pvm_setsbuf( oldbuf );

	pvm_setcontext( tmpctx );

	return( ( cc >= 0 ) ? stvTrue : stvFalse );
}


int
stv_mp_pvm_mcast_msg( int buf, int *dstlist, int numdst,
		int tag, int ctx )
{
	STV_CONTEXT tmpctx;

	int oldbuf;
	int cc;

	if ( dstlist == NULL )
		return( stvFalse );

	tmpctx = pvm_setcontext( ctx );

	oldbuf = pvm_setsbuf( buf );

	cc = pvm_mcast( dstlist, numdst, tag );

	pvm_setsbuf( oldbuf );

	pvm_setcontext( tmpctx );

	return( ( cc >= 0 ) ? stvTrue : stvFalse );
}


int
stv_mp_pvm_bcast_msg( int buf, char *group, int tag, int ctx )
{
	STV_CONTEXT tmpctx;

	int oldbuf;
	int cc;

	if ( group == NULL )
		return( stvFalse );

	tmpctx = pvm_setcontext( ctx );

	oldbuf = pvm_setsbuf( buf );

	cc = pvm_bcast( group, tag );

	pvm_setsbuf( oldbuf );

	pvm_setcontext( tmpctx );

	return( ( cc >= 0 ) ? stvTrue : stvFalse );
}


int
stv_mp_pvm_recv_msg( int src, int tag, int ctx , int *bufp )
{
	STV_CONTEXT tmpctx;

	int oldbuf;

	if ( bufp == NULL )
		return( stvFalse );

	tmpctx = pvm_setcontext( ctx );

	oldbuf = pvm_setrbuf( 0 );

	*bufp = pvm_recv( src, tag );

	pvm_setrbuf( oldbuf );

	pvm_setcontext( tmpctx );

	return( ( *bufp > 0 ) ? stvTrue : stvFalse );
}


int
stv_mp_pvm_recv_msg_noblock( int src, int tag, int ctx, int *bufp )
{
	STV_CONTEXT tmpctx;

	int oldbuf;

	if ( bufp == NULL )
		return( stvFalse );

	tmpctx = pvm_setcontext( ctx );

	oldbuf = pvm_setrbuf( 0 );

	*bufp = pvm_nrecv( src, tag );

	pvm_setrbuf( oldbuf );

	pvm_setcontext( tmpctx );

	return( ( *bufp > 0 ) ? stvTrue : stvFalse );
}


int
stv_mp_pvm_recv_msg_timeout( int src, int tag, int ctx,
		struct timeval *tmoutp, int *bufp )
{
	STV_CONTEXT tmpctx;

	int oldbuf;

	if ( bufp == NULL )
		return( stvFalse );

	tmpctx = pvm_setcontext( ctx );

	oldbuf = pvm_setrbuf( 0 );

	*bufp = pvm_trecv( src, tag, tmoutp );

	pvm_setrbuf( oldbuf );

	pvm_setcontext( tmpctx );

	return( ( *bufp > 0 ) ? stvTrue : stvFalse );
}


/* Error Output */

int
stv_mp_pvm_perr( char *text )
{
	int cc;

	cc = pvm_perror( text );

	return( ( cc >= 0 ) ? stvTrue : stvFalse );
}


/* Other Support Routines */

int
stv_host_config( stv_hostp, nhostp )
STV_HOSTPTR *stv_hostp;
int *nhostp;
{
	struct pvmhostinfo *hostp = 0;

	int nhosts;
	int cc;
	int i;

	cc = pvm_config( &nhosts, (int *) NULL, &hostp );

	if ( cc >= 0 )
	{
		if ( stv_hostp != NULL )
		{
			*stv_hostp = (STV_HOSTPTR) MALLOC( (STV_SIZE_T) nhosts
				* sizeof( STV_HOST ) );
			if ( !(*stv_hostp) )
				return( stvFalse );

			for ( i=0 ; i < nhosts ; i++ )
			{
				strcpy( (*stv_hostp)[i].name, hostp[i].hi_name );
				strcpy( (*stv_hostp)[i].arch, hostp[i].hi_arch );
				(*stv_hostp)[i].tid = hostp[i].hi_tid;
			}
		}

		if ( nhostp != NULL )
			*nhostp = nhosts;

		return( stvTrue );
	}

	return( stvFalse );
}


/* Name Service Routines */

#if ( PVM_MAJOR_VERSION > 3 ) || ( PVM_MINOR_VERSION >= 4 )

/* Do Nothing */

#else

#ifdef CRAY

/* Spank the Cray Name Database...  :-) */

/* Also Necessary for T3E Cray: */
/* setenv PVM_GROUPS_FULL 1     */
/* setenv PVM_PE_LIST all       */

/*
 *         PVM version 3.3:  Parallel Virtual Machine System
 *               University of Tennessee, Knoxville TN.
 *           Oak Ridge National Laboratory, Oak Ridge TN.
 *                   Emory University, Atlanta GA.
 *      Authors:  A. L. Beguelin, J. J. Dongarra, G. A. Geist,
 *    W. C. Jiang, R. J. Manchek, B. K. Moore, and V. S. Sunderam
 *                   (C) 1992 All Rights Reserved
 *
 *                              NOTICE
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted
 * provided that the above copyright notice appear in all copies and
 * that both the copyright notice and this permission notice appear in
 * supporting documentation.
 *
 * Neither the Institutions (Emory University, Oak Ridge National
 * Laboratory, and University of Tennessee) nor the Authors make any
 * representations about the suitability of this software for any
 * purpose.  This software is provided ``as is'' without express or
 * implied warranty.
 *
 * PVM version 3 was funded in part by the U.S. Department of Energy,
 * the National Science Foundation and the State of Tennessee.
 */


/***************
 **  Globals  **
 ***************/

#define TMDB_INSERT     1
#define TMDB_DELETE     2
#define TMDB_LOOKUP     3

#define 	TIDPVMD		0x80000000
#define 	TM_DB 		((int)0x80010001) + 15 

#define BEATASK (pvmmytid == -1 ? ( pvmmytid = pvm_mytid(), 0) : 0)
#define TEV_DO_TRACE(_x) 0
#define TEV_FIN

static int pvmmytid = -1;

int
pvm_delete(name, req)
	char *name;		/* class name */
	int req;		/* class index or -1 for all */
{
	int sbf, rbf, cc;
	int x;

	if (!name || !*name || req < -1) {
		cc = PvmBadParam;

	} else {
		if (!(cc = BEATASK)) {
			sbf = pvm_setsbuf(pvm_mkbuf(PvmDataFoo));
			rbf = pvm_setrbuf(0);
			cc = TMDB_DELETE;
			pvm_pkint(&cc, 1, 1);
			pvm_pkstr(name);
			pvm_pkint(&req, 1, 1);
			if ((cc = msendrecv(TIDPVMD, TM_DB)) > 0) {
				pvm_upkint(&cc, 1, 1);
				pvm_freebuf(pvm_setrbuf(rbf));

			} else
				pvm_setrbuf(rbf);
			pvm_freebuf(pvm_setsbuf(sbf));
		}
	}

	if (cc < 0 && cc != PvmNoEntry)
		lpvmerr("pvm_delete", cc);
	return cc;
}


int
pvm_insert(name, req, data)
	char *name;		/* class name */
	int req;		/* requested class index or -1 for any */
	int data;
{
	int sbf, rbf, cc;
	int x;

	if (!name || !*name || req < -1) {
		cc = PvmBadParam;

	} else {
		if (!(cc = BEATASK)) {
			sbf = pvm_setsbuf(pvm_mkbuf(PvmDataFoo));
			rbf = pvm_setrbuf(0);
			cc = TMDB_INSERT;
			pvm_pkint(&cc, 1, 1);
			pvm_pkstr(name);
			pvm_pkint(&req, 1, 1);
			pvm_pkint(&data, 1, 1);
			if ((cc = msendrecv(TIDPVMD, TM_DB)) > 0) {
				pvm_upkint(&cc, 1, 1);
				pvm_freebuf(pvm_setrbuf(rbf));

			} else
				pvm_setrbuf(rbf);
			pvm_freebuf(pvm_setsbuf(sbf));
		}
	}

	if (cc < 0 && cc != PvmDupEntry)
		lpvmerr("pvm_insert", cc);
	return cc;
}


int
pvm_lookup(name, req, datap)
	char *name;		/* class name */
	int req;		/* req class index or -1 for any */
	int *datap;		/* data return */
{
	int sbf, rbf, cc, cc1;
	int x, nbytes;
	short wfcs;

	if (!name || !*name || req < -1) {
		cc = PvmBadParam;

	} else {
		if (!(cc = BEATASK)) {
			sbf = pvm_setsbuf(pvm_mkbuf(PvmDataFoo));
			rbf = pvm_setrbuf(0);
			cc = TMDB_LOOKUP;
			pvm_pkint(&cc, 1, 1);
			pvm_pkstr(name);
			pvm_pkint(&req, 1, 1);
			if ((cc = msendrecv(TIDPVMD, TM_DB)) >= 0) {
				cc1 = pvm_upkshort(&wfcs, 1, 1);
				cc = wfcs;
				if (cc >= 0 && datap)
				{
					cc = pvm_upkshort(&wfcs, 1, 1);
					*datap = wfcs;
				}
				pvm_freebuf(pvm_setrbuf(rbf));

			} else
				pvm_setrbuf(rbf);
			pvm_freebuf(pvm_setsbuf(sbf));
		}
	}


	if (cc < 0 && cc != PvmNoEntry)
		lpvmerr("pvm_lookup", cc);
	return cc;
}


#ifndef min
#define min(i,j) ((i)<(j)?(i):(j))
#endif


#include <string.h>


ftocstr(ds, dl, ss, sl)
    char *ds, *ss;      /* dst, src ptrs */
    int dl;             /* dst max len */
    int sl;             /* src len */
{
    char *p;

    for (p = ss + sl; --p >= ss && *p == ' '; ) ;
    sl = p - ss + 1;
    dl--;
    ds[0] = 0;
    if (sl > dl)
        return 1;
    strncat(ds, ss, min(sl, dl));
    return 0;
}


ctofstr(ds, dl, ss)
	char *ds;		/* dest space */
	int dl;			/* max dest length */
	char *ss;		/* src string (0-term) */
{
	int sl = strlen(ss);

	if (dl <= sl)
		bcopy(ss, ds, dl);

	else {
		bcopy(ss, ds, sl);
		dl -= sl;
		ds += sl;
		while (dl-- > 0)
			*ds++ = ' ';
	}
	return 0;
}

/* CRAY */
#endif


/* PVM 3.3 */
#endif


/* STV_MP_PVM */
#endif

