
/* $Id: stv_pack.c,v 4.16 2003/09/05 21:44:51 kohl Exp $ */


/*
 *                      CUMULVS version 1.0:
 *      Computational Steering and Interactive Visualization
 *                   in Distributed Applications
 *           Oak Ridge National Laboratory, Oak Ridge TN.
 *      Authors:  James Arthur Kohl and Philip M. Papadopoulos
 *                   (C) 1996 All Rights Reserved
 *
 *                              NOTICE
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted
 * provided that the above copyright notice appear in all copies and
 * that both the copyright notice and this permission notice appear
 * in supporting documentation.
 *
 * Neither the Institution, Oak Ridge National Laboratory, nor the
 * Authors make any representations about the suitability of this
 * software for any purpose.  This software is provided ``as is''
 * without express or implied warranty.
 *
 * CUMULVS was funded by the U.S. Department of Energy.
 */


/* System Header Files */

#include <stdio.h>
#include <math.h>


/* STV Header Files */

#include "stv.h"
#include "stvmac.h"
#include "stv_glob.h"


/* Packing Routines */


/* Pack Up Local SubRegion Elements */

int
stv_pack_SubRegion( V, FG, tag, ctx )
STV_FE_VIEWERPTR V;
STV_FIELD_GROUP_PTR FG;
int tag;
STV_CONTEXT ctx;
{
	STV_PARAM_MAP_PTR PM;

	STV_VIS_REGION VR;

	STV_FIELDPTR F;

	STV_PARAMPTR P;

	int nparams;
	int nfields;
	int nparts;
	int i, j;
	int sbuf;
	int type;
	int tmp;
	int cc;

	/* Pack Region Message */

	V->mp->init_msg_buf( stvDataDefault, &sbuf );

	tmp = stvMsgOp( stvRegion );
	V->mp->pack_int( sbuf, &tmp, 1, 1 );

	/* Pack Field Group ID */

	V->mp->pack_int( sbuf, &(FG->fgid), 1, 1 );

	/* Pack Viewer ID */

	V->mp->pack_int( sbuf, &(V->vid), 1, 1 );

	/* Pack Region ID */

	V->mp->pack_int( sbuf, &(FG->vRegionId), 1, 1 );

	/* Pack Lock Status */

	V->mp->pack_int( sbuf, &STV_APP_LOCK, 1, 1 );

	/* Pack Current Timestamp */

	V->mp->pack_int( sbuf, &STV_TIMESTAMP, 1, 1 );

	/* Pack Instance Number (Nodeid) - for descriptor wrangling... */

	V->mp->pack_int( sbuf, &STV_MYNODEID, 1, 1 );

	/* Pack SubRegion Marker */

	tmp = stvMarkerSubRegion;
	V->mp->pack_int( sbuf, &tmp, 1, 1 );

	/* Count Standard & Particle Fields */

	nfields = 0;

	nparts = 0;

	for ( i=0 ; i < FG->nfields ; i++ )
	{
		F = FG->field_list[i];

		if ( F != NULL && F->data_ready )
		{
			if ( F->decomp != NULL )
				nfields++;

			else if ( F->particle != NULL )
				nparts++;
		}
	}

	/* Pack All Standard Fields */

	V->mp->pack_int( sbuf, &nfields, 1, 1 );

	for ( i=0 ; i < FG->nfields ; i++ )
	{
		F = FG->field_list[i];

		/* Check for Existing Field */

		if ( F != NULL && F->data_ready )
		{
			/* Pack Decomposition Field Only */

			if ( F->decomp != NULL )
			{
				/* Pack Field Marker */

				tmp = stvMarkerField;
				V->mp->pack_int( sbuf, &tmp, 1, 1 );

				/* Pack Viewer Field ID */

				V->mp->pack_int( sbuf, &(FG->field_ids[i]), 1, 1 );

				/* Pack Descriptor Flag */

				V->mp->pack_int( sbuf, &(FG->need_descriptor), 1, 1 );

				if ( FG->need_descriptor )
				{
					/* Pack Data Type */

					V->mp->pack_int( sbuf,
						&(FG->field_types[i]), 1, 1 );

					/* Pack Field Data Storage Info */

					V->mp->pack_int( sbuf, &(F->storage_order), 1, 1 );

					/* Pack Array Dimension */

					V->mp->pack_int( sbuf,
						&(F->decomp->dataDim), 1, 1 );
				}

				/* Pack Current Field Time */

				V->mp->pack_int( sbuf, &(F->time_base), 1, 1 );
				V->mp->pack_int( sbuf, &(F->time_units), 1, 1 );

				/* Pack Data */

				VR = FG->visRegion;

				j = 0;

				while ( VR != NULL )
				{
					if ( FG->field_pdims[i][j] != NULL )
					{
						/* Pack Vis Region Marker */

						tmp = stvMarkerVisRegion;
						V->mp->pack_int( sbuf, &tmp, 1, 1 );

						if ( STV_OPT_VERBOSE & STV_DEBUG_PACK )
						{
							printf( "VR[%d]: ", j );
							stv_dump_pack_dims( FG->field_pdims[i][j],
								F->storage_order,
								FG->field_storage_orders[i] );
							fflush( stdout );
						}

						stv_pack_elems_SubRegion( V->mp, sbuf,
							FG->field_pdims[i][j],
							F, FG->field_types[i],
							FG->need_descriptor );
					}

					else if ( STV_OPT_VERBOSE & STV_DEBUG_PACK )
					{
						printf( "VR[%d]: No Pack Dims\n", j );
						fflush( stdout );
					}

					VR = VR->next;

					j++;
				}

				/* Pack End of Field Marker */

				tmp = stvMarkerFieldEnd;
				V->mp->pack_int( sbuf, &tmp, 1, 1 );
			}
		}
	}

	FG->need_descriptor = stvFalse;

	/* Pack Particle Fields */

	if ( nparts )
		stv_pack_SubRegion_Particles( sbuf, V, FG, nparts );

	else
	{
		tmp = 0;
		V->mp->pack_int( sbuf, &tmp, 1, 1 );
	}

	/* Pack End of SubRegion Marker */

	tmp = stvMarkerSubRegionEnd;
	V->mp->pack_int( sbuf, &tmp, 1, 1 );

	/* Pack Parameter Values Marker */

	tmp = stvMarkerParamValues;
	V->mp->pack_int( sbuf, &tmp, 1, 1 );

	/* Pack Number of Parameter Values */

	PM = V->param_map->next;  /* skip first dummy struct - for Philbo */

	nparams = 0;

	while ( PM != NULL )
	{
		nparams++;

		PM = PM->next;
	}

	V->mp->pack_int( sbuf, &nparams, 1, 1 );

	/* Pack Parameter Values */

	PM = V->param_map->next;  /* skip first dummy struct - for Philbo */

	while ( PM != NULL )
	{
		V->mp->pack_int( sbuf, &(PM->remoteID), 1, 1 );

		P = PM->local;

		V->mp->pack_int( sbuf, &(P->num), 1, 1 );

		for ( i=0 ; i < P->num ; i++ )
		{
			if ( (cc = stv_pack_by_type( V->mp, sbuf, P->vars[i], 1, 1,
				P->types[i] & stvTypeMask )) != stvStatusOk )
			{
				STV_ERROR( "stv_pack_SubRegion", cc,
					"Typed Pack Failed." );
				
				return( cc );
			}
		}

		PM = PM->next;
	}

	/* Pack End of Parameter Values Marker */

	tmp = stvMarkerParamValuesEnd;
	V->mp->pack_int( sbuf, &tmp, 1, 1 );

	/* Send Message */

	V->mp->send_msg( sbuf, V->vtid, tag, ctx );

	V->mp->free_msg_buf( sbuf );

	/* A.O.K. */

	return( stvStatusOk );
}


/* Determine Indices for SubRegion */

STV_PACK_DIM
stv_get_SubRegion_indices( F, VR )
STV_FIELDPTR F;
STV_VIS_REGION VR;
{
	STV_PACK_DIM PDpatch;
	STV_PACK_DIM PDlast;
	STV_PACK_DIM PDtop;
	STV_PACK_DIM PDptr;
	STV_PACK_DIM PDtmp;
	STV_PACK_DIM PD;

	STV_VIS_REGION VRnext;
	STV_VIS_REGION VRck;

	STV_DECOMPPTR D;

	int pindex;
	int bindex;
	int index;

	/* Truncate Vis Region to Field Bounds (Just In Case!) */

	VRck = stv_create_vis_region();

	VRnext = VR->next;

	VR->next = (STV_VIS_REGION) NULL;

	stv_copy_vis_region( VRck, VR );

	VR->next = VRnext;

	stv_check_region( F, VRck );

	/* Initialize Return List */

	PD = (STV_PACK_DIM) NULL;

	/* Get Decomposition */

	D = F->decomp;

	if ( D == NULL )
	{
		sprintf( STVERRTXT, "No Decomp For Field \"%s\"", F->name );

		STV_ERROR( "stv_get_SubRegion_indices", stvStatusBadParam,
			STVERRTXT );

		return( (STV_PACK_DIM) NULL );
	}

	/* a la Fortran */

	if ( F->storage_order == stvColumnMajor )
	{
		if ( D->nbounds > 0 )
		{
			PDtop = (STV_PACK_DIM) NULL;

			for ( bindex=0 ; bindex < D->nbounds ; bindex++ )
			{
				PDpatch = (STV_PACK_DIM) NULL;

				pindex = 0;

				for ( index=0 ; index < D->dataDim ; index++ )
				{
					PDtmp = stv_get_SubRegion_index( F, VRck, index,
						bindex, &pindex );

					if ( PDtmp == NULL )
					{
						sprintf( STVERRTXT,
								"Bad Pack Dimension, Index=%d", index );
						STV_ERROR( "stv_get_SubRegion_indices",
								stvStatusBadParam, STVERRTXT );
						return( (STV_PACK_DIM) NULL );
					}

					if ( PDpatch == NULL )
						PDptr = PDpatch = PDtmp;
					else
						PDptr = PDptr->next_dim = PDtmp;
				}

				if ( bindex < F->numData && F->dataPtrs != NULL )
					PDpatch->dataPtr = F->dataPtrs[ bindex ];
					
				if ( PDtop == NULL )
					PDtop = PD = PDpatch;
				else
					PDtop = PDtop->next = PDpatch;
			}
		}

		else
		{
			pindex = 0;

			for ( index=0 ; index < D->dataDim ; index++ )
			{
				PDtmp = stv_get_SubRegion_index( F, VRck, index, bindex,
					&pindex );

				if ( PDtmp == NULL )
				{
					sprintf( STVERRTXT,
							"Bad Pack Dimension, Index=%d", index );
					STV_ERROR( "stv_get_SubRegion_indices",
							stvStatusBadParam, STVERRTXT );
					return( (STV_PACK_DIM) NULL );
				}

				if ( PD == NULL )
					PDptr = PD = PDtmp;
				else
					PDptr = PDptr->next_dim = PDtmp;
			}
		}
	}

	/* a la C */

	else if ( F->storage_order == stvRowMajor )
	{
		if ( D->nbounds > 0 )
		{
			PDtop = (STV_PACK_DIM) NULL;

			for ( bindex=0 ; bindex < D->nbounds ; bindex++ )
			{
				PDpatch = (STV_PACK_DIM) NULL;

				pindex = D->prank - 1;

				for ( index=(D->dataDim-1) ; index >= 0 ; index-- )
				{
					PDtmp = stv_get_SubRegion_index( F, VRck, index,
						bindex, &pindex );

					if ( PDtmp == NULL )
					{
						sprintf( STVERRTXT,
								"Bad Pack Dimension, Index=%d", index );
						STV_ERROR( "stv_get_SubRegion_indices",
								stvStatusBadParam, STVERRTXT );
						return( (STV_PACK_DIM) NULL );
					}

					if ( PDpatch == NULL )
						PDptr = PDpatch = PDtmp;
					else
						PDptr = PDptr->next_dim = PDtmp;
				}

				if ( bindex < F->numData && F->dataPtrs != NULL )
					PDpatch->dataPtr = F->dataPtrs[ bindex ];
					
				if ( PDtop == NULL )
					PDtop = PD = PDpatch;
				else
					PDtop = PDtop->next = PDpatch;
			}
		}

		else
		{
			pindex = D->prank - 1;

			for ( index=(D->dataDim-1) ; index >= 0 ; index-- )
			{
				PDtmp = stv_get_SubRegion_index( F, VRck, index, bindex,
					&pindex );

				if ( PDtmp == NULL )
				{
					sprintf( STVERRTXT,
							"Bad Pack Dimension, Index=%d", index );
					STV_ERROR( "stv_get_SubRegion_indices",
							stvStatusBadParam, STVERRTXT );
					return( (STV_PACK_DIM) NULL );
				}

				if ( PD == NULL )
					PDptr = PD = PDtmp;
				else
					PDptr = PDptr->next_dim = PDtmp;
			}
		}
	}

	/* Oops, Unknown Layout */

	else
	{
		sprintf( STVERRTXT, "Unknown Data Storage Order, ID = %d",
			F->storage_order );

		STV_ERROR( "stv_get_SubRegion_indices", stvStatusBadParam,
			STVERRTXT );

		return( (STV_PACK_DIM) NULL );
	}

	/* Verify Non-Null Packing Space */

	PDpatch = PD;

	PDlast = (STV_PACK_DIM) NULL;

	while ( PDpatch != NULL )
	{
		PDptr = PDpatch;

		while ( PDptr != NULL )
		{
			if ( PDptr->pindices == NULL )
			{
				if ( STV_OPT_VERBOSE & STV_DEBUG_PACK )
				{
					printf( "%s: Freeing Pack Dims, pindices == NULL\n",
							"stv_get_SubRegion_indices()" );
					fflush( stdout );
				}

				if ( PDlast == NULL )
				{
					PD = PDpatch->next;

					PDpatch->next = (STV_PACK_DIM) NULL;

					stv_free_pack_dim_list( &(PDpatch) );
				}

				else
				{
					PDlast->next = PDpatch->next;

					PDpatch->next = (STV_PACK_DIM) NULL;

					stv_free_pack_dim_list( &(PDpatch) );

					PDpatch = PDlast;
				}

				PDptr = (STV_PACK_DIM) NULL;
			}

			else
				PDptr = PDptr->next_dim;
		}

		PDlast = PDpatch;

		if ( PDpatch == NULL )
			PDpatch = PD;
		else
			PDpatch = PDpatch->next;
	}

	/* Return Pack Dimension List */

	return( PD );
}


/* Determine Dimension Index for SubRegion */

STV_PACK_DIM
stv_get_SubRegion_index( F, VR, index, bindex, pindex )
STV_FIELDPTR F;
STV_VIS_REGION VR;
int index;
int bindex;
int *pindex;
{
	STV_PACK_INDEX PXptr;
	STV_PACK_INDEX PX;

	STV_PACK_DIM PD;

	STV_DECOMPPTR D;

	int gnext, lnext;
	int cyclesize;
	int min, max;
	int arraymax;
	int gindex;
	int offset;
	int tmpmax;
	int first;
	int bogus;
	int delta;
	int cell;
	int q, k;

	/* Get Decomposition */

	D = F->decomp;

	/* Create New Pack Dimension */

	PD = stv_create_pack_dim();

	STV_NO_MEM_TYPE_return( PD, "stv_get_SubRegion_index",
		STV_PACK_DIM );

	PD->type = D->axisType[ index ];

	PD->dim = index;

	/* Create Initial Pack Index */

	PX = PD->pindices = stv_create_pack_index();

	STV_NO_MEM_TYPE_return( PX, "stv_get_SubRegion_index",
		STV_PACK_DIM );

	/* Snag Min & Max for this Dimension */

	max = STV_REGION_MAX( VR->region, index );
	min = STV_REGION_MIN( VR->region, index );

	cell = STV_CELL_OF_DIM( VR->cell, index );

	/* Set Starts, Strides & Max based on Axis Type */

	switch ( D->axisType[ index ] )
	{
		case stvBlock:
		{
			/* Array Max */

			arraymax = D->axisInfo[ index ];

			/* Calculate Block Index */

			gindex = ( F->paddr[ *pindex ] * D->axisInfo[ index ] )
				+ D->glb[ index ];

			if ( max >= gindex + D->axisInfo[ index ] )
				PX->max = arraymax;

			else
				PX->max = max - gindex + 1;

			PX->gstart = STV_START_BLOCK_INDEX( gindex, min, cell );

			while ( PX->gstart < min
				&& PX->gstart - gindex + cell < arraymax )
			{
				PX->gstart += cell;
			}

			bogus = stvTrue;

			if ( PX->gstart >= min && PX->gstart <= max )
			{
				PX->gstride = cell;

				PX->lstart = PX->gstart - gindex;
				PX->lstride = cell;

				if ( PX->lstart <= arraymax )
					bogus = stvFalse;
			}

			if ( bogus )
				stv_free_pack_index( &(PD->pindices) );

			if ( F->storage_order == stvColumnMajor )
				(*pindex)++;
			
			else if ( F->storage_order == stvRowMajor )
				(*pindex)--;

			break;
		}

		case stvCyclic:
		{
			/* Snag Array Max Index */

			arraymax = stv_get_proc_dim_size( index, stvCyclic,
				D->axisInfo[ index ],
				D->axisLBounds, D->axisUBounds, D->nbounds,
				D->gub[ index ] - D->glb[ index ] + 1,
				F->paddr[ *pindex ], D->pshape[ *pindex ] );

			if ( arraymax < 0 )
			{
				STV_ERROR( "stv_get_SubRegion_index", arraymax,
					"stv_get_proc_dim_size() failed." );
				
				return( (STV_PACK_DIM) NULL );
			}

			/* Calculate Cyclic Indices for Each Sub-Cycle */

			cyclesize = D->axisInfo[ index ];

			PXptr = (STV_PACK_INDEX) NULL;

			for ( q=0 ; q < cyclesize ; q++ )
			{
				if ( q )
				{
					if ( PD->pindices == NULL )
						PX = PD->pindices = stv_create_pack_index();

					else
					{
						PXptr = PX;

						PX = PX->next = stv_create_pack_index();
					}

					STV_NO_MEM_TYPE_return( PX,
						"stv_get_SubRegion_index", STV_PACK_DIM );
				}

				/* Find Starting Index */

				k = ( ( min - D->glb[ index ] )
					/ ( cyclesize * D->pshape[ *pindex ] ) ) - 1;

				do
				{
					k++;

					PX->gstart = STV_GLOBAL_CYCLE_INDEX( q, k,
						cyclesize, D->glb[ index ],
						F->paddr[ *pindex ], D->pshape[ *pindex ] );
				}
				while ( PX->gstart < min
					|| ( !STV_DIVISIBLE( PX->gstart - min, cell )
						&& PX->gstart <= max ) );

				/* Verify Start Within Region */

				bogus = stvTrue;

				if ( PX->gstart <= max )
				{
					PX->lstart = STV_LOCAL_CYCLE_INDEX( q, k,
						cyclesize );

					/* Verify Start Within Local Portion */

					if ( PX->lstart < arraymax )
					{
						bogus = stvFalse;

						/* Find Next Index */

						do
						{
							k++;

							gnext = STV_GLOBAL_CYCLE_INDEX( q, k,
								cyclesize, D->glb[ index ],
								F->paddr[ *pindex ],
								D->pshape[ *pindex ] );
						}
						while ( !STV_DIVISIBLE( gnext - min, cell )
							&& gnext <= max );

						/* Calculate Stride */

						if ( gnext <= max )
						{
							PX->gstride = gnext - PX->gstart;

							lnext = STV_LOCAL_CYCLE_INDEX( q, k,
								cyclesize );
						
							PX->lstride = lnext - PX->lstart;

							/* Find Max Index */

							delta = PX->gstride
								/ ( cyclesize * D->pshape[ *pindex ] );

							do
							{
								k += delta;

								gnext = STV_GLOBAL_CYCLE_INDEX( q, k,
									cyclesize, D->glb[ index ],
									F->paddr[ *pindex ],
									D->pshape[ *pindex ] );
							}
							while ( gnext <= max );

							PX->max = STV_LOCAL_CYCLE_INDEX( q, k,
								cyclesize );
						}

						else
						{
							PX->gstride = max - min + 1;

							PX->lstride = arraymax;

							PX->max = arraymax;
						}
					}
				}

				if ( bogus )
				{
					if ( PXptr != NULL )
					{
						stv_free_pack_index( &(PXptr->next) );

						PX = PXptr;
					}

					else
						stv_free_pack_index( &(PD->pindices) );
				}
			}

			if ( bogus && PXptr == NULL && PD->pindices != NULL )
				stv_free_pack_index( &(PD->pindices) );

			if ( F->storage_order == stvColumnMajor )
				(*pindex)++;
			
			else if ( F->storage_order == stvRowMajor )
				(*pindex)--;

			break;
		}

		case stvExplicit:
		{
			/* Calculate Explicit Index */

			first = MAX( min, D->axisLBounds[ bindex ][ index ] );

			PX->gstart = STV_START_BLOCK_INDEX( first, min, cell );
			PX->gstride = cell;

			PX->lstart = PX->gstart - D->axisLBounds[ bindex ][ index ];
			PX->lstride = cell;

			tmpmax = MIN( max, D->axisUBounds[ bindex ][ index ] );

			PX->max = tmpmax - D->axisLBounds[ bindex ][ index ] + 1;

			/* Check for Bogosity */

			bogus = stvTrue;

			if ( PX->lstart < PX->max )
				bogus = stvFalse;

			if ( bogus )
				stv_free_pack_index( &(PD->pindices) );

			if ( F->storage_order == stvColumnMajor )
				(*pindex)++;
		
			else if ( F->storage_order == stvRowMajor )
				(*pindex)--;

			break;
		}

		case stvCollapse:
		{
			/* Calculate Collapse Index */

			PX->gstart = min;
			PX->gstride = cell;

			PX->lstart = PX->gstart - D->glb[ index ];
			PX->lstride = cell;

			PX->max = max - D->glb[ index ] + 1;

			break;
		}
	}

	return( PD );
}


/* Dump Bounds of Local Task's Portion of Global Field Array */

int
stv_dumpLocalFieldBounds( id, verbose )
int id;
int verbose;
{
	STV_PARTICLEPTR PT;

	STV_DECOMPPTR D;

	STV_FIELDPTR F;

	int cyclesize;
	int pindex;
	int gstart;
	int gend;
	int b;
	int k;
	int i;

	/* Lookup Field */

	if ( (F = (STV_FIELDPTR) stv_findField( &STVFIELDS, id ))
		== (STV_FIELDPTR) NULL )
	{
		sprintf( STVERRTXT, "Unknown Field ID %d", id );

		STV_ERROR( "stv_dumpLocalFieldBounds", stvStatusBadParam,
			STVERRTXT );

		return( stvStatusBadParam );
	}

	/* Check for Particle Field */

	if ( F->particle != NULL )
	{
		PT = F->particle;

		/* Dump Global Array Bounds */

		if ( verbose )
		{
			printf(
			"\nGlobal Array Bounds for Particle Field \"%s\":\n\n\t",
				F->name );
		}

		else
			printf( "Global Array \"%s\":  ", F->name );

		for ( i=0 ; i < PT->dataDim ; i++ )
			printf( "[ %d - %d ]", PT->glb[i], PT->gub[i] );

		if ( verbose )
			printf( "\n\n" );
		
		else
			printf( "\n" );
	}

	/* Regular Decomposition */

	else if ( F->decomp != NULL )
	{
		D = F->decomp;

		/* Dump Global Array Bounds */

		if ( verbose )
		{
			printf( "\nGlobal Array Bounds for Field \"%s\":\n\n\t",
				F->name );
		}

		else
			printf( "Global Array \"%s\":  ", F->name );

		for ( i=0 ; i < D->dataDim ; i++ )
			printf( "[ %d - %d ]", D->glb[i], D->gub[i] );
		
		if ( verbose )
			printf( "\n\nLogical array of processors is:\n\n\t" );
		
		else
			printf( "\nProcessor Array:  " );

		for ( i=0 ; i < D->prank ; i++ )
		{
			printf( "%d%s", D->pshape[i],
				( i < ( D->prank - 1 ) ? "  X  " : "" ) );
		}

		if ( verbose )
			printf( "\n\nLocal processor address is:\n\n\t{ ");

		else
			printf( ",  Local Address:  { " );

		for ( i=0 ; i < D->prank ; i++ )
		{
			printf( "%d%s", F->paddr[i],
				( i < ( D->prank - 1 ) ? ", " : " }" ) );
		}

		/* matching } */

		/* Dump Local Decomp Bounds */

		if ( verbose )
		{
			printf(
				"\n\nLocal Portion of Global Space for Field \"%s\":",
				F->name );
		}

		pindex = 0;

		for ( i=0 ; i < D->dataDim ; i++ )
		{
			if ( verbose )
				printf( "\n\n\tBounds for Axis #%d:\n\n\t\t", i + 1 );
			
			else
				printf( "\nAxis #%d:  ", i + 1 );

			switch ( D->axisType[i] )
			{
				case stvBlock:
				{
					/* Calculate Block Index */

					gstart = ( F->paddr[ pindex ] * D->axisInfo[i] )
						+ D->glb[i];

					gend = gstart + D->axisInfo[i] - 1;

					printf( "[ %d - %d ]", gstart, gend );

					pindex++;

					break;
				}

				case stvCyclic:
				{
					/* Calculate Cyclic Indices for Each Sub-Cycle */

					cyclesize = D->axisInfo[i];

					/* Find Starting Index */

					k = 0;

					do
					{
						gstart = STV_GLOBAL_CYCLE_INDEX( 0, k,
							cyclesize, D->glb[i],
							F->paddr[ pindex ], D->pshape[ pindex ] );

						gend = gstart + cyclesize - 1;

						gend = ( gend > D->gub[i] ) ?
							D->gub[i] : gend;

						if ( gstart <= D->gub[i] )
						{
							if ( gend != gstart )
								printf( "[ %d - %d ]", gstart, gend );

							else
								printf( "[ %d ]", gstart );
						}

						k++;
					}
					while ( gstart <= D->gub[i] );

					pindex++;

					break;
				}

				case stvExplicit:
				{
					/* Calculate Explicit Index */

					for ( b=0 ; b < D->nbounds ; b++ )
					{
						gstart = D->axisLBounds[b][i];

						gend = D->axisUBounds[b][i];

						printf( "[ %d - %d ]", gstart, gend );
					}

					pindex++;
			
					break;
				}

				case stvCollapse:
				{
					/* Calculate Collapse Index */

					gstart = D->glb[i];

					gend = D->gub[i];

					printf( "[ %d - %d ]", gstart, gend );

					break;
				}

				default:
				{
					sprintf( STVERRTXT,
						"Unknown Axis Type %d for Axis #%d",
						D->axisType[i], i + 1 );

					STV_ERROR( "stv_dumpLocalFieldBounds",
						stvStatusBadParam, STVERRTXT );

					return( stvStatusBadParam );
				}
			}
		}

		if ( verbose )
			printf( "\n\n" );

		else
			printf( "\n" );
	}

	return( stvStatusOk );
}


/* Pack Data Elements for SubRegion */

int
stv_pack_elems_SubRegion( mp, sbuf, PD, F, type, need_descriptor )
STV_MP_API *mp;
int sbuf;
STV_PACK_DIM PD;
STV_FIELDPTR F;
int type;
int need_descriptor;
{
	STV_PACK_INDEX PX;

	STV_PACK_DIM PDptr;

	STV_VALUE offptr;
	STV_VALUE ptr;

	int offset;
	int bindex;
	int num;
	int tmp;
	int cc;

	bindex = 0;

	while ( PD != NULL )
	{
		/* Initialize Indices */

		stv_init_pack_indices( PD );

		/* Iterate & Pack Stuff */

		do
		{
			/* Pack End of Data Marker */

			tmp = stvMarkerFieldData;
			mp->pack_int( sbuf, &tmp, 1, 1 );

			/* Pack Offset Index */

			if ( need_descriptor )
				stv_pack_offset_index( mp, sbuf, PD );

			/* Get Iterative Index Info */

			PX = PD->current;

			/* Calculate Number of Elements to Pack */

			num = (int) ceil( (double) ( PX->max - PX->lstart )
				/ (double) PX->lstride );

			/* Pack Unpack Info */

			if ( STV_OPT_VERBOSE & STV_DEBUG_PACK_INDEX )
			{
				printf( "%s: packing gstart=%d gstride=%d num=%d\n",
					"stv_pack_elems_SubRegion()",
					PX->gstart, PX->gstride, num );
				fflush( stdout );
			}

			if ( need_descriptor )
			{
				mp->pack_int( sbuf, &(PX->gstart), 1, 1 );
				mp->pack_int( sbuf, &(PX->gstride), 1, 1 );
				mp->pack_int( sbuf, &(num), 1, 1 );
			}

			/* Pack Data */

			offset = stv_get_pack_offset( PD, F, bindex, &offptr );

			ptr = stv_get_typed_ptr_offset( offptr, offset + PX->lstart,
				F->type );

			if ( STV_OPT_VERBOSE & STV_DEBUG_PACK_INDEX )
			{
				printf( "%s: data ptr=0x%lx ",
					"stv_pack_elems_SubRegion()", (long) ptr );
				printf(
					"(offptr=0x%lx offset=%d PX->lstart=%d F->type=%d ",
					(long) offptr, offset, PX->lstart, F->type );
				printf( "PX->lstride=%d).\n", PX->lstride );
				fflush( stdout );
			}

			if ( ptr == NULL )
			{
				STV_ERROR( "stv_pack_elems_SubRegion",
					stvStatusBadParam,
					"stv_get_typed_ptr_offset() failed." );

				return( stvStatusBadParam );
			}

			if ( F->type != type )
			{
				if ( (cc = stv_pack_converted_type( mp, sbuf,
						ptr, num, PX->lstride, F->type, type ))
					!= stvStatusOk )
				{
					STV_ERROR( "stv_pack_elems_SubRegion", cc,
						"Converted Type Pack Failed." );
				
					return( cc );
				}
			}

			else
			{
				if ( (cc = stv_pack_by_type( mp, sbuf,
					ptr, num, PX->lstride, type )) != stvStatusOk )
				{
					STV_ERROR( "stv_pack_elems_SubRegion", cc,
						"Typed Pack Failed." );

					return( cc );
				}
			}

			/* Pack End of Data Marker */

			tmp = stvMarkerFieldDataEnd;
			mp->pack_int( sbuf, &tmp, 1, 1 );
		}
		while ( stv_incr_pack_indices( PD ) );

		PD = PD->next;

		bindex++;
	}

	/* A.O.K. */

	return( stvStatusOk );
}


/* Initialize Pack Indices */

void
stv_init_pack_indices( PD )
STV_PACK_DIM PD;
{
	STV_PACK_DIM ptr;

	ptr = PD;

	while ( ptr != NULL )
	{
		ptr->current = ptr->pindices;

		ptr->gindex = ptr->current->gstart;
		ptr->lindex = ptr->current->lstart;

		ptr = ptr->next_dim;
	}
}


/* Pack N - 1 Offset Index Values */

int
stv_pack_offset_index( mp, sbuf, PD )
STV_MP_API *mp;
int sbuf;
STV_PACK_DIM PD;
{
	STV_PACK_DIM ptr;

	if ( PD == NULL )
	{
		STV_ERROR( "stv_pack_offset_index", stvStatusBadParam,
			"Null Pack Dimension Array" );

		return( stvStatusBadParam );
	}

	ptr = PD;

	/* Skip First Iterative Index */

	ptr = ptr->next_dim;

	/* Pack In Rest */

	if ( STV_OPT_VERBOSE & STV_DEBUG_PACK_INDEX )
		printf( "stv_pack_offset_index():  [X]" );

	while ( ptr != NULL )
	{
		if ( STV_OPT_VERBOSE & STV_DEBUG_PACK_INDEX )
			printf( "[%d]", ptr->gindex );

		mp->pack_int( sbuf, &(ptr->gindex), 1, 1 );

		ptr = ptr->next_dim;
	}

	if ( STV_OPT_VERBOSE & STV_DEBUG_PACK_INDEX )
	{
		printf( "\n" );
		fflush( stdout );
	}

	return( stvStatusOk );
}


/* Get Pack Offset Into Data Array */

int
stv_get_pack_offset( PD, F, bindex, offptr )
STV_PACK_DIM PD;
STV_FIELDPTR F;
int bindex;
STV_VALUE *offptr;
{
	STV_PACK_DIM ptr;

	int dindex;
	int offset;
	int size;

	/* General Case */

	if ( offptr != NULL )
		*offptr = F->dataPtrs[0];

	/* Trivial Case */

	if ( PD == NULL )
		return( 0 );

	/* Calculate Effective Data Index from Bounds Index */

	dindex = ( bindex < F->numData ) ? bindex : 0;

	/* Init Offset & Size */


	if ( F->arrayOffset != NULL && F->arrayOffset[ dindex ] != NULL )
		offset = F->arrayOffset[ dindex ][ PD->dim ];

	else
		offset = 0;

	size = F->arrayDecl[ dindex ][ PD->dim ];

	if ( STV_OPT_VERBOSE & STV_DEBUG_PACK_INDEX )
	{
		printf( "%s: offset=%d size=%d\n",
			"stv_get_pack_offset()", offset, size );
		fflush( stdout );
	}

	/* Check for Explicit Patch Data Ptr */

	if ( PD->dataPtr != NULL && offptr != NULL )
		*offptr = PD->dataPtr;
		
	/* Skip First Iterative Index */

	ptr = PD->next_dim;

	/* Add Up Rest */

	while ( ptr != NULL )
	{
		if ( F->arrayOffset != NULL
				&& F->arrayOffset[ dindex ] != NULL )
		{
			offset += ( ptr->lindex
				+ F->arrayOffset[ dindex ][ ptr->dim ] ) * size;
		}

		else
			offset += ptr->lindex * size;

		size *= F->arrayDecl[ dindex ][ ptr->dim ];

		if ( STV_OPT_VERBOSE & STV_DEBUG_PACK_INDEX )
		{
			printf( "\t-> offset=%d size=%d\n", offset, size );
			fflush( stdout );
		}

		ptr = ptr->next_dim;
	}

	return( offset );
}


/* Increment Packing Indices */

int
stv_incr_pack_indices( PD )
STV_PACK_DIM PD;
{
	STV_PACK_DIM PDptr;

	int first;
	int done;

	if ( STV_OPT_VERBOSE & STV_DEBUG_PACK_INDEX )
		printf( "stv_incr_pack_indices():  " );

	if ( PD == NULL )
	{
		if ( STV_OPT_VERBOSE & STV_DEBUG_PACK_INDEX )
		{
			printf( "Null Pack Dim.\n" );
			fflush( stdout );
		}
		
		return( stvFalse );
	}

	/* Skip Over First Index Unless Cyclic Steps Needed */

	first = 1;

	if ( PD->current->next == NULL && PD->current == PD->pindices )
	{
		if ( STV_OPT_VERBOSE & STV_DEBUG_PACK_INDEX )
			printf( "SKIP,  " );

		PDptr = PD->next_dim;

		first = 0;
	}
	
	/* Else ACTUALLY SET the PDptr to the First Index...  D-Oh! */

	else
		PDptr = PD;

	/* Verify there's anything to increment... */

	if ( PDptr == NULL )
	{
		if ( STV_OPT_VERBOSE & STV_DEBUG_PACK_INDEX )
		{
			printf( "Null Pack Dim.\n" );
			fflush( stdout );
		}
		
		return( stvFalse );
	}

	/* Excrement Away! */

	do
	{
		if ( PDptr->current == NULL )
		{
			if ( STV_OPT_VERBOSE & STV_DEBUG_PACK_INDEX )
			{
				printf( "Current Pack Index Null.\n" );
				fflush( stdout );
			}
		
			return( stvFalse );
		}

		if ( !first )
		{
			PDptr->gindex += PDptr->current->gstride;
			PDptr->lindex += PDptr->current->lstride;

			if ( STV_OPT_VERBOSE & STV_DEBUG_PACK_INDEX )
			{
				printf( "Incr gindex[%d] -> %d, lindex[%d] -> %d,  ",
						PDptr->dim, PDptr->gindex,
						PDptr->dim, PDptr->lindex );
			}
		}

		/* Check for Bump */

		done = stvTrue;

		if ( PDptr->lindex >= PDptr->current->max || first )
		{
			if ( STV_OPT_VERBOSE & STV_DEBUG_PACK_INDEX )
				printf( "BUMP(max=%d),  ", PDptr->current->max );

			/* Check for Cyclic/Explicit Multi-Index Bump */

			if ( PDptr->current->next != NULL )
			{
				if ( STV_OPT_VERBOSE & STV_DEBUG_PACK_INDEX )
					printf( "Next,  " );

				PDptr->current = PDptr->current->next;
			}

			else
			{
				/* Check for Cyclic/Explicit Multi-Index Reset */

				if ( PDptr->current != PDptr->pindices )
				{
					if ( STV_OPT_VERBOSE & STV_DEBUG_PACK_INDEX )
						printf( "Reset,  " );

					PDptr->current = PDptr->pindices;
				}

				done = stvFalse;
			}

			/* Reset Index */

			PDptr->gindex = PDptr->current->gstart;
			PDptr->lindex = PDptr->current->lstart;

			if ( STV_OPT_VERBOSE & STV_DEBUG_PACK_INDEX )
			{
				printf( "gindex[%d]=%d, lindex[%d]=%d,  ",
						PDptr->dim, PDptr->gindex,
						PDptr->dim, PDptr->lindex );
			}

			if ( !done )
			{
				PDptr = PDptr->next_dim;

				if ( !PDptr )
				{
					if ( STV_OPT_VERBOSE & STV_DEBUG_PACK_INDEX )
						printf( "End of Pack Dims.\n" );

					return( stvFalse );
				}
			}
		}

		first = 0;
	}
	while ( !done );

	if ( STV_OPT_VERBOSE & STV_DEBUG_PACK_INDEX )
	{
		printf( "Done.\n" );
		fflush( stdout );
	}

	return( stvTrue );
}


/* Convert the Data to Requested Type & Pack */

int
stv_pack_converted_type( mp, sbuf,
		data, num, stride, data_type, pack_type )
STV_MP_API *mp;
int sbuf;
STV_VALUE data;
int num;
int stride;
int data_type;
int pack_type;
{
	STV_VALUE copy_data;
	STV_VALUE src;
	STV_VALUE dst;

	int cc;
	int i;

	/* Allocate Copy Storage */

	copy_data = (STV_VALUE) MALLOC( (STV_SIZE_T) num
		* STV_SIZEOF( pack_type ) );
	
	if ( !copy_data )
	{
		STV_ERROR( "stv_pack_converted_type", stvStatusNoMem,
			"Couldn't Allocate Conversion Buffer" );
		
		return( stvStatusNoMem );
	}

	/* Copy Data & Cast to Pack Type */

	src = data;
	dst = copy_data;

	for ( i=0 ; i < num ; i++ )
	{
		/* Convert One Data Element */

		if ( (cc = stv_copy_converted_data( src, data_type,
			dst, pack_type )) != stvStatusOk )
		{
			STV_ERROR( "stv_pack_converted_type", cc,
				"Converted Type Copy Failed." );

			FREE( copy_data );

			return( cc );
		}

		/* Advance Ptrs */

		src = stv_get_typed_ptr_offset( src, stride, data_type );

		dst = stv_get_typed_ptr_offset( dst, 1, pack_type );

		if ( src == NULL || dst == NULL )
		{
			STV_ERROR( "stv_pack_converted_type", stvStatusBadParam,
				"stv_get_typed_ptr_offset() failed." );

			FREE( copy_data );

			return( stvStatusBadParam );
		}
	}

	/* Pack Copied Converted Data, No Longer Strided */

	if ( (cc = stv_pack_by_type( mp, sbuf,
		copy_data, num, 1, pack_type )) != stvStatusOk )
	{
		STV_ERROR( "stv_pack_converted_type", cc,
			"Typed Pack Failed." );

		FREE( copy_data );

		return( cc );
	}

	/* Free Copy */

	FREE( copy_data );

	/* A.O.K. */

	return( stvStatusOk );
}


/* Dump the Pack Dimension Indices Info */

void
stv_dump_pack_dims( PD, field_storage_order, view_storage_order )
STV_PACK_DIM PD;
int field_storage_order;
int view_storage_order;
{
	STV_PACK_INDEX PX;

	STV_PACK_DIM PDptr;

	int i;

	printf( "Desired Data Storage Order is %s:\n",
		stv_storage_order_str( view_storage_order ) );

	printf( "Field Data Storage Order is %s:\n",
		stv_storage_order_str( field_storage_order ) );

	PDptr = PD;

	i = 0;

	while ( PDptr != NULL )
	{
		PX = PDptr->pindices;

		printf(
		"\tDIM #%d: gstart=%d gstride=%d lstart=%d lstride=%d max=%d\n",
			i++, PX->gstart, PX->gstride, PX->lstart, PX->lstride,
			PX->max );

		PX = PX->next;

		while ( PX != NULL )
		{
			printf(
			"\t\tgstart=%d gstride=%d lstart=%d lstride=%d max=%d\n",
				PX->gstart, PX->gstride, PX->lstart, PX->lstride,
				PX->max );

			PX = PX->next;
		}
		
		PDptr = PDptr->next_dim;
	}
}


/* Unpack Data Elements from SubRegion */

int
stv_unpack_elems_SubRegion( mp, rbuf, VFG, V )
STV_MP_API *mp;
int rbuf;
STV_VIEW_FIELD_GROUP VFG;
STV_VIEWER V;
{
	STV_VIEW_DESC_INDEX VFI;

	STV_VIEW_DESCRIPTOR VFD;

	STV_VIEW_FIELD VF;

	STV_VIEW_PARAM VP;

	STV_DECOMPPTR D;

	STV_PARAMPTR P;

	STV_VIS_REGION VRck;
	STV_VIS_REGION VR;

	STV_VALUE ptr;

	double dummy[10];

	int storage_stride;
	int i, j, k;
	int nparams;
	int nfields;
	int nparts;
	int index;
	int inst;
	int step;
	int num;
	int tmp;
	int cc;
	int id;

	/* Unpack Instance Number (Nodeid) */

	mp->unpack_int( rbuf, &inst, 1, 1 );

	/* Unpack SubRegion Marker */

	mp->unpack_int( rbuf, &tmp, 1, 1 );

	if ( tmp != stvMarkerSubRegion )
	{
		sprintf( STVERRTXT, "Parse Error: Expected %s = %d, Got %d",
			"stvMarkerSubRegion", stvMarkerSubRegion, tmp );

		STV_ERROR( "stv_unpack_elems_SubRegion",
			stvStatusBadParam, STVERRTXT );

		return( stvStatusBadParam );
	}

	/* Unpack # of Standard Fields */

	mp->unpack_int( rbuf, &nfields, 1, 1 );

	if ( STV_OPT_VERBOSE & STV_DEBUG_PACK )
	{
		printf( "%s:  Unpacking %d Standard Fields\n",
			"stv_unpack_elems_SubRegion", nfields );
		fflush( stdout );
	}

	/* Unpack Fields */

	for ( i=0 ; i < nfields ; i++ )
	{
		/* Unpack Field Marker */

		mp->unpack_int( rbuf, &tmp, 1, 1 );

		if ( tmp != stvMarkerField )
		{
			sprintf( STVERRTXT, "Parse Error: Expected %s = %d, Got %d",
				"stvMarkerField", stvMarkerField, tmp );

			STV_ERROR( "stv_unpack_elems_SubRegion",
				stvStatusBadParam, STVERRTXT );

			return( stvStatusBadParam );
		}

		/* Unpack Field ID */

		mp->unpack_int( rbuf, &id, 1, 1 );

		if ( STV_OPT_VERBOSE & STV_DEBUG_PACK )
		{
			printf( "Unpacked Field ID = %d\n", id );
			fflush( stdout );
		}

		/* Get Field */

		VF = stv_get_view_field_from_group_id( VFG, id, &index );

		if ( VF == NULL )
		{
			sprintf( STVERRTXT, "Unknown Field ID %d", id );

			STV_ERROR( "stv_unpack_elems_SubRegion",
				stvStatusBadParam, STVERRTXT );

			return( stvStatusBadParam );
		}

		if ( STV_OPT_VERBOSE & STV_DEBUG_PACK )
		{
			printf( "Field is \"%s\"\n", VF->field->name );
			fflush( stdout );
		}

		/* Get Decomposition */
	
		D = VF->field->decomp;
	
		/* Unpack Descriptor Flag */

		VFD = VF->descriptors[inst];

		if ( VFD == NULL )
		{
			VFD = VF->descriptors[inst] = stv_create_view_descriptor();
			STV_NO_MEM_return( VFD, "stv_unpack_elems_SubRegion" );
		}

		mp->unpack_int( rbuf, &(VFD->need_descriptor), 1, 1 );

		if ( STV_OPT_VERBOSE & STV_DEBUG_PACK )
		{
			printf( "\tneed_descriptor=%d\n", VFD->need_descriptor );
			fflush( stdout );
		}

		/* Unpack Data Type Check */

		if ( VFD->need_descriptor )
			mp->unpack_int( rbuf, &(VFD->type), 1, 1 );

		if ( VFG->view_types[index] != VFD->type )
		{
			sprintf( STVERRTXT, "Incorrect Unpack Type %s (%d) for %s",
				stv_data_type_str( VFD->type ), VFD->type,
				VF->field->name );

			STV_ERROR( "stv_unpack_elems_SubRegion",
				stvStatusBadParam, STVERRTXT );

			return( stvStatusBadParam );
		}

		/* Unpack Data Storage Order */

		if ( VFD->need_descriptor )
			mp->unpack_int( rbuf, &(VFD->field_storage_order), 1, 1 );

		/* Unpack Data Dimension Check */

		if ( VFD->need_descriptor )
			mp->unpack_int( rbuf, &(VFD->dim), 1, 1 );

		if ( VFD->dim != D->dataDim )
		{
			sprintf( STVERRTXT, "Dim Mismatch %d/%d for %s",
				VFD->dim, D->dataDim, VF->field->name );

			STV_ERROR( "stv_unpack_elems_SubRegion",
				stvStatusBadParam, STVERRTXT );

			return( stvStatusBadParam );
		}

		/* Unpack Current Field Time */

		mp->unpack_int( rbuf, &(VF->field->time_base), 1, 1 );
		mp->unpack_int( rbuf, &(VF->field->time_units), 1, 1 );

		if ( STV_OPT_VERBOSE & STV_DEBUG_TIME )
		{
			printf( "Field %s:  time = ", VF->field->name );
			stv_dump_base_time_str( stdout,
					VF->field->time_base, VF->field->time_units,
					stvTrue );
			fflush( stdout );
		}

		/* Verify Specific Vis Regions for This Field */

		VRck = stv_create_vis_region();

		stv_copy_vis_region( VRck, VFG->visregion );

		stv_check_region( VF->field, VRck );

		/* Unpack Vis Region Marker */

		mp->unpack_int( rbuf, &tmp, 1, 1 );

		/* Initialize View Descriptor Index for Unpack */

		if ( VFD->need_descriptor )
			stv_free_view_desc_index( &(VFD->dindices) );
		
		VFI = VFD->dindices;

		/* Handle All Vis Regions */

		VR = VRck;

		j = 0;

		while ( VR != NULL )
		{
			if ( tmp == stvMarkerFieldEnd )
			{
				if ( STV_OPT_VERBOSE & STV_DEBUG_PACK )
				{
					if ( !j )
						printf( "\tNo Data\n" );
					else
						printf( "\tEnd of Data\n" );
					fflush( stdout );
				}

				break;
			}

			if ( tmp != stvMarkerVisRegion )
			{
				sprintf( STVERRTXT,
					"Parse Error: Expected %s = %d, Got %d",
					"stvMarkerVisRegion", stvMarkerVisRegion, tmp );

				STV_ERROR( "stv_unpack_elems_SubRegion",
					stvStatusBadParam, STVERRTXT );

				return( stvStatusBadParam );
			}

			if ( STV_OPT_VERBOSE & STV_DEBUG_PACK_INDEX )
			{
				printf( "VRck: " );
				for ( k=0 ; k < VF->field->decomp->dataDim ; k++ )
					printf( "[%d - %d]",
						STV_REGION_MIN( VR->region, k ),
						STV_REGION_MAX( VR->region, k ) );
				printf( "\n" );
			}

			/* Unpack Field Data Marker */

			mp->unpack_int( rbuf, &tmp, 1, 1 );

			if ( tmp != stvMarkerFieldData )
			{
				sprintf( STVERRTXT,
					"Parse Error: Expected %s = %d, Got %d",
					"stvMarkerFieldData", stvMarkerFieldData, tmp );

				STV_ERROR( "stv_unpack_elems_SubRegion",
					stvStatusBadParam, STVERRTXT );

				return( stvStatusBadParam );
			}

			/* Unpack Field Data */

			while ( tmp == stvMarkerFieldData )
			{
				/* Unpack Descriptor Index */

				if ( VFD->need_descriptor )
				{
					/* Create Next View Descriptor Index */

					if ( VFI == NULL )
						VFI = VFD->dindices =
							stv_create_view_desc_index();
					else
						VFI = VFI->next = stv_create_view_desc_index();

					/* Get Rest of Offset Indices */

					for ( k=1 ; k < D->dataDim ; k++ )
						mp->unpack_int( rbuf,
							&(VFI->indices[k]), 1, 1 );

					if ( STV_OPT_VERBOSE & STV_DEBUG_PACK_INDEX )
					{
						printf( "Unpacking Index [X]" );
						for ( k=1 ; k < D->dataDim ; k++ )
							printf( "[%d]", VFI->indices[k] );
						printf( ": " );
					}

					/* Get Unpack Info */

					mp->unpack_int( rbuf, &(VFI->indices[0]), 1, 1 );
					mp->unpack_int( rbuf, &(VFI->stride), 1, 1 );
					mp->unpack_int( rbuf, &(VFI->num), 1, 1 );

					if ( STV_OPT_VERBOSE & STV_DEBUG_PACK_INDEX )
					{
						printf( "gstart=%d gstride=%d num=%d\n",
							VFI->indices[0], VFI->stride, VFI->num );
						fflush( stdout );
					}

					/* Tweak Stride */

					VFI->first = stv_get_unpack_offset( VFI->indices,
						VFG->field_arrayDecls[index][j],
						D, VR, VFG->view_storage_orders[index],
						VFD->field_storage_order );

					storage_stride = stv_get_storage_stride(
						VFG->field_arrayDecls[index][j], D,
						VFG->view_storage_orders[index],
						VFD->field_storage_order );

					VFI->stride *= storage_stride;

					if ( STV_OPT_FULL_UNPACK )
						step = 1;

					else
					{
						if ( VFD->field_storage_order
								== stvColumnMajor )
							step = STV_CELL_OF_DIM( VR->cell, 0 );

						else if ( VFD->field_storage_order
								== stvRowMajor )
						{
							step = STV_CELL_OF_DIM( VR->cell,
								D->dataDim - 1 );
						}

						else
						{
							sprintf( STVERRTXT,
								"Unknown View Storage Order %d",
								VFD->field_storage_order );

							STV_ERROR( "stv_unpack_elems_SubRegion",
								stvStatusBadParam, STVERRTXT );

							return( stvStatusBadParam );
						}
					}

					VFI->stride /= step;

					if ( STV_OPT_VERBOSE & STV_DEBUG_PACK_INDEX )
					{
						printf(
						"\tfirst=%d stride=%d ss=%d step=%d type=%d\n",
							VFI->first, VFI->stride, storage_stride,
							step, VFD->type );
						fflush( stdout );
					}
				}

				/* Get Unpack Ptr */

				ptr = stv_get_typed_ptr_offset(
					VFG->field_values[index][j], VFI->first,
					VFD->type );

				if ( ptr == NULL )
				{
					STV_ERROR( "stv_unpack_elems_SubRegion",
						stvStatusBadParam,
						"stv_get_typed_ptr_offset() failed." );

					return( stvStatusBadParam );
				}

				if ( STV_OPT_VERBOSE & STV_DEBUG_PACK_INDEX )
				{
					printf(
						"Data ptr = %lx (field_values[%d][%d] = %lx)\n",
						(long) ptr, index, j,
						(long) VFG->field_values[index][j] );
					fflush( stdout );
				}

				/* Unpack Typed Data */

				if ( (cc = stv_unpack_by_type( mp, rbuf,
						ptr, VFI->num, VFI->stride, VFD->type ))
					!= stvStatusOk )
				{
					STV_ERROR( "stv_unpack_elems_SubRegion", cc,
						"Typed Unpack Failed." );

					return( cc );
				}

				/* Unpack End of Data Marker */

				mp->unpack_int( rbuf, &tmp, 1, 1 );

				if ( tmp != stvMarkerFieldDataEnd )
				{
					sprintf( STVERRTXT,
						"Parse Error: Expected %s = %d, Got %d",
						"stvMarkerFieldDataEnd", stvMarkerFieldDataEnd,
						tmp );

					STV_ERROR( "stv_unpack_elems_SubRegion",
						stvStatusBadParam, STVERRTXT );

					return( stvStatusBadParam );
				}

				/* Unpack Next Marker */

				mp->unpack_int( rbuf, &tmp, 1, 1 );

				/* Increment View Descriptor Index */

				if ( !(VFD->need_descriptor) )
					VFI = VFI->next;
			}

			VR = VR->next;

			j++;
		}

		stv_free_vis_region( &VRck );

		if ( tmp != stvMarkerFieldEnd )
		{
			sprintf( STVERRTXT, "Parse Error: Expected %s = %d, Got %d",
				"stvMarkerFieldEnd", stvMarkerFieldEnd, tmp );

			STV_ERROR( "stv_unpack_elems_SubRegion", stvStatusBadParam,
				STVERRTXT );

			return( stvStatusBadParam );
		}
	}

	/* Unpack # of Particle Fields */

	mp->unpack_int( rbuf, &nparts, 1, 1 );

	if ( STV_OPT_VERBOSE & STV_DEBUG_PACK )
	{
		printf( "%s:  Unpacking %d Particle Fields\n",
			"stv_unpack_elems_SubRegion", nparts );
		fflush( stdout );
	}

	/* Unpack Particle Fields */

	for ( i=0 ; i < nparts ; i++ )
	{
		if ( (cc = stv_unpack_elems_SubRegion_Particles( mp, rbuf,
			VFG )) != stvStatusOk )
		{
			return( cc );
		}
	}

	/* Unpack End of SubRegion Marker */

	mp->unpack_int( rbuf, &tmp, 1, 1 );

	if ( tmp != stvMarkerSubRegionEnd )
	{
		sprintf( STVERRTXT, "Parse Error: Expected %s = %d, Got %d",
			"stvMarkerSubRegionEnd", stvMarkerSubRegionEnd, tmp );

		STV_ERROR( "stv_unpack_elems_SubRegion",
			stvStatusBadParam, STVERRTXT );

		return( stvStatusBadParam );
	}

	/* Check for Parameter Values Update Section */

	mp->unpack_int( rbuf, &tmp, 1, 1 );

	if ( tmp == stvMarkerParamValues )
	{
		/* Unpack # of Parameter Value Updates */

		mp->unpack_int( rbuf, &nparams, 1, 1 );

		if ( STV_OPT_VERBOSE & STV_DEBUG_PACK )
		{
			printf( "%s:  Unpacking %d Parameter Value Updates\n",
				"stv_unpack_elems_SubRegion", nparams );
			fflush( stdout );
		}

		/* Unpack Parameter Value Updates */

		for ( i=0 ; i < nparams ; i++ )
		{
			mp->unpack_int( rbuf, &id, 1, 1 );

			VP = stv_get_view_param_id( id, V->vparams, V->nvparams );

			P = VP->param;

			if ( VP != NULL && P != NULL )
			{
				mp->unpack_int( rbuf, &num, 1, 1 );

				if ( num != P->num )
				{
					sprintf( STVERRTXT,
						"%s:  \"%s\" Num Mismatch (%d != %d) - Ignore.",
						"Parameter Value Update", P->name,
						num, P->num );

					STV_ERROR( "stv_unpack_elems_SubRegion",
						stvStatusBadParam, STVERRTXT );

					return( stvStatusBadParam );
				}

				for ( k=0 ; k < num ; k++ )
				{
					ptr = ( VP->token == stvSteerToken )
						? ( (STV_VALUE) dummy ) : ( P->vars[k] );

					if ( (cc = stv_unpack_by_type( mp, rbuf,
							ptr, 1, 1, P->types[k] & stvTypeMask ))
						!= stvStatusOk )
					{
						sprintf( STVERRTXT,
						"%s: \"%s\" Typed Unpack Failed (%d of %d).",
							"Parameter Value Update", P->name,
							k, num );

						STV_ERROR( "stv_unpack_elems_SubRegion", cc,
							STVERRTXT );

						return( cc );
					}
				}
			}
		}

		/* Unpack End of SubRegion Marker */

		mp->unpack_int( rbuf, &tmp, 1, 1 );

		if ( tmp != stvMarkerParamValuesEnd )
		{
			sprintf( STVERRTXT, "Parse Error: Expected %s = %d, Got %d",
				"stvMarkerParamValuesEnd", stvMarkerParamValuesEnd,
				tmp );

			STV_ERROR( "stv_unpack_elems_SubRegion",
				stvStatusBadParam, STVERRTXT );

			return( stvStatusBadParam );
		}
	}

	return( stvStatusOk );
}


/* Generate Unpack Offset */

int
stv_get_unpack_offset( indices, arrayDecl, D, VR,
	view_storage_order, field_storage_order )
int *indices;
int *arrayDecl;
STV_DECOMPPTR D;
STV_VIS_REGION VR;
int view_storage_order;
int field_storage_order;
{
	int b, e, s;
	int offset;
	int index;
	int size;
	int step;
	int min;
	int i;

	if ( view_storage_order == stvColumnMajor )
	{
		b = 0;
		e = D->dataDim;
		s = 1;
	}

	else if ( view_storage_order == stvRowMajor )
	{
		b = D->dataDim - 1;
		e = -1;
		s = -1;
	}

	offset = 0;

	size = 1;

	if ( STV_OPT_VERBOSE & STV_DEBUG_PACK )
	{
		printf( "%s: offset=%d size=%d\n",
			"stv_get_unpack_offset()", offset, size );
	}

	for ( i=b ; i != e ; i += s )
	{
		if ( STV_OPT_FULL_UNPACK )
		{
			min = D->glb[i];

			step = 1;
		}

		else
		{
			min = STV_REGION_MIN( VR->region, i );

			step = STV_CELL_OF_DIM( VR->cell, i );
		}

		if ( field_storage_order == stvRowMajor )
			index = D->dataDim - i - 1;
		
		else
			index = i;

		offset += ( ( indices[ index ] - min ) / step ) * size;

		size *= arrayDecl[i];

		if ( STV_OPT_VERBOSE & STV_DEBUG_PACK )
		{
			printf( "\t-> offset=%d size=%d ", offset, size );
			printf( "indices[%d]=%d min=%d step=%d arrayDecl[%d]=%d\n",
				index, indices[ index ], min, step, i, arrayDecl[i] );
		}
	}

	return( offset );
}


/* Get Cross-Storage-Order Striding Factor */

int
stv_get_storage_stride( arrayDecl, D,
	view_storage_order, field_storage_order )
int *arrayDecl;
STV_DECOMPPTR D;
int view_storage_order;
int field_storage_order;
{
	int stride;
	int i;

	stride = 1;

	if ( view_storage_order != field_storage_order )
	{
		if ( view_storage_order == stvColumnMajor )
		{
			for ( i=0 ; i < D->dataDim - 1 ; i++ )
				stride *= arrayDecl[i];
		}

		else if ( view_storage_order == stvRowMajor )
		{
			for ( i=(D->dataDim - 1) ; i > 0 ; i-- )
				stride *= arrayDecl[i];
		}
	}

	return( stride );
}

