
/* $Id: stvfulib.m4,v 4.5 2002/11/12 20:14:31 kohl Exp $ */


/*
 *                      CUMULVS version 1.0:
 *      Computational Steering and Interactive Visualization
 *                   in Distributed Applications
 *           Oak Ridge National Laboratory, Oak Ridge TN.
 *      Authors:  James Arthur Kohl and Philip M. Papadopoulos
 *                   (C) 1996 All Rights Reserved
 *
 *                              NOTICE
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted
 * provided that the above copyright notice appear in all copies and
 * that both the copyright notice and this permission notice appear
 * in supporting documentation.
 *
 * Neither the Institution, Oak Ridge National Laboratory, nor the
 * Authors make any representations about the suitability of this
 * software for any purpose.  This software is provided ``as is''
 * without express or implied warranty.
 *
 * CUMULVS was funded by the U.S. Department of Energy.
 */


/* stvfulib.m4 - macros to build FORTRAN interface for stv_ulib  */


#include <signal.h>
#include <stdio.h>

#include "stv.h"
#include "stvmac.h"
#include "stv_glob.h"


void
FUNCTION(stvfinit) ARGS(`STRING_ARG(appname), msgtag, ntasks, nodeid, info')
STRING_ARG_DECL(appname);
int *msgtag, *ntasks, *nodeid, *info;
{
	char cappname[ stvMaxNameLen + 1 ];

	int cc;

	/*
	 * Copy the appname name to make sure there's
	 * a NUL at the end.
	 */

	if ( stv_f_to_c_str( cappname, sizeof(cappname),
			STRING_PTR(appname), STRING_LEN(appname) ) )
	{
		if ( info != NULL )
			*info = stvStatusBadParam;
		return;
	}

	cc = stv_init( cappname, *msgtag, *ntasks, *nodeid );

	if ( info != NULL )
		*info = cc;

	return;
}


void
FUNCTION(stvfdecompdefine) ARGS(`dataDim, axisType, axisInfo1, axisInfo2, glb, gub, prank, pshape, decompId')
int *dataDim, *axisType, *axisInfo1, *axisInfo2, *glb, *gub, *prank, *pshape, *decompId;
{
	int *axisTypeActual;
	int *axisInfo1Actual;
	int *axisInfo2Actual;
	int *pshapeActual;

	int cc;

	/* Check for STVLOCALARRAY */
	if ( axisType[0] < 0 )
	{
		axisTypeActual = stvLocalArray;

		axisInfo1Actual = stvLocalArray;
		axisInfo2Actual = stvLocalArray;
	}

	/* Check for STVDEFAULTINFO */
	else if ( axisType[0] == stvBlock || axisType[0] == stvCyclic )
	{
		axisTypeActual = axisType;

		axisInfo1Actual = ( axisInfo1[0] < 0 )
				? stvDefaultInfo : axisInfo1;

		axisInfo2Actual = ( axisInfo2[0] < 0 )
				? stvDefaultInfo : axisInfo2;
	}

	/* Assume Normal Arguments */
	else
	{
		axisTypeActual = axisType;

		axisInfo1Actual = axisInfo1;
		axisInfo2Actual = axisInfo2;
	}

	/* Check for STVLOCALARRAY */
	pshapeActual = ( pshape[0] < 0 ) ? stvLocalArray : pshape;

	cc = stv_decompDefine( *dataDim, axisTypeActual, 
		axisInfo1Actual, axisInfo2Actual, glb, gub,
		*prank, pshapeActual );

	if ( decompId != NULL )
		*decompId = cc;

	return;
}


void
FUNCTION(stvfdecompaddexplicitbounds) ARGS(`decompId, axis, lowerBound, upperBound, info')
int *decompId, *axis, *lowerBound, *upperBound, *info;
{
	int cc;

	cc = stv_decompAddExplicitBounds( *decompId, *axis, 
		*lowerBound, *upperBound );

	if ( info != NULL )
		*info = cc;

	return;
}


void
FUNCTION(stvfdecompaddexplicitvbounds) ARGS(`decompId, axis, lowerBounds, upperBounds, nbounds, info')
int *decompId, *axis, *lowerBounds, *upperBounds, *nbounds, *info;
{
	int cc;

	cc = stv_decompAddExplicitVBounds( *decompId, *axis, 
		lowerBounds, upperBounds, *nbounds );

	if ( info != NULL )
		*info = cc;

	return;
}


void
FUNCTION(stvfdecompaddexplicitpatch) ARGS(`decompId, lowerBounds, upperBounds, info')
int *decompId, *lowerBounds, *upperBounds, *info;
{
	int cc;

	cc = stv_decompAddExplicitPatch( *decompId,
		lowerBounds, upperBounds );

	if ( info != NULL )
		*info = cc;

	return;
}


void
FUNCTION(stvffielddefine) ARGS(`var, STRING_ARG(name), decompID, arrayOffset, arrayDecl, type, paddr, aflag, fieldId')
STRING_ARG_DECL(name);
STV_VALUE var;
int *decompID, *arrayOffset, *arrayDecl, *type, *paddr, *aflag,
	*fieldId;
{
	char cvarname[ stvMaxNameLen + 1 ];

	int *arrayOffsetActual;
	int *paddrActual;

	int cc;

	/*
	 * Copy the name to make sure there's
	 * a NUL at the end.
	 */

	if ( stv_f_to_c_str( cvarname, sizeof(cvarname),
			STRING_PTR(name), STRING_LEN(name) ) )
	{
		if ( fieldId != NULL )
			*fieldId = stvStatusBadParam;
		return;
	}

	/* Check for STVNOFIELDOFFSETS */
	arrayOffsetActual = ( arrayOffset[0] < 0 )
			? stvNoFieldOffsets : arrayOffset;

	/* Check for STVLOCALARRAY */
	paddrActual = ( paddr[0] < 0 ) ? stvLocalArray : paddr;

	cc = stv_fieldDefine( var, cvarname, *decompID,
			arrayOffsetActual, arrayDecl, *type, paddrActual, *aflag );

	stv_setopt( cc, stvOptFieldStorage, stvColumnMajor );

	if ( fieldId != NULL )
		*fieldId = cc;

	return;
}


void
FUNCTION(stvfparamdefine) ARGS(`var, STRING_ARG(name), type, aflag, paramId')
STRING_ARG_DECL(name);
STV_VALUE var;
int  *type, *aflag, *paramId;
{
	char cvarname[ stvMaxNameLen + 1 ];

	int cc;

	/*
	 * Copy the name to make sure there's
	 * a NUL at the end.
	 */

	if ( stv_f_to_c_str( cvarname, sizeof(cvarname),
			STRING_PTR(name), STRING_LEN(name) ) )
	{
		if ( paramId != NULL )
			*paramId = stvStatusBadParam;
		return;
	}

	cc = stv_paramDefine( cvarname, var, *type, *aflag );

	if ( paramId != NULL )
		*paramId = cc;

	return;
}


void
FUNCTION(stvfparticledefine) ARGS(`STRING_ARG(name), dataDim, glb, gub, ntasks, get_particle, get_particle_arg, make_particle, make_particle_arg, particleId')
STRING_ARG_DECL(name);
int *dataDim, *glb, *gub, *ntasks, *particleId;
STV_GET_PARTICLE get_particle;
STV_VALUE get_particle_arg;
STV_MAKE_PARTICLE make_particle;
STV_VALUE make_particle_arg;
{
	char cvarname[ stvMaxNameLen + 1 ];

	char *tmp;

	int cc;

	/*
	 * Copy the name to make sure there's
	 * a NUL at the end.
	 */

	if ( stv_f_to_c_str( cvarname, sizeof(cvarname),
			STRING_PTR(name), STRING_LEN(name) ) )
	{
		if ( particleId != NULL )
			*particleId = stvStatusBadParam;
		return;
	}

	cc = stv_particleDefine( cvarname, *dataDim, glb, gub, *ntasks,
		get_particle, get_particle_arg,
		make_particle, make_particle_arg );

	if ( particleId != NULL )
		*particleId = cc;

	return;
}


void
FUNCTION(stvfparticlefielddefine) ARGS(`STRING_ARG(name), particleId, type, nelems, get_pfield, set_pfield, aflag, fieldId')
STRING_ARG_DECL(name);
int *particleId, *type, *nelems;
STV_GET_PFIELD get_pfield;
STV_SET_PFIELD set_pfield;
int *aflag, *fieldId;
{
	char cvarname[ stvMaxNameLen + 1 ];

	char *tmp;

	int cc;

	/*
	 * Copy the name to make sure there's
	 * a NUL at the end.
	 */

	if ( stv_f_to_c_str( cvarname, sizeof(cvarname),
			STRING_PTR(name), STRING_LEN(name) ) )
	{
		if ( fieldId != NULL )
			*fieldId = stvStatusBadParam;
		return;
	}

	cc = stv_particleFieldDefine( cvarname, *particleId, 
		*type, *nelems, get_pfield, set_pfield, *aflag );

	if ( fieldId != NULL )
		*fieldId = cc;

	return;
}


void
FUNCTION(stvfpfielddefine) ARGS(`STRING_ARG(name), particleId, type, nelems, get_pfield, get_pfield_arg, set_pfield, set_pfield_arg, aflag, fieldId')
STRING_ARG_DECL(name);
int *particleId, *type, *nelems;
STV_GET_PFIELD get_pfield;
STV_VALUE get_pfield_arg;
STV_SET_PFIELD set_pfield;
STV_VALUE set_pfield_arg;
int *aflag, *fieldId;
{
	char cvarname[ stvMaxNameLen + 1 ];

	char *tmp;

	int cc;

	/*
	 * Copy the name to make sure there's
	 * a NUL at the end.
	 */

	if ( stv_f_to_c_str( cvarname, sizeof(cvarname),
			STRING_PTR(name), STRING_LEN(name) ) )
	{
		if ( fieldId != NULL )
			*fieldId = stvStatusBadParam;
		return;
	}

	cc = stv_pfieldDefine( cvarname,
		*particleId, *type, *nelems,
		get_pfield, get_pfield_arg, set_pfield, set_pfield_arg,
		*aflag );

	if ( fieldId != NULL )
		*fieldId = cc;

	return;
}


void
FUNCTION(stvfparticlesetlocalsubregion) ARGS(`particleId, lbounds, ubounds, info')
int *particleId, *lbounds, *ubounds, *info;
{
	int cc;

	cc = stv_particle_set_local_subregion( *particleId,
		lbounds, ubounds );

	if ( info != NULL )
		*info = cc;

	return;
}


void
FUNCTION(stvfcouplefields) ARGS(`fieldID, STRING_ARG(appname), STRING_ARG(fieldname), dstFreq, srcFreq, dstNoSynch, srcNoSynch, block, couplerId')
STRING_ARG_DECL(appname);
STRING_ARG_DECL(fieldname);
int *fieldID, *dstFreq, *srcFreq, *dstNoSynch, *srcNoSynch, *block,
	*couplerId;
{
	char cappname[ stvMaxNameLen + 1 ];
	char cfieldname[ stvMaxNameLen + 1 ];

	int cc;

	/*
	 * Copy the name to make sure there's
	 * a NUL at the end.
	 */

	if ( stv_f_to_c_str( cappname, sizeof(cappname),
			STRING_PTR(appname), STRING_LEN(appname) ) )
	{
		if ( couplerId != NULL )
			*couplerId = stvStatusBadParam;
		return;
	}

	if ( stv_f_to_c_str( cfieldname, sizeof(cfieldname),
			STRING_PTR(fieldname), STRING_LEN(fieldname) ) )
	{
		if ( couplerId != NULL )
			*couplerId = stvStatusBadParam;
		return;
	}

	cc = stv_couple_fields( *fieldID, cappname, cfieldname,
			*dstFreq, *srcFreq, *dstNoSynch, *srcNoSynch, *block );

	if ( couplerId != NULL )
		*couplerId = cc;

	return;
}


void
FUNCTION(stvfdumplocalfieldbounds) ARGS(`fieldId, verbose, info')
int *fieldId;
int *verbose;
int *info;
{
	int cc;

	cc = stv_dumpLocalFieldBounds( *fieldId, *verbose );

	if ( info != NULL )
		*info = cc;

	return;
}


void
FUNCTION(stvfsendtofe) ARGS(`info')
int *info;
{
	int cc;

	cc = stv_sendToFE();

	if ( info != NULL )
		*info = cc;

	return;
}


void
FUNCTION(stvfsendreadydata) ARGS(`sendflag, info')
int *sendflag, *info;
{
	int cc;

	cc = stv_sendReadyData( *sendflag );

	if ( info != NULL )
		*info = cc;

	return;
}


void
FUNCTION(stvfsendreadydatabegin) ARGS(`sendflag, info')
int *sendflag, *info;
{
	int cc;

	cc = stv_sendReadyDataBegin( *sendflag );

	if ( info != NULL )
		*info = cc;

	return;
}


void
FUNCTION(stvfsendreadydataend) ARGS(`sendflag, info')
int *sendflag, *info;
{
	int cc;

	cc = stv_sendReadyDataEnd( *sendflag );

	if ( info != NULL )
		*info = cc;

	return;
}


void
FUNCTION(stvfsetopt) ARGS(`what, option, value, oldvalue')
int *what;
int *option;
int *value;
int *oldvalue;
{
	int cc;

	cc = stv_setopt( *what, *option, *value );

	if ( oldvalue != NULL )
		*oldvalue = cc;

	return;
}


void
FUNCTION(stvfgetopt) ARGS(`what, option, value')
int *what;
int *option;
int *value;
{
	*value = stv_getopt( *what, *option );

	return;
}


void
FUNCTION(stvftimeof) ARGS(`value, units, tvalue')
int *value;
int *units;
int *tvalue;
{
	*tvalue = stv_time_of( *value, *units );

	return;
}


void
FUNCTION(stvfvalueoftime) ARGS(`tvalue, value')
int *tvalue;
int *value;
{
	*value = stv_value_of_time( *tvalue );

	return;
}


void
FUNCTION(stvfunitoftime) ARGS(`tvalue, units')
int *tvalue;
int *units;
{
	*units = stv_unit_of_time( *tvalue );

	return;
}


void
FUNCTION(stvfistimevalue) ARGS(`tvalue, info')
int *tvalue;
int *info;
{
	*info = stv_is_time_value( *tvalue );

	return;
}


void
FUNCTION(stvfsleep) ARGS(`secs')
int *secs;
{
	stvSleep( (long) *secs );

	return;
}


void
FUNCTION(stvfsigtrap) ARGS(`info')
int *info;
{
	extern void stvsigquit();

	signal( SIGQUIT, stvsigquit );

	signal( SIGTERM, stvsigquit );

	if ( info != NULL )
		*info = 0;

	return;
}


void
FUNCTION(stvfisparamchanged) ARGS(`paramId, info')
int *paramId;
int *info;
{
	*info = stv_isParamChanged(*paramId);

	return;
}


void
FUNCTION(stvfversion) ARGS(`STRING_ARG(version)')
STRING_ARG_DECL(version);
{
	char *rcopy;

	rcopy = stv_version();

	stv_c_to_f_str( STRING_PTR(version), STRING_LEN(version), rcopy );

	return;
}


void
FUNCTION(stvfversionname) ARGS(`STRING_ARG(name)')
STRING_ARG_DECL(name);
{
	char *rcopy;

	rcopy = stv_version_name();

	stv_c_to_f_str( STRING_PTR(name), STRING_LEN(name), rcopy );

	return;
}

